;*********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                *
;*        Solutions for real time microcontroller applications       *
;*********************************************************************
;*                                                                   *
;*      (C) 2002-2004    SEGGER Microcontroller Systeme GmbH         *
;*                       www.segger.com                              *
;*                                                                   *
;*********************************************************************
;
;---------------------------------------------------------------------
;File   : AT91M55800_CStartup.s
;Purpose: startup file for AT91M55800 and OS
;-------- END-OF-HEADER ----------------------------------------------
;

;*********************************************************************
;*
;*      Configuration
;*
;*********************************************************************

#if   defined (TARGET_JTAG)      // Used for debugging.
  #define COPY_CODE_TO_RAM   0   // Code is already in RAM
  #define REMAP              0   // Already done via JTAG
#elif defined (TARGET_ROM)       // Code runs from ROM (a bit slower typically)
  #define COPY_CODE_TO_RAM   0
  #define REMAP              1
#elif defined (TARGET_RAM)       // Code runs from RAM (faster)
  #define COPY_CODE_TO_RAM   1
  #define REMAP              1
#else
  #error TARGET selection define missing
#endif

;********************************************************************
;*
;*      Configuration check
;*
;********************************************************************

#ifndef  COPY_CODE_TO_RAM
  #error COPY_CODE_TO_RAM not defined
#endif

#ifndef  REMAP
  #error REMAP not defined
#endif

;---------------------------------------------------------------
; ?RESET
; Reset Vector.
; Normally, segment INTVEC is linked at address 0.
; For debugging purposes, INTVEC may be placed at other
; addresses.
; A debugger that honors the entry point will start the
; program in a normal way even if INTVEC is not at address 0.
;---------------------------------------------------------------

        MODULE  ?cstartup

        ;; Forward declaration of sections.
        SECTION IRQ_STACK:DATA:NOROOT(3)
        SECTION FIQ_STACK:DATA:NOROOT(3)
        SECTION CSTACK:DATA:NOROOT(3)

        SECTION .intvec:CODE:NOROOT(2)

        PUBLIC  __vector
        PUBLIC  __vector_0x14
        PUBLIC  __iar_program_start
        EXTERN  Undefined_Handler
        EXTERN  SWI_Handler
        EXTERN  Prefetch_Handler
        EXTERN  Abort_Handler
        EXTERN  IRQ_Handler
        EXTERN  FIQ_Handler

        ARM

__iar_init$$done:               ; The vector table is not needed
                                ; until after copy initialization is done

__vector:
        ; All default exception handlers (except reset) are
        ; defined as weak symbol definitions.
        ; If a handler is defined by the application it will take precedence.

        b       __iar_program_start     ; To allow remap, a relative branch is required !

        LDR     PC,Undefined_Addr       ; Undefined instructions
        LDR     PC,SWI_Addr             ; Software interrupt (SWI/SVC)
        LDR     PC,Prefetch_Addr        ; Prefetch abort
        LDR     PC,Abort_Addr           ; Data abort
__vector_0x14:
        DCD     0                       ; RESERVED
        LDR     PC,IRQ_Addr             ; IRQ
        LDR     PC,FIQ_Addr             ; FIQ

        DATA
    
Undefined_Addr: DCD   Undefined_Handler
SWI_Addr:       DCD   SWI_Handler
Prefetch_Addr:  DCD   Prefetch_Handler
Abort_Addr:     DCD   Abort_Handler
IRQ_Addr:       DCD   IRQ_Handler
FIQ_Addr:       DCD   FIQ_Handler

__vector_end:

;---------------------------------------------------------------
; CSTARTUP
;---------------------------------------------------------------

        SECTION .text:CODE:NOROOT(2)

        EXTERN  ?main
        REQUIRE __vector

        ARM

__iar_program_start:

#if REMAP
        ; The memory controller is initialized immediately before the remap
        ldr     r10, =EBI_init_table ; EBI register initialization table
        ; If pc > 0x100000
        movs    r0, pc, LSR #20
        ; Mask the 12 highest bits of the address
        moveq   r10, r10, LSL #12
        moveq   r10, r10, LSR #12

        ; Load the address where to jump
        ldr     r12, =after_remap ; get the real jump address ( after remap )

        ; Copy chip select register image to memory controller and command remap
        ldmia   r10!, {r0-r9,r11}       ; load the complete image and the EBI base
        stmia   r11!, {r0-r9}           ; store the complete image with the remap command

        ; jump to ROM at its new address
        ; this instruction was loaded into the pipeline before the remap was done
        mov         pc, r12             ; jump and break the pipeline

        ; Put constant table here.
        LTORG

        ; EBI initialization table
        ; 32,768 MHz master clock assumed for timing

EBI_init_table:
        dc32    0x01000000+0x2529       ; CS0: Flash at 0x01000000, 16MB, 2 hold, 16 bits, 3 WS
        dc32    0x04000000+0x30A1       ; CS1: RAM   at 0x04000000, 16MB, 0 hold, 16 bits, 1 WS
        dc32    0x20003431              ; CS2: Ethernet controller on Phycore board
        dc32    0x30000000              ; CS3:
        dc32    0x40000000              ; CS4:
        dc32    0x06002429              ; CS5: LCD controller on SEGGER hardware
        dc32    0x60000000              ; CS6: unused
        dc32    0x70000000              ; CS7: unused
        dc32    0x00000001              ; REMAP command
        dc32    0x00000010              ; early read
        dc32    0xFFE00000              ; EBI Base address

after_remap:

// COPY vectors to RAM
        ldr     r0, = __vector
        ldr     r1, = 0x00000
        ldr     r2, = (__vector_end - __vector)/4 ; Copy words

        ldr     r3,[r0], #+4
        str     r3,[R1], #+4
        ldr     r3,[r0], #+4
        str     r3,[R1], #+4
        ldr     r3,[r0], #+4
//        str     r3,[R1], #+4    // Skip SWI vector, because debugger might have set a breakpoint there when RAM target is used
        add     r1,r1, #+4

        subs    r2,r2,#+0x3
copy_vector_loop:
        ldr     r3,[r0], #+4
        str     r3,[R1], #+4
        subs    r2,r2,#+0x1
        bne     copy_vector_loop
#endif

; Execute normal C startup code.
; Add initialization nedded before setup of stackpointers here


; Initialize the stack pointers.
; The pattern below can be used for any of the exception stacks:
; FIQ, IRQ, SVC, ABT, UND, SYS.
; The USR mode uses the same stack as SYS.
; The stack segments must be defined in the linker command file,
; and be declared above.

?cstartup:
        ;; Set up the interrupt stack pointer.

        msr     cpsr_c,#0xd2            ; Change mode to IRQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(IRQ_STACK)     ; End of IRQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary
	
        ;; Set up the fast interrupt stack pointer.

        msr     cpsr_c,#0xd1            ; Change mode to FIQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(FIQ_STACK)     ; End of FIQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary

        ;; Set up the normal stack pointer.

        msr     cpsr_c,#0xdf            ; Change mode to SYS mode, FIQ and IRQ disabled
        ldr     sp, =SFE(CSTACK)        ; End of CSTACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary


; Continue to ?main for more IAR specific system startup

        ldr     r0,=?main
        bx      r0


; ********************************************************************

        END

; ***** End of File **************************************************
