/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : UART implementation

Additional information:

  Device : AT91R55800
  Board  : AT91EB55

  Unit | UART   | Board connector
  ===============================
  0    | USART0 | Serial A
  1    | USART1 | Serial B
  2    | USART2 | -

*/

#include "BSP_UART.h"
#include "RTOS.h"      // For OS_ARM_* and OS_INT_* functions.

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/

#define SYSTEM_FREQ     (32000000u)
#define USART_FREQ      (SYSTEM_FREQ)

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/
//
// Peripheral clock enable register
//
#define PCER                  (*(volatile unsigned int*)0xFFFF4010u)
//
// PIO control register
//
#define PIO_PDR               (*(volatile unsigned int*)0xFFFEC004u)
#define PIO_PDR_USART0        ((1u << 15) | (1u << 16))
#define PIO_PDR_USART1        ((1u << 18) | (1u << 19))
#define PIO_PDR_USART2        ((1u << 21) | (1u << 22))
//
// Advanced interrupt controller (AIC)
//
#define AIC_SMR2              (*(volatile unsigned int*)0xFFFFF008u)
#define AIC_SMR3              (*(volatile unsigned int*)0xFFFFF00Cu)
#define AIC_SMR4              (*(volatile unsigned int*)0xFFFFF010u)
#define AIC_SVR2              (*(volatile unsigned int*)0xFFFFF088u)
#define AIC_SVR3              (*(volatile unsigned int*)0xFFFFF08Cu)
#define AIC_SVR4              (*(volatile unsigned int*)0xFFFFF090u)
#define AIC_ICCR              (*(volatile unsigned int*)0xFFFFF128u)
//
// USART0
//
#define USART0_BASE_ADDR      (0xFFFC0000u)
#define US0_CR                (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x00u))
#define US0_MR                (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x04u))
#define US0_IER               (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x08u))
#define US0_IDR               (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x0Cu))
#define US0_IMR               (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x10u))
#define US0_CSR               (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x14u))
#define US0_RHR               (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x18u))
#define US0_THR               (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x1Cu))
#define US0_BRGR              (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x20u))
#define US0_RTOR              (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x24u))
#define US0_TTGR              (*(volatile unsigned int*)(USART0_BASE_ADDR + 0x28u))
//
// USART1
//
#define USART1_BASE_ADDR      (0xFFFC4000u)
#define US1_CR                (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x00u))
#define US1_MR                (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x04u))
#define US1_IER               (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x08u))
#define US1_IDR               (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x0Cu))
#define US1_IMR               (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x10u))
#define US1_CSR               (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x14u))
#define US1_RHR               (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x18u))
#define US1_THR               (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x1Cu))
#define US1_BRGR              (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x20u))
#define US1_RTOR              (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x24u))
#define US1_TTGR              (*(volatile unsigned int*)(USART1_BASE_ADDR + 0x28u))
//
// USART2
//
#define USART2_BASE_ADDR      (0xFFFC8000u)
#define US2_CR                (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x00u))
#define US2_MR                (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x04u))
#define US2_IER               (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x08u))
#define US2_IDR               (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x0Cu))
#define US2_IMR               (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x10u))
#define US2_CSR               (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x14u))
#define US2_RHR               (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x18u))
#define US2_THR               (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x1Cu))
#define US2_BRGR              (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x20u))
#define US2_RTOR              (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x24u))
#define US2_TTGR              (*(volatile unsigned int*)(USART2_BASE_ADDR + 0x28u))
//
//  Advanced interrupt controller interrupt sources
//
#define USART0_IRQ_ID         (2)      // USART Channel 0 interrupt
#define USART1_IRQ_ID         (3)      // USART Channel 1 interrupt
#define USART2_IRQ_ID         (4)      // USART Channel 2 interrupt

#define USART_IRQ_PRIO        (0)

#define USART_RXRDY           (0x01u)  // Rx status flag
#define USART_TXRDY           (0x02u)  // Tx Status flag
#define USART_RX_ERROR_FLAGS  (0xE0u)  // Parity, framing, overrun error

#define USART0                (0)
#define USART1                (1)
#define USART2                (2)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static BSP_UART_TX_CB* _pfWriteCB;
static BSP_UART_RX_CB* _pfReadCB;

/*********************************************************************
*
*       Prototypes
*
*  Declare ISR handler here to avoid "no prototype" warning.
*  They are not declared in any CMSIS header.
*
**********************************************************************
*/

#if defined(__cplusplus)
  extern "C" {                // Make sure we have C-declarations in C++ programs.
#endif

static void USART0_RX_TX_IRQHandler(void);
static void USART1_RX_TX_IRQHandler(void);
static void USART2_RX_TX_IRQHandler(void);

#if defined(__cplusplus)
}                             // Make sure we have C-declarations in C++ programs.
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _SetBaudrate()
*
*  Function description
*    Configures the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
static void _SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Baudrate);
}

/*********************************************************************
*
*       Global functions, IRQ handler
*
**********************************************************************
*/

/*********************************************************************
*
*       USART0_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void USART0_RX_TX_IRQHandler(void) {
  unsigned int Status;
  unsigned int Unit;
  unsigned int Data;

  Unit = USART0;
  do {
    Status = US0_CSR;                       // Examine status register
    if (Status & USART_RXRDY) {             // Data received?
      Data = US0_RHR;
      if (Status & USART_RX_ERROR_FLAGS) {  // Any error ?
        US0_CR = (1u << 8);                 // RSTSTA: Reset Status Bits PARE, FRAME, OVRE and RXBRK
      } else {
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);            // Forward the received byte to the callback.
        }
      }
    }
    if (Status & US0_IMR & USART_TXRDY) {   // Check Tx status => Send next character
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {
        US0_IDR = USART_TXRDY;              // Disable further tx interrupts
      }
    }
  } while (Status & US0_IMR & (USART_TXRDY | USART_RXRDY));
}

/*********************************************************************
*
*       USART1_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void USART1_RX_TX_IRQHandler(void) {
  unsigned int Status;
  unsigned int Unit;
  unsigned int Data;

  Unit = USART1;
  do {
    Status = US1_CSR;                       // Examine status register
    if (Status & USART_RXRDY) {             // Data received?
      Data = US1_RHR;
      if (Status & USART_RX_ERROR_FLAGS) {  // Any error ?
        US1_CR = (1u << 8);                 // RSTSTA: Reset Status Bits PARE, FRAME, OVRE and RXBRK
      } else {
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);            // Forward the received byte to the callback.
        }
      }
    }
    if (Status & US1_IMR & USART_TXRDY) {   // Check Tx status => Send next character
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {
        US1_IDR = USART_TXRDY;              // Disable further tx interrupts
      }
    }
  } while (Status & US1_IMR & (USART_TXRDY | USART_RXRDY));
}

/*********************************************************************
*
*       USART2_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void USART2_RX_TX_IRQHandler(void) {
  unsigned int Status;
  unsigned int Unit;
  unsigned int Data;

  Unit = USART2;
  do {
    Status = US2_CSR;                       // Examine status register
    if (Status & USART_RXRDY) {             // Data received?
      Data = US2_RHR;
      if (Status & USART_RX_ERROR_FLAGS) {  // Any error ?
        US2_CR = (1u << 8);                 // RSTSTA: Reset Status Bits PARE, FRAME, OVRE and RXBRK
      } else {
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);            // Forward the received byte to the callback.
        }
      }
    }
    if (Status & US2_IMR & USART_TXRDY) {   // Check Tx status => Send next character
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {
        US2_IDR = USART_TXRDY;              // Disable further tx interrupts
      }
    }
  } while (Status & US2_IMR & (USART_TXRDY | USART_RXRDY));
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_UART_Init()
*
*  Function description
*    Initializes the UART peripheral.
*
*  Parameters
*    Unit       : Unit number (typically zero-based).
*    Baudrate   : Baud rate to configure [Hz].
*    NumDataBits: Number of data bits to use.
*    Parity     : One of the following values:
*                   * BSP_UART_PARITY_NONE
*                   * BSP_UART_PARITY_ODD
*                   * BSP_UART_PARITY_EVEN
*    NumStopBits: Number of stop bits to use.
*/
void BSP_UART_Init(unsigned int Unit, unsigned long Baudrate, unsigned char NumDataBits, unsigned char Parity, unsigned char NumStopBits) {
  BSP_UART_USE_PARA(NumDataBits);
  BSP_UART_USE_PARA(Parity);
  BSP_UART_USE_PARA(NumStopBits);
  OS_INT_IncDI();
  switch (Unit) {
  case USART0:
    PCER     = (1u << USART0_IRQ_ID);  // Enable peripheral clock for selected USART
    PIO_PDR  = PIO_PDR_USART0;         // Enable peripheral output signals (disable PIO Port A)
    US0_CR   = (1u <<  2)              // RSTRX:  Reset Receiver: 1 = The receiver logic is reset.
             | (1u <<  3);             // RSTTX:  Reset Transmitter: 1 = The transmitter logic is reset.
    US0_CR   = (0u <<  2)              // RSTRX:  Reset Receiver: 1 = The receiver logic is reset.
             | (0u <<  3)              // RSTTX:  Reset Transmitter: 1 = The transmitter logic is reset.
             | (1u <<  4)              // RXEN:   Receiver Enable: 1 = The receiver is enabled if RXDIS is 0.
             | (0u <<  5)              // RXDIS:  Receiver Disable: 0 = Noeffect.
             | (1u <<  6)              // TXEN:   Transmitter Enable: 1 = The transmitter is enabled if TXDIS is 0.
             | (0u <<  7)              // TXDIS:  Transmitter Disable: 0 = Noeffect.
             | (1u <<  8)              // RSTSTA: Reset Status Bits: 1 = Resets the status bits PARE, FRAME, OVRE and RXBRK in the US_CSR.
             | (0u <<  9)              // STTBRK: Start Break: 0 = Noeffect.
             | (0u << 10)              // STPBRK: Stop Break: 0 = Noeffect.
             | (0u << 11)              // STTTO:  Start Time-out: 0 = Noeffect.
             | (0u << 12);             // SENDA:  Send Address: 0 = Noeffect.
    US0_MR   = (0u <<  4)              // USCLKS: Clock Selection: 0 = MCK
             | (3u <<  6)              // CHRL:   Character Length: 3 = Eight bits
             | (0u <<  8)              // SYNC:   Synchronous Mode Select: 0 = USART operates in Asynchronous Mode.
             | (4u <<  9)              // PAR:    Parity Type: 0x4 = No parity
             | (0u << 12)              // NBSTOP: Number of Stop Bits: 0 = 1 stop bit
             | (0u << 14)              // CHMODE: Channel Mode: 0 = Normal mode
             | (0u << 17)              // MODE9:  9-bit Character Length: 0 = CHRL defines character length.
             | (0u << 18);             // CKLO:   Clock Output Select: 0 = The USART does not drive the SCK pin.
    US0_BRGR = (USART_FREQ / (Baudrate * 16));
    US0_IER  = (1u <<  0)              // Enable Rx Interrupt
             | (1u <<  1);             // (Enable Tx Interrupt
    //
    // Setup interrupt controller for USART
    //
    OS_ARM_InstallISRHandler(USART0_IRQ_ID, &USART0_RX_TX_IRQHandler);  // OS UART interrupt handler vector
    OS_ARM_ISRSetPrio(USART0_IRQ_ID, USART_IRQ_PRIO);                   // Level sensitive, selected priority.
    AIC_ICCR = (1u << USART0_IRQ_ID);                                   // Clear OS usart interrupt.
    OS_ARM_EnableISR(USART0_IRQ_ID);                                    // Enable OS usart interrupts
    break;
  case USART1:
    PCER     = (1u << USART1_IRQ_ID);  // Enable peripheral clock for selected USART
    PIO_PDR  = PIO_PDR_USART1;         // Enable peripheral output signals (disable PIO Port A)
    US1_CR   = (1u <<  2)              // RSTRX:  Reset Receiver: 1 = The receiver logic is reset.
             | (1u <<  3);             // RSTTX:  Reset Transmitter: 1 = The transmitter logic is reset.
    US1_CR   = (0u <<  2)              // RSTRX:  Reset Receiver: 1 = The receiver logic is reset.
             | (0u <<  3)              // RSTTX:  Reset Transmitter: 1 = The transmitter logic is reset.
             | (1u <<  4)              // RXEN:   Receiver Enable: 1 = The receiver is enabled if RXDIS is 0.
             | (0u <<  5)              // RXDIS:  Receiver Disable: 0 = Noeffect.
             | (1u <<  6)              // TXEN:   Transmitter Enable: 1 = The transmitter is enabled if TXDIS is 0.
             | (0u <<  7)              // TXDIS:  Transmitter Disable: 0 = Noeffect.
             | (1u <<  8)              // RSTSTA: Reset Status Bits: 1 = Resets the status bits PARE, FRAME, OVRE and RXBRK in the US_CSR.
             | (0u <<  9)              // STTBRK: Start Break: 0 = Noeffect.
             | (0u << 10)              // STPBRK: Stop Break: 0 = Noeffect.
             | (0u << 11)              // STTTO:  Start Time-out: 0 = Noeffect.
             | (0u << 12);             // SENDA:  Send Address: 0 = Noeffect.
    US1_MR   = (0u <<  4)              // USCLKS: Clock Selection: 0 = MCK
             | (3u <<  6)              // CHRL:   Character Length: 3 = Eight bits
             | (0u <<  8)              // SYNC:   Synchronous Mode Select: 0 = USART operates in Asynchronous Mode.
             | (4u <<  9)              // PAR:    Parity Type: 0x4 = No parity
             | (0u << 12)              // NBSTOP: Number of Stop Bits: 0 = 1 stop bit
             | (0u << 14)              // CHMODE: Channel Mode: 0 = Normal mode
             | (0u << 17)              // MODE9:  9-bit Character Length: 0 = CHRL defines character length.
             | (0u << 18);             // CKLO:   Clock Output Select: 0 = The USART does not drive the SCK pin.
    US1_BRGR = (USART_FREQ / (Baudrate * 16));
    US1_IER  = (1u <<  0)              // Enable Rx Interrupt
             | (1u <<  1);             // (Enable Tx Interrupt
    //
    // Setup interrupt controller for USART
    //
    OS_ARM_InstallISRHandler(USART1_IRQ_ID, &USART1_RX_TX_IRQHandler);  // OS UART interrupt handler vector
    OS_ARM_ISRSetPrio(USART1_IRQ_ID, USART_IRQ_PRIO);                   // Level sensitive, selected priority.
    AIC_ICCR = (1u << USART1_IRQ_ID);                                   // Clear OS usart interrupt.
    OS_ARM_EnableISR(USART1_IRQ_ID);                                    // Enable OS usart interrupts
    break;
  case USART2:
    PCER     = (1u << USART2_IRQ_ID);  // Enable peripheral clock for selected USART
    PIO_PDR  = PIO_PDR_USART2;         // Enable peripheral output signals (disable PIO Port A)
    US2_CR   = (1u <<  2)              // RSTRX:  Reset Receiver: 1 = The receiver logic is reset.
             | (1u <<  3);             // RSTTX:  Reset Transmitter: 1 = The transmitter logic is reset.
    US2_CR   = (0u <<  2)              // RSTRX:  Reset Receiver: 1 = The receiver logic is reset.
             | (0u <<  3)              // RSTTX:  Reset Transmitter: 1 = The transmitter logic is reset.
             | (1u <<  4)              // RXEN:   Receiver Enable: 1 = The receiver is enabled if RXDIS is 0.
             | (0u <<  5)              // RXDIS:  Receiver Disable: 0 = Noeffect.
             | (1u <<  6)              // TXEN:   Transmitter Enable: 1 = The transmitter is enabled if TXDIS is 0.
             | (0u <<  7)              // TXDIS:  Transmitter Disable: 0 = Noeffect.
             | (1u <<  8)              // RSTSTA: Reset Status Bits: 1 = Resets the status bits PARE, FRAME, OVRE and RXBRK in the US_CSR.
             | (0u <<  9)              // STTBRK: Start Break: 0 = Noeffect.
             | (0u << 10)              // STPBRK: Stop Break: 0 = Noeffect.
             | (0u << 11)              // STTTO:  Start Time-out: 0 = Noeffect.
             | (0u << 12);             // SENDA:  Send Address: 0 = Noeffect.
    US2_MR   = (0u <<  4)              // USCLKS: Clock Selection: 0 = MCK
             | (3u <<  6)              // CHRL:   Character Length: 3 = Eight bits
             | (0u <<  8)              // SYNC:   Synchronous Mode Select: 0 = USART operates in Asynchronous Mode.
             | (4u <<  9)              // PAR:    Parity Type: 0x4 = No parity
             | (0u << 12)              // NBSTOP: Number of Stop Bits: 0 = 1 stop bit
             | (0u << 14)              // CHMODE: Channel Mode: 0 = Normal mode
             | (0u << 17)              // MODE9:  9-bit Character Length: 0 = CHRL defines character length.
             | (0u << 18);             // CKLO:   Clock Output Select: 0 = The USART does not drive the SCK pin.
    US2_BRGR = (USART_FREQ / (Baudrate * 16));
    US2_IER  = (1u <<  0)              // Enable Rx Interrupt
             | (1u <<  1);             // (Enable Tx Interrupt
    //
    // Setup interrupt controller for USART
    //
    OS_ARM_InstallISRHandler(USART2_IRQ_ID, &USART2_RX_TX_IRQHandler);  // OS UART interrupt handler vector
    OS_ARM_ISRSetPrio(USART2_IRQ_ID, USART_IRQ_PRIO);                   // Level sensitive, selected priority.
    AIC_ICCR = (1u << USART2_IRQ_ID);                                   // Clear OS usart interrupt.
    OS_ARM_EnableISR(USART2_IRQ_ID);                                    // Enable OS usart interrupts
    break;
  }
  OS_INT_DecRI();
}

/*********************************************************************
*
*       BSP_UART_DeInit()
*
*  Function description
*    De-initializes the UART peripheral.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*/
void BSP_UART_DeInit(unsigned int Unit) {
  BSP_UART_USE_PARA(Unit);
}

/*********************************************************************
*
*       BSP_UART_SetBaudrate()
*
*  Function description
*    Configures/changes the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
void BSP_UART_SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  _SetBaudrate(Unit, Baudrate);
}

/*********************************************************************
*
*       BSP_UART_SetReadCallback()
*
*  Function description
*    Sets the callback to execute upon an Rx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetReadCallback(unsigned Unit, BSP_UART_RX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfReadCB = pf;
}

/*********************************************************************
*
*       BSP_UART_SetWriteCallback()
*
*  Function description
*    Sets the callback to execute upon a Tx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetWriteCallback(unsigned int Unit, BSP_UART_TX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfWriteCB = pf;
}

/*********************************************************************
*
*       BSP_UART_Write1()
*
*  Function description
*    Sends one byte via UART.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    Data: (First) data byte to send.
*
*  Additional information
*    The first byte of a transfer is typically sent from application
*    context. Further bytes of the transfer are then sent from the
*    Tx interrupt handler by also calling this function from interrupt
*    context.
*/
void BSP_UART_Write1(unsigned int Unit, unsigned char Data) {
  switch (Unit) {
  case USART0:
    US0_THR = Data;         // Send character
    US0_IER = USART_TXRDY;  // enable Tx Empty interrupt
    break;
  case USART1:
    US1_THR = Data;         // Send character
    US1_IER = USART_TXRDY;  // enable Tx Empty interrupt
    break;
  case USART2:
    US2_THR = Data;         // Send character
    US2_IER = USART_TXRDY;  // enable Tx Empty interrupt
    break;
  }
}

/*************************** End of file ****************************/
