/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   AIC_DEBUG_MODE
  #define AIC_DEBUG_MODE  (1)
#endif

/*********************************************************************
*
*       Clock settings
*/
#define OS_SYSTEM_FREQ   (179712000u)

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ    (OS_SYSTEM_FREQ / 3 / 2)
#define OS_TICK_FREQ     (1000u)
#define OS_INT_FREQ      (OS_TICK_FREQ)
#define OS_TIMER_RELOAD  (OS_TIMER_FREQ / OS_INT_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_DISABLED  // Disabled to reduce application size
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define SDRAM_BASE_ADDR  (0x20000000)  // base address to access memory on CS1

#define DELAY_PLL        (100)
#define DELAY_MAIN_FREQ  (100)

#define PLLAR_VALUE      (0x2026BF04)  // 179,712000 MHz for PCK
#define PLLBR_VALUE      (0x10483F0E)  // 48,054857 MHz (divide by 2 for USB clock)
#define MCKR_VALUE       (0x00000202)  // PCK/3 = MCK Master Clock = 59,904000MHz with PLLA selected

/*********************************************************************
*
*       peripheral ID definitions for AT91RM9200
*/
#define AT91C_ID_FIQ      ( 0u)  // Advanced Interrupt Controller (FIQ)
#define AT91C_ID_SYS      ( 1u)  // System Peripheral
#define AT91C_ID_PIOA     ( 2u)  // Parallel IO Controller A
#define AT91C_ID_PIOB     ( 3u)  // Parallel IO Controller B
#define AT91C_ID_PIOC     ( 4u)  // Parallel IO Controller C
#define AT91C_ID_PIOD     ( 5u)  // Parallel IO Controller D
#define AT91C_ID_US0      ( 6u)  // USART 0
#define AT91C_ID_US1      ( 7u)  // USART 1
#define AT91C_ID_US2      ( 8u)  // USART 2
#define AT91C_ID_US3      ( 9u)  // USART 3
#define AT91C_ID_MCI      (10u)  // Multimedia Card Interface
#define AT91C_ID_UDP      (11u)  // USB Device Port
#define AT91C_ID_TWI      (12u)  // Two-Wire Interface
#define AT91C_ID_SPI      (13u)  // Serial Peripheral Interface
#define AT91C_ID_SSC0     (14u)  // Serial Synchronous Controller 0
#define AT91C_ID_SSC1     (15u)  // Serial Synchronous Controller 1
#define AT91C_ID_SSC2     (16u)  // Serial Synchronous Controller 2
#define AT91C_ID_TC0      (17u)  // Timer Counter 0
#define AT91C_ID_TC1      (18u)  // Timer Counter 1
#define AT91C_ID_TC2      (19u)  // Timer Counter 2
#define AT91C_ID_TC3      (20u)  // Timer Counter 3
#define AT91C_ID_TC4      (21u)  // Timer Counter 4
#define AT91C_ID_TC5      (22u)  // Timer Counter 5
#define AT91C_ID_UHP      (23u)  // USB Host port
#define AT91C_ID_EMAC     (24u)  // Ethernet MAC
#define AT91C_ID_IRQ0     (25u)  // Advanced Interrupt Controller (IRQ0)
#define AT91C_ID_IRQ1     (26u)  // Advanced Interrupt Controller (IRQ1)
#define AT91C_ID_IRQ2     (27u)  // Advanced Interrupt Controller (IRQ2)
#define AT91C_ID_IRQ3     (28u)  // Advanced Interrupt Controller (IRQ3)
#define AT91C_ID_IRQ4     (29u)  // Advanced Interrupt Controller (IRQ4)
#define AT91C_ID_IRQ5     (30u)  // Advanced Interrupt Controller (IRQ5)
#define AT91C_ID_IRQ6     (31u)  // Advanced Interrupt Controller (IRQ6)
//
// Timer sfr definitions
//
#define TC0_BASE_ADDR   (0xfffa0000u)
#define TC1_BASE_ADDR   (0xfffa0040u)
#define TC2_BASE_ADDR   (0xfffa0080u)
#define TC3_BASE_ADDR   (0xfffa4000u)
#define TC4_BASE_ADDR   (0xfffa4040u)
#define TC5_BASE_ADDR   (0xfffa4080u)
//
// assign timer used for OS
//
#define OS_TC_BASE_ADDR   (TC0_BASE_ADDR)

#define OS_TC_CCR  (*(volatile OS_U32*)(OS_TC_BASE_ADDR + 0x00u))
#define OS_TC_CMR  (*(volatile OS_U32*)(OS_TC_BASE_ADDR + 0x04u))
#define OS_TC_CV   (*(volatile OS_U32*)(OS_TC_BASE_ADDR + 0x10u))
#define OS_TC_RC   (*(volatile OS_U32*)(OS_TC_BASE_ADDR + 0x1cu))
#define OS_TC_SR   (*(volatile OS_U32*)(OS_TC_BASE_ADDR + 0x20u))
#define OS_TC_IER  (*(volatile OS_U32*)(OS_TC_BASE_ADDR + 0x24u))
#define OS_TC_IDR  (*(volatile OS_U32*)(OS_TC_BASE_ADDR + 0x28u))

#if   (OS_TC_BASE_ADDR == TC0_BASE_ADDR)
  #define OS_TIMER_ID (AT91C_ID_TC0)
#elif (OS_TC_BASE_ADDR == TC1_BASE_ADDR)
  #define OS_TIMER_ID (AT91C_ID_TC1)
#elif (OS_TC_BASE_ADDR == TC2_BASE_ADDR)
  #define OS_TIMER_ID (AT91C_ID_TC2)
#elif (OS_TC_BASE_ADDR == TC3_BASE_ADDR)
  #define OS_TIMER_ID (AT91C_ID_TC3)
#elif (OS_TC_BASE_ADDR == TC4_BASE_ADDR)
  #define OS_TIMER_ID (AT91C_ID_TC4)
#elif (OS_TC_BASE_ADDR == TC5_BASE_ADDR)
  #define OS_TIMER_ID (AT91C_ID_TC5)
#endif

#if (OS_TC_BASE_ADDR < TC3_BASE_ADDR)
  #define OS_TC_BCR    (*(volatile OS_U32*)(TC0_BASE_ADDR + 0xc0u))
  #define OS_TC_BMR    (*(volatile OS_U32*)(TC0_BASE_ADDR + 0xc4u))
#else
  #define OS_TC_BCR    (*(volatile OS_U32*)(TC3_BASE_ADDR + 0xc0u))
  #define OS_TC_BMR    (*(volatile OS_U32*)(TC3_BASE_ADDR + 0xc4u))
#endif
//
// Advanced interrupt controller (AIC)
//
#define AIC_BASE_ADDR      (0xfffff000u)
#define AIC_SMR_BASE_ADDR  (AIC_BASE_ADDR + 0x000u)
#define AIC_SVR_BASE_ADDR  (AIC_BASE_ADDR + 0x080u)

#define AIC_SMR0           (*(volatile OS_U32*)(AIC_SMR_BASE_ADDR))
#define AIC_SVR0           (*(volatile OS_U32*)(AIC_SVR_BASE_ADDR))
#define AIC_IVR            (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x100u))
#define AIC_ISR            (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x108u))
#define AIC_IPR            (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x10cu))
#define AIC_IDCR           (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x124u))
#define AIC_ICCR           (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x128u))
#define AIC_IECR           (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x120u))
#define AIC_EOICR          (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x130u))
#define AIC_SPU            (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x134u))
#define AIC_DCR            (*(volatile OS_U32*)(AIC_BASE_ADDR + 0x138u))

#ifndef   NUM_INT_SOURCES
  #define NUM_INT_SOURCES  (32)
#endif

#define INT_PRIORITY_MASK  (0x07)
#define NUM_INT_PRIORITIES (8)
//
// SDRAM controller interface
//
#define SDRC_BASE_ADDR     (0xFFFFFF90u) // (SDRC) Base Address
#define SDRC_MR            (*(volatile OS_U32*)(SDRC_BASE_ADDR + 0x00u))
#define SDRC_TR            (*(volatile OS_U32*)(SDRC_BASE_ADDR + 0x04u))
#define SDRC_CR            (*(volatile OS_U32*)(SDRC_BASE_ADDR + 0x08u))
//
// PIO control register
//
#define PIOA_BASE_ADDR     (0xfffff400u)
#define PIOB_BASE_ADDR     (0xfffff600u)
#define PIOC_BASE_ADDR     (0xfffff800u)
#define PIOD_BASE_ADDR     (0xfffffA00u)

#define PIO_ASR_OFFS       (0x70u)
#define PIO_BSR_OFFS       (0x74u)
#define PIO_PDR_OFFS       (0x04u)

#define SDRAM_PIO_ASR      (*(volatile OS_U32*)(PIOC_BASE_ADDR + PIO_ASR_OFFS))
#define SDRAM_PIO_BSR      (*(volatile OS_U32*)(PIOC_BASE_ADDR + PIO_BSR_OFFS))
#define SDRAM_PIO_PDR      (*(volatile OS_U32*)(PIOC_BASE_ADDR + PIO_PDR_OFFS))
//
// Power management controller (PMC)
//
#define PMC_BASE_ADDR      (0xfffffc00u)
#define PMC_PCER           (*(volatile OS_U32*) (PMC_BASE_ADDR + 0x10u))
#define PMC_PCDR           (*(volatile OS_U32*) (PMC_BASE_ADDR + 0x14u))
#define PMC_CKGR_MOR       (*(volatile OS_U32*) (PMC_BASE_ADDR + 0x20u))  // main oscillator register
#define PMC_CKGR_MCFR      (*(volatile OS_U32*) (PMC_BASE_ADDR + 0x24u))  // main clock frequency register
#define PMC_CKGR_PLLAR     (*(volatile OS_U32*) (PMC_BASE_ADDR + 0x28u))  // PLLA register
#define PMC_CKGR_PLLBR     (*(volatile OS_U32*) (PMC_BASE_ADDR + 0x2cu))  // PLLB register
#define PMC_SR             (*(volatile OS_U32*) (PMC_BASE_ADDR + 0x68u))  // status register
#define PMC_MCKR           (*(volatile OS_U32*) (PMC_BASE_ADDR + 0x30u))  // Master clock register

#define CKGR_MAINRDY_BIT   (1u << 16)
#define PMC_LOCKA_BIT      (1u <<  1)
#define PMC_LOCKB_BIT      (1u <<  2)
#define PMC_MCKRDY_BIT     (1u <<  3)

#define PMC_CSS_MAIN_CLK_SELECT (1u)
//
// External bus interface
//
#define EBI_BASE_ADDR      (0xffffff60u)
#define EBI_CSA            (*(volatile OS_U32*)(EBI_BASE_ADDR + 0x00u))
#define EBI_CFGR           (*(volatile OS_U32*)(EBI_BASE_ADDR + 0x04u))
//
// Static memory controller interface
//
#define SMC_BASE_ADDR      (0xffffff70u)
#define SMC_CSR_BASE_ADDR  (SMC_BASE_ADDR)
#define SMC_CSR0           (*(volatile OS_U32*)(SMC_BASE_ADDR + 0x00u))

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

#if (defined(__ICCARM__))
/*********************************************************************
*
*       OS_GetPC()
*/
#pragma diag_suppress=Pe940 // IAR specific: No warning for missing return
static OS_U32 OS_GetPC(void) {
  #if (__VER__ < 6300000)
    __asm("mov  r0, lr");
    __asm("subs r0, r0, #4");
  #else
    __asm volatile("mov  r0, lr");
    __asm volatile("subs r0, r0, #4");
  #endif
}
#elif (defined __CC_ARM)
static __asm OS_U32 OS_GetPC(void) {
  mov  r0, lr
  subs r0, r0, #4
  bx   lr
}
#elif (defined __GNUC__)
OS_U32 OS_GetPC(void);
#endif

/*********************************************************************
*
*       _DefaultFiqHandler()
*
*  Function description
*    A dummy FIQ handler
*/
static void _DefaultFiqHandler(void) {
  while (1) {
    ;     /* Not implemented */
  }
}

/*********************************************************************
*
*       _DefaultIrqHandler()
*
*  Function description
*    A dummy IRQ handler
*
*  Notes
*    (1) This handler is initially written into all AIC interrupt vectors
*        It is called if no interrupt vector was installed for specific
*        interrupt source.
*        May be used during debugging to detect uninstalled interrupts.
*/
static void _DefaultIrqHandler(void) {
  OS_U32 IrqSource;

  IrqSource = AIC_ISR;  // detect source of uninstalled interrupt
  while (IrqSource == AIC_ISR);
}

/*********************************************************************
*
*       _SpuriousIrqHandler()
*
*  Function description
*    default spurious IRQ handler
*
*  Notes
*    A spurious interrupt is called under one of the following
*    circumstances:
*    1. An external interrupt source is programmed in level-sensitive mode
*       and an active level occurs for only a short time.
*    2. An internal interrupt source is programmed in level sensitive
*       and the output signal of the corresponding embedded peripheral
*       is activated for a short time. (As in the case for the Watchdog.)
*    3. An interrupt occurs just a few cycles before the software begins
*       to mask it, thus resulting in a pulse on the interrupt source.
*
*    The spurious handler is called to detect these errors.
*/
static void _SpuriousIrqHandler(void) {
#if (OS_DEBUG != 0)
  static OS_U32 _SpuriousIrqCnt;

  _SpuriousIrqCnt++;
#endif
}

/*********************************************************************
*
*       _InitAIC()
*
*  Function description
*    Initialize interrupt controller by setting default vectors
*    and clearing all interrupts
*
*  Notes
*    (1) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*/
static void _InitAIC(void) {
  unsigned int     i;
  OS_ISR_HANDLER** papISR;

  AIC_IDCR = 0xFFFFFFFFu;                    // Disable all interrupts
  AIC_ICCR = 0xFFFFFFFFu;                    // Clear all interrupts
  AIC_SVR0 = (int)_DefaultFiqHandler;        // Dummy FIQ handler
  AIC_SPU  = (int) _SpuriousIrqHandler;      // Dummy spurious handler
  papISR   = (OS_ISR_HANDLER**)AIC_SVR_BASE_ADDR;
  for (i = 1u; i < NUM_INT_SOURCES; i++)  {  // Initially set all sources
    *(papISR + i) = &_DefaultIrqHandler;     // to dummy irq handler
  }
  for (i = 0; i < NUM_INT_PRIORITIES; i++) {
    AIC_EOICR = 0;                           // Reset interrupt controller
  }
#if OS_DEBUG                                 // For debugging activate AIC protected mode
  AIC_DCR |= 0x01u;                          // Enable AIC protected mode
#endif
}

/*********************************************************************
*
*       _InitFlash()
*
*  Function description
*    Initialize wait states and BUS interface for external Flash access
*    Needs to be called before main clock is switched to high speed
*    Depends on specific hardware and has to be implemented by user.
*
*  Notes
*    (1) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*    (2) Settings used here are for AT91RM9200-EK eval board
*        and may have to be modified for other target board
*/
static void _InitFlash(void) {
/*
  EBI_CSA = 0x00
           |(0 << 0)        // CS0    assigned to SMC
           |(0 << 1)        // CS1    assigned to SMC
           |(0 << 3)        // CS3    assigned to SMC
           |(0 << 4)        // CS4..6 assigned to SMC
           ;
*/
  EBI_CFGR = 0x00
           |(0 << 0)        // Data BUS pull up disabled
           |(0 << 1)        // Data BUS sharing disabled
           ;
  SMC_CSR0 = 0x00           // Settings for CS0
           | 0x04           // Number of wait states
           | (1 << 7)       // Enable wait states
           | (2 << 8)       // Two data float cycles
           | (1 << 12)      // Flash is 16 bit type
           | (1 << 13)      // 16 bit data bus width
           | (0 << 15)      // standard read protocol
           | (0 << 16)      // standard CS pulse
           | (0 << 24)      // smallest RW setup time
           | (0 << 28)      // smallest RW hold time
           ;
}

/*********************************************************************
*
*       _InitClocks()
*
*  Function description
*    Initialize main clock and PLLs
*
*  Notes
*    (1) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*    (2) Settings used here are for AT91RM9200-DK eval board
*        and may have to be modified for other target board or frequency
*/
static void _InitClocks(OS_U32 PLLA_Val, OS_U32 PLLB_Val, OS_U32 MCKR_Val) {
  volatile int Cnt;

  //
  // Eanble main clock OSC and wait until delay counter expired
  //
  PMC_CKGR_MOR = 0x0000FF01;            // PMC_MOR: MOSCEN = 1, enable main clock
  while((PMC_CKGR_MCFR & CKGR_MAINRDY_BIT) == 0) {
  }
  //
  //  Initialize PLL A for Main clock
  //
  PMC_CKGR_PLLAR = PLLA_Val;
  //
  // Wait for PLLA stabilization LOCKA bit in PMC_SR
  //
  Cnt = 0;
  while(((PMC_SR & PMC_LOCKA_BIT) == 0) && (Cnt < DELAY_PLL) ) {
    Cnt++;
  }
  //
  //  Initialize PLL B for USB clock
  //
  PMC_CKGR_PLLBR = PLLB_Val;
  //
  // Wait for PLLB stabilization LOCKB bit in PMC_SR
  //
  Cnt = 0;
  while(((PMC_SR & PMC_LOCKB_BIT) == 0) && (Cnt < DELAY_PLL) ) {
    Cnt++;
  }
  //
  // Select Master Clock MCK and Processor Clock PCK
  // Write in the MCKR dirty value concerning the clock selection CSS then overwrite it in a second sequence
  //
  PMC_MCKR = PMC_CSS_MAIN_CLK_SELECT;
  //
  // Wait until the master clock is stable
  //
  Cnt = 0;
  while(((PMC_SR & PMC_MCKRDY_BIT) == 0) && (Cnt < DELAY_MAIN_FREQ) ) {
    Cnt++;
  }
  PMC_MCKR = MCKR_Val;
  //
  // Wait until the master clock is stable
  //
  Cnt = 0;
  while(((PMC_SR & PMC_MCKRDY_BIT) == 0) && (Cnt < DELAY_MAIN_FREQ) ) {
    Cnt++;
  }
}

/*********************************************************************
*
*       _InitSDRAM()
*
*  Function description
*    Initialize SDRAM controller
*
*  Notes
*    (1) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*    (2) Settings used here are for AT91RM9200-DK eval board
*        and may have to be modified for other target boards
*/
static void _InitSDRAM(void) {
  int           i;
  volatile int* pSDRAM;

  pSDRAM = (int*)SDRAM_BASE_ADDR;

  //
  // Configure PIO for peripheral function used for SDRAM access
  //
  if ((EBI_CSA  & (1 << 1)) == 0) {   // Make sure we do not init SDRAM if program is already running in SDRAM
    SDRAM_PIO_ASR = 0xffff0000;        // Select peripheral A function: D16..D31 used as data signals for SDRAM */
    SDRAM_PIO_BSR = 0x0;               // Disable peripheral B function */
    SDRAM_PIO_PDR = 0xffff0000;        // disable PIO for D16..D31 */
    //
    // Setup MEMC to support CS1 = SDRAM
    //
    EBI_CSA |= (1 << 1);               // assign CS1
    //
    // Init SDRAM controller
    //
    SDRC_CR = 0x01                     // 9 bit
            | (0x1 << 2)               // 12 bit
            | (0x1 << 4)               // 4 Banks
            | (0x2 << 5)               // 2 CAS cycles
            | 0x100
            | 0x4000
            | 0x8000
            | 0x880000
            | 0x21000000
            ;
    //
    // 2. Send Precharge All command to SDRAM
    //
    SDRC_MR = 0x02;
    *pSDRAM = 0;
    //
    // 3. Eight Auto-refresh are provided
    //
    SDRC_MR = 0x04;
    for(i = 0; i < 8; i++) {
      *pSDRAM = 0;
    }
    //
    // 4. Send mode register command to program the SDRAM parameters
    //
    SDRC_MR        = 0x03;
    *(pSDRAM+0x80) = 0;
    //
    // 5. Write refresh rate into SDRAMC refresh timer COUNT register
    //
    SDRC_TR = 0x2E0;
    *pSDRAM = 0;
    //
    // 6. Normal Mode Command is provided, 3 clocks after tMRD is set
    //
    SDRC_MR = 0x00;
    *pSDRAM = 0;
  }
}
/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return OS_TC_CV;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return AIC_IPR & (1u << OS_TIMER_ID);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  volatile OS_U32 Dummy;

  Dummy = OS_TC_SR;    // Reset Timer Interrupt to reset/reenable further interrupts
  OS_TICK_Handle();     // Call OS tick handler
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_DOWNCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  volatile OS_U32 Dummy;

  OS_INT_IncDI();
  _InitAIC();                           // Initialize and reset AIC
  //
  // Initialize selected timer for embOS, assuming PLL is already initialized
  //
  PMC_PCER    = (1u << OS_TIMER_ID);   // Enable peripheral clock for timer 0
  OS_TC_IDR   = 0xFFu;                 // Disable all timer/counter 0 interrupts.
  OS_TC_BMR   = 0x02u;                 // Runs on internal clock
  OS_TC_CMR   = 0x00004000u;           // Capture mode, CPCTRG=1, TCCLKS=0 (=> MCLK/2)
  OS_TC_RC    = OS_TIMER_RELOAD;       // Set compare register
  OS_TC_CCR   = 1u;                    // Enable the clock.
  OS_TC_CCR   = 5u;                    // Software trigger.
  Dummy       = OS_TC_SR;              // Read timer/counter 0 status register to clear flags.
  OS_TC_IER   = (1u << 4);             // Interrupt on RC compare.
  //
  // Setup interrupt controller for OS timer interrupt using selected timer
  //
  OS_ARM_InstallISRHandler(OS_TIMER_ID, &_OS_ISR_Tick);  // Timer/counter 0 interrupt vector.
  OS_ARM_ISRSetPrio(OS_TIMER_ID, 0x00);                   // Timer/counter 0 interrupt level sensitive at lowest priority (0).
  AIC_ICCR = (1u << OS_TIMER_ID);                         // Clears timer/counter 0 interrupt.
  OS_ARM_EnableISR(OS_TIMER_ID);                          // Enable timer/counter 0 interrupt.
  //
  // Setup values for usec precise system time functions
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;

  pISR = (OS_ISR_HANDLER*)AIC_IVR;  // Read interrupt vector to release NIRQ to CPU core
#if (OS_DEBUG != 0)
  AIC_IVR = (OS_U32)pISR;           // Write back any value to IVR register to allow interrupt stacking in protected mode
#endif
  OS_INT_EnterNestable();
  pISR();                           // Call interrupt service routine
  AIC_EOICR = 0;                    // interrupt controller =>  Restore previous priority
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  papISR = (OS_ISR_HANDLER**)AIC_SVR_BASE_ADDR;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  AIC_IECR = (1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  AIC_IDCR = (1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pPrio   = (OS_U32*)AIC_SMR_BASE_ADDR;
  OldPrio = pPrio[ISRIndex];
  pPrio[ISRIndex] = (OldPrio & ~INT_PRIORITY_MASK) | (Prio & INT_PRIORITY_MASK);
  OS_INT_EnableConditional();
  return OldPrio & INT_PRIORITY_MASK;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  //
  // Setup waitstates for Flash and external BUS interface before switching to high speed
  //
  _InitFlash();
  //
  // Initialize main clock oscillator and PLLs
  //
  _InitClocks(PLLAR_VALUE, PLLBR_VALUE, MCKR_VALUE);
  //
  // Initialize SDRAM
  //
  _InitSDRAM();
  //
  // Make sure vector area is o.k.
  //
  if (OS_GetPC() & 0x20000000) {   // Running in SDRAM ?
    memcpy((void*)0, (void*)0x20000000, 0x40);
  }
  return 1;                        // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
