/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AT91SAM7X-EK
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define _PMC_BASE_ADDR   (0xFFFFFC00u)
#define _PMC             (*(volatile unsigned int*)(_PMC_BASE_ADDR + 0x00u))

#define _PIOB_BASE_ADDR  (0xFFFFF600u)
#define _PIOB_ID         (3)

#define _LED0_BIT        (19) /* LED1 on SAM7X-EK board, active low */
#define _LED1_BIT        (20) /* LED2 on SAM7X-EK board, active low */
#define _LED2_BIT        (21) /* LED2 on SAM7X-EK board, active low */
#define _LED3_BIT        (22) /* LED2 on SAM7X-EK board, active low */

#define _LEDPORT_PER     (*(volatile unsigned int*)(_PIOB_BASE_ADDR + 0x00u))
#define _LEDPORT_OER     (*(volatile unsigned int*)(_PIOB_BASE_ADDR + 0x10u))
#define _LEDPORT_SODR    (*(volatile unsigned int*)(_PIOB_BASE_ADDR + 0x30u))
#define _LEDPORT_CODR    (*(volatile unsigned int*)(_PIOB_BASE_ADDR + 0x34u))
#define _LEDPORT_STATE   (*(volatile unsigned int*)(_PIOB_BASE_ADDR + 0x38u))

#define _LEDMASK_ALL     ((1 << _LED0_BIT) | (1 << _LED1_BIT)) | ((1 << _LED2_BIT) | (1 << _LED3_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  _PMC |= (1 << _PIOB_ID);        /* Enable peripheral clock for LED-Port    */
  _LEDPORT_PER   = _LEDMASK_ALL;  /* enable Port           */
  _LEDPORT_OER   = _LEDMASK_ALL;  /* switch to output mode */
  _LEDPORT_SODR |= _LEDMASK_ALL;  /* Initially clear LEDs  */
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  _LEDPORT_CODR = (1 << (_LED0_BIT + Index));         /* Switch on LED 0  */
}

void BSP_ClrLED(int Index) {
  _LEDPORT_SODR = (1 << (_LED0_BIT + Index));         /* Switch off LED 0 */
}

void BSP_ToggleLED(int Index) {
  if (_LEDPORT_STATE & (1 << (_LED0_BIT + Index))) {  /* LED is switched off */
    BSP_SetLED(Index);
  } else {
    BSP_ClrLED(Index);
  }
}

/****** EOF *********************************************************/
