/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AT91SAM9G20-EK
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/

#define PIOA_ID          (2)
#define PIOB_ID          (3)
#define PIOC_ID          (4)

#define PIOA_BASE_ADDR   (0xfffff400)
#define PIOB_BASE_ADDR   (0xfffff600)
#define PIOC_BASE_ADDR   (0xfffff800)

#define PIO_PER_OFFS     (0x00)
#define PIO_OER_OFFS     (0x10)
#define PIO_SODR_OFFS    (0x30)       // Set output data
#define PIO_CODR_OFFS    (0x34)       // Clear output data register
#define PIO_ODSR_OFFS    (0x38)       // output data status register
#define PIO_PDSR_OFFS    (0x3c)       // pin data status register

#define PMC_BASE_ADDR         (0xFFFFFC00)
#define PMC_PCER              (*(volatile unsigned int*) (PMC_BASE_ADDR + 0x10))

#define LEDPORT0_BASE_ADDR    (PIOA_BASE_ADDR)
#define LEDPORT1_BASE_ADDR    (PIOA_BASE_ADDR)
#define LEDPORT0_ID           (PIOA_ID)
#define LEDPORT1_ID           (PIOA_ID)

#define LED0_BIT              ( 6) // User  LED on AT91SAM9G20-EK board, active low
#define LED1_BIT              ( 9) // Power LED on AT91SAM9G20-EK board, active low

#define LEDPORT0_PER          (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_PER_OFFS))
#define LEDPORT0_OER          (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_OER_OFFS))
#define LEDPORT0_SODR         (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_SODR_OFFS))
#define LEDPORT0_CODR         (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_CODR_OFFS))
#define LEDPORT0_STATE        (*(volatile unsigned int*) (LEDPORT0_BASE_ADDR + PIO_ODSR_OFFS))
#define LEDPORT1_PER          (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_PER_OFFS))
#define LEDPORT1_OER          (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_OER_OFFS))
#define LEDPORT1_SODR         (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_SODR_OFFS))
#define LEDPORT1_CODR         (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_CODR_OFFS))
#define LEDPORT1_STATE        (*(volatile unsigned int*) (LEDPORT1_BASE_ADDR + PIO_ODSR_OFFS))

#define LEDMASK0_ALL          (1uL << LED0_BIT)
#define LEDMASK1_ALL          (1uL << LED1_BIT)

/*********************************************************************
*
*       Static const
*
**********************************************************************
*/
typedef struct {
  volatile unsigned int * pSODR;
  volatile unsigned int * pCODR;
  volatile unsigned int * pSTATE;
  char                    Id;
} LED_TYPE;

static const LED_TYPE _aLED[] = {
  {&LEDPORT0_SODR, &LEDPORT0_CODR, &LEDPORT0_STATE, LED0_BIT},
  {&LEDPORT1_SODR, &LEDPORT1_CODR, &LEDPORT1_STATE, LED1_BIT}
};

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PMC_PCER     |= (1uL << LEDPORT0_ID) /* Enable peripheral clock for LED-Ports    */
               | (1uL << LEDPORT1_ID)
               ;
  LEDPORT0_PER   = LEDMASK0_ALL;       /* Enable Port           */
  LEDPORT0_OER   = LEDMASK0_ALL;       /* Switch to output mode */
  LEDPORT0_SODR |= LEDMASK0_ALL;       /* Initially clear LEDs  */
  LEDPORT1_PER   = LEDMASK1_ALL;       /* Enable Port           */
  LEDPORT1_OER   = LEDMASK1_ALL;       /* Switch to output mode */
  LEDPORT1_SODR |= LEDMASK1_ALL;       /* Initially clear LEDs  */
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  if(Index > 1) {
    return;
  }
  *_aLED[Index].pCODR = (1uL << _aLED[Index].Id);    /* Switch on LED  */
}

void BSP_ClrLED(int Index) {
  if(Index > 1) {
    return;
  }
  *_aLED[Index].pSODR = (1uL << _aLED[Index].Id);    /* Switch off LED */
}

void BSP_ToggleLED(int Index) {
  if(Index > 1) {
    return;
  }
  if (*_aLED[Index].pSTATE & (1 << _aLED[Index].Id)) {  /* LED is switched off */
    BSP_SetLED(Index);
  } else {
    BSP_ClrLED(Index);
  }
}

/****** End of file *************************************************/

