/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AT91SAM9X5-EK eval board and AT91SAM9X35 CPU module
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define PIOB_ID           (2)
#define PIOB_BASE_ADDR    (0xFFFFF600)

#define PIOB_PER          (*(volatile unsigned int*) (PIOB_BASE_ADDR + 0x00))
#define PIOB_OER          (*(volatile unsigned int*) (PIOB_BASE_ADDR + 0x10))
#define PIOB_SODR         (*(volatile unsigned int*) (PIOB_BASE_ADDR + 0x30))  // Set output data
#define PIOB_CODR         (*(volatile unsigned int*) (PIOB_BASE_ADDR + 0x34))  // Clear output data register
#define PIOB_ODSR         (*(volatile unsigned int*) (PIOB_BASE_ADDR + 0x38))  // Output data status register
#define PIOB_PDSR         (*(volatile unsigned int*) (PIOB_BASE_ADDR + 0x3C))  // Pin data status register

#define PMC_BASE_ADDR     (0xFFFFFC00)
#define PMC_PCER          (*(volatile unsigned int*) (PMC_BASE_ADDR + 0x10))
#define LED0_BIT          (18) // LED B18

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PMC_PCER  |= (1uL << PIOB_ID);     // Enable peripheral clock for LED-Ports
  PIOB_PER   = (1uL << LED0_BIT);    // Enable Port
  PIOB_OER   = (1uL << LED0_BIT);    // Switch to output mode
  PIOB_SODR |= (1uL << LED0_BIT);    // Initially clear LEDs
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    PIOB_SODR = (1uL << LED0_BIT);         // Switch on LED
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    PIOB_CODR = (1uL << LED0_BIT);         // Switch off LED
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if (PIOB_ODSR & (1uL << LED0_BIT)) {  // LED is switched on
      BSP_ClrLED(Index);
    } else {
      BSP_SetLED(Index);
    }
  }
}


/****** End Of File *************************************************/
