/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Atmel SAMA5D3x-MB eval board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define LED_RED_PIN                   (24u) // Led DL1 on Ronetix CPU module, active high
#define LED_BLUE_PIN                  (25u) // Led DL2 on Ronetix CPU module, active low

#define PIOE_BASE_ADDR                (0xFFFFFA00u)
#define PIOE_PER                      (*(volatile unsigned int*) (PIOE_BASE_ADDR + 0x00u)) // PIO Enable Register
#define PIOE_OER0                     (*(volatile unsigned int*) (PIOE_BASE_ADDR + 0x10u)) // Output Enable Register
#define PIOE_SODR                     (*(volatile unsigned int*) (PIOE_BASE_ADDR + 0x30u)) // Set Output Data Register
#define PIOE_CODR                     (*(volatile unsigned int*) (PIOE_BASE_ADDR + 0x34u)) // Clear Output Data Register
#define PIOE_ODSR                     (*(volatile unsigned int*) (PIOE_BASE_ADDR + 0x38u)) // Output Data Status Register
#define PIOE_WPMR                     (*(volatile unsigned int*) (PIOE_BASE_ADDR + 0xE4u)) // Output Data Status Register
#define PIO_WPKEY                     (0x50494Fu)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  unsigned int Protection;

  //
  // Unlock access to PIO configuration
  //
  Protection = (PIOE_WPMR) & (1u << 0);
  PIOE_WPMR  = ((PIO_WPKEY) << 8);
  //
  // Enable led port pins and set to output, switch off LEDs initially
  //
  PIOE_PER  = (1u << LED_RED_PIN) | (1u << LED_BLUE_PIN);
  PIOE_OER0 = (1u << LED_RED_PIN) | (1u << LED_BLUE_PIN);
  PIOE_CODR = (1u << LED_RED_PIN);
  PIOE_SODR = (1u << LED_BLUE_PIN);
  //
  // Finally restore protection state
  //
  if (Protection != 0) {
    PIOE_WPMR  = ((PIO_WPKEY) << 8) | (1u << 0);
  }
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    PIOE_CODR = (1u << LED_BLUE_PIN);    /* Blue LED is active low  */
  } else if (Index == 1) {
    PIOE_SODR = (1u << LED_RED_PIN);     /* Red LED is active high  */
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    PIOE_SODR = (1u << LED_BLUE_PIN);    /* Blue LED is active low  */
  } else if (Index == 1) {
    PIOE_CODR = (1u << LED_RED_PIN);     /* Red LED is active high  */
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if ((PIOE_ODSR & (1u << LED_BLUE_PIN)) == (1u << LED_BLUE_PIN)) {
      PIOE_CODR = (1u << LED_BLUE_PIN);
    } else {
      PIOE_SODR = (1u << LED_BLUE_PIN);
    }
  } else if (Index == 1) {
    if ((PIOE_ODSR & (1u << LED_RED_PIN)) == (1u << LED_RED_PIN)) {
      PIOE_CODR = (1u << LED_RED_PIN);
    } else {
      PIOE_SODR = (1u << LED_RED_PIN);
    }
  }
}

/****** End Of File *************************************************/
