/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Atmel ATSAMA5D44 eval board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define LED_BLUE_PIN        (8u)
#define LED_RED_PIN         (9u)
#define LED_GREEN_PIN      (28u)

#define PIOE_BASE_ADDR     (0xFC06D000u)
#define PIOE_ID            (26u)

#define PIO_PER_OFFS       (0x00u)
#define PIO_PDR_OFFS       (0x04u)
#define PIO_PSR_OFFS       (0x08u)
#define PIO_OER_OFFS       (0x10u)
#define PIO_ODR_OFFS       (0x14u)
#define PIO_OSR_OFFS       (0x18u)
#define PIO_SODR_OFFS      (0x30u)  /* Set output data                */
#define PIO_CODR_OFFS      (0x34u)  /* Clear output data register     */
#define PIO_ODSR_OFFS      (0x38u)  /* Output data status register    */
#define PIO_PDSR_OFFS      (0x3Cu)  /* Pin data status register       */
#define PIO_IDR_OFFS       (0x44u)  /* Interrupt disable register     */
#define PIO_PUDR_OFFS      (0x60u)  /* Pull up disable register       */
#define PIO_ABCDSR1_OFFS   (0x70u)  /* Peripheral select register 1   */
#define PIO_ABCDSR2_OFFS   (0x74u)  /* Peripheral select register 2   */
#define PIO_PPDDR_OFFS     (0x90u)  /* Pad pull down disable register */
#define PIO_WPMR_OFFS      (0xE4u)  /* Write protect mode register    */

#define PIO_WPKEY          (0x50494Fu)

/****** Target specific configuration *******************************/

/****** Assign LEDs to Ports ****************************************/

#define LEDPORT_BASE_ADDR  (PIOE_BASE_ADDR)
#define LEDPORT_PCER       (PMC_PCER0)
#define LEDPORT_PCER_BIT   (PIOE_ID)
#define LED0_BIT           (LED_RED_PIN)   /* LED D9  (Red)  on SAMA5D4X board, active high */
#define LED1_BIT           (LED_BLUE_PIN)  /* LED D10 (Blue) on SAMA5D4X board, active low  */

#define LEDPORT_PER        (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_PER_OFFS))
#define LEDPORT_OER        (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OER_OFFS))
#define LEDPORT_SODR       (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_SODR_OFFS))
#define LEDPORT_CODR       (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_CODR_OFFS))
#define LEDPORT_STATE      (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_ODSR_OFFS))
#define LEDPORT_WPMR       (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_WPMR_OFFS))
#define LEDMASK_ALL        ((1uL << LED0_BIT) | (1uL << LED1_BIT))

#define PIOE_PER           (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_PER_OFFS)) // PIO Enable Register
#define PIOE_PDR0          (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_PDR_OFFS)) // PIO Disable Register
#define PIOE_PSR0          (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_PSR_OFFS)) // PIO Status Register
#define PIOE_OER0          (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_OER_OFFS)) // Output Enable Register
#define PIOE_ODR0          (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_ODR_OFFS)) // Output Disable Register
#define PIOE_OSR0          (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_OSR_OFFS)) // Output Status Register
#define PIOE_SODR          (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_SODR_OFFS)) // Set Output Data Register
#define PIOE_CODR          (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_CODR_OFFS)) // Clear Output Data Register
#define PIOE_ODSR          (*(volatile unsigned int*) (PIOE_BASE_ADDR + PIO_ODSR_OFFS)) // Output Data Status Register

/* Power management controller */
#define PMC_BASE_ADDR      (0xF0018000u)
#define PMC_SCER           (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x000u)) // PMC System Clock Enable Register
#define PMC_SCSR           (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x008u)) // PMC System Clock Enable Register
#define PMC_PCER0          (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x010u)) // PMC Peripheral Clock Enable Register 0
#define PMC_PCDR0          (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x014u)) // PMC Peripheral Clock Disable Register 0
#define PMC_PCSR0          (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x018u)) // PMC Peripheral status register 0
#define CKGR_UCKR          (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x01Cu)) // PMC UTMI Clock Configuration Register
#define CKGR_PLLAR         (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x028u)) // PMC Clock Generator PLLA Register
#define PMC_MCKR           (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x030u)) // PMC Master Clock Register
#define PMC_USB            (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x038u)) // PMC USB Clock Register
#define PMC_SR             (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x068u)) // PMC Status Register
#define PMC_WPMR           (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x0E4u)) // PMC Write Protect Mode Register
#define PMC_PCER1          (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x100u)) // PMC Peripheral Clock Enable Register 1
#define PMC_PCDR1          (*(volatile unsigned int*)(PMC_BASE_ADDR + 0x104u)) // PMC Peripheral Clock Disable Register 1

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  unsigned int Protection;

  //
  // Unlock access to PIO configuration
  //
  Protection   = (LEDPORT_WPMR) & (1u << 0);
  LEDPORT_WPMR = (PIO_WPKEY) << 8;
  //
  // Enable led port pins and set to output
  //
  LEDPORT_PER   = LEDMASK_ALL;    /* Enable Port           */
  LEDPORT_OER   = LEDMASK_ALL;    /* Switch to output mode */
  //
  // Restore protection
  //
  if (Protection != 0) {
    LEDPORT_WPMR = ((PIO_WPKEY) << 8) | (1u << 0);
  }
  //
  // Initially clear all LEDs
  //
  LEDPORT_CODR  = (1u << LED0_BIT);
  LEDPORT_SODR  = (1u << LED1_BIT);
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    LEDPORT_SODR = (1uL << LED0_BIT);         /* Switch on LED 0  */
  } else if (Index == 1) {
    LEDPORT_CODR = (1uL << LED1_BIT);         /* Switch on LED 1  */
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    LEDPORT_CODR = (1uL << LED0_BIT);        /* Switch off LED 0  */
  } else if (Index == 1) {
    LEDPORT_SODR = (1uL << LED1_BIT);        /* Switch off LED 1  */
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if ((LEDPORT_STATE & (1u << LED0_BIT)) == (1u << LED0_BIT)) {
      LEDPORT_CODR = (1u << LED0_BIT);
    } else {
      LEDPORT_SODR = (1u << LED0_BIT);
    }
  } else if (Index == 1) {
    if ((LEDPORT_STATE & (1u << LED1_BIT)) == (1u << LED1_BIT)) {
      LEDPORT_CODR = (1u << LED1_BIT);
    } else {
      LEDPORT_SODR = (1u << LED1_BIT);
    }
  }
}

/****** End Of File *************************************************/
