;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Part one of the system initialization code,
;; contains low-level
;; initialization.
;;
;; Copyright 2007 IAR Systems. All rights reserved.
;;
;; $Revision: 49919 $
;;


FIQ_MODE        DEFINE  0x11            ; Fast Interrupt Request mode

IRQ_MODE        DEFINE  0x12            ; Interrupt Request mode
SVC_MODE        DEFINE  0x13            ; Supervisor mode
SYS_MODE        DEFINE  0x1F            ; System mode
FLAG_I          DEFINE  0x80            ; IRQ disable flag
FLAG_F          DEFINE  0x40            ; FIQ disable flag
FLAG_T          DEFINE  0x20            ; Thumb mode flag
FLAG_MODE       DEFINE  0x1F            ; All mode bits


        MODULE  ?cstartup

        ;; Forward declaration of sections.
        SECTION IRQ_STACK:DATA:NOROOT(3)
        SECTION FIQ_STACK:DATA:NOROOT(3)
        SECTION CSTACK:DATA:NOROOT(3)

;
; The module in this file are included in the libraries, and may be
; replaced by any user-defined modules that define the PUBLIC symbol
; __iar_program_start or a user defined start symbol.
;
; To override the cstartup defined in the library, simply add your
; modified version to the workbench project.

        SECTION .intvec:CODE:NOROOT(2)

        PUBLIC  __vector
        PUBLIC  __iar_program_start
        EXTERN  Undefined_Handler
        EXTERN  SWI_Handler
        EXTERN  Prefetch_Handler
        EXTERN  Abort_Handler
        EXTERN  IRQ_Handler
        PUBLIC  FIQ_Handler

        DATA

__iar_init$$done:               ; The vector table is not needed
                                ; until after copy initialization is done

__vector:                       ; Make this a DATA label, so that stack usage
                                ; analysis doesn't consider it an uncalled fun

        ARM

        ; All default exception handlers (except reset) are
        ; defined as weak symbol definitions.
        ; If a handler is defined by the application it will take precedence.
        LDR     PC,Reset_Addr           ; Reset
        LDR     PC,Undefined_Addr       ; Undefined instructions
        LDR     PC,SWI_Addr             ; Software interrupt (SWI/SVC)
        LDR     PC,Prefetch_Addr        ; Prefetch abort
        LDR     PC,Abort_Addr           ; Data abort
        DCD     0                       ; RESERVED
        LDR     PC,IRQ_Addr             ; IRQ
        LDR     PC,FIQ_Addr             ; FIQ

        DATA

Reset_Addr:     DCD   __iar_program_start
Undefined_Addr: DCD   Undefined_Handler
SWI_Addr:       DCD   SWI_Handler
Prefetch_Addr:  DCD   Prefetch_Handler
Abort_Addr:     DCD   Abort_Handler
IRQ_Addr:       DCD   IRQ_Handler
FIQ_Addr:       DCD   FIQ_Handler


; --------------------------------------------------
; ?cstartup -- low-level system initialization code.
;
; After a reset execution starts here, the mode is ARM, supervisor
; with interrupts disabled.
;



        SECTION .text:CODE:NOROOT(2)

        EXTERN  __cmain
        REQUIRE __vector
        EXTWEAK __iar_init_core
        EXTWEAK __iar_init_vfp

        ARM

__iar_program_start:
?cstartup:

; Disable Addr translation, D cache and I cache disabled
        MRC     p15,0,R1,C1,C0,0
        LDR     R0,=0xFFFFEFFA
        AND     R1,R1,R0
        ORR     R1,R1,#(1<<12)
        MCR     p15,0,R1,C1,C0,0
;
; Add initialization needed before setup of stackpointers here.
;

;
; Initialize the stack pointers.
; The pattern below can be used for any of the exception stacks:
; FIQ, IRQ, SVC, ABT, UND, SYS.
; The USR mode uses the same stack as SYS.
; The stack segments must be defined in the linker command file,
; and be declared above.
;


; --------------------
; Mode, correspords to bits 0-5 in CPSR

#define MODE_MSK 0x1F            ; Bit mask for mode bits in CPSR
//#define FIQ_MODE 0x11            ; Fast Interrupt Request mode

#if 0

#define USR_MODE 0x10            ; User mode
#define IRQ_MODE 0x12            ; Interrupt Request mode
#define SVC_MODE 0x13            ; Supervisor mode
#define ABT_MODE 0x17            ; Abort mode
#define UND_MODE 0x1B            ; Undefined Instruction mode
#define SYS_MODE 0x1F            ; System mode
#endif

        MRS     r0, cpsr                ; Original PSR value

        ;; Set up the interrupt stack pointer.

        BIC     r0, r0, #MODE_MSK       ; Clear the mode bits
        ORR     r0, r0, #IRQ_MODE       ; Set IRQ mode bits
        MSR     cpsr_c, r0              ; Change the mode
        LDR     sp, =SFE(IRQ_STACK)     ; End of IRQ_STACK
        BIC     sp,sp,#0x7              ; Make sure SP is 8 aligned

        ;; Set up the fast interrupt stack pointer.

        BIC     r0, r0, #MODE_MSK       ; Clear the mode bits
        ORR     r0, r0, #FIQ_MODE       ; Set FIR mode bits
        MSR     cpsr_c, r0              ; Change the mode
        LDR     sp, =SFE(FIQ_STACK)     ; End of FIQ_STACK
        BIC     sp,sp,#0x7              ; Make sure SP is 8 aligned

        ;; Set up the normal stack pointer.

        BIC     r0 ,r0, #MODE_MSK       ; Clear the mode bits
        ORR     r0 ,r0, #SYS_MODE       ; Set System mode bits
        MSR     cpsr_c, r0              ; Change the mode
        LDR     sp, =SFE(CSTACK)        ; End of CSTACK
        BIC     sp,sp,#0x7              ; Make sure SP is 8 aligned

        ;; Turn on core features assumed to be enabled.
          FUNCALL __iar_program_start, __iar_init_core
        BL      __iar_init_core

        ;; Initialize VFP (if needed).
          FUNCALL __iar_program_start, __iar_init_vfp
        BL      __iar_init_vfp

;;;
;;; Add more initialization here
;;;

;;; Continue to __cmain for C-level initialization.

          FUNCALL __iar_program_start, __cmain
        B       __cmain


;------------------------------------------------------------------------------
;- Function             : FIQ_Handler
;- Treatments           : FIQ Controller Interrupt Handler.
;- Called Functions     : AIC_IVR[interrupt]
;------------------------------------------------------------------------------
//SAIC    DEFINE      0xFC068400
//AIC_FVR DEFINE      0x14

//#define AIC_IVR     0x10
//#define AIC_EOICR   0x38

//#define I_BIT            0x80
//#define F_BIT            0x40

        EXTERN  OS_fiq_handler

        SECTION .text:CODE:NOROOT(2)
        ARM
FIQ_Handler:
//
// Save temp. registers
//
        stmdb   SP!,{R0-R3,R12,LR}             // Push all scratch register
//
// push SPSR (req. if we allow nested interrupts)
//
        mrs     R0, SPSR                       // Get SPSR
        stmdb   SP!,{R0}                       // Push SPSR_irq on FIQ stack
//
// Switch to supervisor mode (svc mode), keep interrupt disabled, don't modify FIQ bit
//
        msr     CPSR_c, #(SVC_MODE | FLAG_I | FLAG_F) // Write CPSR: MODE = SVC, interrupts disabled
        stmdb   SP!, {R0, LR}                  // Push LR_svc and dummy for 8 byte alignment on SVC stack
//
// Call "C" interrupt handler in RTOSINIT.c
//
        blx     OS_fiq_handler                 // The high level fiq handler runs on SVC stack (embOS system stack)

        ldmia   SP!, {R0, LR}                  // pop LR_svc from svc stack
//
// Switch back to FIQ mode, interrupts disabled
//
        msr     CPSR_c, #(FIQ_MODE | FLAG_I | FLAG_F)      // Write CPSR: MODE = FIQ, interrupts disabled
//
// pop SPSR
//
        ldmia   SP!, {R1}                      // Pop SPSR_irq from FIQ stack
        msr     SPSR_cxfs, R1
//
// Restore temp registers
//
        ldmia   SP!, {R0-R3,R12,LR}            // Pop all scratch register
        subs    PC, LR, #4                     // RETI


        END
