/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   SDRAM_BASE_ADDR
  #define SDRAM_BASE_ADDR  (0xA0000000u)
#endif

#ifndef   DATABUS_32BIT
  #define DATABUS_32BIT  (0)
#endif

#ifndef   OS_USE_INTERNAL_RC
  #define OS_USE_INTERNAL_RC  (0)
#endif

#ifndef   OS_INIT_PLL
  #define OS_INIT_PLL  (1)  // Init PLL during startup
#endif

#ifndef   OS_INIT_MAM
  #define OS_INIT_MAM  (1)  // Init memory accelerator
#endif

#ifndef   OS_IGNORE_UNDEFINED_INTERRUPT
  #define OS_IGNORE_UNDEFINED_INTERRUPT  (0)
#endif

/*********************************************************************
*
*       Clock settings
*/
#if (OS_USE_INTERNAL_RC == 1)
  #ifndef   OS_PLL_MULTIPLIER
    #define OS_PLL_MULTIPLIER  (36)
  #endif
#else
  #ifndef   OS_PLL_MULTIPLIER
    #define OS_PLL_MULTIPLIER  (12)
  #endif
#endif

#define OS_SYSTEM_FREQ  (72000000u)

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ     (OS_SYSTEM_FREQ / 2)
#define OS_TICK_FREQ      (1000u)
#define OS_INT_FREQ       (OS_TICK_FREQ)
#define OS_TIMER_RELOAD   (OS_TIMER_FREQ / OS_INT_FREQ)

#if   (OS_SYSTEM_FREQ == OS_TIMER_FREQ)
  #define PCLKSEL_VALUE   (0x55555555u)
#elif ((OS_SYSTEM_FREQ / OS_TIMER_FREQ) == 2)
  #define PCLKSEL_VALUE   (0xAAAAAAAAu)
#elif ((OS_SYSTEM_FREQ / OS_TIMER_FREQ) == 4)
  #define PCLKSEL_VALUE   (0x00000000u)
#else
  #error "Unsupported peripheral clock divider, code has to be modified"
#endif

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Memory accelerator settings
*/

#if (OS_INIT_MAM == 1)  // Define timer value for MAM
  #if   (OS_SYSTEM_FREQ < 2000000)
    #define OS_MAMTIM_VALUE  (2)
  #elif (OS_SYSTEM_FREQ < 4000000)
    #define OS_MAMTIM_VALUE  (3)
  #else
    #define OS_MAMTIM_VALUE  (5)
  #endif
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/

//
// Port, port mode settings
//
#define PINSEL_BASE_ADDR        (0xE002C000u)
#define PINSEL0                 (*(volatile OS_U32*)(PINSEL_BASE_ADDR + 0x00u))
#define PINSEL4                 (*(volatile OS_U32*)(PINSEL_BASE_ADDR + 0x10u))
#define PINSEL5                 (*(volatile OS_U32*)(PINSEL_BASE_ADDR + 0x14u))
#define PINSEL6                 (*(volatile OS_U32*)(PINSEL_BASE_ADDR + 0x18u))
#define PINSEL7                 (*(volatile OS_U32*)(PINSEL_BASE_ADDR + 0x1Cu))
#define PINSEL8                 (*(volatile OS_U32*)(PINSEL_BASE_ADDR + 0x20u))
#define PINSEL9                 (*(volatile OS_U32*)(PINSEL_BASE_ADDR + 0x24u))
//
// External memory controller
//
#define EMC_BASE_ADDR           (0xFFE08000u)
#define EMCCONTROL              (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0000u))
#define EMCDYNAMIC_CONTROL      (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0020u))
#define EMCDYNAMIC_REFRESH      (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0024u))
#define EMCDYNAMIC_READCONFIG   (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0028u))
#define EMCDYNAMIC_RP           (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0030u))
#define EMCDYNAMIC_RAS          (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0034u))
#define EMCDYNAMIC_SREX         (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0038u))
#define EMCDYNAMIC_APR          (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x003Cu))
#define EMCDYNAMIC_DAL          (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0040u))
#define EMCDYNAMIC_WR           (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0044u))
#define EMCDYNAMIC_RC           (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0048u))
#define EMCDYNAMIC_RFC          (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x004Cu))
#define EMCDYNAMIC_XSR          (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0050u))
#define EMCDYNAMIC_RRD          (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0054u))
#define EMCDYNAMIC_MRD          (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0058u))
#define EMCDYNAMIC_CONFIG0      (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0100u))
#define EMCDYNAMIC_RASCAS0      (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0104u))
#define EMCSTATIC_CONFIG0       (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0200u))
#define EMCSTATIC_WAITWEN0      (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0204u))
#define EMCSTATIC_WAITOEN0      (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0208u))
#define EMCSTATIC_WAITRD0       (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x020Cu))
#define EMCSTATIC_WAITPAGE0     (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0210u))
#define EMCSTATIC_WAITWR0       (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0214u))
#define EMCSTATIC_WAITTURN0     (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0218u))
#define EMCSTATIC_CONFIG1       (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0220u))
#define EMCSTATIC_WAITWEN1      (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0224u))
#define EMCSTATIC_WAITOEN1      (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0228u))
#define EMCSTATIC_WAITRD1       (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x022Cu))
#define EMCSTATIC_WAITPAGE1     (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0230u))
#define EMCSTATIC_WAITWR1       (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0234u))
#define EMCSTATIC_WAITTURN1     (*(volatile unsigned long *)(EMC_BASE_ADDR + 0x0238u))
//
// Power, reset clock control unit register
//
#define SCS                     (*(volatile OS_U32*)0xE01FC1A0u)
#define PLLCON                  (*(volatile OS_U32*)0xE01FC080u)
#define PLLCFG                  (*(volatile OS_U32*)0xE01FC084u)
#define PLLSTAT                 (*(volatile OS_U32*)0xE01FC088u)
#define PLLFEED                 (*(volatile OS_U32*)0xE01FC08Cu)
#define PCON                    (*(volatile OS_U32*)0xE01FC0C0u)
#define PCONP                   (*(volatile OS_U32*)0xE01FC0C4u)
#define CPUCLKCFG               (*(volatile OS_U32*)0xE01FC104u)
#define USBCLKCFG               (*(volatile OS_U32*)0xE01FC108u)
#define CLKSRCSEL               (*(volatile OS_U32*)0xE01FC10Cu)
#define PCLKSEL0                (*(volatile OS_U32*)0xE01FC1A8u)
#define PCLKSEL1                (*(volatile OS_U32*)0xE01FC1ACu)
//
// PLL and memeory timing
//
#define MAMCR                   (*(volatile OS_U32*)(0xE01FC000u))
#define MAMTIM                  (*(volatile OS_U32*)(0xE01FC004u))
//
// Timer sfr definition
//
#define TIM0_BASE_ADDR          (0xE0004000u)
#define TIM1_BASE_ADDR          (0xE0008000u)

#define TIM_IR_OFFS             (0x00u)
#define TIM_TCR_OFFS            (0x04u)
#define TIM_TC_OFFS             (0x08u)
#define TIM_PR_OFFS             (0x0Cu)
#define TIM_MCR_OFFS            (0x14u)
#define TIM_MR0_OFFS            (0x18u)
//
// Assign timer sfrs used for OS timer
// initially, we use timer 0
//
#define OS_TIM_BASE_ADDR        TIM0_BASE_ADDR
#define OS_TIM_IR_BIT           (0u)  // Used to reset timer interrupt (Timer IR register), Match channel 0
#define OS_TIMER_INT_SOURCE     (4u)
#define TIMER_PCONP_BIT         (1u)

#define OS_TIM_IR               (*(volatile OS_U32*)(OS_TIM_BASE_ADDR + TIM_IR_OFFS))
#define OS_TIM_TCR              (*(volatile OS_U32*)(OS_TIM_BASE_ADDR + TIM_TCR_OFFS))
#define OS_TIM_TC               (*(volatile OS_U32*)(OS_TIM_BASE_ADDR + TIM_TC_OFFS))
#define OS_TIM_PR               (*(volatile OS_U32*)(OS_TIM_BASE_ADDR + TIM_PR_OFFS))
#define OS_TIM_MCR              (*(volatile OS_U32*)(OS_TIM_BASE_ADDR + TIM_MCR_OFFS))
#define OS_TIM_MR0              (*(volatile OS_U32*)(OS_TIM_BASE_ADDR + TIM_MR0_OFFS))
//
// Vectored interrupt controller (VIC)
//
#define VIC_BASE_ADDR           (0xFFFFF000u)
#define VIC_IRQSTATUS           (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0000u))
#define VIC_FIQSTATUS           (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0004u))
#define VIC_RAWINTERRUPT        (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0008u))
#define VIC_INTSELECT           (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x000Cu))
#define VIC_INTENABLE           (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0010u))
#define VIC_INTENABLECLEAR      (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0014u))
#define VIC_SOFTINT             (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0018u))
#define VIC_SOFTINTCLEAR        (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x001Cu))
#define VIC_VECTORADDR          (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0F00u))

#define VIC_VECT_BASE_ADDR      (VIC_BASE_ADDR + 0x100u)
#define VIC_PRIO_BASE_ADDR      (VIC_BASE_ADDR + 0x200u)

#define NUM_INT_SOURCES         (32u)

#define NUM_INT_PRIORITIES      (16u)
#define INT_PRIORITY_MASK       (NUM_INT_PRIORITIES - 1)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _InitPLL()
*
*  Function description
*    Initialize PLL
*    WARNING: Improper settings may lock CPU
*    Please examine whether init sequence fits your hardware configuration
*    We assume a CPU running with internal oscillator.
*/
static void _InitPLL(void) {
#if (OS_INIT_PLL != 0)
  if (PLLSTAT & (1u << 25)) {
    PLLCON  |= (1u << 1);
    PLLFEED  = 0xAAu;
    PLLFEED  = 0x55u;
  }
  PLLCON     = 0;                        // Set PLL disabled and disconnected
  PLLFEED    = 0xAAu;
  PLLFEED    = 0x55u;
#if (OS_USE_INTERNAL_RC == 1)
  CLKSRCSEL  = 0;                        // Select internal RC oscillator 4 MHz as clock source
#else
  SCS       &= ~(1u << 4);               // The frequency range of the main oscillator is 1 MHz to 20 MHz.
  SCS       |=  (1u << 5);               // Main oscillator enable.
  CLKSRCSEL  = 1;                        // Select main oscillator 12 MHz as clock source
#endif
  PLLCFG     = (OS_PLL_MULTIPLIER - 1);  // MSEL: Multiplier - 1
  PLLFEED    = 0xAAu;
  PLLFEED    = 0x55u;
  PLLCON     = 1;                        // Enable PLL, Disconnect
  PLLFEED    = 0xAAu;
  PLLFEED    = 0x55u;
  while ((PLLSTAT & (1u << 26)) == 0);   // Check PLOCK bit
  CPUCLKCFG  = 3u;                       // Set CPU clock divider value (which is divider - 1); has to be an odd value!
  USBCLKCFG  = 5u;                       // Set USB clock divider value (which is divider - 1); has to be an odd value!
  PCLKSEL0   = PCLKSEL_VALUE;
  PCLKSEL1   = PCLKSEL_VALUE;
  PLLCON     = (1u << 0) | (1u << 1);    // Enable PLL and select as clock source
  PLLFEED    = 0xAAu;
  PLLFEED    = 0x55u;
#endif
}

/*********************************************************************
*
*       _Init_VIC()
*
*  Function description
*    Clears and disables all interrupt vectors in the VIC.
*/
static void _Init_VIC(void) {
  unsigned int Index;

  VIC_INTENABLECLEAR = 0xFFFFFFFFu;         // Disable all source interrupts
  VIC_VECTORADDR     = 0;                   // Write to vector, reset VIC
  for (Index = 0; Index < NUM_INT_SOURCES; Index++) {
    OS_ARM_InstallISRHandler(Index, NULL);  // Set vector to 0;
  }
  VIC_SOFTINTCLEAR   = 0xFFFFFFFFu;         // Disable all soft interrupts
  VIC_INTSELECT      = 0;                   // Set all channels to generate IRQ
}

/*********************************************************************
*
*       _EMC_Init()
*
*  Function description
*    Initializes external memory controller for SDRAM, NAND- and NOR-flash
*/
static void _EMC_Init(void) {
  volatile unsigned long i;
  volatile unsigned long Dummy;

  EMCCONTROL  = (1u <<  0);  // EMC enable
  PCONP      |= (1u << 11);  // Turn on EMC peripheral clock
  PINSEL4     = 0
              | (1u << 28)   // Reserved
              | (1u << 30)   // Reserved
              ;
#if (DATABUS_32BIT == 1)
  PINSEL5     = 0
              | (1u <<  0)  // CAS
              | (1u <<  2)  // RAS
              | (1u <<  4)  // CLKOUT0
              | (1u <<  8)  // DYCS0
              | (1u << 16)  // CKEOUT0
              | (1u << 24)  // DQMOUT0
              | (1u << 26)  // DQMOUT1
              | (1u << 28)  // DQMOUT2
              | (1u << 30)  // DQMOUT3
              ;
  PINSEL7     = 0
              | (1u <<  0)  // D16
              | (1u <<  2)  // D17
              | (1u <<  4)  // D18
              | (1u <<  6)  // D19
              | (1u <<  8)  // D20
              | (1u << 10)  // D21
              | (1u << 12)  // D22
              | (1u << 14)  // D23
              | (1u << 16)  // D24
              | (1u << 18)  // D25
              | (1u << 20)  // D26
              | (1u << 22)  // D27
              | (1u << 24)  // D28
              | (1u << 26)  // D29
              | (1u << 28)  // D30
              | (1u << 30)  // D31
              ;
#else
  PINSEL5     = 0
              | (1u <<  0)  // CAS
              | (1u <<  2)  // RAS
              | (1u <<  4)  // CLKOUT0
              | (1u <<  6)  // CLKOUT1
              | (1u <<  8)  // DYCS0
              | (1u << 10)  // DYCS1
              | (1u << 16)  // CKEOUT0
              | (1u << 18)  // CKEOUT1
              | (1u << 24)  // DQMOUT0
              | (1u << 26)  // DQMOUT1
              ;
#endif
  PINSEL6     = 0
              | (1u <<  0)  // D0
              | (1u <<  2)  // D1
              | (1u <<  4)  // D2
              | (1u <<  6)  // D3
              | (1u <<  8)  // D4
              | (1u << 10)  // D5
              | (1u << 12)  // D6
              | (1u << 14)  // D7
              | (1u << 16)  // D8
              | (1u << 18)  // D9
              | (1u << 20)  // D10
              | (1u << 22)  // D11
              | (1u << 24)  // D12
              | (1u << 26)  // D13
              | (1u << 28)  // D14
              | (1u << 30)  // D15
              ;
  PINSEL8     = 0
              | (1u <<  0)  // A0
              | (1u <<  2)  // A1
              | (1u <<  4)  // A2
              | (1u <<  6)  // A3
              | (1u <<  8)  // A4
              | (1u << 10)  // A5
              | (1u << 12)  // A6
              | (1u << 14)  // A7
              | (1u << 16)  // A8
              | (1u << 18)  // A9
              | (1u << 20)  // A10
              | (1u << 22)  // A11
              | (1u << 24)  // A12
              | (1u << 26)  // A13
              | (1u << 28)  // A14
              | (1u << 30)  // A15
              ;
  PINSEL9     = 0
              | (1u <<  0)  // A16
              | (1u <<  2)  // A17
              | (1u <<  4)  // A18
              | (1u <<  6)  // A19
              | (1u <<  8)  // A20
              | (1u << 10)  // A21
              | (1u << 12)  // A22
              | (1u << 14)  // A23
              | (1u << 16)  // OE
              | (1u << 18)  // WE
              | (1u << 20)  // BLS0
              | (1u << 22)  // BLS1
              | (1u << 28)  // CS0
              | (0u << 30)  // P4.31 (NAND CE)
              ;
#if (DATABUS_32BIT == 1)
  //
  // Set memory cycles
  //
  EMCDYNAMIC_RP         = 1;
  EMCDYNAMIC_RAS        = 3;
  EMCDYNAMIC_SREX       = 5;
  EMCDYNAMIC_APR        = 1;
  EMCDYNAMIC_DAL        = 5;
  EMCDYNAMIC_WR         = 1;
  EMCDYNAMIC_RC         = 5;
  EMCDYNAMIC_RFC        = 5;
  EMCDYNAMIC_XSR        = 5;
  EMCDYNAMIC_RRD        = 1;
  EMCDYNAMIC_MRD        = 1;
  EMCDYNAMIC_READCONFIG = 1;
  EMCDYNAMIC_RASCAS0    = 0
                        | (2u << 0)  // RAS latency, two CCLK cycles
                        | (2u << 8)  // CAS latency, two CCLK cycles
                        ;
  EMCDYNAMIC_CONFIG0    = 0x5488u;   // 128MB (8Mx16), 4 banks, row length = 12, column length = 9
#else
  //
  // Set memory cycles
  //
  EMCDYNAMIC_RP         = 2;
  EMCDYNAMIC_RAS        = 3;
  EMCDYNAMIC_SREX       = 7;
  EMCDYNAMIC_APR        = 2;
  EMCDYNAMIC_DAL        = 5;
  EMCDYNAMIC_WR         = 1;
  EMCDYNAMIC_RC         = 5;
  EMCDYNAMIC_RFC        = 5;
  EMCDYNAMIC_XSR        = 7;
  EMCDYNAMIC_RRD        = 1;
  EMCDYNAMIC_MRD        = 2;
  EMCDYNAMIC_READCONFIG = 1;
  EMCDYNAMIC_RASCAS0    = 0
                        | (3u << 0)  // RAS latency, three CCLK cycles
                        | (3u << 8)  // CAS latency, three CCLK cycles
                        ;
  EMCDYNAMIC_CONFIG0    = 0x0680u;   // 256MB (16Mx16), 4 banks, row length = 13, column length = 9
#endif
  //
  // Delay at least 100 us
  //
  for (i = 0; i < 5000; i++) {      // Delay
    ;
  }
  EMCDYNAMIC_CONTROL = 0
                       | (1u << 0)  // Dynamic memory clock enable (CE), all clock enables are driven high continuously
                       | (1u << 1)  // Dynamic memory clock control (CS), CLKOUT runs continuously
                       | (3u << 7)  // Issue SDRAM NOP command
                       ;
  //
  // Delay at least 200 us
  //
  for (i = 0; i < 10000; i++) {     // Delay
    ;
  }
  EMCDYNAMIC_CONTROL = 0
                     | (1u << 0)    // Dynamic memory clock enable (CE), all clock enables are driven high continuously
                     | (1u << 1)    // Dynamic memory clock control (CS), CLKOUT runs continuously
                     | (1u << 8)    // Issue SDRAM PALL (precharge all) command
                     ;
  EMCDYNAMIC_REFRESH = 2u;          // n x 16 CCLKs between SDRAM refresh cycles
  //
  // Delay at least 100 us
  //
  for (i = 0; i < 5000; i++) {      // Delay
    ;
  }
  EMCDYNAMIC_REFRESH = 0x1Cu;       // Set correct refresh period, n x 16 CCLKs between SDRAM refresh cycles
  EMCDYNAMIC_CONTROL = 0
                     | (1u << 0)    // Dynamic memory clock enable (CE), all clock enables are driven high continuously
                     | (1u << 1)    // Dynamic memory clock control (CS), CLKOUT runs continuously
                     | (1u << 7)    // Issue SDRAM MODE command
                     ;
#if (DATABUS_32BIT == 1)
  Dummy = *((volatile unsigned long*)(SDRAM_BASE_ADDR | (0x22u << 11)));  // Set mode register in SDRAM
#else
  Dummy = *((volatile unsigned long*)(SDRAM_BASE_ADDR | (0x33u << 12)));  // Set mode register in SDRAM
#endif
  OS_USEPARA(Dummy);
  EMCDYNAMIC_CONTROL  = 0;           // Issue SDRAM NORMAL operation command
  EMCDYNAMIC_CONFIG0 |= (1u << 19);  // Enable buffer
  //
  // Initial system delay, at least 100 us
  //
  for (i = 0; i < (5000); i++) {     // Delay
    ;
  }
  EMCSTATIC_WAITWEN0  = 0x02u;       // n + 1 CCLK cycle delay between assertion of chip select and write enable
  EMCSTATIC_WAITOEN0  = 0x02u;       // n cycle delay.
  EMCSTATIC_WAITRD0   = 0x1Fu;       // 32 CCLK cycles for read accesses
  EMCSTATIC_WAITPAGE0 = 0x1Fu;       // 32 CCLK cycle read access time
  EMCSTATIC_WAITWR0   = 0x1Fu;       // 33 CCLK cycle write access time
  EMCSTATIC_WAITTURN0 = 0x0Fu;       // 16 CCLK turnaround cycles
  EMCSTATIC_CONFIG0   = 0
                      | (1u << 0)    // 16bit memory width
                      | (1u << 7)    // Bit lane state
                      ;
  EMCSTATIC_WAITWEN1  = 0x02u;       // n + 1 CCLK cycle delay between assertion of chip select and write enable
  EMCSTATIC_WAITOEN1  = 0x02u;       // n cycle delay.
  EMCSTATIC_WAITRD1   = 0x08u;       // n + 1 CCLK cycle delay
  EMCSTATIC_WAITPAGE1 = 0x1Fu;       // 32 CCLK cycle read access time
  EMCSTATIC_WAITWR1   = 0x08u;       // n + 2 CCLK cycle write access time
  EMCSTATIC_WAITTURN1 = 0x0Fu;       // 16 CCLK turnaround cycles
  EMCSTATIC_CONFIG1   = (1u << 7);   // Bit lane state
  //
  // Delay, at least 10 us
  //
  for (i = 0; i < (500); i++) {      // Delay
    ;
  }
}

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when an not installed interrupt was detected.
*    As interrupt pending condition of peripherals has to be reset,
*    program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(void) {
#if (OS_IGNORE_UNDEFINED_INTERRUPT == 0)
  volatile int Dummy;

  Dummy = 1;
  while (Dummy > 0) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
#endif
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return OS_TIM_TC;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return OS_TIM_IR & (1u << OS_TIM_IR_BIT);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  OS_TIM_IR = (1u << OS_TIM_IR_BIT);  // Clear OS timer interrupt flag
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  _Init_VIC();                               // Initialize VIC, clear and disable all interrupts
  //
  // Initialize timer for OS
  //
  PCONP      |=  (1u << TIMER_PCONP_BIT);  // Enable Clock for OS timer
  OS_TIM_TCR &= ~(1u << 0);                // Disable timer
  OS_TIM_IR   =  (1u << OS_TIM_IR_BIT);    // Clear OS timer interrupt flag
  OS_TIM_PR   = 0;                         // Count on every pclk
  OS_TIM_MR0  = OS_TIMER_RELOAD;           // Initialize match register 1
  OS_TIM_MCR  =  (1u << 0)                 // Interrupt on match channel 0
              |  (1u << 1);                // Counter reset on match channel 0
  OS_TIM_TC   = 0;                         // Clear counter
  OS_TIM_TCR |=  (1u << 0);                // Enable timer
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler(OS_TIMER_INT_SOURCE, &_OS_ISR_Tick);  // Timer/counter interrupt vector.
  OS_ARM_EnableISR(OS_TIMER_INT_SOURCE);                         // Enable timer/counter 0 interrupt.
  OS_ARM_ISRSetPrio(OS_TIMER_INT_SOURCE, 15);                    // Set lowest priority
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;

  pISR = (OS_ISR_HANDLER*)VIC_VECTORADDR;  // Get current interrupt handler
  OS_INT_EnterNestable();
  if (pISR != NULL) {
    pISR();                                // Call installed interrupt service routine
  } else {
    _OS_ISR_Undefined();
  }
  VIC_VECTORADDR = 0;                      // Clear current interrupt pending condition, reset VIC
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  papISR = (OS_ISR_HANDLER**)VIC_VECT_BASE_ADDR;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  VIC_INTENABLE = (1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  VIC_INTENABLECLEAR = (1UL << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if OS_DEBUG
  if ((unsigned)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pPrio = (OS_U32*)VIC_PRIO_BASE_ADDR;
  OldPrio = pPrio[ISRIndex];
  pPrio[ISRIndex] = (OldPrio & ~INT_PRIORITY_MASK) | (Prio & INT_PRIORITY_MASK);
  OS_INT_EnableConditional();
  return OldPrio & INT_PRIORITY_MASK;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  _InitPLL();
#if (OS_INIT_MAM == 1)       // Speedup CPU
  MAMCR  = 0x00;             // Disable Memory accelerator module
  MAMTIM = OS_MAMTIM_VALUE;  // Depends on OS_FSYS
  MAMCR  = 0x01u;            // Enable Memory accelerator module partially , refer to [1], MAM.1 operation
#endif
  //
  // Init SDRAM, NAND- and NOR-flash
  //
  _EMC_Init();
  return 1;                    // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
