;*********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                *
;*        Solutions for real time microcontroller applications       *
;*********************************************************************
;*                                                                   *
;*      (C) 2005- 2007 SEGGER Microcontroller Systeme GmbH           *
;*                  www.segger.com                                   *
;*                                                                   *
;*********************************************************************
;
;---------------------------------------------------------------------
;File   : Startup.s
;Purpose: Startup file for embOS and IAR embedded workbench V5xx
;-------- END-OF-HEADER ----------------------------------------------
;

CP_DIS_MASK DEFINE ~5     ; MMU Disable Mask
MODE_MSK    DEFINE 0x1F   ; Bit mask for mode bits in CPSR
SVC_MODE    DEFINE 0x13   ; Supervisor mode

        MODULE  ?cstartup

        ;; Forward declaration of sections.
        SECTION PROGRAM_DATA:CODE
        SECTION ROM_END:CODE
        SECTION IRQ_STACK:DATA:NOROOT(3)
        SECTION FIQ_STACK:DATA:NOROOT(3)
        SECTION CSTACK:DATA:NOROOT(3)
        SECTION BTL_STACK:DATA:NOROOT(3)

        SECTION .intvec:CODE:NOROOT(2)

        PUBLIC  __vector
        PUBLIC  __vector_0x14
        PUBLIC  __iar_program_start
        EXTERN  IRQ_Handler

        ARM

; This initial handler is only for reset, a real application will
; replace this exception handler in memory with a more capable one
__vector:
                     ldr PC,Reset_Addr             ; Reset
                     DCD   0x41676d69              ; Image magic number
                     DCD   0,0,0
__vector_0x14:
                     DCD   0,0
image_type:
                     DCD   0x0000000A              ; No CRC check
sizeOfPermanentCode:
                     DCD   SFE(ROM_END) - 0x11029000 ; Image size including 0x80 header
                     DCD   0,0
bootparameter:
                     DCD   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

        LTORG            ; Constant table entries (for ldr pc) will be placed at 0x80
Reset_Addr:          DCD   __iar_program_start
__vector_end:

; --------------------------------------------------
; ?cstartup -- low-level system initialization code.
;
; After a reser execution starts here, the mode is ARM, supervisor
; with interrupts disabled.
;


        SECTION .text:CODE:NOROOT(2)

        EXTERN  ?main
        EXTERN  bootloader
        REQUIRE __vector

        ARM

__iar_program_start:

/**********************************************************************
* ?CSTARTUP
*
* Execution starts here.
* After a reset, the mode is ARM, Supervisor, interrupts disabled.
*/
?cstartup:

        mrc     p15, 0, r0, c1, c0,0                ; read cp15 control register (cp15 r1) r0
        ldr     r3, =0xC0001085                     ; Disable Cache, Disable MMU
        ldr     r4, =0x40004000                     ; Round Robin replacement
        bic     r0, r0, r3
        orr     r0, r0, r4
        mcr     p15, 0, r0, c1, c0,0                ; write cp15 control register (cp15 r1)

; Start second level bootloader
#ifdef SECOND_LVL_BTL
        mrs     r0, cpsr                            ; Original PSR value
        bic     r0, r0, #MODE_MSK                   ; Clear the mode bits
        orr     r0, r0, #SVC_MODE                   ; Set Supervisor mode bits
        msr     cpsr_c, r0                          ; Change the mode
        ldr     sp,=SFE(BTL_STACK)                 ; End of bootloader stack ()

        ldr     r0, =bootloader
        blx     r0
#endif

; Initialize the stack pointers.
        ;; Set up the interrupt stack pointer.

        msr     cpsr_c,#0xd2            ; Change mode to IRQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(IRQ_STACK)     ; End of IRQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary
	
        ;; Set up the fast interrupt stack pointer.

        msr     cpsr_c,#0xd1            ; Change mode to FIQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(FIQ_STACK)     ; End of FIQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary

        ;; Set up the normal stack pointer.

        msr     cpsr_c,#0xdf            ; Change mode to SYS mode, FIQ and IRQ disabled
        ldr     sp, =SFE(CSTACK)        ; End of CSTACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary

; Add more initialization here if required


; Jump to startup code in library
        ldr     r0,=?main
        bx      r0

        LTORG

        SECTION ROM_END:CODE:NOROOT(9)

        END

; ***** EOF **********************************************************



