;*********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                *
;*        Solutions for real time microcontroller applications       *
;*********************************************************************
;*                                                                   *
;*      (C) 2005- 2007 SEGGER Microcontroller Systeme GmbH           *
;*                  www.segger.com                                   *
;*                                                                   *
;*********************************************************************
;
;---------------------------------------------------------------------
;File   : LPC3180_Startup.s
;Purpose: Startup file for embOS and IAR embedded workbench V5.10
;-------- END-OF-HEADER ----------------------------------------------
;

CP_DIS_MASK DEFINE ~5     ; MMU Disable Mask

        MODULE  ?cstartup

        ;; Forward declaration of sections.
        SECTION PROGRAM_DATA:CODE
        SECTION IRQ_STACK:DATA:NOROOT(3)
        SECTION FIQ_STACK:DATA:NOROOT(3)
        SECTION CSTACK:DATA:NOROOT(3)

        SECTION .intvec:CODE:NOROOT(2)

        PUBLIC  __vector
        PUBLIC  __vector_0x14
        PUBLIC  __iar_program_start
        EXTERN  IRQ_Handler

        ARM

__vector:
                     ldr PC,Reset_Addr        ; Reset
undef_handler:       b   undef_handler        ; Handler for undefined instructions
swi_handler:         b   swi_handler          ; Handler for Software interrupts.
prefetch_handler:    b   prefetch_handler     ; Handle Errors(Aborts) on instruction fetch
data_handler:        b   data_handler         ; Handle Errors(Aborts) on data read or write
__vector_0x14:
                     DCD    0                 ; undefined vector

irq_hanlder:
                     ldr pc,=IRQ_Handler      ; IRQ handler, OS irq handler has to be called here !
fiq_handler:         b   fiq_handler          ; FIQ handler

        LTORG            ; Constant table entries (for ldr pc) will be placed at 0x20
Reset_Addr:          DCD   __iar_program_start
__vector_end:

; --------------------------------------------------
; ?cstartup -- low-level system initialization code.
;
; After a reser execution starts here, the mode is ARM, supervisor
; with interrupts disabled.
;


        SECTION .text:CODE:NOROOT(2)

        EXTERN  ?main
        REQUIRE __vector

        ARM

__iar_program_start:

/**********************************************************************
* ?CSTARTUP
*
* Execution starts here.
* After a reset, the mode is ARM, Supervisor, interrupts disabled.
*/
?cstartup:

        mrc     p15, 0, r0, c1, c0,0                ; read cp15 control register (cp15 r1) r0
        ldr     r3, =0xC0001085                     ; Disable Cache, Disable MMU
        ldr     r4, =0x40004000                     ; Round Robin replacement
        bic     r0, r0, r3
        orr     r0, r0, r4
        mcr     p15, 0, r0, c1, c0,0                ; write cp15 control register (cp15 r1)

; Initialize the stack pointers.
        ;; Set up the interrupt stack pointer.

        msr     cpsr_c,#0xd2            ; Change mode to IRQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(IRQ_STACK)     ; End of IRQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary
	
        ;; Set up the fast interrupt stack pointer.

        msr     cpsr_c,#0xd1            ; Change mode to FIQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(FIQ_STACK)     ; End of FIQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary

        ;; Set up the normal stack pointer.

        msr     cpsr_c,#0xdf            ; Change mode to SYS mode, FIQ and IRQ disabled
        ldr     sp, =SFE(CSTACK)        ; End of CSTACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary


#ifdef __ARMVFP__
; Enable the VFP coprocessor.
        mov     r0, #0x40000000                 ; Set EN bit in VFP
        fmxr    fpexc, r0                       ; FPEXC, clear others.

; Disable underflow exceptions by setting flush to zero mode.
; For full IEEE 754 underflow compliance this code should be removed
; and the appropriate exception handler installed.
        mov     r0, #0x01000000                 ; Set FZ bit in VFP
        fmxr    fpscr, r0                       ; FPSCR, clear others.

#endif

; Add more initialization here if required


; Jump to startup code in library
        ldr     r0,=?main
        bx      r0

        LTORG
        END

; ***** EOF **********************************************************



