/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for LPC3250 Starter boards and IAR compiler
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** SFRs used for LED-Port **************************************/
#define PORT3_BASE_ADDR     0x40028000
#define LED_PORT_BASE_ADDR  PORT3_BASE_ADDR
#define PIO_OUTP_SET        *(volatile unsigned int*)(LED_PORT_BASE_ADDR + 0x4)
#define PIO_OUTP_CLR        *(volatile unsigned int*)(LED_PORT_BASE_ADDR + 0x8)
#define PIO_OUTP_STATE      *(volatile unsigned int*)(LED_PORT_BASE_ADDR + 0xC)

/****** Assign LEDs to Ports ****************************************/

#define LED0_BIT       1    // LED1 on Eval board
#define LED1_BIT      14    // LED2 on Eval board
#define LED0_MASK     (1 << LED0_BIT)
#define LED1_MASK     (1 << LED1_BIT)
#define LED_MASK_ALL  (LED0_MASK | LED1_MASK)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void BSP_Init(void) {  // Initially switch off LEDs
  PIO_OUTP_SET = LED_MASK_ALL;  // LEDS are active low
}

/*********************************************************************
*
*       LED switching routines
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    PIO_OUTP_CLR = LED0_MASK;    // Switch on LED 0
  } else {
    PIO_OUTP_CLR = LED1_MASK;    // Switch on LED 1
  }
}

void BSP_ClrLED(int Index) {
  if (Index == 0) {
    PIO_OUTP_SET = LED0_MASK;    // Switch off LED 0
  } else {
    PIO_OUTP_SET = LED1_MASK;    // Switch off LED 1
  }
}

void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if (PIO_OUTP_STATE & LED0_MASK) {
      BSP_SetLED(0);
    } else {
      BSP_ClrLED(0);
    }
  } else {
    if (PIO_OUTP_STATE & LED1_MASK) {
      BSP_SetLED(1);
    } else {
      BSP_ClrLED(1);
    }
  }
}

/****** EOF *********************************************************/

