/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for custom hardware
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define CCM_CCGR2        (*(volatile unsigned int*)(0x4006B048u))             // CCM Clock Gating Register 2
#define IOMUXC_PTA27     (*(volatile unsigned int*)(0x40048044u))             // Software MUX Pad Control Register 18
#define IOMUXC_PTA28     (*(volatile unsigned int*)(0x40048048u))             // Software MUX Pad Control Register 18
#define IOMUXC_PTB2      (*(volatile unsigned int*)(0x40048060u))             // Software MUX Pad Control Register 18
#define IOMUXC_PTB7      (*(volatile unsigned int*)(0x40048074u))             // Software MUX Pad Control Register 18

#define GPIO0_BASE_ADDR  (0x400FF000u)
#define GPIO0_PSOR       (*(volatile unsigned int*)(GPIO0_BASE_ADDR + 0x04u)) // Port Set Output Register
#define GPIO0_PCOR       (*(volatile unsigned int*)(GPIO0_BASE_ADDR + 0x08u)) // Port Clear Output Register
#define GPIO0_PTOR       (*(volatile unsigned int*)(GPIO0_BASE_ADDR + 0x0Cu)) // Port Toggle Output Register

#define LED0             (24u)                                                // Bit position of LED0 (Red led CR1, PTB2)
#define LED1             (29u)                                                // Bit position of LED1 (Green led CR2, PTB7)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  CCM_CCGR2  |= (3u << 16u);   // Enable clock for IOMUX
  IOMUXC_PTB2 = 0x00000182u;   // Set to GPIO and output
  IOMUXC_PTB7 = 0x00000182u;   // Set to GPIO and output
  GPIO0_PCOR |= (1u << LED0);  // Initially turn off LED0
  GPIO0_PCOR |= (1u << LED1);  // Initially turn off LED1
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    GPIO0_PSOR |= (1u << LED0);
  } else if (Index == 1) {
    GPIO0_PSOR |= (1u << LED1);
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    GPIO0_PCOR |= (1u << LED0);
  } else if (Index == 1) {
    GPIO0_PCOR |= (1u << LED1);
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    GPIO0_PTOR |= (1u << LED0);
  } else if (Index == 1) {
    GPIO0_PTOR |= (1u << LED1);
  }
}

/****** End Of File *************************************************/
