/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   OS_IGNORE_UNDEFINED_INTERRUPT
  #define OS_IGNORE_UNDEFINED_INTERRUPT  (0)
#endif

/*********************************************************************
*
*       Clock settings
*/

#define OS_SYSTEM_FREQ    (452000000u)

/*********************************************************************
*
*       System tick settings
*/

#define OS_TIMER_FREQ     (OS_SYSTEM_FREQ / 4)
#define OS_TICK_FREQ      (1000u)
#define OS_INT_FREQ       (OS_TICK_FREQ)
#define OS_TIMER_RELOAD   (OS_TIMER_FREQ / OS_INT_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
//
// Clock configuration
//
#define CCM_CSR                       (*(volatile unsigned int*)(0x4006B004u))                           // CCM Status Register
#define CCM_CCSR                      (*(volatile unsigned int*)(0x4006B008u))                           // CCM Clock Switcher Register
#define CCM_CACRR                     (*(volatile unsigned int*)(0x4006B00Cu))                           // CCM ARM Clock Root Register
#define CCM_CLPCR                     (*(volatile unsigned int*)(0x4006B02Cu))                           // CCM Low Power Control Register
#define CCM_CCGR1                     (*(volatile unsigned int*)(0x4006B044u))                           // CCM Clock Gating Register 1
#define CCM_CCGR2                     (*(volatile unsigned int*)(0x4006B048u))                           // CCM Clock Gating Register 2
#define CCM_CCGR3                     (*(volatile unsigned int*)(0x4006B04Cu))                           // CCM Clock Gating Register 3

#define ANADIG_PLL1_CTRL              (*(volatile unsigned int*)(0x40050270u))                           // PLL1 Control register
#define ANADIG_PLL2_CTRL              (*(volatile unsigned int*)(0x40050030u))                           // PLL2 Control register
#define ANADIG_PLL3_CTRL              (*(volatile unsigned int*)(0x40050010u))                           // PLL3 Control register
#define ANADIG_PLL4_CTRL              (*(volatile unsigned int*)(0x40050070u))                           // PLL4 Control register
#define ANADIG_PLL5_CTRL              (*(volatile unsigned int*)(0x400500E0u))                           // PLL5 Control register
#define ANADIG_PLL6_CTRL              (*(volatile unsigned int*)(0x400500A0u))                           // PLL6 Control register
#define ANADIG_PLL7_CTRL              (*(volatile unsigned int*)(0x40050020u))                           // PLL7 Control register

#define CCM_CLPCR_XOSC_PWRDWN_MASK    (1u << 11)
#define CCM_CSR_FXOSC_RDY_MASK        (1u <<  5)
#define CCM_CCSR_FAST_CLK_SEL_MASK    (1u <<  5)

#define ANADIG_PLL1_CTRL_LOCK_MASK    (1u << 31)
#define ANADIG_PLL2_CTRL_LOCK_MASK    (1u << 31)
#define ANADIG_PLL3_CTRL_LOCK_MASK    (1u << 31)
#define ANADIG_PLL4_CTRL_LOCK_MASK    (1u << 31)
#define ANADIG_PLL5_CTRL_LOCK_MASK    (1u << 31)
#define ANADIG_PLL6_CTRL_LOCK_MASK    (1u << 31)
#define ANADIG_PLL7_CTRL_LOCK_MASK    (1u << 31)

//
// Interrupt router
//
#define MSCM_BASE_ADDR                (0x40001000u)
#define MSCM_IRSPRC_BASE_ADDR         (MSCM_BASE_ADDR + 0x880u)                                          // Interrupt Router Shared Peripheral Routing Control

//
// ARM Generic interrupt controller (GIC)
//
#define GIC_INTERFACE_BASE_ADDR       (0x40002100u)
#define GIC_ICCICR                    (*(volatile unsigned int*)(GIC_INTERFACE_BASE_ADDR + 0x0000u))     // CPU Interface Control Register
#define GIC_ICCPMR                    (*(volatile unsigned int*)(GIC_INTERFACE_BASE_ADDR + 0x0004u))     // Interrupt Priority Mask Register
#define GIC_ICCIAR                    (*(volatile unsigned int*)(GIC_INTERFACE_BASE_ADDR + 0x000Cu))     // Interrupt Acknowledge Register
#define GIC_ICCEOIR                   (*(volatile unsigned int*)(GIC_INTERFACE_BASE_ADDR + 0x0010u))     // End of Interrupt Register

#define GIC_DISTRIBUTION_BASE_ADDR    (0x40003000u)
#define GIC_ICDDCR                    (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0000u))  // Interrupt distributor Control Register
#define GIC_ICDISR0                   (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0080u))  // Interrupt security registers 0
#define GIC_ICDISR1                   (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0084u))  // Interrupt security registers 1
#define GIC_ICDISR2                   (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0088u))  // Interrupt security registers 2
#define GIC_ICDISR3                   (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x008Cu))  // Interrupt security registers 3
#define GIC_ICDISR4                   (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0090u))  // Interrupt security registers 4
#define GIC_ICDISER_ADDR              (GIC_DISTRIBUTION_BASE_ADDR + 0x0100u)                             // Interrupt set enable registers base address
#define GIC_ICDICER_ADDR              (GIC_DISTRIBUTION_BASE_ADDR + 0x0180u)                             // Interrupt clear enable registers base address
#define GIC_ICDICER0                  (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0180u))  // Interrupt enable clear registers 0
#define GIC_ICDICER1                  (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0184u))  // Interrupt enable clear registers 1
#define GIC_ICDICER2                  (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0188u))  // Interrupt enable clear registers 2
#define GIC_ICDICER3                  (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x018Cu))  // Interrupt enable clear registers 3
#define GIC_ICDICER4                  (*(volatile unsigned int*)(GIC_DISTRIBUTION_BASE_ADDR + 0x0190u))  // Interrupt enable clear registers 4
#define GIC_ICDIPR_ADDR               (GIC_DISTRIBUTION_BASE_ADDR + 0x0400u)                             // Interrupt priority register address

#define NUM_INT_SOURCES               (112u)

//
// PIT timer
//
#define PIT_BASE_ADDR                 (0x40037000u)
#define PIT_MCR                       (*(volatile unsigned int*)(PIT_BASE_ADDR + 0x000u))                // PIT Module Control Register
#define PIT_LDVAL0                    (*(volatile unsigned int*)(PIT_BASE_ADDR + 0x100u))                // Timer Load Value Register
#define PIT_CVAL0                     (*(volatile unsigned int*)(PIT_BASE_ADDR + 0x104u))                // Current Timer Value Register
#define PIT_TCTRL0                    (*(volatile unsigned int*)(PIT_BASE_ADDR + 0x108u))                // Timer Control Register
#define PIT_TFLG0                     (*(volatile unsigned int*)(PIT_BASE_ADDR + 0x10Cu))                // Timer Flag Register
#define PIT_MCR_MDIS_MASK             (0x02u)
#define PIT_TFLG0_TIF_MASK            (0x01u)
#define PIT_INT_ID                    ( 71u)
#define PIT_INT_PRIO                  (240u) // With Vybrid VF3 5 priority bits are implemented, thus this is the lowest priority

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/

extern unsigned int __vector;

static void _OS_ISR_Undefined(void);

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       MMU and cache configuration
*
*  Notes
*    The translation table has to be aligned on 16384 bytes boundary
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=16384
  static __no_init OS_U32 _TranslationTable[0x1000];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(16384) static OS_U32 _TranslationTable[0x1000];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_U32 _TranslationTable[0x1000] __attribute__ ((aligned (16384), section (".no_init")));
#endif

/*********************************************************************
*
*       Interrupt vector table
*
*  Notes
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=4
  static __no_init OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(4) static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES] __attribute__ ((aligned (4), section (".no_init")));
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _SpuriousIrqHandler()
*
*  Function description
*    default spurious IRQ handler
*
*  Notes
*    A spurious interrupt is called under one of the following
*    circumstances:
*    1. An external interrupt source is programmed in level-sensitive mode
*       and an active level occurs for only a short time.
*    2. An internal interrupt source is programmed in level sensitive
*       and the output signal of the corresponding embedded peripheral
*       is activated for a short time. (As in the case for the Watchdog.)
*    3. An interrupt occurs just a few cycles before the software begins
*       to mask it, thus resulting in a pulse on the interrupt source.
*
*    The spurious handler is called to detect these errors.
*/
static void _SpuriousIrqHandler(void) {
#if (OS_DEBUG != 0)
  static OS_U32 _SpuriousIrqCnt;

  _SpuriousIrqCnt++;
#endif
}

/*********************************************************************
*
*       _InitVIC()
*
*  Function description
*    Initialize interrupt controller by setting default vectors
*    and clearing all interrupts
*/
static void _InitVIC(OS_U32 addr) {
  unsigned int i;
  __MCR(15, 0, addr, 12, 0, 0);               // Set interrupt vector table base addr
  //
  // Configure all interrupts as non-secure
  //
  GIC_ICDISR0 = 0xFFFFFFFFu;
  GIC_ICDISR1 = 0xFFFFFFFFu;
  GIC_ICDISR2 = 0xFFFFFFFFu;
  GIC_ICDISR3 = 0xFFFFFFFFu;
  GIC_ICDISR4 = 0xFFFFFFFFu;
  //
  // Disable all interrupts
  //
  GIC_ICDICER0 = 0xFFFFFFFFu;
  GIC_ICDICER1 = 0xFFFFFFFFu;
  GIC_ICDICER2 = 0xFFFFFFFFu;
  GIC_ICDICER3 = 0xFFFFFFFFu;
  GIC_ICDICER4 = 0xFFFFFFFFu;
  //
  // Configure priority mask to lowest value (highest number) - enables all priorities
  //
  GIC_ICCPMR = 0xFF;
  //
  // Enable both secure and non-secure interrupts at CPU interface & distributor
  // Also enable secure id_read & acknowledge of non-secure interrupts
  //
  GIC_ICCICR = 0x07u;
  GIC_ICDDCR = 0x03u;
  //
  // Program the interrupt router.
  // Route all interrupts to CA5
  //
  for (i = 0 ; i < NUM_INT_SOURCES ; i++) {
    (*(volatile unsigned short*)(MSCM_IRSPRC_BASE_ADDR + (i * 0x2u))) |= 0x01u; // MSCM.IRSPRC[i].B.CP0 = 1;
  }
  //
  // Set default interrupt handler for all interrupts
  //
  for (i = 0 ; i < NUM_INT_SOURCES ; i++) {
    _apIsrHandler[i] = _OS_ISR_Undefined;
  }
}

/*********************************************************************
*
*       _PLL_Init()
*
*  Function description
*    Initialize the PLL Unit
*/
static void _PLL_Init(void) {
  //
  // Set XTAL as SYSCLK
  //
  CCM_CLPCR &= ~CCM_CLPCR_XOSC_PWRDWN_MASK;                         // Power up FXOSC
  while ((CCM_CSR & CCM_CSR_FXOSC_RDY_MASK) == 0x00u);              // Wait until FXOSC ready
  CCM_CCSR |= CCM_CCSR_FAST_CLK_SEL_MASK;                           // Set XTAL as fast clock source

  CCM_CCGR3 |= 0x03u;                                               // Enable clock for ANADIG

  ANADIG_PLL1_CTRL = 0x00002001u;                                   // Set PLL1
  while ((ANADIG_PLL1_CTRL & ANADIG_PLL1_CTRL_LOCK_MASK) == 0x00u); // Wait until lock

  CCM_CACRR = 0x00600808u;                                          // Set ARMCLKDIV = 1, BUSDIV=2, IPG_CLK_DIV=2
  CCM_CCSR  = 0xF002ff64u;                                          // Enable all PFDs sources and set SYSCLK as PLL1 PFD2 for 452MHz (no divider)
}

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when an not installed interrupt was detected.
*    As interrupt pending condition of peripherals has to be reset,
*    program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(void) {
#if (OS_IGNORE_UNDEFINED_INTERRUPT == 0)
  volatile int Dummy;

  Dummy = 1;
  while (Dummy > 0) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
#endif
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return PIT_CVAL0;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return ((PIT_TFLG0 & PIT_TFLG0_TIF_MASK) == PIT_TFLG0_TIF_MASK);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  PIT_TFLG0 |= PIT_TFLG0_TIF_MASK;  // Clear timer interrupt pending flag
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_DOWNCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize interrupt vector controller
  //
  _InitVIC((OS_U32)&__vector);
  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  CCM_CCGR1  |= (0x03u << 14u);     // Make sure the PIT clock is enabled
  PIT_TCTRL0  = 0x00u;              // Stop PIT
  PIT_MCR    &= ~PIT_MCR_MDIS_MASK; // Enable clock for PIT
  PIT_LDVAL0  = OS_TIMER_RELOAD;    // Set timer reload value
  PIT_TCTRL0  = 0x03u;              // Interrupt enable and timer enable
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler(PIT_INT_ID, _OS_ISR_Tick);
  OS_ARM_ISRSetPrio(PIT_INT_ID, PIT_INT_PRIO);
  OS_ARM_EnableISR(PIT_INT_ID);
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  OS_U32          Id;
  OS_U32          Vector;

  Id = GIC_ICCIAR;                       // Read interrupt ID
  if ((Id == 1022u) || (Id == 1023u)) {  // Check for spurious interrupts
    _SpuriousIrqHandler();
  } else {
    Vector = Id & 0x3FF;                 // Mask ID bits 0..9
    pISR = _apIsrHandler[Vector];        // Get interrupt vector handler
    OS_INT_EnterNestable();
    pISR();                              // Call interrupt service routine
    OS_INT_LeaveNestable();
  }
  GIC_ICCEOIR = Id;                      // End interrupt
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pOldHandler             = _apIsrHandler[ISRIndex];
  _apIsrHandler[ISRIndex] = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
  OS_U32* pReg;
  OS_U32  Off;
  OS_U32  Shift;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  //
  // Calculate register index and offset position
  //
  Off       = (ISRIndex / 32u);
  Shift     = (ISRIndex % 32u);
  //
  // Set interrupt enable flag
  //
  pReg      = (OS_U32*)GIC_ICDISER_ADDR;
  pReg[Off] = (1uL << Shift);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
  OS_U32* pReg;
  OS_U32  Off;
  OS_U32  Shift;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  //
  // Calculate register index and offset position
  //
  Off       = (ISRIndex / 32u);
  Shift     = (ISRIndex % 32u);
  //
  // Clear interrupt enable flag
  //
  pReg      = (OS_U32*)GIC_ICDICER_ADDR;
  pReg[Off] = (1u << Shift);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  OS_U32  OldPrio;
  OS_U32  Off;
  OS_U32  Shift;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  //
  // Calculate register index and offset position
  //
  Off         = (ISRIndex / 4u);
  Shift       = (ISRIndex % 4u) * 8u;
  //
  // Read current priority, delete priority bits and set new priority
  //
  pPrio       = (OS_U32*) GIC_ICDIPR_ADDR;
  OldPrio     = (pPrio[Off] & (0xFFu << Shift)) >> Shift;
  pPrio[Off] &= ~(0xFFu << Shift);
  pPrio[Off] |= (Prio << Shift);
  OS_INT_EnableConditional();
  return (int)OldPrio;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
int __low_level_init(void) {
  //
  // Initialize PLL
  //
  _PLL_Init();
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Memory mapping should be complete, meaning 4096 entries.
  // Code below fills in ascending VAddr order
  //
  OS_ARM_MMU_InitTT(_TranslationTable);
  //                                         Mode                                Access permissions        VAddr  PAddr  Size[MB]
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x000, 0x000, 0x200);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_WRITE_BACK_ALLOC,                            0x200, 0x200, 0x004);    // QSPI flash
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x204, 0x204, 0x1EC);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_WRITE_BACK_ALLOC,                            0x3F0, 0x3F0, 0x001);    // OCRAM - SysRAM0, 0x3F000000-0x3F03FFFF and OCRAM - SysRAM1, 0x3F040000-0x3F07FFFF, 1 MB (program and data area)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x3F1, 0x3F1, 0x00F);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_STRONGLY_ORDERED | OS_ARM_MMU_EXECUTE_NEVER, 0x400, 0x400, 0x001);    // SFRs, non cacheable, non bufferable
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x401, 0x401, 0xBFF);    // Unmapped area
  OS_ARM_MMU_Enable      (_TranslationTable);
  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();
  return 1;  // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
