#ifndef _QUADSPI_BOOT_H_
#define _QUADSPI_BOOT_H_

#define byte_swap16(word) \
  ( ((word&0x00FF) << 8) |			\
  ((word&0xFF00) >> 8) )


/*************************************
 *  QuadSPI Configuration Information
 *************************************/
#define  COMAND_SEQ_LEN            128 //interms of instruction word size

/* MACROS */
#define IMAGE_ENTRY_FUNCTION  __iar_program_start
#define DCD_ADDRESS           device_config_data
#define BOOT_DATA_ADDRESS     &boot_data
#define CSF_ADDRESS           0

#if (QSPI_CFG == 0)
  /* QuadSPI0 Configuration */
  #define FLASH_BASE            0x20000000
  #define FLASH_END             0x20400000
#else
  /* QuadSPI1 Configuration */
  #define FLASH_BASE            0x50000000
  #define FLASH_END             0x50400000
#endif

#define PLUGIN_FLAG           (unsigned int)0


/*Quad SPI configuration structures */
typedef struct __sflash_configuration_parameter
{
  unsigned int dqs_loopback;              //Sets DQS LoopBack Mode to enable Dummy Pad MCR[24]
  unsigned int rsvd[4];
  unsigned int cs_hold_time;              //CS hold time in terms of serial clock.(for example 1 serial clock cyle)
  unsigned int cs_setup_time;             //CS setup time in terms of serial clock.(for example 1 serial clock cyle)
  unsigned int sflash_A1_size;            //interms of Bytes
  unsigned int sflash_A2_size;            //interms of Bytes
  unsigned int sflash_B1_size;            //interms of Bytes
  unsigned int sflash_B2_size;            //interms of Bytes
  unsigned int sclk_freq;                 //In  00 - 22MHz, 01 - 66MHz, 10 - 80MHz, 11 - 104MHz (only for SDR Mode)
  unsigned int rsvd5;                     //Reserved for Future Use
  unsigned char sflash_type;               //0-Single,1--Dual 2--Quad
  unsigned char sflash_port;               //0--Only Port-A,1--Both PortA and PortB
  unsigned char ddr_mode_enable;          //Enable DDR mode if set to TRUE
  unsigned char dqs_enable;               //Enable DQS mode if set to TRUE.
  unsigned char parallel_mode_enable;     //Enable Individual or parrallel mode.
  unsigned char portA_cs1;
  unsigned char portB_cs1;
  unsigned char fsphs;
  unsigned char fsdly;
  unsigned char ddrsmp;
  unsigned short  command_seq[COMAND_SEQ_LEN]; //set of seq to perform optimum read on SFLASH as as per vendor SFLASH

}SFLASH_CONFIGURATION_PARAM,*SFLASH_CONFIGURATION_PARAM_PTR;



/*************************************
 *  DCD Data
 *************************************/
#define DCD_TAG_HEADER            (0xD2)
#define DCD_TAG_HEADER_SHIFT      (24)
#define DCD_VERSION               (0x40)
#define DCD_ARRAY_SIZE             1


/*************************************
 *  IVT Data
 *************************************/
typedef struct _ivt_ {
    /** @ref hdr with tag #HAB_TAG_IVT, length and HAB version fields
     *  (see @ref data)
     */
    unsigned int hdr;
    /** Absolute address of the first instruction to execute from the
     *  image
     */
    unsigned int entry;
    /** Reserved in this version of HAB: should be NULL. */
    unsigned int reserved1;
    /** Absolute address of the image DCD: may be NULL. */
    unsigned int dcd;
    /** Absolute address of the Boot Data: may be NULL, but not interpreted
     *  any further by HAB
     */
    unsigned int boot_data;
    /** Absolute address of the IVT.*/
    unsigned int self;
    /** Absolute address of the image CSF.*/
    unsigned int csf;
    /** Reserved in this version of HAB: should be zero. */
    unsigned int reserved2;
} ivt;

#define IVT_MAJOR_VERSION           0x4
#define IVT_MAJOR_VERSION_SHIFT     0x4
#define IVT_MAJOR_VERSION_MASK      0xF
#define IVT_MINOR_VERSION           0x1
#define IVT_MINOR_VERSION_SHIFT     0x0
#define IVT_MINOR_VERSION_MASK      0xF

#define IVT_VERSION(major, minor)   \
  ((((major) & IVT_MAJOR_VERSION_MASK) << IVT_MAJOR_VERSION_SHIFT) |  \
  (((minor) & IVT_MINOR_VERSION_MASK) << IVT_MINOR_VERSION_SHIFT))


#define IVT_TAG_HEADER        (0xD1)       /**< Image Vector Table */
#define IVT_SIZE              byte_swap16(sizeof(ivt))
#define IVT_PAR               IVT_VERSION(IVT_MAJOR_VERSION, IVT_MINOR_VERSION)

#define IVT_HEADER          (IVT_TAG_HEADER | (IVT_SIZE << 8) | (IVT_PAR << 24))
#define IVT_RSVD            (unsigned int)(0x00000000)


/*************************************
 *  Boot Data
 *************************************/
typedef struct _boot_data_ {
  unsigned int start;           /* boot start location */
  unsigned int size;            /* size */
  unsigned int plugin;          /* plugin flag - 1 if downloaded application is plugin */
}BOOT_DATA_T;

/* External Variables */
extern const unsigned int device_config_data[];
extern const BOOT_DATA_T boot_data;
extern void _start(void);


#endif
