/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for IAR KSK-iMX25 eval board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Ports *******************************************************/

#define IOMUX_BASE_ADDR                   0x43FAC000uL

#define IOMUXC_SW_MUX_CTL_PAD_A14         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x010))
#define IOMUXC_SW_MUX_CTL_PAD_A15         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x014))
#define IOMUXC_SW_MUX_CTL_PAD_A16         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x018))
#define IOMUXC_SW_MUX_CTL_PAD_A17         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x01C))
#define IOMUXC_SW_MUX_CTL_PAD_A18         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x020))
#define IOMUXC_SW_MUX_CTL_PAD_A19         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x024))
#define IOMUXC_SW_MUX_CTL_PAD_A20         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x028))
#define IOMUXC_SW_MUX_CTL_PAD_A21         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x02C))
#define IOMUXC_SW_MUX_CTL_PAD_A22         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x030))
#define IOMUXC_SW_MUX_CTL_PAD_A23         (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x034))
#define IOMUXC_SW_MUX_CTL_PAD_UART2_RTS   (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x188))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_MDC     (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1C8))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_MDIO    (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1CC))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_TDATA0  (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1D0))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_TDATA1  (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1D4))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_TX_EN   (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1D8))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_RDATA0  (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1DC))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_RDATA1  (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1E0))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_RX_DV   (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1E4))
#define IOMUXC_SW_MUX_CTL_PAD_FEC_TX_CLK  (*(volatile unsigned long *)(IOMUX_BASE_ADDR + 0x1E8))

#define GPIO2_BASE_ADDR                   0x53FD0000uL
#define GPIO2_DR                          (*(volatile unsigned long *)(GPIO2_BASE_ADDR + 0x00))
#define GPIO2_GDIR                        (*(volatile unsigned long *)(GPIO2_BASE_ADDR + 0x04))

#define GPIO3_BASE_ADDR                   0x53FA4000uL
#define GPIO3_DR                          (*(volatile unsigned long *)(GPIO3_BASE_ADDR + 0x00))
#define GPIO3_GDIR                        (*(volatile unsigned long *)(GPIO3_BASE_ADDR + 0x04))

#define GPIO4_BASE_ADDR                   0x53F9C000uL
#define GPIO4_DR                          (*(volatile unsigned long *)(GPIO4_BASE_ADDR + 0x00))
#define GPIO4_GDIR                        (*(volatile unsigned long *)(GPIO4_BASE_ADDR + 0x04))

#define ALT5_MUX_MODE       5

/****** LED-ports ***************************************************/

#define LED0_BIT            0
#define LED1_BIT            1
#define LED2_BIT            2
#define LED3_BIT            3
#define LED4_BIT            4
#define LED5_BIT            5
#define LED6_BIT            6
#define LED7_BIT            7

#define LED0_MASK           (1uL << 0)
#define LED1_MASK           (1uL << 1)
#define LED2_MASK           (1uL << 2)
#define LED3_MASK           (1uL << 3)
#define LED4_MASK           (1uL << 4)  // Shared with Ethernet
#define LED5_MASK           (1uL << 5)  // Shared with Ethernet
#define LED6_MASK           (1uL << 6)  // Shared with Ethernet
#define LED7_MASK           (1uL << 7)  // Shared with Ethernet

#define LED_MASK_ALL        ( LED0_MASK | LED1_MASK | LED2_MASK | LED3_MASK \
                            | LED4_MASK | LED5_MASK | LED6_MASK | LED7_MASK )

#define LED_PORT            GPIO2_DR
#define LED_DIR             GPIO2_GDIR

#define NUM_LEDS            4 

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // Initialize port for LEDs, LEDs are switched on with low level on port lines
  //
  IOMUXC_SW_MUX_CTL_PAD_A14 |= ALT5_MUX_MODE;
  IOMUXC_SW_MUX_CTL_PAD_A15 |= ALT5_MUX_MODE;
  IOMUXC_SW_MUX_CTL_PAD_A16 |= ALT5_MUX_MODE;
  IOMUXC_SW_MUX_CTL_PAD_A17 |= ALT5_MUX_MODE;

  LED_PORT |= LED_MASK_ALL; // Initially all LEDs OFF
  LED_DIR  |= LED_MASK_ALL; // All LED ports switched to output mode
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index >= NUM_LEDS) {
    return;
  }
  LED_PORT &= ~(1uL << Index);        /* Switch on selected LED  */
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index >= NUM_LEDS) {
    return;
  }
  LED_PORT |= (1uL << Index);         /* Switch off selected LED */
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index >= NUM_LEDS) {
    return;
  }
  if (LED_PORT & (1uL << Index)) {    /* Selected LED is switched off */
    BSP_SetLED(Index);
  } else {
    BSP_ClrLED(Index);
  }
}

/****** End Of File *************************************************/
