/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   SDRAM_TARGET
  #define SDRAM_TARGET  (0)
#endif

/*********************************************************************
*
*       Clock settings
*/
#define OS_SYSTEM_FREQ     (266918181u)

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ      (OS_SYSTEM_FREQ / 2 / 4)
#define OS_TICK_FREQ       (1000u)
#define OS_INT_FREQ        (OS_TICK_FREQ)
#define OS_TIMER_RELOAD    ((OS_TIMER_FREQ / OS_INT_FREQ) - 1)

/*********************************************************************
*
*       Memory settings
*/
#define   DDR_RAM_TYPE  (1)
#define   DDR_CS_ADDR   (0x20000000u)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
//
// ASIC
//
#define INTCNTL       *((volatile OS_U32 *)(0x68000000uL))
#define NIMASK        *((volatile OS_U32 *)(0x68000004uL))
#define INTENNUM      *((volatile OS_U32 *)(0x68000008uL))
#define INTDISNUM     *((volatile OS_U32 *)(0x6000000CuL))
#define INTENABLEH    *((volatile OS_U32 *)(0x68000010uL))
#define INTENABLEL    *((volatile OS_U32 *)(0x68000014uL))
#define INTTYPEH      *((volatile OS_U32 *)(0x68000018uL))
#define INTTYPEL      *((volatile OS_U32 *)(0x6800001CuL))
#define NIPRIORITY7   *((volatile OS_U32 *)(0x68000020uL))
#define NIPRIORITY6   *((volatile OS_U32 *)(0x68000024uL))
#define NIPRIORITY5   *((volatile OS_U32 *)(0x68000028uL))
#define NIPRIORITY4   *((volatile OS_U32 *)(0x6800002CuL))
#define NIPRIORITY3   *((volatile OS_U32 *)(0x68000030uL))
#define NIPRIORITY2   *((volatile OS_U32 *)(0x68000034uL))
#define NIPRIORITY1   *((volatile OS_U32 *)(0x68000038uL))
#define NIPRIORITY0   *((volatile OS_U32 *)(0x6800003CuL))
#define NIVECSR       *((volatile OS_U32 *)(0x68000040uL))
#define FIVECSR       *((volatile OS_U32 *)(0x68000044uL))
#define INTSRCH       *((volatile OS_U32 *)(0x68000048uL))
#define INTSRCL       *((volatile OS_U32 *)(0x6800004CuL))
#define INTFRCH       *((volatile OS_U32 *)(0x68000050uL))
#define INTFRCL       *((volatile OS_U32 *)(0x68000054uL))
#define NIPNDH        *((volatile OS_U32 *)(0x68000058uL))
#define NIPNDL        *((volatile OS_U32 *)(0x6800005CuL))
#define FIPNDH        *((volatile OS_U32 *)(0x68000060uL))
#define FIPNDL        *((volatile OS_U32 *)(0x68000064uL))

#define NIPRIORITY_BASE_ADDR  0x68000020uL
//
// CRM
//
#define MPCTL         *((volatile OS_U32 *)(0x53F80000uL))
#define UPCTL         *((volatile OS_U32 *)(0x53F80004uL))
#define CCTL          *((volatile OS_U32 *)(0x53F80008uL))
#define CGCR0         *((volatile OS_U32 *)(0x53F8000CuL))
#define CGCR1         *((volatile OS_U32 *)(0x53F80010uL))
#define CGCR2         *((volatile OS_U32 *)(0x53F80014uL))
#define PCDR0         *((volatile OS_U32 *)(0x53F80018uL))
#define PCDR1         *((volatile OS_U32 *)(0x53F8001CuL))
#define PCDR2         *((volatile OS_U32 *)(0x53F80020uL))
#define PCDR3         *((volatile OS_U32 *)(0x53F80024uL))
#define RCSR          *((volatile OS_U32 *)(0x53F80028uL))
#define CRDR          *((volatile OS_U32 *)(0x53F8002CuL))
#define DCVR0         *((volatile OS_U32 *)(0x53F80030uL))
#define DCVR1         *((volatile OS_U32 *)(0x53F80034uL))
#define DCVR2         *((volatile OS_U32 *)(0x53F80038uL))
#define DCVR3         *((volatile OS_U32 *)(0x53F8003CuL))
#define LTR0          *((volatile OS_U32 *)(0x53F80040uL))
#define LTR1          *((volatile OS_U32 *)(0x53F80044uL))
#define LTR2          *((volatile OS_U32 *)(0x53F80048uL))
#define LTR3          *((volatile OS_U32 *)(0x53F8004CuL))
#define LTBR0         *((volatile OS_U32 *)(0x53F80050uL))
#define LTBR1         *((volatile OS_U32 *)(0x53F80054uL))
#define PMCR0         *((volatile OS_U32 *)(0x53F80058uL))
#define PMCR1         *((volatile OS_U32 *)(0x53F8005CuL))
#define PMCR2         *((volatile OS_U32 *)(0x53F80060uL))
#define MCR           *((volatile OS_U32 *)(0x53F80064uL))
#define LPIMR0        *((volatile OS_U32 *)(0x53F80068uL))
#define LPIMR1        *((volatile OS_U32 *)(0x53F8006CuL))

// EPIT1
#define EPIT1CR       *((volatile OS_U32 *)(0x53F94000uL))
#define EPIT1SR       *((volatile OS_U32 *)(0x53F94004uL))
#define EPIT1LR       *((volatile OS_U32 *)(0x53F94008uL))
#define EPIT1CMPR     *((volatile OS_U32 *)(0x53F9400CuL))
#define EPIT1CNR      *((volatile OS_U32 *)(0x53F94010uL))

// MAX
#define MPR0          *((volatile OS_U32 *)(0x43F04000uL))
#define SGPCR0        *((volatile OS_U32 *)(0x43F04010uL))
#define MPR1          *((volatile OS_U32 *)(0x43F04100uL))
#define SGPCR1        *((volatile OS_U32 *)(0x43F04110uL))
#define MPR2          *((volatile OS_U32 *)(0x43F04200uL))
#define SGPCR2        *((volatile OS_U32 *)(0x43F04210uL))
#define MPR3          *((volatile OS_U32 *)(0x43F04300uL))
#define SGPCR3        *((volatile OS_U32 *)(0x43F04310uL))
#define MPR4          *((volatile OS_U32 *)(0x43F04400uL))
#define SGPCR4        *((volatile OS_U32 *)(0x43F04410uL))
#define MGPCR0        *((volatile OS_U32 *)(0x43F04800uL))
#define MGPCR1        *((volatile OS_U32 *)(0x43F04900uL))
#define MGPCR2        *((volatile OS_U32 *)(0x43F04A00uL))
#define MGPCR3        *((volatile OS_U32 *)(0x43F04B00uL))
#define MGPCR4        *((volatile OS_U32 *)(0x43F04C00uL))

// ESDRAMC
#define ESDCTL0       *((volatile OS_U32 *)(0xB8001000uL))
#define ESDCFG0       *((volatile OS_U32 *)(0xB8001004uL))
#define ESDCTL1       *((volatile OS_U32 *)(0xB8001008uL))
#define ESDCFG1       *((volatile OS_U32 *)(0xB800100CuL))
#define ESDMISC       *((volatile OS_U32 *)(0xB8001010uL))
#define ESDCDLY1      *((volatile OS_U32 *)(0xB8001020uL))
#define ESDCDLY2      *((volatile OS_U32 *)(0xB8001024uL))
#define ESDCDLY5      *((volatile OS_U32 *)(0xB8001030uL))
#define ESDCDLYL      *((volatile OS_U32 *)(0xB8001034uL))
#define ESDCDLY6      *((volatile OS_U32 *)(0xB8001038uL))

#define NUM_INT_SOURCES     0x40u
#define NUM_INT_PRIORITIES  0x10u
#define INT_PRIORITY_MASK   0x0fu
#define INT_ID_EPIT1        28u

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       MMU and cache configuration
*
*  Notes
*    The translation table has to be aligned on 16384 bytes boundary
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=16384
  static __no_init OS_U32 _TranslationTable[0x1000];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(16384) static OS_U32 _TranslationTable[0x1000];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))   // GCC
  static OS_U32 _TranslationTable[0x1000] __attribute__ ((aligned (16384), section (".no_init")));
#endif

/*********************************************************************
*
*       Interrupt vector table
*
*  Notes
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=4
  static __no_init OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(4) static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES] __attribute__ ((aligned (4), section (".no_init")));
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _DefaultIrqHandler()
*
*  Function description
*    a dummy IRQ handler
*    This handler is initially written into all AIC interrupt vectors
*    It is called, if no interrupt vector was installed for
*    specific interrupt source.
*    May be used during debugging to detect uninstalled interrupts
*/
static void _DefaultIrqHandler(void) {
  OS_U32 IrqSource;

  IrqSource = (NIVECSR >> 16);  /* detect source of uninstalled interrupt */
  while(IrqSource == NIVECSR);
}

/*********************************************************************
*
*       _InitASIC()
*
*  Function description
*    Initialize interrupt controller by setting default vectors
*    and clearing all interrupts
*
*  NOTES:
*    This function may be called from __low_level_init() and therefore
*    must not use or call any function which relies on any variable,
*    because variables are not initialized before __low_level_init()
*    is called !
*/
static void _InitASIC(void) {
  int  i;
  OS_ISR_HANDLER** papISR;

  INTCNTL   |= (1uL << 21);  // Disable all fast interrupts
  INTCNTL   |= (1uL << 22);  // Disable all normal interrupts

  // Mask interrupt by priority
  NIMASK     = 0;
  INTENABLEH = 0;
  INTENABLEL = 0;   // All interrupts are disable
  INTTYPEH   = 0;
  INTTYPEL   = 0;   // All interrupt lines are routed to the normal interrupt
  papISR     = (OS_ISR_HANDLER**) &_apIsrHandler;
  for (i = 0; i < (int)NUM_INT_SOURCES; i++)  {  // Initially set all sources
    *(papISR + i) = &_DefaultIrqHandler;         // to dummy irq handler
  }
  INTCNTL &= ~(1uL << 21);  // Enable all fast interrupts
  INTCNTL &= ~(1uL << 22);  // Enable all normal interrupts
}

/*********************************************************************
*
*       _InitFlash()
*
*  Function description
*    Initialize wait states and BUS interface for external Flash access
*    Needs to be called before main clock is switched to high speed
*    Depends on specific hardware and has to be implemented by user.
*
*  Notes
*    (1) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*/
static void _InitFlash(void) {
}

/*********************************************************************
*
*       _InitClocks()
*
*  Function description
*    Initialize main clock and PLLs
*
*  Notes
*    (1) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*    (2) Settings used here are for KSK i.MX25 eval board
*        and may have to be modified for other target board or frequency
*
*
*/
static void _InitClocks(void) {
  //
  // configure peripheral enables for CGCR0
  //
  CGCR0 = 0x1FFFFFFF;
  //
  // configure peripheral enables for CGCR1
  //
  CGCR1 = 0xFFFFFFFF;
  //
  // configure peripheral enables for CGCR2
  //
  CGCR2 = 0x000FDFFF;
  //
  // Set Peripheral Clock Divider Registers
  //
  PCDR0 = 0x23C80303;
  PCDR1 = 0x03030303;
  PCDR2 = 0x01010103;
  PCDR3 = 0x01010101;

  // AIPS
  //*(unsigned long *)0x43f00000 = 0x77777777;
  //*(unsigned long *)0x43f00004 = 0x77777777;
  //*(unsigned long *)0x53f00000 = 0x77777777;
  //*(unsigned long *)0x53f00004 = 0x77777777;

  // MUX
  // Init Slave General Purpose Control Register
  //
  SGPCR0 = SGPCR1 = SGPCR2 = SGPCR3 = SGPCR4 = 0x00000010;
  //
  // Init Master Priority Register
  //
  MPR0 = MPR1 = MPR2 =  MPR3 = MPR4 = 0x00043210;
  //
  // Init Master General Purpose Control Register
  //
  MGPCR0 = MGPCR1 = MGPCR2 = MGPCR3 = MGPCR4 = 0;
  //
  // Set maximum clock division
  //
  CCTL = (0 << 14)  // ARMSRC
       | (0 << 15)  // CGCTL
       | (0 << 22)  // MPLLBYPASS
                    // FCLK = 266Hz, HCLK = 133 MHz
       | (1 << 30)  // ARMCLKDIV - Clock divide by 2
       | (1 << 28)  // AHBCLKDIV - Clock divide by 2
       ;

}

/*********************************************************************
*
*       _InitSDRAM()
*
*  Function description
*    Initialize SDRAM controller
*
*  Notes
*    (1) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*    (2) Settings used here are for KSK i.MX25 eval board
*        and may have to be modified for other target boards
*/
static void _InitSDRAM(void) {
#if (SDRAM_TARGET == 0)
  // Init SDRAM 0x80000000 - 0x87FFFFFF
  // SDRAM 4x16Mx16 x 1
  // Row Address = 13
  // Col Address = 9
  // ESDMISC: enable SDR mode
  ESDMISC = 0x00000002;
  ESDMISC = 0x00000000;
  // ESDCFG0: timing config
  ESDCFG0 = 0x006AC73A;
  // ESDCTL0: Precharge Mode
  ESDCTL0 = 0x92116080;
  // issue Precharge All command
  *(volatile unsigned int *)0x80000400 = 0;
  // ESDCTL0: AutoRefresh Mode
  ESDCTL0 = 0xA2116080;
  // issue Autorefresh command
  *(volatile unsigned int *)0x80000000 = 0;
  *(volatile unsigned int *)0x80000000 = 0;
  *(volatile unsigned int *)0x80000000 = 0;
  *(volatile unsigned int *)0x80000000 = 0;
  *(volatile unsigned int *)0x80000000 = 0;
  *(volatile unsigned int *)0x80000000 = 0;
  *(volatile unsigned int *)0x80000000 = 0;
  *(volatile unsigned int *)0x80000000 = 0;
  // ESDCTL0: Load Mode register Mode
  ESDCTL0 = 0xB2116080;
  // issue Load Mode Register command
  *(volatile unsigned char *)0x80000033= 0;
  // ESDCTL0: Normal Mode
  // Row Address  = 13
  // Col Address  = 9
  // Data width   = 16-bit
  // Refresh Rate = 8192/64ms
  // Burst Length = 8
  // - 2 : 16 bit mode
  //     : 16 bit - D[15:00]
  ESDCTL0 = 0x82116080;
  // issue normal access
  *(volatile unsigned int *)0x80000000 = 0;
#endif
}

/*********************************************************************
*
*       _InitSMC()
*
*  Function description
*    Initialize static memory controller
*
*  Notes
*    (1) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*/
static void _InitSMC(void) {
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return EPIT1CNR;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return EPIT1SR & (1u << 0);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  if (EPIT1SR & (1u << 0)) {  // Timer interupt pending?
    EPIT1SR = (1u << 0);      // Reset interrupt pending condition
    OS_TICK_Handle();         // Call OS tick handler
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
    JLINKDCC_Process();
#endif
  }
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Ensure caches are synchronized
  //
  OS_ARM_CACHE_Sync();
  //
  // Initialize EPIT1 as OS timer, enable timer + timer interrupt
  //
  EPIT1CR = 0;                  // Disable timer
  EPIT1SR = 1;                  // Remove interrupt pending
  EPIT1CR = (1u << 16);         // Do a soft-reset of the timer
  while (EPIT1CR & (1u << 16)); // Wait for the reset to be released

  EPIT1LR = OS_TIMER_RELOAD;
  EPIT1CR = (2u << 24)   // Use peripheral clock
          | (1u << 21)   // Run timer even in STOP mode
          | (0u <<  4)   // Prescaler is to 1
          | (1u <<  3)   // Counter reloads the value after reaching 0
          | (1u <<  2)   // Enable interrupt
          | (1u <<  1)   // Enable counter
          ;
  EPIT1CR |=  (1 <<  0);   // Enable EPIT
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler(INT_ID_EPIT1, _OS_ISR_Tick);
  OS_ARM_EnableISR(INT_ID_EPIT1);
  OS_ARM_ISRSetPrio(INT_ID_EPIT1, 1);
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  OS_U32          Vector;

  Vector = NIVECSR >> 16;
  pISR   = _apIsrHandler[Vector];  // Read interrupt vector to release NIRQ to CPU core
  OS_INT_Enter();                  // Inform OS that interrupt handler is running
  pISR();                          // Call interrupt service routine
  OS_INT_Disable();                // Disable interrupts and unlock
  OS_INT_Leave();                  // Leave interrupt, perform task switch if required
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pOldHandler             = _apIsrHandler[ISRIndex];
  _apIsrHandler[ISRIndex] = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  INTENNUM = ISRIndex;
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  INTDISNUM = ISRIndex;
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;
  int     Off;
  int     Shift;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  Off     = 7 - ((ISRIndex) >> 3);
  Shift   = ISRIndex & 0x07;
  pPrio   = (OS_U32*) (NIPRIORITY_BASE_ADDR + 4u * Off);
  OldPrio = pPrio[ISRIndex] & (7 << Shift);
  *pPrio |= ((OldPrio & ~INT_PRIORITY_MASK) << (4 * Shift)) | ((Prio & INT_PRIORITY_MASK) << (4 * Shift));
  OS_INT_EnableConditional();
  return OldPrio & INT_PRIORITY_MASK;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  //
  //  Setup waitstates for external BUS interface before switching to high speed
  //
  _InitFlash();
  //
  //  Initialize main clock oscillator and PLLs
  //
  _InitClocks();
  //
  //  Initialize SDRAM
  //
  _InitSDRAM();
  //
  //  Initialize static memory controller
  //
  _InitSMC();
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Memory mapping should be complete, meaning 4096 entries.
  // Code below fills in ascending VAddr order
  //
  //                                             Mode                    VAddr  PAddr  Size[MB]
#if (SDRAM_TARGET != 0)
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x000, 0x800, 0x001);    // SDRAM, the first MB remapped to 0 to map vectors to correct address
#else
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x000, 0x780, 0x001);    // Internal SRAM
#endif
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x001, 0x000, 0x001);    // ROM
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x004, 0x004, 0x001);    // ROM
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x005, 0x005, 0x0fc);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x100, 0x100, 0x100);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x200, 0x200, 0x100);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x300, 0x300, 0x100);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x400, 0x400, 0x03f);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x43f, 0x43f, 0x001);    // sfrs
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x440, 0x440, 0x0C0);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x500, 0x500, 0x040);    // sfrs
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x540, 0x540, 0x0C0);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x600, 0x600, 0x080);    // ARM926 platform ROMPATCH
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x680, 0x680, 0x080);    // ARM926 platform ASIC
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x700, 0x700, 0x080);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x780, 0x780, 0x080);    // Internal SRAM
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x800, 0x800, 0x020);    // SDRAM bank 0, 32 MByte cached, buffered
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x820, 0x800, 0x020);    // SDRAM bank 0, 32 MByte non-cahced, non-buffered
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x840, 0x840, 0x0C0);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x900, 0x900, 0x100);    // SDRAM bank 1
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0xA00, 0xA00, 0x080);    // WEIM CS0
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0xA80, 0xA80, 0x080);    // WEIM CS1
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0xB00, 0xB00, 0x020);    // WEIM CS2
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0xB20, 0xB20, 0x020);    // WEIM CS3
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0xB40, 0xB40, 0x020);    // WEIM CS4
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0xB60, 0xB60, 0x020);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0xB80, 0xB80, 0x001);    // sfrs
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0xB81, 0xB81, 0x030);    // reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0xBB0, 0xBB0, 0x001);    // NAND flash controller
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0xBB1, 0xBB0, 0x04F);    // Reserved
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0xC00, 0xC00, 0x400);    // Reserved
  OS_ARM_MMU_Enable      (&_TranslationTable[0]);
  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();
  //
  // Initialize interrupt controller
  //
  _InitASIC();
  return 1;  // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
