/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for NXP i.MX6S phyCORE-i.MX6
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define LED0 (4u)

#define IOMUXC_BASE_ADDR              (0x020E0000u)
#define IOMUXC_SW_MUX_CTL_PAD_GPIO04  (*(volatile unsigned int*)(IOMUXC_BASE_ADDR + 0x22Cu))

#define GPIO1_BASE_ADDR               (0x0209C000u)
#define GPIO1_DR                      (*(volatile unsigned int*)(GPIO1_BASE_ADDR + 0x0u))
#define GPIO1_DIR                     (*(volatile unsigned int*)(GPIO1_BASE_ADDR + 0x4u))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  IOMUXC_SW_MUX_CTL_PAD_GPIO04  =  (5u <<    0);  // ALT5: Select signal GPIO1_IO04
  GPIO1_DR                     &= ~(1u << LED0);  // Set GPIO pin to output
  GPIO1_DIR                    |=  (1u << LED0);  // Set GPIO pin to output
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    GPIO1_DR |= (1u << LED0);
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    GPIO1_DR &= ~(1u << LED0);
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    GPIO1_DR ^= (1u << LED0);
  }
}

/****** End Of File *************************************************/
