/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : SWI_Handler.c
Purpose : Implements an SWI handler for ARM cores which can be used
          to replace the dummy SWI handler from the startup code.
          This handler is useful when printf is used for debug outputs
          and semihosting is used.
          Using this SWI handler allows running a debug application
          with printf and semihosting standalone.
          This handler can NOT be used as SWI handler when the
          application wants to use SWI (SVC) calls.
          Usage of SVC calls requires an SWI dispatcher written in
          assembly language.
          About using SVC in an application, read the compiler
          reference.

--------  END-OF-HEADER  ---------------------------------------------
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

__swi __arm void SWI_Handler(void);

/*********************************************************************
*
*       SWI_Handler()
*
* Function description
*   This just returns. It can not be used as SWI handler when
*   SWI is needed by the application.
*   It is useful when IAR semihosting and printf is used
*   and the application shall run standalone without debugger.
*/
__swi __arm void SWI_Handler(void) {
  //
  // Do nothing here, just return...
  //
}

/*************************** End of file ****************************/

