/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Fiel    : BSP.c
Purpose : BSP for custom hardware
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define NUM_LEDS                     (4)  // All LEDs: BGPIO1B, LED1: RED D3, LED2: GREEN D3, LED3: RED D1, LED4: GREEN D1
#define LED_BASE_BIT                 (7)

//
// SYSCTRL
//
#define SYSCTRL_BASE_ADDR            (0x4000C000u)
#define PWRCTRL_PG1_1                (*(volatile unsigned int*)(SYSCTRL_BASE_ADDR + 0x040u))     // Power Management Control #1 for PG1
#define PWRSTAT_PG1_1                (*(volatile unsigned int*)(SYSCTRL_BASE_ADDR + 0x158u))     // Power Management Status for PG1

//
// MUX Level1
//
#define MUX_LEVEL1_BASE_ADDR         (0x40067000u)
#define RGPIOS_LEVEL1_CONFIGA_13     (*(volatile unsigned int*)(MUX_LEVEL1_BASE_ADDR + 0x034u))  // GPIO13 GMII Multiplexing Level1 Configuration Register
#define RGPIOS_LEVEL1_CONFIGA_14     (*(volatile unsigned int*)(MUX_LEVEL1_BASE_ADDR + 0x038u))  // GPIO14 GMII Multiplexing Level1 Configuration Register
#define RGPIOS_LEVEL1_CONFIGA_17     (*(volatile unsigned int*)(MUX_LEVEL1_BASE_ADDR + 0x044u))  // GPIO17 GMII Multiplexing Level1 Configuration Register
#define RGPIOS_LEVEL1_CONFIGA_18     (*(volatile unsigned int*)(MUX_LEVEL1_BASE_ADDR + 0x048u))  // GPIO18 GMII Multiplexing Level1 Configuration Register
#define RGPIOS_LEVEL1_STATUSPROTECT  (*(volatile unsigned int*)(MUX_LEVEL1_BASE_ADDR + 0x400u))  // GPIO Multiplexing Level 1 Status and Protect Register

//
// MUX Level2
//
#define MUX_LEVEL2_BASE_ADDR         (0x51000000u)
#define RGPIOS_LEVEL2_CONFIGA_13     (*(volatile unsigned int*)(MUX_LEVEL2_BASE_ADDR + 0x034u))  // GPIO13 GMII Multiplexing Level1 Configuration Register
#define RGPIOS_LEVEL2_CONFIGA_14     (*(volatile unsigned int*)(MUX_LEVEL2_BASE_ADDR + 0x038u))  // GPIO14 GMII Multiplexing Level1 Configuration Register
#define RGPIOS_LEVEL2_CONFIGA_17     (*(volatile unsigned int*)(MUX_LEVEL2_BASE_ADDR + 0x044u))  // GPIO17 GMII Multiplexing Level1 Configuration Register
#define RGPIOS_LEVEL2_CONFIGA_18     (*(volatile unsigned int*)(MUX_LEVEL2_BASE_ADDR + 0x048u))  // GPIO18 GMII Multiplexing Level1 Configuration Register
#define RGPIOS_LEVEL2_STATUSPROTECT  (*(volatile unsigned int*)(MUX_LEVEL2_BASE_ADDR + 0x400u))  // GPIO Multiplexing Level 1 Status and Protect Register

//
// BGPIO1
//
#define BGPIO1_BASE_ADDR             (0x5000B000u)
#define BGPIO1_SWPORTB_DR            (*(volatile unsigned int*)(BGPIO1_BASE_ADDR + 0x0Cu))       // Data output register
#define BGPIO1_SWPORTB_DDR           (*(volatile unsigned int*)(BGPIO1_BASE_ADDR + 0x10u))       // Data direction register

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  int i;

  //
  // Power control
  //
  PWRCTRL_PG1_1 |= (1u << 20);                         // Activate BGPIO1 clock
  while ((PWRSTAT_PG1_1 & (1u << 6)) == 0) {
    ;                                                  // Wait for BGPIO1 to be clocked
  }
  //
  // MUX Level1
  //
  RGPIOS_LEVEL1_STATUSPROTECT = 0x40067400u;           // Remove register protection
  RGPIOS_LEVEL1_CONFIGA_13   |= (15u << 0);            // IO Multiplex are controlled by Level2
  RGPIOS_LEVEL1_CONFIGA_14   |= (15u << 0);
  RGPIOS_LEVEL1_CONFIGA_17   |= (15u << 0);
  RGPIOS_LEVEL1_CONFIGA_18   |= (15u << 0);
  RGPIOS_LEVEL1_STATUSPROTECT = 0x40067401u;           // Set register protection
  //
  // MUX Level2
  //
  RGPIOS_LEVEL2_STATUSPROTECT = 0x51000400u;           // Remove register protection
  RGPIOS_LEVEL2_CONFIGA_13   |= (48u << 0);            // GGPIO
  RGPIOS_LEVEL2_CONFIGA_14   |= (48u << 0);
  RGPIOS_LEVEL2_CONFIGA_17   |= (48u << 0);
  RGPIOS_LEVEL2_CONFIGA_18   |= (48u << 0);
  RGPIOS_LEVEL2_STATUSPROTECT = 0x51000401u;           // Set register protection
  //
  // BGPIO1
  //
  for (i = 0; i < NUM_LEDS; i++) {
    BGPIO1_SWPORTB_DDR |= (1u << (i + LED_BASE_BIT));  // Set pin to output
  }
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  BGPIO1_SWPORTB_DR |= (1u << (Index + LED_BASE_BIT));
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  BGPIO1_SWPORTB_DR &= ~(1u << (Index + LED_BASE_BIT));
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  BGPIO1_SWPORTB_DR ^= (1u << (Index + LED_BASE_BIT));
}

/*************************** End of file ****************************/
