/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : UART implementation

Additional information:

  Device : STR710
  Board  : STR710 Eval

  Unit | UART   | Board connector
  ===============================
  0    | UART0  | RS-232-A
  1    | UART1  | RS-232-B, SW9: 1-2
  2    | UART2  | RS-232-B, SW9: 2-3
  3    | UART3  | -

*/

#include "BSP_UART.h"
#include "RTOS.h"      // For OS_ARM_* and OS_INT_* functions.

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/

#define SYSTEM_FREQ           (48000000L)
#define UART_FREQ             (SYSTEM_FREQ / 2uL)

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/

#define UART0_BASE_ADDR            (0xC0004000uL)
#define UART0_BR                   (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x00uL))
#define UART0_TxBUFR               (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x04uL))
#define UART0_RxBUFR               (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x08uL))
#define UART0_CR                   (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x0CuL))
#define UART0_IER                  (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x10uL))
#define UART0_SR                   (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x14uL))
#define UART0_GTR                  (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x18uL))
#define UART0_TOR                  (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x1CuL))
#define UART0_TxRSTR               (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x20uL))
#define UART0_RxRSTR               (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x24uL))

#define UART1_BASE_ADDR            (0xC0005000uL)
#define UART1_BR                   (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x00uL))
#define UART1_TxBUFR               (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x04uL))
#define UART1_RxBUFR               (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x08uL))
#define UART1_CR                   (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x0CuL))
#define UART1_IER                  (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x10uL))
#define UART1_SR                   (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x14uL))
#define UART1_GTR                  (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x18uL))
#define UART1_TOR                  (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x1CuL))
#define UART1_TxRSTR               (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x20uL))
#define UART1_RxRSTR               (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x24uL))

#define UART2_BASE_ADDR            (0xC0006000uL)
#define UART2_BR                   (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x00uL))
#define UART2_TxBUFR               (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x04uL))
#define UART2_RxBUFR               (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x08uL))
#define UART2_CR                   (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x0CuL))
#define UART2_IER                  (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x10uL))
#define UART2_SR                   (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x14uL))
#define UART2_GTR                  (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x18uL))
#define UART2_TOR                  (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x1CuL))
#define UART2_TxRSTR               (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x20uL))
#define UART2_RxRSTR               (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x24uL))

#define UART3_BASE_ADDR            (0xC0007000uL)
#define UART3_BR                   (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x00uL))
#define UART3_TxBUFR               (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x04uL))
#define UART3_RxBUFR               (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x08uL))
#define UART3_CR                   (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x0CuL))
#define UART3_IER                  (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x10uL))
#define UART3_SR                   (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x14uL))
#define UART3_GTR                  (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x18uL))
#define UART3_TOR                  (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x1CuL))
#define UART3_TxRSTR               (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x20uL))
#define UART3_RxRSTR               (*(volatile unsigned short*)(UART3_BASE_ADDR + 0x24uL))

#define UART0_RX_PIN_MASK          (1uL <<  8)
#define UART0_TX_PIN_MASK          (1uL <<  9)
#define UART1_RX_PIN_MASK          (1uL << 10)
#define UART1_TX_PIN_MASK          (1uL << 11)
#define UART2_RX_PIN_MASK          (1uL << 13)
#define UART2_TX_PIN_MASK          (1uL << 14)
#define UART3_RX_PIN_MASK          (1uL <<  1)
#define UART3_TX_PIN_MASK          (1uL <<  0)

#define UART0_INT_NUMBER           ( 9uL)
#define UART1_INT_NUMBER           (10uL)
#define UART2_INT_NUMBER           (11uL)
#define UART3_INT_NUMBER           (12uL)

#define UART0                      (0uL)
#define UART1                      (1uL)
#define UART2                      (2uL)
#define UART3                      (3uL)

#define UART_BAUDDIVIDE(Baudrate)  ((UART_FREQ + (Baudrate) * 8uL) / ((Baudrate) * 16uL))

#define UART_TX_EMPTY_INT_SHIFT    (2)
#define UART_RX_FULL_INT_SHIFT     (0)
#define UART_TXRDY_MASK            (1 << UART_TX_EMPTY_INT_SHIFT)
#define UART_RXRDY_MASK            (1 << UART_RX_FULL_INT_SHIFT)
#define UART_RX_ERROR_MASK         ((1uL << 3) | (1uL << 4) | (1uL << 5))  // Parity, framing, overrun error
#define UART_PRIO                  (1uL)                                   // lowest priority for OS UART interrupts

#define IOPORT0_BASE_ADDR          (0xE0003000uL)
#define GPIO0_PC0                  (*(volatile unsigned short*)(IOPORT0_BASE_ADDR + 0x00uL))
#define GPIO0_PC1                  (*(volatile unsigned short*)(IOPORT0_BASE_ADDR + 0x04uL))
#define GPIO0_PC2                  (*(volatile unsigned short*)(IOPORT0_BASE_ADDR + 0x08uL))
#define GPIO0_PD                   (*(volatile unsigned short*)(IOPORT0_BASE_ADDR + 0x0CuL))

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static BSP_UART_TX_CB* _pfWriteCB;
static BSP_UART_RX_CB* _pfReadCB;

/*********************************************************************
*
*       Prototypes
*
*  Declare ISR handler here to avoid "no prototype" warning.
*  They are not declared in any CMSIS header.
*
**********************************************************************
*/

#if defined(__cplusplus)
  extern "C" {                // Make sure we have C-declarations in C++ programs.
#endif

static void UART0_RX_TX_IRQHandler(void);
static void UART1_RX_TX_IRQHandler(void);
static void UART2_RX_TX_IRQHandler(void);
static void UART3_RX_TX_IRQHandler(void);

#if defined(__cplusplus)
}                             // Make sure we have C-declarations in C++ programs.
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _SetBaudrate()
*
*  Function description
*    Configures the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
static void _SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Baudrate);
}

/*********************************************************************
*
*       Global functions, IRQ handler
*
**********************************************************************
*/

/*********************************************************************
*
*       UART0_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void UART0_RX_TX_IRQHandler(void) {
  volatile unsigned int Data;
           unsigned int Status;
           unsigned int Unit;

  Unit   = UART0;
  Status = UART0_SR;                                     // Examine status register
  do {
    if (Status & UART_RXRDY_MASK) {                      // Data received?
      if (Status & UART_RX_ERROR_MASK) {                 // Any error ?
        Data = UART0_RxBUFR;                             // => Discard data
      } else {
        Data = UART0_RxBUFR;
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);                         // Forward the received byte to the callback.
        }
      }
    }
    if (Status & UART_TXRDY_MASK) {                      // Check Tx status => Send next character
      UART0_SR &= ~UART_TXRDY_MASK;                      // Clear Tx Int
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {
        UART0_IER &= ~(1uL << UART_TX_EMPTY_INT_SHIFT);  // Disable further tx interrupts
      }
    }
    Status = UART0_SR;                                   // Examine status register
  } while (Status & UART0_IER & (UART_TXRDY_MASK | UART_RXRDY_MASK));
}

/*********************************************************************
*
*       UART1_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void UART1_RX_TX_IRQHandler(void) {
  volatile unsigned int Data;
           unsigned int Status;
           unsigned int Unit;

  Unit   = UART1;
  Status = UART1_SR;                                     // Examine status register
  do {
    if (Status & UART_RXRDY_MASK) {                      // Data received?
      if (Status & UART_RX_ERROR_MASK) {                 // Any error ?
        Data = UART1_RxBUFR;                             // => Discard data
      } else {
        Data = UART1_RxBUFR;
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);                         // Forward the received byte to the callback.
        }
      }
    }
    if (Status & UART_TXRDY_MASK) {                      // Check Tx status => Send next character
      UART1_SR &= ~UART_TXRDY_MASK;                      // Clear Tx Int
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {
        UART1_IER &= ~(1uL << UART_TX_EMPTY_INT_SHIFT);  // Disable further tx interrupts
      }
    }
    Status = UART1_SR;                                   // Examine status register
  } while (Status & UART1_IER & (UART_TXRDY_MASK | UART_RXRDY_MASK));
}

/*********************************************************************
*
*       UART2_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void UART2_RX_TX_IRQHandler(void) {
  volatile unsigned int Data;
           unsigned int Status;
           unsigned int Unit;

  Unit   = UART2;
  Status = UART2_SR;                                     // Examine status register
  do {
    if (Status & UART_RXRDY_MASK) {                      // Data received?
      if (Status & UART_RX_ERROR_MASK) {                 // Any error ?
        Data = UART2_RxBUFR;                             // => Discard data
      } else {
        Data = UART2_RxBUFR;
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);                         // Forward the received byte to the callback.
        }
      }
    }
    if (Status & UART_TXRDY_MASK) {                      // Check Tx status => Send next character
      UART2_SR &= ~UART_TXRDY_MASK;                      // Clear Tx Int
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {
        UART2_IER &= ~(1uL << UART_TX_EMPTY_INT_SHIFT);  // Disable further tx interrupts
      }
    }
    Status = UART2_SR;                                   // Examine status register
  } while (Status & UART2_IER & (UART_TXRDY_MASK | UART_RXRDY_MASK));
}

/*********************************************************************
*
*       UART3_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void UART3_RX_TX_IRQHandler(void) {
  volatile unsigned int Data;
           unsigned int Status;
           unsigned int Unit;

  Unit   = UART3;
  Status = UART3_SR;                                     // Examine status register
  do {
    if (Status & UART_RXRDY_MASK) {                      // Data received?
      if (Status & UART_RX_ERROR_MASK) {                 // Any error ?
        Data = UART3_RxBUFR;                             // => Discard data
      } else {
        Data = UART3_RxBUFR;
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);                         // Forward the received byte to the callback.
        }
      }
    }
    if (Status & UART_TXRDY_MASK) {                      // Check Tx status => Send next character
      UART3_SR &= ~UART_TXRDY_MASK;                      // Clear Tx Int
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {
        UART3_IER &= ~(1uL << UART_TX_EMPTY_INT_SHIFT);  // Disable further tx interrupts
      }
    }
    Status = UART3_SR;                                   // Examine status register
  } while (Status & UART3_IER & (UART_TXRDY_MASK | UART_RXRDY_MASK));
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_UART_Init()
*
*  Function description
*    Initializes the UART peripheral.
*
*  Parameters
*    Unit       : Unit number (typically zero-based).
*    Baudrate   : Baud rate to configure [Hz].
*    NumDataBits: Number of data bits to use.
*    Parity     : One of the following values:
*                   * BSP_UART_PARITY_NONE
*                   * BSP_UART_PARITY_ODD
*                   * BSP_UART_PARITY_EVEN
*    NumStopBits: Number of stop bits to use.
*/
void BSP_UART_Init(unsigned int Unit, unsigned long Baudrate, unsigned char NumDataBits, unsigned char Parity, unsigned char NumStopBits) {
  BSP_UART_USE_PARA(NumDataBits);
  BSP_UART_USE_PARA(Parity);
  BSP_UART_USE_PARA(NumStopBits);
  OS_INT_IncDI();
  switch (Unit) {
  case UART0:
    //
    // Setup Port-Mode to alternate function, TX: push-pull, RX: open drain, CMOS input
    //
    GPIO0_PC0   &= ~(UART0_RX_PIN_MASK);       // Rx set to open drain
    GPIO0_PC0   |=   UART0_TX_PIN_MASK;        // Tx set to push pull
    GPIO0_PC1   |=  (UART0_RX_PIN_MASK
                |    UART0_TX_PIN_MASK);       // Select alternate function
    GPIO0_PC2   &= ~(UART0_RX_PIN_MASK);       // Select CMOS input
    GPIO0_PC2   |=   UART0_TX_PIN_MASK;        // Select output
    UART0_BR     = UART_BAUDDIVIDE(Baudrate);  // Setup baudrate generator
    UART0_CR     = (1uL <<  0)                 // Setup UART control register, 8 data bits
                 | (1uL <<  3)                 // 1 Stop bit (bit 4:3 = 01)
                 | (0uL <<  5)                 // Parity, don't care
                 | (0uL <<  6)                 // Loopback disabled
                 | (1uL <<  7)                 // Baudrate generator on
                 | (1uL <<  8)                 // Rx Enable
                 | (0uL <<  9)                 // Smartcard mode disabled
                 | (0uL << 10);                // FIFO disabled
    UART0_TxRSTR = 0;                          // Clear Rx FIFO
    UART0_RxRSTR = 0;                          // Clear Tx FIFO
    //
    //  Install OS UART interrupt handler
    //
    OS_ARM_InstallISRHandler(UART0_INT_NUMBER, UART0_RX_TX_IRQHandler);  // UART interrupt vector.
    OS_ARM_ISRSetPrio       (UART0_INT_NUMBER, UART_PRIO);               // UART interrupt level.
    OS_ARM_EnableISR        (UART0_INT_NUMBER);                          // Enable OS UART interrupt
    UART0_IER |= (1uL << UART_RX_FULL_INT_SHIFT);                        // Enable Rx interrupts
    break;
  case UART1:
    //
    // Setup Port-Mode to alternate function, TX: push-pull, RX: open drain, CMOS input
    //
    GPIO0_PC0   &= ~(UART1_RX_PIN_MASK);       // Rx set to open drain
    GPIO0_PC0   |=   UART1_TX_PIN_MASK;        // Tx set to push pull
    GPIO0_PC1   |=  (UART1_RX_PIN_MASK
                |    UART1_TX_PIN_MASK);       // Select alternate function
    GPIO0_PC2   &= ~(UART1_RX_PIN_MASK);       // Select CMOS input
    GPIO0_PC2   |=   UART1_TX_PIN_MASK;        // Select output
    UART1_BR     = UART_BAUDDIVIDE(Baudrate);  // Setup baudrate generator
    UART1_CR     = (1uL <<  0)                 // Setup UART control register, 8 data bits
                 | (1uL <<  3)                 // 1 Stop bit (bit 4:3 = 01)
                 | (0uL <<  5)                 // Parity, don't care
                 | (0uL <<  6)                 // Loopback disabled
                 | (1uL <<  7)                 // Baudrate generator on
                 | (1uL <<  8)                 // Rx Enable
                 | (0uL <<  9)                 // Smartcard mode disabled
                 | (0uL << 10);                // FIFO disabled
    UART1_TxRSTR = 0;                          // Clear Rx FIFO
    UART1_RxRSTR = 0;                          // Clear Tx FIFO
    //
    //  Install OS UART interrupt handler
    //
    OS_ARM_InstallISRHandler(UART1_INT_NUMBER, UART1_RX_TX_IRQHandler);  // UART interrupt vector.
    OS_ARM_ISRSetPrio       (UART1_INT_NUMBER, UART_PRIO);               // UART interrupt level.
    OS_ARM_EnableISR        (UART1_INT_NUMBER);                          // Enable OS UART interrupt
    UART1_IER |= (1uL << UART_RX_FULL_INT_SHIFT);                        // Enable Rx interrupts
    break;
  case UART2:
    //
    // Setup Port-Mode to alternate function, TX: push-pull, RX: open drain, CMOS input
    //
    GPIO0_PC0   &= ~(UART2_RX_PIN_MASK);       // Rx set to open drain
    GPIO0_PC0   |=   UART2_TX_PIN_MASK;        // Tx set to push pull
    GPIO0_PC1   |=  (UART2_RX_PIN_MASK
                |    UART2_TX_PIN_MASK);       // Select alternate function
    GPIO0_PC2   &= ~(UART2_RX_PIN_MASK);       // Select CMOS input
    GPIO0_PC2   |=   UART2_TX_PIN_MASK;        // Select output
    UART2_BR     = UART_BAUDDIVIDE(Baudrate);  // Setup baudrate generator
    UART2_CR     = (1uL <<  0)                 // Setup UART control register, 8 data bits
                 | (1uL <<  3)                 // 1 Stop bit (bit 4:3 = 01)
                 | (0uL <<  5)                 // Parity, don't care
                 | (0uL <<  6)                 // Loopback disabled
                 | (1uL <<  7)                 // Baudrate generator on
                 | (1uL <<  8)                 // Rx Enable
                 | (0uL <<  9)                 // Smartcard mode disabled
                 | (0uL << 10);                // FIFO disabled
    UART2_TxRSTR = 0;                          // Clear Rx FIFO
    UART2_RxRSTR = 0;                          // Clear Tx FIFO
    //
    //  Install OS UART interrupt handler
    //
    OS_ARM_InstallISRHandler(UART2_INT_NUMBER, UART2_RX_TX_IRQHandler);  // UART interrupt vector.
    OS_ARM_ISRSetPrio       (UART2_INT_NUMBER, UART_PRIO);               // UART interrupt level.
    OS_ARM_EnableISR        (UART2_INT_NUMBER);                          // Enable OS UART interrupt
    UART2_IER |= (1uL << UART_RX_FULL_INT_SHIFT);                        // Enable Rx interrupts
    break;
  case UART3:
    //
    // Setup Port-Mode to alternate function, TX: push-pull, RX: open drain, CMOS input
    //
    GPIO0_PC0   &= ~(UART3_RX_PIN_MASK);       // Rx set to open drain
    GPIO0_PC0   |=   UART3_TX_PIN_MASK;        // Tx set to push pull
    GPIO0_PC1   |=  (UART3_RX_PIN_MASK
                |    UART3_TX_PIN_MASK);       // Select alternate function
    GPIO0_PC2   &= ~(UART3_RX_PIN_MASK);       // Select CMOS input
    GPIO0_PC2   |=   UART3_TX_PIN_MASK;        // Select output
    UART3_BR     = UART_BAUDDIVIDE(Baudrate);  // Setup baudrate generator
    UART3_CR     = (1uL <<  0)                 // Setup UART control register, 8 data bits
                 | (1uL <<  3)                 // 1 Stop bit (bit 4:3 = 01)
                 | (0uL <<  5)                 // Parity, don't care
                 | (0uL <<  6)                 // Loopback disabled
                 | (1uL <<  7)                 // Baudrate generator on
                 | (1uL <<  8)                 // Rx Enable
                 | (0uL <<  9)                 // Smartcard mode disabled
                 | (0uL << 10);                // FIFO disabled
    UART3_TxRSTR = 0;                          // Clear Rx FIFO
    UART3_RxRSTR = 0;                          // Clear Tx FIFO
    //
    //  Install OS UART interrupt handler
    //
    OS_ARM_InstallISRHandler(UART3_INT_NUMBER, UART3_RX_TX_IRQHandler);  // UART interrupt vector.
    OS_ARM_ISRSetPrio       (UART3_INT_NUMBER, UART_PRIO);               // UART interrupt level.
    OS_ARM_EnableISR        (UART3_INT_NUMBER);                          // Enable OS UART interrupt
    UART3_IER |= (1uL << UART_RX_FULL_INT_SHIFT);                        // Enable Rx interrupts
    break;
  }
  OS_INT_DecRI();
}

/*********************************************************************
*
*       BSP_UART_DeInit()
*
*  Function description
*    De-initializes the UART peripheral.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*/
void BSP_UART_DeInit(unsigned int Unit) {
  BSP_UART_USE_PARA(Unit);
}

/*********************************************************************
*
*       BSP_UART_SetBaudrate()
*
*  Function description
*    Configures/changes the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
void BSP_UART_SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  _SetBaudrate(Unit, Baudrate);
}

/*********************************************************************
*
*       BSP_UART_SetReadCallback()
*
*  Function description
*    Sets the callback to execute upon an Rx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetReadCallback(unsigned Unit, BSP_UART_RX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfReadCB = pf;
}

/*********************************************************************
*
*       BSP_UART_SetWriteCallback()
*
*  Function description
*    Sets the callback to execute upon a Tx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetWriteCallback(unsigned int Unit, BSP_UART_TX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfWriteCB = pf;
}

/*********************************************************************
*
*       BSP_UART_Write1()
*
*  Function description
*    Sends one byte via UART.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    Data: (First) data byte to send.
*
*  Additional information
*    The first byte of a transfer is typically sent from application
*    context. Further bytes of the transfer are then sent from the
*    Tx interrupt handler by also calling this function from interrupt
*    context.
*/
void BSP_UART_Write1(unsigned int Unit, unsigned char Data) {
  switch (Unit) {
  case UART0:
    UART0_TxBUFR = Data;                              // Send character
    UART0_IER   |= (1uL << UART_TX_EMPTY_INT_SHIFT);  // enable Tx Empty interrupt
    break;
  case UART1:
    UART1_TxBUFR = Data;                              // Send character
    UART1_IER   |= (1uL << UART_TX_EMPTY_INT_SHIFT);  // enable Tx Empty interrupt
    break;
  case UART2:
    UART2_TxBUFR = Data;                              // Send character
    UART2_IER   |= (1uL << UART_TX_EMPTY_INT_SHIFT);  // enable Tx Empty interrupt
    break;
  case UART3:
    UART3_TxBUFR = Data;                              // Send character
    UART3_IER   |= (1uL << UART_TX_EMPTY_INT_SHIFT);  // enable Tx Empty interrupt
    break;
  }
}

/*************************** End of file ****************************/
