/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   OS_INIT_PLL
  #define OS_INIT_PLL  (1)
#endif

#ifndef   OS_IGNORE_UNDEFINED_INTERRUPT
  #define OS_IGNORE_UNDEFINED_INTERRUPT  (0)
#endif

/*********************************************************************
*
*       Clock settings
*/
#define OS_SYSTEM_FREQ   (48000000u)

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ    (OS_SYSTEM_FREQ / 2)
#define OS_TICK_FREQ     (1000u)
#define OS_INT_FREQ      (OS_TICK_FREQ)
#define OS_TIMER_RELOAD  ((OS_TIMER_FREQ / OS_INT_FREQ) - 1)
#define OS_TIMER_ID      (19u)    // Assign to TC1 global interrupt
#define OS_TIMER_PRIO    (0x01u)  // lowest priority

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
//
// GPIO register
//
#define IOPORT0_BASE_ADDR   (0xE0003000u)
#define GPIO0_PC0           (*(volatile OS_U16*)(IOPORT0_BASE_ADDR + 0x00u))
#define GPIO0_PC1           (*(volatile OS_U16*)(IOPORT0_BASE_ADDR + 0x04u))
#define GPIO0_PC2           (*(volatile OS_U16*)(IOPORT0_BASE_ADDR + 0x08u))
#define GPIO0_PD            (*(volatile OS_U16*)(IOPORT0_BASE_ADDR + 0x0Cu))
//
// Power, reset clock control unit register
//
#define PRCCU_CFR           (*(volatile OS_U32*)0xA0000008u)
#define PRCCU_PLL1CR        (*(volatile OS_U32*)0xA0000018u)
#define PRCCU_PDIVR         (*(volatile OS_U32*)0xA0000044u)
//
// Timer sfr definition
//
#define TIM0_BASE_ADDR      (0xE0009000u)
#define TIM1_BASE_ADDR      (0xE000A000u)
#define TIM2_BASE_ADDR      (0xE000B000u)
#define TIM3_BASE_ADDR      (0xE000C000u)

#define TIM_ICAR_OFFS       (0x00)
#define TIM_ICBR_OFFS       (0x04)
#define TIM_OCAR_OFFS       (0x08)
#define TIM_OCBR_OFFS       (0x0C)
#define TIM_CNTR_OFFS       (0x10)
#define TIM_CR1_OFFS        (0x14)
#define TIM_CR2_OFFS        (0x18)
#define TIM_SR_OFFS         (0x1C)
//
// Assign timer sfrs used for OS timer
// initially, we use timer 1
//
#define OS_TIM_BASE_ADDR    TIM1_BASE_ADDR
#define OS_TIM_OCR          (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_OCAR_OFFS))
#define OS_TIM_CR1          (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_CR1_OFFS))
#define OS_TIM_CR2          (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_CR2_OFFS))
#define OS_TIM_CNTR         (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_CNTR_OFFS))
#define OS_TIM_SR           (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_SR_OFFS))

#define TIMER_INT_BIT_NO    (14u)
//
// Enhanced interrupt controller (EIC)
//
#define EIC_ICR             (*(volatile OS_U32*)0xfffff800u)
#define EIC_CICR            (*(volatile OS_U32*)0xfffff804u)
#define EIC_CPIR            (*(volatile OS_U32*)0xfffff808u)
#define EIC_IVR             (*(volatile OS_U32*)0xfffff818u)
#define EIC_FIR             (*(volatile OS_U32*)0xfffff81Cu)
#define EIC_IER             (*(volatile OS_U32*)0xfffff820u)
#define EIC_IPR             (*(volatile OS_U32*)0xfffff840u)
#define EIC_SIR0            (*(volatile OS_U32*)0xfffff860u)
//
// Enhanced interrupt controller interrupt sources
//
#define T0_EFTI_VECT_ID     ( 0u)  // IRQ0  T0.EFTI Timer 0 global interrupt 5
#define FLASH_VECT_ID       ( 1u)  // IRQ1  FLASH FLASH global interrupt
#define PRCCU_VECT_ID       ( 2u)  // IRQ2  PRCCU PRCCU global interrupt
#define RTC_VECT_ID         ( 3u)  // IRQ3  RTC Real Time Clock global interrupt 2
#define WDG_VECT_ID         ( 4u)  // IRQ4  WDG.IRQ Watchdog timer interrupt 1
#define XTI_VECT_ID         ( 5u)  // IRQ5  XTI.IRQ XTI external interrupt 16
#define USB_VECT_ID         ( 6u)  // IRQ6  USB.HPIRQ USB high priority event interrupt 0-7
#define I2C0_ERR_VECT_ID    ( 7u)  // IRQ7  I2C0.ITERR I2C 0 error interrupt
#define I2C1_ERR_VECT_ID    ( 8u)  // IRQ8  I2C1.ITERR I2C 1 error interrupt
#define UART0_VECT_ID       ( 9u)  // IRQ9  UART0.IRQ UART 0 global interrupt 9
#define UART1_VECT_ID       (10u)  // IRQ10 UART1.IRQ UART 1 global interrupt 9
#define UART2_VECT_ID       (11u)  // IRQ11 UART2.IRQ UART 2 global interrupt 9
#define UART3_VECT_ID       (12u)  // IRQ12 UART3.IRQ UART 3 global interrupt 9
#define SPI0_VECT_ID        (13u)  // IRQ13 SPI0.IRQ BSPI 0 global interrupt 5
#define SPI1_VECT_ID        (14u)  // IRQ14 SPI1.IRQ BSPI 1 global interrupt 5
#define I2C0_VECT_ID        (15u)  // IRQ15 I2C0.Iobal interrupt 32
#define ADC_VECT_ID         (18u)  // IRQ18 ADC.IRQ ADC sample ready interrupt 1
#define T1_GI_VECT_ID       (19u)  // IRQ19 T1.GI Timer 1 global interrupt 5
#define T2_GI_VECT_ID       (20u)  // IRQ20 T2.GI Timer 2 global interrupt 5
#define T3_GI_VECT_ID       (21u)  // IRQ21 T3.GI Timer 3 global interrupt 5
#define Reserved1_VECT_ID   (22u)  // IRQ22 Reserved
#define Reserved2_VECT_ID   (23u)  // IRQ23 Reserved
#define Reserved3_VECT_ID   (24u)  // IRQ24 Reserved
#define HDLC_IR_VECT_ID     (25u)  // IRQ25 HDLC.IRQ HDLC global interrupt
#define USB_LPI_VECT_ID     (26u)  // IRQ26 USB.LPIRQ USB low priority event interrupt 7-15
#define Reserved4_VECT_ID   (27u)  // IRQ27 Reserved
#define Reserved5_VECT_ID   (28u)  // IRQ28 Reserved
#define T0_TOI_VECT_ID      (29u)  // IRQ29 T0.TOI Timer 0 Overflow interrupt 1
#define T0_OC1_VECT_ID      (30u)  // IRQ30 T0.OC1 Timer 0 Output Compare 1 interrupt 1
#define T0_OC2_VECT_ID      (31u)  // IRQ31 T0.OC2 Timer 0 Output Compare 2 interrupt 1

#define NUM_INT_SOURCES     (32u)
#define INT_CHANNEL_MASK    (0x1Fu)
#define INT_PRIORITY_MASK   (0x0Fu)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Interrupt vector table
*
*  Notes
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=4
  static __no_init OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(4) static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES] __attribute__ ((aligned (4), section (".no_init")));
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _InitPLL()
*
*  Function description
*    Initialize PLL to speed up system
*    This function is called from __low_level_init() before
*    any variables are initialized.
*    Therefore, it can not rely on any variable content
*    and can not set any variables (except no-init variables).
*/
static void _InitPLL(void) {
  #if (OS_INIT_PLL == 1)
    PRCCU_PDIVR  = (1u <<  8) | (1u << 0);  // Divide peripheral clocks by two (to ensure max. clock limit)
    PRCCU_PLL1CR = (PRCCU_PLL1CR & 0xFF00u) | 0x0051u;
    PRCCU_CFR   |= (1u << 15) | (1u << 3) | (1u << 0);
  #endif
}

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when an not installed interrupt was detected.
*    As interrupt pending condition of peripherals has to be reset,
*    program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(int Index) {
#if (OS_IGNORE_UNDEFINED_INTERRUPT == 0)
  volatile int Dummy;

  Dummy = Index;
  while ((unsigned int)Dummy < (INT_CHANNEL_MASK + 1)) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
#endif
  OS_USEPARA(Index);
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  OS_I16 Cnt;

  Cnt = OS_TIM_CNTR;
  Cnt = OS_TIM_OCR - Cnt;
  if (Cnt < 0) {
    Cnt = 0 - Cnt;
  }
  return Cnt;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  OS_U16 Cnt;

  Cnt = OS_TIM_CNTR;
  return (OS_TIM_OCR - Cnt) <= 0;
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  OS_TIM_SR  &= ~(1 << TIMER_INT_BIT_NO);  // Clear embOS timer interrupt flag
  OS_TIM_OCR += OS_TIMER_RELOAD;           // Advance to next count;
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize Peripheral clock divider
  //
  PRCCU_PDIVR = (1u << 8) | (1u << 0);  // Divide peripheral clocks by two (to ensure max. clock limit)
  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  OS_TIM_CR1 = 0;                                        // Stop timer, reset mode
  OS_TIM_CR2 = 0;                                        // Disable all interrupts, clear prescaler
  OS_TIM_OCR = OS_TIMER_RELOAD;                          // Setup compare register, initially use 1000 ticks per second
  OS_TIM_CR1 = (1u << 15);                               // Start timer
  OS_TIM_CR2 = (1u << 14);                               // Enable output capture interrupt A
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler(OS_TIMER_ID, &_OS_ISR_Tick);  // Timer/counter interrupt vector.
  OS_ARM_ISRSetPrio(OS_TIMER_ID, OS_TIMER_PRIO);         // Timer/counter interrupt priority.
  OS_ARM_EnableISR(OS_TIMER_ID);                         // Enable timer/counter 0 interrupt.
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  EIC_ICR |= 0x01u; // Enable interrupt controller
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  int             ISRIndex;

  ISRIndex = EIC_IVR;                      // Perform a dummy vector read to update CICR
  ISRIndex = EIC_CICR & INT_CHANNEL_MASK;  // Examine current interrupt source (channel number)
  pISR = _apIsrHandler[ISRIndex];          // Read interrupt vector
  OS_INT_EnterNestable();
  if (pISR != NULL) {
    pISR();                                // Call installed interrupt service routine
  } else {
    _OS_ISR_Undefined(ISRIndex);
  }
  EIC_IPR |= (1u << ISRIndex);             // Clear current interrupt pending bit, reset EIC
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pOldHandler             = _apIsrHandler[ISRIndex];
  _apIsrHandler[ISRIndex] = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  EIC_IER |= (1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  EIC_IER &= ~(1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pPrio           = (OS_U32*)&EIC_SIR0;
  OldPrio         = pPrio[ISRIndex];
  pPrio[ISRIndex] = (OldPrio & ~INT_PRIORITY_MASK) | (Prio & INT_PRIORITY_MASK);
  OS_INT_EnableConditional();
  return OldPrio & INT_PRIORITY_MASK;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  _InitPLL();
  return 1;  // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
