/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   OS_INIT_PLL
  #define OS_INIT_PLL  (1)
#endif

#ifndef   OS_IGNORE_UNDEFINED_INTERRUPT
  #define OS_IGNORE_UNDEFINED_INTERRUPT  (0)
#endif

/*********************************************************************
*
*       Clock settings
*/
#define OS_XTAL_FREQUENCY  (4000000uL)
#define OS_SYSTEM_FREQ     (60000000uL)
#define PLL_MUL            MRCC_PLL_MUL_15


/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_PRESCALE  (1u)
#define OS_TIMER_FREQ      (OS_SYSTEM_FREQ / OS_TIMER_PRESCALE / 2)
#define OS_TICK_FREQ       (1000u)
#define OS_INT_FREQ        (OS_TICK_FREQ)
#define OS_TIMER_RELOAD    (OS_TIMER_FREQ / OS_INT_FREQ)
#define OS_TIMER_PRIO      (0x01)  // lowest priority

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
//
// GPIO register
//

#define GPIO0_BASE_ADDR             (0xFFFFE400uL)
#define GPIO1_BASE_ADDR             (0xFFFFE440uL)
#define GPIO2_BASE_ADDR             (0xFFFFE480uL)

#define GPIO0_PC0                   (*(volatile OS_U32*)(GPIO0_BASE_ADDR + 0x00uL))
#define GPIO0_PC1                   (*(volatile OS_U32*)(GPIO0_BASE_ADDR + 0x04uL))
#define GPIO0_PC2                   (*(volatile OS_U32*)(GPIO0_BASE_ADDR + 0x08uL))
#define GPIO0_PD                    (*(volatile OS_U32*)(GPIO0_BASE_ADDR + 0x0CuL))
//
// Power, reset, config, clock control unit register
//
#define CFG_GLCONF                  (*(volatile OS_U32*)(0x60000010uL))

#define MRCC_BASE_ADDR              (0x60000020uL)
#define MRCC_CLKCTL                 (*(volatile OS_U32*)(MRCC_BASE_ADDR + 0x00uL))
#define MRCC_RFSR                   (*(volatile OS_U32*)(MRCC_BASE_ADDR + 0x04uL))
#define MRCC_PWRCTRL                (*(volatile OS_U32*)(MRCC_BASE_ADDR + 0x08uL))
#define MRCC_PCLKEN                 (*(volatile OS_U32*)(MRCC_BASE_ADDR + 0x10uL))
#define MRCC_PSWRES                 (*(volatile OS_U32*)(MRCC_BASE_ADDR + 0x14uL))

#define MRCC_GPIO_EN_BIT            (24)
#define MRCC_UART0_EN_BIT           (20)
#define MRCC_UART1_EN_BIT           (21)
#define MRCC_UART2_EN_BIT           (22)
#define MRCC_NCKDF_BIT              (18)
#define MRCC_CKOSCSEL_BIT           (21)
#define MRCC_CKSEL_BIT              (23)
#define MRCC_PLLEN_BIT              (24)
#define MRCC_BCOUNT_BIT             (24)
#define MRCC_OSC4MOFF_BIT           (17)
#define MRCC_XTDIV2_BIT             (15)
#define MRCC_LOCK_BIT               (31)
#define MRCC_EN33V_BIT              (16)

#define MRCC_HPRESC_RESET_MASK      (0xFFFFFFE7uL)
#define MRCC_PPRESC_RESET_MASK      (0xFFFFFFFCuL)
#define MRCC_PPRESC2_RESET_MASK     (0xFFFFFFFBuL)
#define MRCC_MX_MASK                (0xE7FFFFFFuL)

#define MRCC_PLL_MUL_16             (0uL << 27)
#define MRCC_PLL_MUL_15             (1uL << 27)
#define MRCC_PLL_MUL_14             (2uL << 27)
#define MRCC_PLL_MUL_12             (3uL << 27)

#define OSC4M_STARTUP_TIMEOUT       (254uL)
//
// Timer sfr definition
//
#define TB_BASE_ADDR                (0xFFFF8800uL)
#define TIM0_BASE_ADDR              (0xFFFF8C00uL)
#define TIM1_BASE_ADDR              (0xFFFF9000uL)
#define TIM2_BASE_ADDR              (0xFFFF9400uL)

#define TIM_CR_OFFS                 (0x00uL)
#define TIM_SCR_OFFS                (0x04uL)
#define TIM_IMCR_OFFS               (0x08uL)
#define TIM_RSR_OFFS                (0x18uL)
#define TIM_RER_OFFS                (0x1CuL)
#define TIM_ISR_OFFS                (0x20uL)
#define TIM_CNT_OFFS                (0x24uL)
#define TIM_PSC_OFFS                (0x28uL)
#define TIM_ARR_OFFS                (0x30uL)
#define TIM_ICR1_OFFS               (0x4CuL)
//
// Assign timer sfrs used for OS timer
// Per default, we use the time base timer
//
#define OS_TIM_BASE_ADDR            TB_BASE_ADDR
#define OS_TIM_CR                   (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_CR_OFFS))
#define OS_TIM_SCR                  (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_SCR_OFFS))
#define OS_TIM_IMCR                 (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_IMCR_OFFS))
#define OS_TIM_RSR                  (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_RSR_OFFS))
#define OS_TIM_RER                  (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_RER_OFFS))
#define OS_TIM_ISR                  (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_ISR_OFFS))
#define OS_TIM_CNT                  (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_CNT_OFFS))
#define OS_TIM_PSC                  (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_PSC_OFFS))
#define OS_TIM_ARR                  (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_ARR_OFFS))
#define OS_TIM_ICR1                 (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_ICR1_OFFS))

#if   OS_TIM_BASE_ADDR == TB_BASE_ADDR
  #define MRCC_TIM_BIT    (1)
  #define OS_TIMER_ID     (31)
#elif OS_TIM_BASE_ADDR == TIM0_BASE_ADDR
  #define MRCC_TIM_BIT    (2)
  #define OS_TIMER_ID     (12)
#elif OS_TIM_BASE_ADDR == TIM1_BASE_ADDR
  #define MRCC_TIM_BIT    (3)
  #define OS_TIMER_ID     (8)
#elif OS_TIM_BASE_ADDR == TIM2_BASE_ADDR
  #define MRCC_TIM_BIT    (4)
  #define OS_TIMER_ID     (4)
#endif
//
// Enhanced interrupt controller (EIC)
//
#define EIC_ICR                     (*(volatile OS_U32*)0xfffff800uL)
#define EIC_CICR                    (*(volatile OS_U32*)0xfffff804uL)
#define EIC_CPIR                    (*(volatile OS_U32*)0xfffff808uL)
#define EIC_IVR                     (*(volatile OS_U32*)0xfffff818uL)
#define EIC_FIR                     (*(volatile OS_U32*)0xfffff81CuL)
#define EIC_IER                     (*(volatile OS_U32*)0xfffff820uL)
#define EIC_IPR                     (*(volatile OS_U32*)0xfffff840uL)
#define EIC_SIR0                    (*(volatile OS_U32*)0xfffff860uL)
//
// Enhanced interrupt controller interrupt sources
//
#define NUM_INT_SOURCES             (32uL)
#define INT_CHANNEL_MASK            (0x1FuL)
#define INT_PRIORITY_MASK           (0x0FuL)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Interrupt vector table
*
*  Notes
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=4
  static __no_init OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(4) static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES] __attribute__ ((aligned (4), section (".no_init")));
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

#if (OS_INIT_PLL != 0)
/*********************************************************************
*
*       WriteCKOSCSELBit()
*
*  Function description
*    Executes the CKOSCSEL bit writing sequence
*/
static void _WriteCKOSCSELBit(void) {
  volatile OS_U32 Tmp;
  OS_U32          Tmp1;
  OS_U32          Tmp2;

  Tmp         = MRCC_CLKCTL;
  Tmp1        = Tmp |  (1uL << MRCC_CKOSCSEL_BIT);
  Tmp2        = Tmp & ~(1uL << MRCC_CKOSCSEL_BIT);
  MRCC_CLKCTL = Tmp1;  // Set CKOSCSEL bit
  MRCC_CLKCTL = Tmp1;  // Set CKOSCSEL bit
  MRCC_CLKCTL = Tmp2;  // Reset CKOSCSEL bit
  MRCC_CLKCTL = Tmp1;  // Set CKOSCSEL bit
  Tmp = MRCC_CLKCTL;   // Read CKOSCSEL bit
}
#endif

/*********************************************************************
*
*       _InitPLL()
*
*  Function description
*    Initialize PLL to speed up system
*    This function is called from __low_level_init() before
*    any variables are initialized.
*    Therefore, it can not rely on any variable content
*    and can not set any variables (except no-init variables).
*/
static void _InitPLL(void) {
#if (OS_INIT_PLL == 1)
  OS_U32 Cnt;
  OS_U32 Temp;

  //
  // Set Flash Burst mode
  //
  CFG_GLCONF |= (1uL << 8);
  //
  // Reset MRCC system (for debug purpose)
  //
  MRCC_CLKCTL &= ~(1uL << MRCC_NCKDF_BIT);                  // Try to clear NCKDF bit
  if ((MRCC_CLKCTL & (1uL << MRCC_NCKDF_BIT)) != 0) {       // No clock detected on OSC4M
    //
    // Reset LOCKIE, LOCKIF, CKUSBSEL, NCKDIE, OSC4MOFF, OSC4MBYP, MCOS[1:0],
    // MCOP, HPRESC[1:0], PPRES[2:0] bits
    //
    MRCC_CLKCTL &= 0x9FB40000uL;
    if ((MRCC_CLKCTL & (1u << MRCC_CKOSCSEL_BIT)) != 0)  {  // Clear CKOSCSEL bit
      _WriteCKOSCSELBit();                                  // Execute CKOSCSEL bit writing sequence
    }
  } else {                                                  // Clock is present on OSC4M
    if ((MRCC_CLKCTL & (1uL << MRCC_CKOSCSEL_BIT)) != 0)  {
      MRCC_CLKCTL &= ~(1uL << MRCC_CKSEL_BIT);              // Reset CKSEL bit
      _WriteCKOSCSELBit();                                  // Clear CKOSCSEL bit by special sequence */
    }
    if ((MRCC_CLKCTL & (1uL << MRCC_CKSEL_BIT)) == 0) {
      MRCC_CLKCTL |= (1uL << MRCC_CKSEL_BIT);               // Set CKSEL bit to select Oscillator as clock source
    }
    MRCC_CLKCTL &= ~(1uL << MRCC_PLLEN_BIT);                // Disable PLL
    MRCC_CLKCTL &= 0x87B70000uL;                            // Reset LOCKIE, LOCKIF, MX[1:0], CKUSBSEL, NCKDIE, MCOS[1:0], MCOP, HPRESC[1:0], PPRES[2:0]
    MRCC_CLKCTL &= ~(1u << MRCC_CKSEL_BIT);                 // Reset CKSEL bit
    MRCC_CLKCTL &= 0xFFFCFFFFuL;                            // Reset OSC4MOFF and OSC4MBYP bits
  }
  //
  // Reset RTCM, EN33V, LP_PARAM[15:13], WFI_FLASH_EN, LPMC_DBG and LPMC[1:0] bits
  //
  MRCC_PWRCTRL &= 0xFBFE1FE1uL;
  MRCC_PCLKEN   = 0x00uL;                                   // Reset PCLKEN, => disable all peripheral clocks
  MRCC_PSWRES   = 0x00uL;                                   // Reset PSWRES register bits
  MRCC_CLKCTL  &= ~(1uL << MRCC_NCKDF_BIT);                 // Clear NCKDF bit

  //
  // Wait for OSC4M start-up
  //
  Cnt = 0;
  do {   // Clear No Clock Detected flag
    if(MRCC_CLKCTL & (1uL << MRCC_NCKDF_BIT) != 0) {
      MRCC_CLKCTL &= ~(1uL << MRCC_NCKDF_BIT);
    }
    Cnt++;
  } while ((MRCC_RFSR & (1uL << MRCC_BCOUNT_BIT) == 0) && (Cnt <= OSC4M_STARTUP_TIMEOUT));
  //
  // Set HCLK prescaler
  //
  Temp         = MRCC_CLKCTL & MRCC_HPRESC_RESET_MASK;
  Temp        |= (0uL << 3);    // Set HCLK divider to 1 (b4..3 = 0)
  MRCC_CLKCTL  = Temp;
  //
  // Set CKTIM prescaler
  //
  Temp         = MRCC_CLKCTL & MRCC_PPRESC_RESET_MASK;
  Temp        |= (1uL << 0);    // Set CK_TIM divider to 2 (b1..0 = 1)
  MRCC_CLKCTL  = Temp;
  //
  // Set PCLK divider
  //
  Temp         = MRCC_CLKCTL & MRCC_PPRESC2_RESET_MASK;
  Temp        |= (0uL << 2);    // Set PCLK divider to 1   (b2 = 0)
  MRCC_CLKCTL  = Temp;
  //
  // Set CKSYS and enable PLL
  //
  if ((MRCC_CLKCTL & ((1uL << MRCC_NCKDF_BIT) | (1uL << MRCC_OSC4MOFF_BIT))) == 0)  {  // Check whether Main osc clock is present
    if (((MRCC_CLKCTL & (1uL << MRCC_PLLEN_BIT)) != 0) && ((MRCC_CLKCTL & ~MRCC_MX_MASK) == PLL_MUL)) {
      //
      // If PLL is already setup, select CK_PLL1 as CK_SYS clock source
      //
      MRCC_CLKCTL &= ~(1uL << MRCC_CKSEL_BIT);
    } else {
      //
      // If CK_RTC is selected as CK_OSC clock source, switch to Oscillator
      //
      if ((MRCC_CLKCTL & (1uL << MRCC_CKOSCSEL_BIT)) != 0) {
        if ((MRCC_CLKCTL & (1uL << MRCC_CKSEL_BIT)) != 0)  {
          MRCC_CLKCTL &= ~(1uL << MRCC_CKSEL_BIT);
        }
        _WriteCKOSCSELBit();                                   // Clear CKOSCSEL bit by special CKOSCSEL bit writing sequence
        if ((MRCC_CLKCTL & (1uL << MRCC_CKOSCSEL_BIT)) != 0) {  // Check if CKOSCSEL is set to 0
          goto ExitPLLInit;                                    // Error, do not initialize PLL
        }
      }
      MRCC_CLKCTL |=  (1uL << MRCC_CKSEL_BIT);         // Select CK_OSC as CK_SYS clock source
      MRCC_CLKCTL &= ~(1uL << MRCC_PLLEN_BIT);         // Disable PLL
      MRCC_CLKCTL &= MRCC_MX_MASK;                    // Configure PLL multiplier, clear MX-bits
      if (PLL_MUL != MRCC_PLL_MUL_16) {               // Set MX[1:0] bits according to PLL_Mul value
        MRCC_CLKCTL |= PLL_MUL;
      }
      if(OS_XTAL_FREQUENCY == 4000000uL) {             // 4 MHz external Quartz oscillator used as main oscillator
        MRCC_CLKCTL &= ~(1uL << MRCC_XTDIV2_BIT);      // Disable Oscillator Divider by 2
      } else if (OS_XTAL_FREQUENCY == 8000000uL) {     // 8 MHz external Quartz oscillator used as main oscillator
        MRCC_CLKCTL |=  (1uL << MRCC_XTDIV2_BIT);      // Enable Oscillator Divider by 2
      }
      //
      // Now Enable PLL and wait until PLL is locked
      //
      MRCC_CLKCTL |= (1uL << MRCC_PLLEN_BIT);
      while ((MRCC_CLKCTL & (1uL << MRCC_LOCK_BIT)) == 0) {
        //
        // If OSC4M clock disapear or the PLL is disabled, exit
        //
        if (((MRCC_CLKCTL & (1uL << MRCC_NCKDF_BIT)) != 0) ||
            ((MRCC_CLKCTL & (1uL << MRCC_PLLEN_BIT)) == 0)) {
          goto ExitPLLInit;
        }
      }
      MRCC_CLKCTL &= ~(1uL << MRCC_CKSEL_BIT);  // Finally Select CK_PLL1 as CK_SYS clock source
    }
  }
  ExitPLLInit:
  //
  // GPIO pins configuration, optimized for 3V3 operation
  //
  MRCC_PWRCTRL |= (1uL << MRCC_EN33V_BIT);
#endif
}

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when an not installed interrupt was detected.
*    As interrupt pending condition of peripherals has to be reset,
*    program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(int Index) {
#if (OS_IGNORE_UNDEFINED_INTERRUPT == 0)
  volatile unsigned int Dummy;

  Dummy = Index;
  while (Dummy < (INT_CHANNEL_MASK + 1)) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
#endif
  OS_USEPARA(Index);
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return OS_TIM_CNT;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return OS_TIM_ISR & (1uL << 0);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  OS_TIM_ISR &= ~(1uL << 0);  // Clear OS timer interrupt flag
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize timer for OS
  //
  OS_TIM_CR    = 0;                                      // Disable timer
  MRCC_PCLKEN |=  (1uL << MRCC_TIM_BIT);                 // Enable timer clock
  MRCC_PSWRES &= ~(1uL << MRCC_TIM_BIT);                 // Clear software reset
  OS_TIM_PSC   = (OS_TIMER_PRESCALE - 1);                // Init prescaler for timer
  OS_TIM_SCR   = 0                                       // b15..10, b7..b5, b1..b0 always 0
               | (0uL << 2)                              // 0: Slave mode disabled
               | (0uL << 3)                              // b4..3: Slave mod selection, don't care
               | (0uL << 5)                              // b7..b5: Always 0
               | (0uL << 8)                              // b9..8: Trigger selection, 0 => internal
               ;
  OS_TIM_CR    = 0;                                      // Disable timer
  OS_TIM_CR    = 0                                       // Setup counting mode
               | (1uL << 4)                              // Select Up/Down counting mode. 1 => Down count
               ;
  OS_TIM_ARR   = OS_TIMER_RELOAD;
  OS_TIM_ISR   = 0;                                      // Clear all timer interrupt flags
  OS_TIM_CR   |= (1uL << 0);                             // Enable timer update interrupt
  OS_TIM_RSR   = (1uL << 0);                             // Request update interrupt only
  OS_TIM_RER   = (1uL << 0);                             // Enable update interrupt request
  OS_TIM_CR   |= (1uL << 2);                             // Enable and start timer
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler(OS_TIMER_ID, &_OS_ISR_Tick);  // Timer/counter interrupt vector.
  OS_ARM_ISRSetPrio(OS_TIMER_ID, OS_TIMER_PRIO);         // Timer/counter interrupt priority.
  OS_ARM_EnableISR(OS_TIMER_ID);                         // Enable timer/counter 0 interrupt.
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  EIC_ICR |= 0x01uL;  // Enable interrupt controller
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  int             ISRIndex;

  ISRIndex = EIC_IVR;                       // Perform a dummy vector read to update CICR
  ISRIndex = EIC_CICR & INT_CHANNEL_MASK;  // Examine current interrupt source (channel number)
  pISR     = _apIsrHandler[ISRIndex];     // Read interrupt vector
  OS_INT_EnterNestable();
  if (pISR != NULL) {
    pISR();                                  // Call installed interrupt service routine
  } else {
    _OS_ISR_Undefined(ISRIndex);
  }
  EIC_IPR |= (1u << ISRIndex);             // Clear current interrupt pending bit, reset EIC
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER* pOldHandler;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pOldHandler             = _apIsrHandler[ISRIndex];
  _apIsrHandler[ISRIndex] = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  EIC_IER |= (1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  EIC_IER &= ~(1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pPrio           = (OS_U32*)&EIC_SIR0;
  OldPrio         = pPrio[ISRIndex];
  pPrio[ISRIndex] = (OldPrio & ~INT_PRIORITY_MASK) | (Prio & INT_PRIORITY_MASK);
  OS_INT_EnableConditional();
  return OldPrio & INT_PRIORITY_MASK;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  _InitPLL();
  return 1;  // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
