/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : UART implementation

Additional information:

  Device : STR912
  Board  : STR912 SK

  Unit | UART   | Board connector
  ===============================
  0    | UART0  | UART0
  1    | UART1  | -
  2    | UART2  | -

*/

#include "BSP_UART.h"
#include "RTOS.h"      // For OS_ARM_* and OS_INT_* functions.

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/

#define SYSTEM_FREQ                (96000000uL)
#define UART_FREQ                  (SYSTEM_FREQ / 2uL)

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/

#define UART0                      (0)
#define UART1                      (1)
#define UART2                      (2)

#define UART0_INT_ID               (16)  // IRQ21 UART0.IRQ UART 0 global interrupt
#define UART1_INT_ID               (17)  // IRQ20 UART1.IRQ UART 1 global interrupt
#define UART2_INT_ID               (18)  // IRQ19 UART2.IRQ UART 2 global interrupt

#define UART_VECT_INDEX            (31)  // lowest priority, allowed range 16..31, UART ints are connected to VIC1

#define UART0_BASE_ADDR            (0x5C004000uL)
#define UART0_DR                   (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x00uL))
#define UART0_RSR                  (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x04uL))
#define UART0_FR                   (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x18uL))
#define UART0_ILPR                 (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x20uL))
#define UART0_IBRD                 (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x24uL))
#define UART0_FBRD                 (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x28uL))
#define UART0_LCR                  (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x2CuL))
#define UART0_CR                   (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x30uL))
#define UART0_IFLS                 (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x34uL))
#define UART0_IMSC                 (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x38uL))
#define UART0_RIS                  (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x3CuL))
#define UART0_MIS                  (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x40uL))
#define UART0_ICR                  (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x44uL))
#define UART0_DMACR                (*(volatile unsigned short*)(UART0_BASE_ADDR + 0x48uL))

#define UART1_BASE_ADDR            (0x5C005000uL)
#define UART1_DR                   (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x00uL))
#define UART1_RSR                  (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x04uL))
#define UART1_FR                   (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x18uL))
#define UART1_ILPR                 (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x20uL))
#define UART1_IBRD                 (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x24uL))
#define UART1_FBRD                 (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x28uL))
#define UART1_LCR                  (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x2CuL))
#define UART1_CR                   (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x30uL))
#define UART1_IFLS                 (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x34uL))
#define UART1_IMSC                 (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x38uL))
#define UART1_RIS                  (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x3CuL))
#define UART1_MIS                  (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x40uL))
#define UART1_ICR                  (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x44uL))
#define UART1_DMACR                (*(volatile unsigned short*)(UART1_BASE_ADDR + 0x48uL))

#define UART2_BASE_ADDR            (0x5C006000uL)
#define UART2_DR                   (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x00uL))
#define UART2_RSR                  (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x04uL))
#define UART2_FR                   (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x18uL))
#define UART2_ILPR                 (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x20uL))
#define UART2_IBRD                 (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x24uL))
#define UART2_FBRD                 (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x28uL))
#define UART2_LCR                  (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x2CuL))
#define UART2_CR                   (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x30uL))
#define UART2_IFLS                 (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x34uL))
#define UART2_IMSC                 (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x38uL))
#define UART2_RIS                  (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x3CuL))
#define UART2_MIS                  (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x40uL))
#define UART2_ICR                  (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x44uL))
#define UART2_DMACR                (*(volatile unsigned short*)(UART2_BASE_ADDR + 0x48uL))

//#define UART0_CLK_EN_SHIFT         (20)
//#define UART1_CLK_EN_SHIFT         (21)
//#define UART2_CLK_EN_SHIFT         (22)

#define UART0_RX_PIN_SHIFT         (1)
#define UART0_TX_PIN_SHIFT         (4)
#define UART1_RX_PIN_SHIFT         (2)
#define UART1_TX_PIN_SHIFT         (3)
#define UART2_RX_PIN_SHIFT         (1)
#define UART2_TX_PIN_SHIFT         (5)

#define UART_TX_EMPTY_INT_SHIFT    (5)
#define UART_RX_FULL_INT_SHIFT     (4)
#define UART_TXRDY_MASK            (1uL << UART_TX_EMPTY_INT_SHIFT)
#define UART_RXRDY_MASK            (1uL << UART_RX_FULL_INT_SHIFT)
#define UART_PRIO                  (1)  // Lowest prio

#define UART_BAUDDIVIDE(Baudrate)  ((UART_FREQ + (Baudrate) * 8uL) / ((Baudrate) * 16uL))

#define GPIO3_BASE_ADDR            (0x58009000uL)
#define GPIO3_DIR                  (*(volatile unsigned long*)(GPIO3_BASE_ADDR + 0x400uL))
#define GPIO3_SEL                  (*(volatile unsigned long*)(GPIO3_BASE_ADDR + 0x41CuL))

#define SCU_BASE_ADDR              (0x5C002000uL)
#define SCU_PCGR1                  (*(volatile unsigned long*)(SCU_BASE_ADDR + 0x18uL))
#define SCU_PRR1                   (*(volatile unsigned long*)(SCU_BASE_ADDR + 0x20uL))
#define SCU_GPIOIN3                (*(volatile unsigned long*)(SCU_BASE_ADDR + 0x50uL))
#define SCU_GPIOIN5                (*(volatile unsigned long*)(SCU_BASE_ADDR + 0x58uL))
#define SCU_GPIOOUT3               (*(volatile unsigned long*)(SCU_BASE_ADDR + 0x70uL))

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static BSP_UART_TX_CB* _pfWriteCB;
static BSP_UART_RX_CB* _pfReadCB;

/*********************************************************************
*
*       Prototypes
*
*  Declare ISR handler here to avoid "no prototype" warning.
*  They are not declared in any CMSIS header.
*
**********************************************************************
*/

#if defined(__cplusplus)
  extern "C" {                // Make sure we have C-declarations in C++ programs.
#endif

static void UART0_RX_TX_IRQHandler(void);
static void UART1_RX_TX_IRQHandler(void);
static void UART2_RX_TX_IRQHandler(void);

#if defined(__cplusplus)
}                             // Make sure we have C-declarations in C++ programs.
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _SetBaudrate()
*
*  Function description
*    Configures the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
static void _SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Baudrate);
}

/*********************************************************************
*
*       Global functions, IRQ handler
*
**********************************************************************
*/

/*********************************************************************
*
*       UART0_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void UART0_RX_TX_IRQHandler(void) {
  volatile unsigned short Data;
           unsigned int   Status;
           unsigned int   Unit;

  Unit   = UART0;
  Status = UART0_MIS;                                      // Examine status register
  do {
    if (Status & UART_RXRDY_MASK) {                        // Data received?
      UART0_ICR &= ~UART_RXRDY_MASK;                       // Clear Interrupt pending condition
      Data = UART0_DR;                                     // Get current Data
      if ((Data & 0xFF00u) == 0) {                         // check error flags
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);
        }                                                  // Process actual byte
      } else {
        UART0_RSR = 0;                                     // Reset all errors
      }
    }
    if (Status & UART_TXRDY_MASK) {                        // Check Tx status => Send next character
      UART0_ICR = UART_TXRDY_MASK;                         // Clear Tx Int
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {  // No more characters to send ?
        UART0_IMSC &= ~(1uL << UART_TX_EMPTY_INT_SHIFT);   // Disable further tx interrupts
      }
    }
    Status = UART0_MIS;                                    // Examine current interrupt status
  } while (Status & UART0_MIS & (UART_TXRDY_MASK | UART_RXRDY_MASK));
}

/*********************************************************************
*
*       UART1_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void UART1_RX_TX_IRQHandler(void) {
  volatile unsigned short Data;
           unsigned int   Status;
           unsigned int   Unit;

  Unit   = UART1;
  Status = UART1_MIS;                                      // Examine status register
  do {
    if (Status & UART_RXRDY_MASK) {                        // Data received?
      UART1_ICR &= ~UART_RXRDY_MASK;                       // Clear Interrupt pending condition
      Data = UART1_DR;                                     // Get current Data
      if ((Data & 0xFF00u) == 0) {                         // check error flags
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);
        }                                                  // Process actual byte
      } else {
        UART1_RSR = 0;                                     // Reset all errors
      }
    }
    if (Status & UART_TXRDY_MASK) {                        // Check Tx status => Send next character
      UART1_ICR = UART_TXRDY_MASK;                         // Clear Tx Int
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {  // No more characters to send ?
        UART1_IMSC &= ~(1uL << UART_TX_EMPTY_INT_SHIFT);   // Disable further tx interrupts
      }
    }
    Status = UART1_MIS;                                    // Examine current interrupt status
  } while (Status & UART1_MIS & (UART_TXRDY_MASK | UART_RXRDY_MASK));
}

/*********************************************************************
*
*       UART2_RX_TX_IRQHandler()
*
*  Function description
*    UART Rx & Tx interrupt handler.
*/
static void UART2_RX_TX_IRQHandler(void) {
  volatile unsigned short Data;
           unsigned int   Status;
           unsigned int   Unit;

  Unit   = UART2;
  Status = UART2_MIS;                                      // Examine status register
  do {
    if (Status & UART_RXRDY_MASK) {                        // Data received?
      UART2_ICR &= ~UART_RXRDY_MASK;                       // Clear Interrupt pending condition
      Data = UART2_DR;                                     // Get current Data
      if ((Data & 0xFF00u) == 0) {                         // check error flags
        if (_pfReadCB != 0) {
          _pfReadCB(Unit, Data);
        }                                                  // Process actual byte
      } else {
        UART2_RSR = 0;                                     // Reset all errors
      }
    }
    if (Status & UART_TXRDY_MASK) {                        // Check Tx status => Send next character
      UART2_ICR = UART_TXRDY_MASK;                         // Clear Tx Int
      if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {  // No more characters to send ?
        UART2_IMSC &= ~(1uL << UART_TX_EMPTY_INT_SHIFT);   // Disable further tx interrupts
      }
    }
    Status = UART2_MIS;                                    // Examine current interrupt status
  } while (Status & UART2_MIS & (UART_TXRDY_MASK | UART_RXRDY_MASK));
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_UART_Init()
*
*  Function description
*    Initializes the UART peripheral.
*
*  Parameters
*    Unit       : Unit number (typically zero-based).
*    Baudrate   : Baud rate to configure [Hz].
*    NumDataBits: Number of data bits to use.
*    Parity     : One of the following values:
*                   * BSP_UART_PARITY_NONE
*                   * BSP_UART_PARITY_ODD
*                   * BSP_UART_PARITY_EVEN
*    NumStopBits: Number of stop bits to use.
*/
void BSP_UART_Init(unsigned int Unit, unsigned long Baudrate, unsigned char NumDataBits, unsigned char Parity, unsigned char NumStopBits) {
  volatile unsigned long Dummy;

  BSP_UART_USE_PARA(NumDataBits);
  BSP_UART_USE_PARA(Parity);
  BSP_UART_USE_PARA(NumStopBits);
  OS_INT_IncDI();
  switch (Unit) {
  case UART0:
    SCU_GPIOIN5  |= (1uL << UART0_RX_PIN_SHIFT);  // Setup GPIOIN to alternate function for Rx
    SCU_GPIOOUT3 |= (1uL << UART0_TX_PIN_SHIFT);  // Setup GPIOOUT to alternate function for Tx
    GPIO3_DIR    |= (1uL << UART0_TX_PIN_SHIFT);  // Setup GPIO port for Tx to output mode
    //
    // Enable peripheral clock and clear reset condition for selected UART
    //
    SCU_PCGR1 |= (1uL << 3);
    SCU_PRR1  |= (1uL << 3);
    //
    // Setup UART mode
    //
    UART0_CR  = (0u << 0);      // Initially disable the UART
    UART0_LCR = (0u << 0)       // Reserved, always 0
              | (0u << 1)       // No parity
              | (0u << 2)       // ODD Parity select (don't care)
              | (0u << 3)       // One stop bit
              | (1u << 4)       // 0: FIFO disabled, initially enable FIFO (!)
              | (3u << 5)       // 8 data bits (0=>5; 1=>5; 2=>7; 3=>8)
              | (0u << 7)       // Stick parity (MARK/SPACE), don't care
              | (0u << 8)       // LIN master mode disabled
              | (0u << 9)       // b12..9: LIN break len, don't care
              ;
    UART0_IBRD = UART_BAUDDIVIDE(Baudrate);  // Setup baudrate generator, integer part
    //
    // fractions should not be necessary on low baudrates
    //
    UART0_FBRD  = 0;            // Setup baudrate generator, fractional part
    UART0_CR    = UART0_CR
                | (1uL << 8)    // Enable Tx
                | (1uL << 9)    // Enable Rx
                ;
    UART0_IMSC  = 0;            // Disable all UART interrupts
    UART0_ICR   = 0xFFFFu;      // Clear all UART interrupts
    UART0_IFLS &= ~(0x3u);      // Clear TX FIFO level
    //
    //  Install embOS UART interrupt handler
    //
    OS_ARM_InstallISRHandler(UART_VECT_INDEX, UART0_RX_TX_IRQHandler);  // Install UART interrupt vector.
    OS_ARM_AssignISRSource  (UART_VECT_INDEX, UART0_INT_ID);
    OS_ARM_EnableISR        (UART_VECT_INDEX);                          // Enable UART interrupt vector.
    OS_ARM_EnableISRSource  (UART0_INT_ID);                             // Enable UART interrupt.
    Dummy     = UART0_DR;         // Clear Rx data register
    UART0_RSR = 0;                // Reset all errors
    //
    // Enable UART interrupt
    //
    UART0_IMSC = (1uL << UART_RX_FULL_INT_SHIFT);  // Enable UART Rx interrupt
    //
    // Flush the FIFO to enable Tx
    //
    UART0_LCR &= ~(1uL << 4);  // Disable FIFO
    UART0_CR  |=  (1uL << 0);  // Enable UART
    break;
  case UART1:
    SCU_GPIOIN3  |= (1uL << UART1_RX_PIN_SHIFT);  // Setup GPIOIN to alternate function for Rx
    SCU_GPIOOUT3 |= (1uL << UART1_TX_PIN_SHIFT);  // Setup GPIOOUT to alternate function for Tx
    GPIO3_DIR    |= (1uL << UART1_TX_PIN_SHIFT);  // Setup GPIO port for Tx to output mode
    //
    // Enable peripheral clock and clear reset condition for selected UART
    //
    SCU_PCGR1 |= (1uL << 4);
    SCU_PRR1  |= (1uL << 4);
    //
    // Setup UART mode
    //
    UART1_CR  = (0u << 0);      // Initially disable the UART
    UART1_LCR = (0u << 0)       // Reserved, always 0
              | (0u << 1)       // No parity
              | (0u << 2)       // ODD Parity select (don't care)
              | (0u << 3)       // One stop bit
              | (1u << 4)       // 0: FIFO disabled, initially enable FIFO (!)
              | (3u << 5)       // 8 data bits (0=>5; 1=>5; 2=>7; 3=>8)
              | (0u << 7)       // Stick parity (MARK/SPACE), don't care
              | (0u << 8)       // LIN master mode disabled
              | (0u << 9)       // b12..9: LIN break len, don't care
              ;
    UART1_IBRD = UART_BAUDDIVIDE(Baudrate);  // Setup baudrate generator, integer part
    //
    // fractions should not be necessary on low baudrates
    //
    UART1_FBRD  = 0;            // Setup baudrate generator, fractional part
    UART1_CR    = UART1_CR
                | (1uL << 8)    // Enable Tx
                | (1uL << 9)    // Enable Rx
                ;
    UART1_IMSC  = 0;            // Disable all UART interrupts
    UART1_ICR   = 0xFFFFu;      // Clear all UART interrupts
    UART1_IFLS &= ~(0x3u);      // Clear TX FIFO level
    //
    //  Install embOS UART interrupt handler
    //
    OS_ARM_InstallISRHandler(UART_VECT_INDEX, UART1_RX_TX_IRQHandler);  // Install UART interrupt vector.
    OS_ARM_AssignISRSource  (UART_VECT_INDEX, UART1_INT_ID);
    OS_ARM_EnableISR        (UART_VECT_INDEX);                          // Enable UART interrupt vector.
    OS_ARM_EnableISRSource  (UART1_INT_ID);                             // Enable UART interrupt.
    Dummy     = UART1_DR;         // Clear Rx data register
    UART1_RSR = 0;                // Reset all errors
    //
    // Enable UART interrupt
    //
    UART1_IMSC = (1uL << UART_RX_FULL_INT_SHIFT);  // Enable UART Rx interrupt
    //
    // Flush the FIFO to enable Tx
    //
    UART1_LCR &= ~(1uL << 4);  // Disable FIFO
    UART1_CR  |=  (1uL << 0);  // Enable UART
    break;
  case UART2:
    SCU_GPIOIN3  |= (1uL << UART2_RX_PIN_SHIFT);  // Setup GPIOIN to alternate function for Rx
    SCU_GPIOOUT3 |= (1uL << UART2_TX_PIN_SHIFT);  // Setup GPIOOUT to alternate function for Tx
    GPIO3_DIR    |= (1uL << UART2_TX_PIN_SHIFT);  // Setup GPIO port for Tx to output mode
    //
    // Enable peripheral clock and clear reset condition for selected UART
    //
    SCU_PCGR1 |= (1uL << 5);
    SCU_PRR1  |= (1uL << 5);
    //
    // Setup UART mode
    //
    UART2_CR  = (0u << 0);      // Initially disable the UART
    UART2_LCR = (0u << 0)       // Reserved, always 0
              | (0u << 1)       // No parity
              | (0u << 2)       // ODD Parity select (don't care)
              | (0u << 3)       // One stop bit
              | (1u << 4)       // 0: FIFO disabled, initially enable FIFO (!)
              | (3u << 5)       // 8 data bits (0=>5; 1=>5; 2=>7; 3=>8)
              | (0u << 7)       // Stick parity (MARK/SPACE), don't care
              | (0u << 8)       // LIN master mode disabled
              | (0u << 9)       // b12..9: LIN break len, don't care
              ;
    UART2_IBRD = UART_BAUDDIVIDE(Baudrate);  // Setup baudrate generator, integer part
    //
    // fractions should not be necessary on low baudrates
    //
    UART2_FBRD  = 0;            // Setup baudrate generator, fractional part
    UART2_CR    = UART2_CR
                | (1uL << 8)    // Enable Tx
                | (1uL << 9)    // Enable Rx
                ;
    UART2_IMSC  = 0;            // Disable all UART interrupts
    UART2_ICR   = 0xFFFFu;      // Clear all UART interrupts
    UART2_IFLS &= ~(0x3u);      // Clear TX FIFO level
    //
    //  Install embOS UART interrupt handler
    //
    OS_ARM_InstallISRHandler(UART_VECT_INDEX, UART2_RX_TX_IRQHandler);  // Install UART interrupt vector.
    OS_ARM_AssignISRSource  (UART_VECT_INDEX, UART2_INT_ID);
    OS_ARM_EnableISR        (UART_VECT_INDEX);                          // Enable UART interrupt vector.
    OS_ARM_EnableISRSource  (UART2_INT_ID);                             // Enable UART interrupt.
    Dummy     = UART2_DR;         // Clear Rx data register
    UART2_RSR = 0;                // Reset all errors
    //
    // Enable UART interrupt
    //
    UART2_IMSC = (1uL << UART_RX_FULL_INT_SHIFT);  // Enable UART Rx interrupt
    //
    // Flush the FIFO to enable Tx
    //
    UART2_LCR &= ~(1uL << 4);  // Disable FIFO
    UART2_CR  |=  (1uL << 0);  // Enable UART
    break;
  }
  OS_INT_DecRI();
}

/*********************************************************************
*
*       BSP_UART_DeInit()
*
*  Function description
*    De-initializes the UART peripheral.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*/
void BSP_UART_DeInit(unsigned int Unit) {
  BSP_UART_USE_PARA(Unit);
}

/*********************************************************************
*
*       BSP_UART_SetBaudrate()
*
*  Function description
*    Configures/changes the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
void BSP_UART_SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  _SetBaudrate(Unit, Baudrate);
}

/*********************************************************************
*
*       BSP_UART_SetReadCallback()
*
*  Function description
*    Sets the callback to execute upon an Rx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetReadCallback(unsigned Unit, BSP_UART_RX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfReadCB = pf;
}

/*********************************************************************
*
*       BSP_UART_SetWriteCallback()
*
*  Function description
*    Sets the callback to execute upon a Tx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetWriteCallback(unsigned int Unit, BSP_UART_TX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfWriteCB = pf;
}

/*********************************************************************
*
*       BSP_UART_Write1()
*
*  Function description
*    Sends one byte via UART.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    Data: (First) data byte to send.
*
*  Additional information
*    The first byte of a transfer is typically sent from application
*    context. Further bytes of the transfer are then sent from the
*    Tx interrupt handler by also calling this function from interrupt
*    context.
*/
void BSP_UART_Write1(unsigned int Unit, unsigned char Data) {
  switch (Unit) {
  case UART0:
    UART0_DR    = Data;                              // Send character
    UART0_IMSC |= (1uL << UART_TX_EMPTY_INT_SHIFT);  // enable Tx Empty interrupt
    break;
  case UART1:
    UART1_DR    = Data;                              // Send character
    UART1_IMSC |= (1uL << UART_TX_EMPTY_INT_SHIFT);  // enable Tx Empty interrupt
    break;
  case UART2:
    UART2_DR    = Data;                              // Send character
    UART2_IMSC |= (1uL << UART_TX_EMPTY_INT_SHIFT);  // enable Tx Empty interrupt
    break;
  }
}

/*************************** End of file ****************************/
