/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   OS_INIT_PLL
  #define OS_INIT_PLL  (1)
#endif

#ifndef   OS_IGNORE_UNDEFINED_INTERRUPT
  #define OS_IGNORE_UNDEFINED_INTERRUPT  (0)
#endif

/*********************************************************************
*
*       Clock settings
*/
#define OS_SYSTEM_FREQ     (96000000u)
#define OS_EXTERNAL_FREQ   (25000000u)

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_PRESCALE  (8)
#define OS_TIMER_FREQ      (OS_SYSTEM_FREQ / OS_TIMER_PRESCALE / 2)
#define OS_TICK_FREQ       (1000u)
#define OS_INT_FREQ        (OS_TICK_FREQ)
#define OS_TIMER_RELOAD    (OS_TIMER_FREQ / OS_INT_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
//
// FMI registers
//
#define FMI_BASE_ADDR      (0x54000000u)
#define FMI_BBSR           (*(volatile OS_U32*)(FMI_BASE_ADDR + 0x00u))
#define FMI_NBBSR          (*(volatile OS_U32*)(FMI_BASE_ADDR + 0x04u))
#define FMI_BBADR          (*(volatile OS_U32*)(FMI_BASE_ADDR + 0x0Cu))
#define FMI_NBBADR         (*(volatile OS_U32*)(FMI_BASE_ADDR + 0x10u))
#define FMI_CR             (*(volatile OS_U32*)(FMI_BASE_ADDR + 0x18u))

#define FMI_READ_WAIT_STATE_1     (0x0000u)        // One read wait state
#define FMI_READ_WAIT_STATE_2     (0x2000u)        // Two read wait states
#define FMI_READ_WAIT_STATE_3     (0x4000u)        // Three read wait states
#define FMI_WRITE_WAIT_STATE_0    (0xFFFFFEFFu)    // Zero wait state
#define FMI_WRITE_WAIT_STATE_1    (0x00000100u)    // One wait state
#define FMI_PWD_ENABLE            (0x1000u)        // FMI Power Down Enable
#define FMI_PWD_DISABLE           (0x0000u)        // FMI Power Down Disable
#define FMI_LVD_ENABLE            (0x0000u)        // FMI Low Voltage Detector Enable
#define FMI_LVD_DISABLE           (0x0800u)        // FMI Low Voltage Detector Disable
#define FMI_FREQ_LOW              (0x0000u)        // FMI Low bus working frequency
#define FMI_FREQ_HIGH             (0x0040u)        // FMI High bus working gfrequency

#define BANK1_STARTADDR          (0x80000u)
//
// Peripheral Bus base addresses
//
#define APB0_BASE_ADDR     (0x58000000u)
#define APB1_BASE_ADDR     (0x5C000000u)
//
// GPIO register
//
#define GPIO_BASE_ADDR     (APB0_BASE_ADDR + 0x6000u)
#define GPIO0_BASE_ADDR    (APB0_BASE_ADDR + 0x6000u)
#define GPIO1_BASE_ADDR    (APB0_BASE_ADDR + 0x7000u)
#define GPIO2_BASE_ADDR    (APB0_BASE_ADDR + 0x8000u)
#define GPIO3_BASE_ADDR    (APB0_BASE_ADDR + 0x9000u)
#define GPIO4_BASE_ADDR    (APB0_BASE_ADDR + 0xA000u)
#define GPIO5_BASE_ADDR    (APB0_BASE_ADDR + 0xB000u)
#define GPIO6_BASE_ADDR    (APB0_BASE_ADDR + 0xC000u)
#define GPIO7_BASE_ADDR    (APB0_BASE_ADDR + 0xD000u)
#define GPIO8_BASE_ADDR    (APB0_BASE_ADDR + 0xE000u)
#define GPIO9_BASE_ADDR    (APB0_BASE_ADDR + 0xF000u)

#define GPIO_DIR_OFFS      (0x400u)
#define GPIO_SEL_OFFS      (0x41Cu)
//
// system, reset clock control unit register
//
#define SCU_BASE_ADDR          (APB1_BASE_ADDR + 0x2000u)
#define SCU_CLKCNTR            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x00u))
#define SCU_PLLCONF            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x04u))
#define SCU_SYSSTATUS          (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x08u))
#define SCU_PCGR0              (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x14u))
#define SCU_PCGR1              (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x18u))
#define SCU_PRR0               (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x1Cu))
#define SCU_PRR1               (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x20u))
#define SCU_SCR0               (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x34u))
#define SCU_SCR1               (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x38u))
#define SCU_SCR2               (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x3Cu))

#define SCU_GPIOOUT_BASE_ADDR  (SCU_BASE_ADDR + 0x44u)
#define SCU_GPIOOUT0           (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x44u))
#define SCU_GPIOOUT1           (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x48u))
#define SCU_GPIOOUT2           (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x4Cu))
#define SCU_GPIOOUT3           (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x50u))
#define SCU_GPIOOUT4           (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x54u))
#define SCU_GPIOOUT5           (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x58u))
#define SCU_GPIOOUT6           (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x5Cu))
#define SCU_GPIOOUT7           (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x60u))

#define SCU_GPIOIN_BASE_ADDR   (SCU_BASE_ADDR + 0x64u)
#define SCU_GPIOIN0            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x64u))
#define SCU_GPIOIN1            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x68u))
#define SCU_GPIOIN2            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x6Cu))
#define SCU_GPIOIN3            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x70u))
#define SCU_GPIOIN4            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x74u))
#define SCU_GPIOIN5            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x78u))
#define SCU_GPIOIN6            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x7Cu))
#define SCU_GPIOIN7            (*(volatile OS_U32*)(SCU_BASE_ADDR + 0x80u))

#define SCU_CLKSEL_PLL         (0x00u)
#define SCU_CLKSEL_RTC         (0x01u)
#define SCU_CLKSEL_MAIN        (0x02u)

#define SCU_PLLEN              (0x80000u)
#define SCU_PCLK_Div1          (0xFFFFFE7Fu)
#define SCU_PCLK_Div2          (0x80u)
#define SCU_FLAG_LOCK          (0x01u)

#define SCU_VIC_ENABLE_BIT     (5u)
#define SCU_TIM0_ENABLE_BIT    (0u)
#define SCU_TIM1_ENABLE_BIT    (0u)
#define SCU_TIM2_ENABLE_BIT    (1u)
#define SCU_TIM3_ENABLE_BIT    (1u)
//
// Timer sfr definition
//
#define TIM0_BASE_ADDR         (APB0_BASE_ADDR + 0x2000u)
#define TIM1_BASE_ADDR         (APB0_BASE_ADDR + 0x3000u)
#define TIM2_BASE_ADDR         (APB0_BASE_ADDR + 0x4000u)
#define TIM3_BASE_ADDR         (APB0_BASE_ADDR + 0x5000u)

#define TIM_ICAR_OFFS          (0x00u)
#define TIM_ICBR_OFFS          (0x04u)
#define TIM_OCAR_OFFS          (0x08u)
#define TIM_OCBR_OFFS          (0x0Cu)
#define TIM_CNTR_OFFS          (0x10u)
#define TIM_CR1_OFFS           (0x14u)
#define TIM_CR2_OFFS           (0x18u)
#define TIM_SR_OFFS            (0x1Cu)

#define TIMER_INT_BIT_NO       (14u)  // Output compare match flag 1
//
// Assign timer sfrs used for embOS timer
//
#define OS_TIM_BASE_ADDR       TIM1_BASE_ADDR
#define OS_TIM_OCR             (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_OCAR_OFFS))
#define OS_TIM_CR1             (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_CR1_OFFS))
#define OS_TIM_CR2             (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_CR2_OFFS))
#define OS_TIM_CNTR            (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_CNTR_OFFS))
#define OS_TIM_SR              (*(volatile OS_U16*)(OS_TIM_BASE_ADDR + TIM_SR_OFFS))

#if   (OS_TIM_BASE_ADDR == TIM0_BASE_ADDR)
  #define  OS_TIMER_INT_ID     VIC_IRQ_TIM0_ID
  #define  SCU_TIM_ENABLE_BIT  SCU_TIM0_ENABLE_BIT
  #define  SCU_TIM_PRESCALE    SCU_SCR1
#elif (OS_TIM_BASE_ADDR == TIM1_BASE_ADDR)
  #define  OS_TIMER_INT_ID     VIC_IRQ_TIM1_ID
  #define  SCU_TIM_ENABLE_BIT  SCU_TIM1_ENABLE_BIT
  #define  SCU_TIM_PRESCALE    SCU_SCR1
#elif (OS_TIM_BASE_ADDR == TIM2_BASE_ADDR)
  #define  OS_TIMER_INT_ID     VIC_IRQ_TIM2_ID
  #define  SCU_TIM_ENABLE_BIT  SCU_TIM2_ENABLE_BIT
  #define  SCU_TIM_PRESCALE    SCU_SCR2
#elif (OS_TIM_BASE_ADDR == TIM3_BASE_ADDR)
  #define  OS_TIMER_INT_ID     VIC_IRQ_TIM3_ID
  #define  SCU_TIM_ENABLE_BIT  SCU_TIM3_ENABLE_BIT
  #define  SCU_TIM_PRESCALE    SCU_SCR2
#else
  #error "Unsupported assignment of embOS timer"
#endif
//
// Vectored interrupt controller (VIC)
//
#define VIC0_BASE_ADDR            (0xFFFFF000u)
#define VIC1_BASE_ADDR            (0xFC000000u)

#define VIC0_IRQ_STATUS           (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0000u))
#define VIC0_FIQ_STATUS           (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0004u))
#define VIC0_RAW_INTR             (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0008u))
#define VIC0_INT_SELECT           (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x000Cu))
#define VIC0_INT_ENABLE           (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0010u))
#define VIC0_INT_ENCLEAR          (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0014u))
#define VIC0_SOFT_INT             (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0018u))
#define VIC0_SOFT_INT_CLEAR       (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x001Cu))
#define VIC0_PROTECTION           (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0020u))
#define VIC0_CURR_VECTADDR        (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0030u))
#define VIC0_DEF_VECTADDR         (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0034u))
#define VIC0_VECT_BASE_ADDR       (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0100u))
#define VIC0_VECT_CNTL_BASE_ADDR  (*(volatile OS_U32*)(VIC0_BASE_ADDR + 0x0200u))

#define VIC1_IRQ_STATUS           (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0000u))
#define VIC1_FIQ_STATUS           (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0004u))
#define VIC1_RAW_INTR             (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0008u))
#define VIC1_INT_SELECT           (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x000Cu))
#define VIC1_INT_ENABLE           (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0010u))
#define VIC1_INT_ENCLEAR          (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0014u))
#define VIC1_SOFT_INT             (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0018u))
#define VIC1_SOFT_INT_CLEAR       (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x001Cu))
#define VIC1_PROTECTION           (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0020u))
#define VIC1_CURR_VECTADDR        (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0030u))
#define VIC1_DEF_VECTADDR         (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0034u))
#define VIC1_VECT_BASE_ADDR       (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0100u))
#define VIC1_VECT_CNTL_BASE_ADDR  (*(volatile OS_U32*)(VIC1_BASE_ADDR + 0x0200u))
//
// Vectored interrupt controller sources
//
#define VIC_IRQ_WD_ID      ( 0u)  // IRQ0  Watchdog
#define VIC_IRQ_SWI_ID     ( 1u)  // IRQ1  Software interrupt
#define VIC_IRQ_CPU_RX_ID  ( 2u)  // IRQ2  Debug receive command
#define VIC_IRQ_CPU_TX_ID  ( 3u)  // IRQ3  Debug transmit command
#define VIC_IRQ_TIM0_ID    ( 4u)  // IRQ4  Timer 0
#define VIC_IRQ_TIM1_ID    ( 5u)  // IRQ4  Timer 0
#define VIC_IRQ_TIM2_ID    ( 6u)  // IRQ4  Timer 0
#define VIC_IRQ_TIM3_ID    ( 7u)  // IRQ4  Timer 0
#define VIC_IRQ_USBH_ID    ( 8u)  // IRQ8  USB high priority
#define VIC_IRQ_USBL_ID    ( 9u)  // IRQ9  USB low priority
#define VIC_IRQ_SCU_ID     (10u)  // IRQ10 SCU
#define VIC_IRQ_MAC_ID     (11u)  // IRQ11 MAC
#define VIC_IRQ_DMA_ID     (12u)  // IRQ12 DMA
#define VIC_IRQ_CAN_ID     (13u)  // IRQ13 CAN
#define VIC_IRQ_IMC_ID     (14u)  // IRQ14 Induction motor control
#define VIC_IRQ_ADC_ID     (15u)  // IRQ15 A/D converter
#define VIC_IRQ_UART0_ID   (16u)  // IRQ16 UART0
#define VIC_IRQ_UART1_ID   (17u)  // IRQ17 UART1
#define VIC_IRQ_UART2_ID   (18u)  // IRQ18 UART2
#define VIC_IRQ_I2C0_ID    (19u)  // IRQ19 I2C channel 0
#define VIC_IRQ_I2C1_ID    (20u)  // IRQ20 I2C channel 1
#define VIC_IRQ_SSP0_ID    (21u)  // IRQ21 SSP channel 0
#define VIC_IRQ_SSP1_ID    (22u)  // IRQ22 SSP channel 1
#define VIC_IRQ_SCUB_ID    (23u)  // IRQ23 SCU brown out
#define VIC_IRQ_RTC_ID     (24u)  // IRQ24 SCU brown out
#define VIC_IRQ_WIUA_ID    (25u)  // IRQ25 WIU all
#define VIC_IRQ_WIU0_ID    (26u)  // IRQ26 WIU group 0
#define VIC_IRQ_WIU1_ID    (27u)  // IRQ27 WIU group 1
#define VIC_IRQ_WIU2_ID    (28u)  // IRQ28 WIU group 2
#define VIC_IRQ_WIU3_ID    (29u)  // IRQ29 WIU group 3
#define VIC_IRQ_USBW_ID    (30u)  // IRQ30 USB wake up
#define VIC_IRQ_PFQ_ID     (31u)  // IRQ31 Prefetch queue and branch

#ifndef   NUM_INT_SOURCES
  #define NUM_INT_SOURCES  (32u)
#endif

#ifndef   NUM_INT_VECTORS
  #define NUM_INT_VECTORS  (32u)
#endif

#ifndef   NUM_VECTORS_PER_VIC
  #define NUM_VECTORS_PER_VIC  (16u)
#endif

#ifndef   NUM_INTS_PER_VIC
  #define NUM_INTS_PER_VIC  (16u)
#endif

#define INT_SOURCE_MASK  (0x0Fu)

#define INT_PRIORITY_MASK  (0x0Fu)

#define OS_TIMER_VECT_INDEX  (15u)  // lowest priority, allowed range 0..15, Timer ints are connected to VIC0

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

#if (defined(__ICCARM__))
/*********************************************************************
*
*       OS_GetPC()
*/
#pragma diag_suppress=Pe940 // IAR specific: No warning for missing return
static OS_U32 OS_GetPC(void) {
  #if (__VER__ < 6300000)
    __asm("mov  r0, lr");
    __asm("subs r0, r0, #4");
  #else
    __asm volatile("mov  r0, lr");
    __asm volatile("subs r0, r0, #4");
  #endif
}
#endif

/*********************************************************************
*
*       _InitVICs()
*
*  Function description
*    Clears and disables all interrut vectors in VIC.
*    CPU interrupt has to be disabled before calling this function.
*/
static void _InitVICs(void) {
  unsigned int Index;

  SCU_PRR0          &= ~(1u << SCU_VIC_ENABLE_BIT);  // reset VIC
  SCU_PCGR0         |=  (1u << SCU_VIC_ENABLE_BIT);  // Enable clock to VIC
  SCU_PRR0          |=  (1u << SCU_VIC_ENABLE_BIT);  // Cancel reset state of VIC
  VIC0_INT_ENCLEAR   = 0xFFFFFFFFu;                  // Disable all source interrupts
  VIC1_INT_ENCLEAR   = 0xFFFFFFFFu;                  // Disable all source interrupts
  for (Index = 0; Index < NUM_INT_VECTORS; Index++) {
    OS_ARM_DisableISR(Index);                        // Disable channel
    OS_ARM_AssignISRSource(Index, 0);                // Assign to hardware channel 0
    OS_ARM_InstallISRHandler(Index, NULL);           // Set vector to 0;
  }
  VIC0_SOFT_INT_CLEAR  = 0xFFFFFFFFu;                // Disable all soft interrupts
  VIC0_INT_SELECT      = 0;                          // Set all channels to generate IRQ
  VIC0_DEF_VECTADDR    = 0;                          // Set default vector to 0
  VIC1_SOFT_INT_CLEAR  = 0xFFFFFFFFu;                // Disable all soft interrupts
  VIC1_INT_SELECT      = 0;                          // Set all channels to generate IRQ
  VIC1_DEF_VECTADDR    = 0;                          // Set default vector to 0
}

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when an not installed interrupt was detected.
*    As interrupt pending condition of peripherals has to be reset,
*    program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(void) {
#if (OS_IGNORE_UNDEFINED_INTERRUPT == 0)
  volatile int Dummy;

  Dummy = 1;
  while (Dummy > 0) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
#endif
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  OS_I16 Cnt;

  Cnt = OS_TIM_CNTR;
  Cnt = OS_TIM_OCR - Cnt;
  if (Cnt < 0) {
    Cnt = 0 - Cnt;
  }
  return Cnt;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  OS_U16 Cnt;

  Cnt = OS_TIM_CNTR;
  return (OS_TIM_OCR - Cnt) <= 0;
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  OS_TIM_SR  &= ~(1 << TIMER_INT_BIT_NO);  // Clear embOS timer interrupt flag
  OS_TIM_OCR += OS_TIMER_RELOAD;           // Advance to next count;
  OS_TICK_Handle();                        // Call embOS tick handler
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  _InitVICs();                                         // Initialize VICs, clear and disable all interrupts

  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  SCU_PCGR1 |= (1u << SCU_TIM_ENABLE_BIT);            // Enable clock to timer module
  SCU_PRR1  |= (1u << SCU_TIM_ENABLE_BIT);            // Cancel reset state of timer module

  OS_TIM_CR1  = 0;                                     // Stop timer, reset mode
  OS_TIM_CR2  = 0;                                     // Disable all interrupts, clear prescaler
  OS_TIM_OCR  = OS_TIMER_RELOAD;                       // Setup compare register, initially use 1000 ticks per second
  OS_TIM_CR2  = (1u << 14) | (OS_TIMER_PRESCALE - 1);  // Enable output capture interrupt A, set selected prescaler
  OS_TIM_CNTR = 0;
  OS_TIM_CR1  = (1u << 15);                            // Start timer
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler(OS_TIMER_VECT_INDEX, &_OS_ISR_Tick);  // Timer/counter interrupt vector.
  OS_ARM_AssignISRSource(OS_TIMER_VECT_INDEX, OS_TIMER_INT_ID);
  OS_ARM_EnableISR(OS_TIMER_VECT_INDEX);                         // Enable timer/counter interrupt.
  OS_ARM_EnableISRSource(OS_TIMER_INT_ID);                       // Enable timer/counter interrupt.
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  OS_ISR_HANDLER* pISR1;

  pISR  = (OS_ISR_HANDLER*)VIC0_CURR_VECTADDR;  // Get current interrupt handler
  pISR1 = (OS_ISR_HANDLER*)VIC1_CURR_VECTADDR;  // Get current interrupt handler of VIC1
  OS_INT_EnterNestable();
  if (pISR != NULL) {
    pISR();
  } else {
    if (pISR1 != NULL) {
      pISR1();
    } else {
      _OS_ISR_Undefined();
    }
  }
  VIC0_CURR_VECTADDR = (OS_U32)pISR;   // Clear current interrupt pending condition, reset VIC
  VIC1_CURR_VECTADDR = (OS_U32)pISR1;  // Clear current interrupt pending condition, reset VIC
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  if ((unsigned int)ISRIndex < NUM_VECTORS_PER_VIC) {
    papISR = (OS_ISR_HANDLER**)&VIC0_VECT_BASE_ADDR;
  } else {
    ISRIndex -= NUM_VECTORS_PER_VIC;
    papISR = (OS_ISR_HANDLER**)&VIC1_VECT_BASE_ADDR;
  }
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_AssignISRSource()
*/
void OS_ARM_AssignISRSource(int ISRIndex, int Source) {
  OS_U32* pCntlRegister;
  OS_U32 ControlReg;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  if ((unsigned int)ISRIndex < NUM_VECTORS_PER_VIC) {
    pCntlRegister = (OS_U32*)&VIC0_VECT_CNTL_BASE_ADDR;
  } else {
    ISRIndex     -= NUM_VECTORS_PER_VIC;
    pCntlRegister = (OS_U32*)&VIC1_VECT_CNTL_BASE_ADDR;
  }
  OS_INT_Disable();
  ControlReg                  = *(pCntlRegister + ISRIndex) & ~INT_SOURCE_MASK;
  *(pCntlRegister + ISRIndex) = (ControlReg | (Source & INT_SOURCE_MASK));
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
  OS_U32* pCntlRegister;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  if ((unsigned int)ISRIndex < NUM_VECTORS_PER_VIC) {
    pCntlRegister = (OS_U32*)&VIC0_VECT_CNTL_BASE_ADDR;
  } else {
    ISRIndex     -= NUM_VECTORS_PER_VIC;
    pCntlRegister = (OS_U32*)&VIC1_VECT_CNTL_BASE_ADDR;
  }
  OS_INT_Disable();
  *(pCntlRegister + ISRIndex) |= (1u << 5);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
  OS_U32* pCntlRegister;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  if ((unsigned int)ISRIndex < NUM_VECTORS_PER_VIC) {
    pCntlRegister = (OS_U32*)&VIC0_VECT_CNTL_BASE_ADDR;
  } else {
    ISRIndex     -= NUM_VECTORS_PER_VIC;
    pCntlRegister = (OS_U32*)&VIC1_VECT_CNTL_BASE_ADDR;
  }
  OS_INT_Disable();
  *(pCntlRegister + ISRIndex) &= ~(1u << 5);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_EnableISRSource()
*/
void OS_ARM_EnableISRSource(int SourceIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)SourceIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  if ((unsigned int)SourceIndex < NUM_INTS_PER_VIC) {
    VIC0_INT_ENABLE |= (1u << SourceIndex);
  } else {
    SourceIndex     -= NUM_INTS_PER_VIC;
    VIC1_INT_ENABLE |= (1u << SourceIndex);
  }
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISRSource()
*/
void OS_ARM_DisableISRSource(int SourceIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned)SourceIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
  OS_INT_Disable();
  if ((unsigned int)SourceIndex < NUM_INTS_PER_VIC) {
    VIC0_INT_ENCLEAR = (1u << SourceIndex);
  } else {
    SourceIndex -= NUM_INTS_PER_VIC;
    VIC1_INT_ENCLEAR = (1u << SourceIndex);
  }
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  //
  // Not supported for CPU with VIC
  //
  OS_USEPARA(ISRIndex);  // Suppress warning
  OS_USEPARA(Prio);      // Suppress warning
  return 0;
}


/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  OS_U32 CLKCNTR_Value;
  int    i;

  //
  // Check if vector jump table is programmed at 0x0 for RAM targets
  //
  if ((OS_GetPC() & 0x04000000u) && ((*(OS_U32*)0x18u != 0xE59FF010u) || (*(OS_U32*)0x30u != 0x04000018u))) {
    while (1) {  // Fatal error! Vector jump table not programmed at 0x0. See ReadMe.txt for more info.
      ;
    }
  }
  //
  // Initialize Clock source and PLL
  //
  CLKCNTR_Value  = SCU_CLKCNTR;      // Get the current Clock control value
  CLKCNTR_Value &= ~0x3u;            // Clear field MCLKSEL
  CLKCNTR_Value |= SCU_CLKSEL_MAIN;  // Select Main oscillator as source
  SCU_CLKCNTR    = CLKCNTR_Value;    // Update CLKCNTR register value
  //
  // Write a write flash configuration register command and configure Flash configuration
  //
  *(volatile unsigned short*) (FMI_NBBADR << 2) = 0x60u;
  *(volatile unsigned short*)((FMI_NBBADR << 2) | FMI_READ_WAIT_STATE_2 | FMI_PWD_ENABLE | FMI_LVD_ENABLE | FMI_FREQ_HIGH) = 0x03u;
  //
  // Set the PCLK Clock to MCLK/2, because peripheral clock is limited to 48 MHz
  //
  SCU_CLKCNTR &= SCU_PCLK_Div1;  // Clear APBDIV[1:0] field
  SCU_CLKCNTR |= SCU_PCLK_Div2;  // Update field with PCLK Divisor
  //
  // Setup PLL config registers
  //
  #if   (OS_SYSTEM_FREQ == 48000000)
    #define PLL_P  (3u)
  #elif (OS_SYSTEM_FREQ == 96000000)
    #define PLL_P  (2u)
  #else
    #error "Unsupported PLL frequency, please modify PLL init function"
  #endif
  #define PLL_N  (0x18 * 8)
  #define PLL_M  (OS_EXTERNAL_FREQ / 1000000)

  if (SCU_CLKCNTR & 3u) {            // Check if PLL not already selected as clock
    for(i = 10 ; i > 0; i--) {       // Add a short delay before disabling PLL
      ;
    }
    SCU_PLLCONF &= ~SCU_PLLEN;     // Disable PLL
    SCU_PLLCONF  = 0;              // Clear PLLCONF register
    SCU_PLLCONF |= (PLL_N <<  8);  // Update PLLN field
    SCU_PLLCONF |= PLL_M;          // Update PLLM field
    SCU_PLLCONF |= (PLL_P << 16);  // Update PLLP field
  }
  //
  // Enable PLL if not already enabled
  //
  if (((SCU_PLLCONF & SCU_PLLEN) && (SCU_SYSSTATUS & SCU_FLAG_LOCK)) == 0) {
    SCU_SYSSTATUS |= SCU_FLAG_LOCK;                 // Clear LOCK bit
    SCU_PLLCONF   |= SCU_PLLEN;                     // Enable PLL
    while ((SCU_SYSSTATUS & SCU_FLAG_LOCK) == 0) {  // Wait until PLL locks
      ;
    }
  }
  //
  // Select PLL as clock source
  //
  CLKCNTR_Value  = SCU_CLKCNTR;  // Get current CLKCNTR register value
  CLKCNTR_Value &= ~3u;          // Clear MCLKSEL
  //
  // Check if PLL is enabled & locked
  //
  if (((SCU_PLLCONF & SCU_PLLEN) && (SCU_SYSSTATUS & SCU_FLAG_LOCK))) {
    CLKCNTR_Value |= SCU_CLKSEL_PLL;  // Switch to PLL as clock source
    SCU_CLKCNTR    = CLKCNTR_Value;   // Update CLKCNTR register
  }
  return 1;  // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
