;*********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                *
;*        Solutions for real time microcontroller applications       *
;*********************************************************************
;*                                                                   *
;*      (C) 2005- 2007 SEGGER Microcontroller Systeme GmbH           *
;*                  www.segger.com                                   *
;*                                                                   *
;*********************************************************************
;
;---------------------------------------------------------------------
;File   : STR912_Startup.s
;Purpose: Startup file for embOS and IAR embedded workbench V5.x
;
;-------- END-OF-HEADER ----------------------------------------------
;

;*********************************************************************

        MODULE  ?cstartup

SCRO_AHB_UNB    EQU     0x5C002034

        ;; Forward declaration of sections.
        SECTION IRQ_STACK:DATA:NOROOT(3)
        SECTION FIQ_STACK:DATA:NOROOT(3)
        SECTION CSTACK:DATA:NOROOT(3)


        SECTION .intvec:CODE:NOROOT(2)

        PUBLIC  __vector
        PUBLIC  __vector_0x14
        PUBLIC  __iar_program_start
        EXTERN  Undefined_Handler
        EXTERN  SWI_Handler
        EXTERN  Prefetch_Handler
        EXTERN  Abort_Handler
        EXTERN  IRQ_Handler
        EXTERN  FIQ_Handler

        ARM

__iar_init$$done:               ; The vector table is not needed
                                ; until after copy initialization is done

__vector:
        ; All default exception handlers (except reset) are
        ; defined as weak symbol definitions.
        ; If a handler is defined by the application it will take precedence.

        ldr     PC,Reset_Addr           ; To allow remap, a relative branch is required !

        ldr     PC,Undefined_Addr       ; Undefined instructions
        ldr     PC,SWI_Addr             ; Software interrupt (SWI/SVC)
        ldr     PC,Prefetch_Addr        ; Prefetch abort
        ldr     PC,Abort_Addr           ; Data abort
__vector_0x14:
        DCD     0                       ; Reserved

        ldr     PC,IRQ_Addr             ; IRQ
        ldr     PC,FIQ_Addr             ; FIQ

        DATA

Reset_Addr:     DCD   __iar_program_start
Undefined_Addr: DCD   Undefined_Handler
SWI_Addr:       DCD   SWI_Handler
Prefetch_Addr:  DCD   Prefetch_Handler
Abort_Addr:     DCD   Abort_Handler
IRQ_Addr:       DCD   IRQ_Handler
FIQ_Addr:       DCD   FIQ_Handler

__vector_end:


; --------------------------------------------------
; ?cstartup -- low-level system initialization code.
;
; After a reser execution starts here, the mode is ARM, supervisor
; with interrupts disabled.
;
; The startup function is placed in a special section which has to be located in the
; first 32KBytes of the program memory to ensure, the code can be reached as long as the Flash memory
; mapping is not initialized which is the case after reset
;

        SECTION .cstartup:CODE:NOROOT(2)

        EXTERN  ?main
        REQUIRE __vector

        ARM

__iar_program_start:

/**********************************************************************
* ?CSTARTUP
*
* Execution starts here.
* After a reset, the mode is ARM, Supervisor, interrupts disabled.
*/
?cstartup:
        ldr     pc, =NextInst

NextInst
        nop   ; execute some instructions to access CPU registers after wake
        nop   ; up from Reset, while waiting for OSC stabilization
        nop
        nop
        nop
        nop
        nop
        nop
        nop


; --- Remap Flash Bank 0 at address 0x0 and Bank 1 at address 0x80000,
;     when the bank 0 is the boot bank, then enable the Bank 1.

        ldr R6, =0x54000000    ; BOOT BANK Size = 512KB
        ldr R7, =0x4           ; (2^4) * 32 = 512KB
        str R7, [R6]

        ldr R6, =0x54000004    ; NON BOOT BANK Size = 32KB
        ldr R7, =0x2           ; (2^2) * 8 = 32KB
        str R7, [R6]

        ldr R6, =0x5400000C    ; BOOT BANK Address = 0x0
        ldr R7, =0x0
        str R7, [R6]

        ldr R6, =0x54000010    ; NON BOOT BANK Address = 0x80000
        ldr R7, =0x20000       ; need to put 0x20000 because FMI bus on A[25:2] of CPU bus
        str R7, [R6]

        ldr R6, =0x54000018    ; Enable CS on both banks
        ldr R7, =0x18
        str R7, [R6]

; --- Enable 96K RAM

        ldr     R0, = SCRO_AHB_UNB
        ldr     R1, = 0x0191   ; PFQBC enabled / DTCM & AHB wait-states disabled
        str     R1, [R0]

; --- Initialize Stack pointer registers

; Initialize the stack pointers.
        ;; Set up the interrupt stack pointer.

        msr     cpsr_c,#0xd2            ; Change mode to IRQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(IRQ_STACK)     ; End of IRQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary
	
        ;; Set up the fast interrupt stack pointer.

        msr     cpsr_c,#0xd1            ; Change mode to FIQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(FIQ_STACK)     ; End of FIQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary

        ;; Set up the normal stack pointer.

        msr     cpsr_c,#0xdf            ; Change mode to SYS mode, FIQ and IRQ disabled
        ldr     sp, =SFE(CSTACK)        ; End of CSTACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary

; --- Set bits 17-18 (DTCM/ITCM order bits) of the Core Configuration Control Register

        MOV     r0, #0x60000
        MCR     p15,0x1,r0,c15,c1,0

; Jump to startup code in library
        ldr     r0,=?main
        bx      r0

        LTORG
        END

; ***** EOF **********************************************************
