/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   OS_INIT_PLL
  #define OS_INIT_PLL  (1)
#endif

#ifndef   ALLOW_NESTED_INTERRUPTS
  #define ALLOW_NESTED_INTERRUPTS  (0)
#endif

/*********************************************************************
*
*       Clock settings
*/
#define OS_SYSTEM_FREQ   (400000000u)

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ    (25000000u)
#define OS_TICK_FREQ     (1000u)
#define OS_INT_FREQ      (OS_TICK_FREQ)
#define OS_TIMER_RELOAD  (OS_TIMER_FREQ / OS_INT_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
//
// Interrupt Vector Controller
//
#define AINTC_REVID                   (*(volatile OS_U32*)(0xFFFEE000u))               // Revision Identification Register
#define AINTC_CR                      (*(volatile OS_U32*)(0xFFFEE004u))               // Control Register
#define AINTC_GER                     (*(volatile OS_U32*)(0xFFFEE010u))               // Global Enable Register
#define AINTC_GNLR                    (*(volatile OS_U32*)(0xFFFEE01Cu))               // Global Nesting Level Register
#define AINTC_SISR                    (*(volatile OS_U32*)(0xFFFEE020u))               // System Interrupt Status Indexed Set Register
#define AINTC_SICR                    (*(volatile OS_U32*)(0xFFFEE024u))               // System Interrupt Status Indexed Clear Register
#define AINTC_EISR                    (*(volatile OS_U32*)(0xFFFEE028u))               // System Interrupt Enable Indexed Set
#define AINTC_EICR                    (*(volatile OS_U32*)(0xFFFEE02Cu))               // System Interrupt Enable Indexed Clear
#define AINTC_HIEISR                  (*(volatile OS_U32*)(0xFFFEE034u))               // Host Interrupt Enable Indexed Set Register
#define AINTC_HIEICR                  (*(volatile OS_U32*)(0xFFFEE038u))               // Host Interrupt Enable Indexed Clear Register
#define AINTC_VBR                     (*(volatile OS_U32*)(0xFFFEE050u))               // Vector Base Register
#define AINTC_VSR                     (*(volatile OS_U32*)(0xFFFEE054u))               // Vector Size Register
#define AINTC_VNR                     (*(volatile OS_U32*)(0xFFFEE058u))               // Vector Null Register
#define AINTC_GPIR                    (*(volatile OS_U32*)(0xFFFEE080u))               // Global Prioritized Index Register
#define AINTC_GPVR                    (*(volatile OS_U32*)(0xFFFEE084u))               // Global Prioritized Vector Register
#define AINTC_SRSR1                   (*(volatile OS_U32*)(0xFFFEE200u))               // System Interrupt Status Raw/Set Register 1
#define AINTC_SRSR2                   (*(volatile OS_U32*)(0xFFFEE204u))               // System Interrupt Status Raw/Set Register 2
#define AINTC_SRSR3                   (*(volatile OS_U32*)(0xFFFEE208u))               // System Interrupt Status Raw/Set Register 3
#define AINTC_SRSR4                   (*(volatile OS_U32*)(0xFFFEE20Cu))               // System Interrupt Status Raw/Set Register 4

#define AINTC_SECR1_ADDR              (0xFFFEE280u)
#define AINTC_SECR1                   (*(volatile OS_U32*)(0xFFFEE280u))               // System Interrupt Status Enabled/Clear Register 1
#define AINTC_SECR2                   (*(volatile OS_U32*)(0xFFFEE284u))               // System Interrupt Status Enabled/Clear Register 2
#define AINTC_SECR3                   (*(volatile OS_U32*)(0xFFFEE288u))               // System Interrupt Status Enabled/Clear Register 3
#define AINTC_SECR4                   (*(volatile OS_U32*)(0xFFFEE28Cu))               // System Interrupt Status Enabled/Clear Register 4
#define AINTC_ESR1                    (*(volatile OS_U32*)(0xFFFEE300u))               // System Interrupt Enable Set Register 1
#define AINTC_ESR2                    (*(volatile OS_U32*)(0xFFFEE304u))               // System Interrupt Enable Set Register 2
#define AINTC_ESR3                    (*(volatile OS_U32*)(0xFFFEE308u))               // System Interrupt Enable Set Register 3
#define AINTC_ESR4                    (*(volatile OS_U32*)(0xFFFEE30Cu))               // System Interrupt Enable Set Register 4
#define AINTC_ECR1                    (*(volatile OS_U32*)(0xFFFEE380u))               // System Interrupt Enable Clear Register 1
#define AINTC_ECR2                    (*(volatile OS_U32*)(0xFFFEE384u))               // System Interrupt Enable Clear Register 2
#define AINTC_ECR3                    (*(volatile OS_U32*)(0xFFFEE388u))               // System Interrupt Enable Clear Register 3
#define AINTC_ECR4                    (*(volatile OS_U32*)(0xFFFEE38Cu))               // System Interrupt Enable Clear Register 4

#define AINTC_CMR0_ADDR               (0xFFFEE400u)                                    // Channel Map Registers 0-25
#define AINTC_HIPIR1                  (*(volatile OS_U32*)(0xFFFEE900u))               // Host Interrupt Prioritized Index Register 1
#define AINTC_HIPIR2                  (*(volatile OS_U32*)(0xFFFEE904u))               // Host Interrupt Prioritized Index Register 2
#define AINTC_HINLR1                  (*(volatile OS_U32*)(0xFFFEF100u))               // Host Interrupt Nesting Level Register 1
#define AINTC_HINLR2                  (*(volatile OS_U32*)(0xFFFEF104u))               // Host Interrupt Nesting Level Register 2
#define AINTC_HIER                    (*(volatile OS_U32*)(0xFFFEF500u))               // Host Interrupt Enable Register
#define AINTC_HIPVR1                  (*(volatile OS_U32*)(0xFFFEF600u))               // Host Interrupt Prioritized Vector Register 1
#define AINTC_HIPVR2                  (*(volatile OS_U32*)(0xFFFEF604u))               // Host Interrupt Prioritized Vector Register 2

#define NUM_INT_VECTORS               (101u)

#define AINTC_CR_NESTMODE             (0x0000000Cu)
#define AINTC_CR_NESTMODE_SHIFT       (0x00000002u)
#define AINTC_CR_NESTMODE_NONEST      (0x00000000u)
#define AINTC_CR_NESTMODE_INDIVIDUAL  (0x00000001u)
#define AINTC_CR_NESTMODE_GLOBAL      (0x00000002u)
#define AINTC_CR_NESTMODE_MANUAL      (0x00000003u)
//
// Timer
//
#define TIMER0_BASE_ADDR              (0x01C20000u)
#define TIMER1_BASE_ADDR              (0x01C21000u)
#define TIMER0_REVID                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x00u))  // Revision ID Register
#define TIMER0_EMUMGT                 (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x04u))  // Emulation Management Register
#define TIMER0_GPINTGPEN              (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x08u))  // GPIO Interrupt and GPIO Enable Register
#define TIMER0_GPDATGPDIR             (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x0Cu))  // GPIO Data and GPIO Direction Register
#define TIMER0_TIM12                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x10u))  // Timer Counter Register 12
#define TIMER0_TIM34                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x14u))  // Timer Counter Register 34
#define TIMER0_PRD12                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x18u))  // Timer Period Register 12
#define TIMER0_PRD34                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x1Cu))  // Timer Period Register 34
#define TIMER0_TCR                    (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x20u))  // Timer Control Register
#define TIMER0_TGCR                   (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x24u))  // Timer Global Control Register
#define TIMER0_WDTCR                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x28u))  // Watchdog Timer Control Register
#define TIMER0_REL12                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x34u))  // Timer Reload Register 12
#define TIMER0_REL34                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x38u))  // Timer Reload Register 34
#define TIMER0_CAP12                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x3Cu))  // Timer Capture Register 12
#define TIMER0_CAP34                  (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x40u))  // Timer Capture Register 34
#define TIMER0_INTCTLSTAT             (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x44u))  // Timer Interrupt Control and Status Register
#define TIMER0_CMP0                   (*(volatile OS_U32*)(TIMER0_BASE_ADDR + 0x60u))  // Compare Register 0
#define TIMER1_IRQ_PRIO               ( 2u)                                            // Use channel 2 in AINTC
#define TIMER0_IRQ_ID                 (21u)
#define TIMMODE_32BIT_UNCHAINED       (2u << 1)
#define TIM12RS                       (1u << 0)
#define ENAMODE12_CONT                (0x00000080u)
#define PRDINTEN12                    (1u << 0)
#define EMUMGT_SOFT                   (1u << 1)
#define EMUMGT_FREE                   (1u << 0)
#define PRDINTSTAT12                  (1u << 1)

#define TIMER1_REVID                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x00u))  // Revision ID Register
#define TIMER1_EMUMGT                 (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x04u))  // Emulation Management Register
#define TIMER1_GPINTGPEN              (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x08u))  // GPIO Interrupt and GPIO Enable Register
#define TIMER1_GPDATGPDIR             (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x0Cu))  // GPIO Data and GPIO Direction Register
#define TIMER1_TIM12                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x10u))  // Timer Counter Register 12
#define TIMER1_TIM34                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x14u))  // Timer Counter Register 34
#define TIMER1_PRD12                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x18u))  // Timer Period Register 12
#define TIMER1_PRD34                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x1Cu))  // Timer Period Register 34
#define TIMER1_TCR                    (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x20u))  // Timer Control Register
#define TIMER1_TGCR                   (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x24u))  // Timer Global Control Register
#define TIMER1_WDTCR                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x28u))  // Watchdog Timer Control Register
#define TIMER1_REL12                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x34u))  // Timer Reload Register 12
#define TIMER1_REL34                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x38u))  // Timer Reload Register 34
#define TIMER1_CAP12                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x3Cu))  // Timer Capture Register 12
#define TIMER1_CAP34                  (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x40u))  // Timer Capture Register 34
#define TIMER1_INTCTLSTAT             (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x44u))  // Timer Interrupt Control and Status Register
#define TIMER1_CMP0                   (*(volatile OS_U32*)(TIMER1_BASE_ADDR + 0x60u))  // Compare Register 0
//
// PLL
//
#define PLLC0_REVID                   (*(volatile OS_U32*)(0x01C11000u))               // PLLC0 Revision Identification Register Section 7.3.1
#define PLLC0_RSTYPE                  (*(volatile OS_U32*)(0x01C110E4u))               // PLLC0 Reset Type Status Register Section 7.3.3
#define PLLC0_RSCTRL                  (*(volatile OS_U32*)(0x01C110E8u))               // PLLC0 Reset Control Register Section 7.3.4
#define PLLC0_PLLCTL                  (*(volatile OS_U32*)(0x01C11100u))               // PLLC0 Control Register Section 7.3.5
#define PLLC0_OCSEL                   (*(volatile OS_U32*)(0x01C11104u))               // PLLC0 PLLC0 OBSCLK Select Register Section 7.3.7
#define PLLC0_PLLM                    (*(volatile OS_U32*)(0x01C11110u))               // PLLC0 PLL Multiplier Control Register Section 7.3.9
#define PLLC0_PREDIV                  (*(volatile OS_U32*)(0x01C11114u))               // PLLC0 Pre-Divider Control Register Section 7.3.10
#define PLLC0_PLLDIV1                 (*(volatile OS_U32*)(0x01C11118u))               // PLLC0 Divider 1 Register Section 7.3.11
#define PLLC0_PLLDIV2                 (*(volatile OS_U32*)(0x01C1111Cu))               // PLLC0 Divider 2 Register Section 7.3.13
#define PLLC0_PLLDIV3                 (*(volatile OS_U32*)(0x01C11120u))               // PLLC0 Divider 3 Register Section 7.3.15
#define PLLC0_OSCDIV                  (*(volatile OS_U32*)(0x01C11124u))               // PLLC0 Oscillator Divider 1 Register Section 7.3.21
#define PLLC0_POSTDIV                 (*(volatile OS_U32*)(0x01C11128u))               // PLLC0 PLL Post-Divider Control Register Section 7.3.23
#define PLLC0_PLLCMD                  (*(volatile OS_U32*)(0x01C11138u))               // PLLC0 PLL Controller Command Register Section 7.3.24
#define PLLC0_PLLSTAT                 (*(volatile OS_U32*)(0x01C1113Cu))               // PLLC0 PLL Controller Status Register Section 7.3.25
#define PLLC0_ALNCTL                  (*(volatile OS_U32*)(0x01C11140u))               // PLLC0 Clock Align Control Register Section 7.3.26
#define PLLC0_DCHANGE                 (*(volatile OS_U32*)(0x01C11144u))               // PLLC0 PLLDIV Ratio Change Status Register Section 7.3.28
#define PLLC0_CKEN                    (*(volatile OS_U32*)(0x01C11148u))               // PLLC0 Clock Enable Control Register Section 7.3.30
#define PLLC0_CKSTAT                  (*(volatile OS_U32*)(0x01C1114Cu))               // PLLC0 Clock Status Register Section 7.3.32
#define PLLC0_SYSTAT                  (*(volatile OS_U32*)(0x01C11150u))               // PLLC0 SYSCLK Status Register Section 7.3.34
#define PLLC0_PLLDIV4                 (*(volatile OS_U32*)(0x01C11160u))               // PLLC0 Divider 4 Register Section 7.3.17
#define PLLC0_PLLDIV5                 (*(volatile OS_U32*)(0x01C11164u))               // PLLC0 Divider 5 Register Section 7.3.18
#define PLLC0_PLLDIV6                 (*(volatile OS_U32*)(0x01C11168u))               // PLLC0 Divider 6 Register Section 7.3.19
#define PLLC0_PLLDIV7                 (*(volatile OS_U32*)(0x01C1116Cu))               // PLLC0 Divider 7 Register Section 7.3.20
#define PLLC0_EMUCNT0                 (*(volatile OS_U32*)(0x01C111F0u))               // PLLC0 Emulation Performance Counter 0 Register Section 7.3.36
#define PLLC0_EMUCNT1                 (*(volatile OS_U32*)(0x01C111F4u))               // PLLC0 Emulation Performance Counter 1 Register Section 7.3.37

#define PLLC1_REVID                   (*(volatile OS_U32*)(0x01E1A000u))               // PLLC1 Revision Identification Register Section 7.3.2
#define PLLC1_PLLCTL                  (*(volatile OS_U32*)(0x01E1A100u))               // PLLC1 Control Register Section 7.3.6
#define PLLC1_OCSEL                   (*(volatile OS_U32*)(0x01E1A104u))               // PLLC1 OBSCLK Select Register Section 7.3.8
#define PLLC1_PLLM                    (*(volatile OS_U32*)(0x01E1A110u))               // PLLC1 PLL Multiplier Control Register Section 7.3.9
#define PLLC1_PLLDIV1                 (*(volatile OS_U32*)(0x01E1A118u))               // PLLC1 Divider 1 Register Section 7.3.12
#define PLLC1_PLLDIV2                 (*(volatile OS_U32*)(0x01E1A11Cu))               // PLLC1 Divider 2 Register Section 7.3.14
#define PLLC1_PLLDIV3                 (*(volatile OS_U32*)(0x01E1A120u))               // PLLC1 Divider 3 Register Section 7.3.16
#define PLLC1_OSCDIV                  (*(volatile OS_U32*)(0x01E1A124u))               // PLLC1 Oscillator Divider 1 Register Section 7.3.22
#define PLLC1_POSTDIV                 (*(volatile OS_U32*)(0x01E1A128u))               // PLLC1 PLL Post-Divider Control Register Section 7.3.23
#define PLLC1_PLLCMD                  (*(volatile OS_U32*)(0x01E1A138u))               // PLLC1 PLL Controller Command Register Section 7.3.24
#define PLLC1_PLLSTAT                 (*(volatile OS_U32*)(0x01E1A13Cu))               // PLLC1 PLL Controller Status Register Section 7.3.25
#define PLLC1_ALNCTL                  (*(volatile OS_U32*)(0x01E1A140u))               // PLLC1 Clock Align Control Register Section 7.3.27
#define PLLC1_DCHANGE                 (*(volatile OS_U32*)(0x01E1A144u))               // PLLC1 PLLDIV Ratio Change Status Register Section 7.3.29
#define PLLC1_CKEN                    (*(volatile OS_U32*)(0x01E1A148u))               // PLLC1 Clock Enable Control Register Section 7.3.31
#define PLLC1_CKSTAT                  (*(volatile OS_U32*)(0x01E1A14Cu))               // PLLC1 Clock Status Register Section 7.3.33
#define PLLC1_SYSTAT                  (*(volatile OS_U32*)(0x01E1A150u))               // PLLC1 SYSCLK Status Register Section 7.3.35
#define PLLC1_EMUCNT0                 (*(volatile OS_U32*)(0x01E1A1F0u))               // PLLC1 Emulation Performance Counter 0 Register Section 7.3.36
#define PLLC1_EMUCNT1                 (*(volatile OS_U32*)(0x01E1A1F4u))               // PLLC1 Emulation Performance Counter 1 Register Section 7.3.37
//
// Fixing PLL0 register values, 456 MHz
//
#define PLL_CLK_SRC                             ( 0u)
#define PLL0_MUL                                (15u)
#define PLL0_PREDIV                             ( 0u)
#define PLL0_POSTDIV                            ( 0u)
#define PLL0_DIV1                               ( 0u)
#define PLL0_DIV3                               ( 2u)
#define PLL0_DIV7                               ( 7u)
//
// Fixing PLL1 register values
//
#define PLL1_MUL                                (21u)
#define PLL1_POSTDIV                            ( 1u)
#define PLL1_DIV1                               ( 0u)
#define PLL1_DIV2                               ( 1u)
#define PLL1_DIV3                               ( 2u)

#define PLLC_PLLCTL_EXTCLKSRC                   (0x00000200u)
#define PLLC_PLLCTL_EXTCLKSRC_SHIFT             (0x00000009u)
#define PLLC_PLLCTL_CLKMODE                     (0x00000100u)
#define PLLC_PLLCTL_CLKMODE_SHIFT               (0x00000008u)
#define PLLC_PLLCTL_PLLENSRC                    (0x00000020u)
#define PLLC_PLLCTL_PLLENSRC_SHIFT              (0x00000005u)
#define PLLC_PLLCTL_PLLDIS                      (0x00000010u)
#define PLLC_PLLCTL_PLLDIS_SHIFT                (0x00000004u)
#define PLLC_PLLCTL_PLLRST                      (0x00000008u)
#define PLLC_PLLCTL_PLLRST_SHIFT                (0x00000003u)
#define PLLC_PLLCTL_PLLPWRDN                    (0x00000002u)
#define PLLC_PLLCTL_PLLPWRDN_SHIFT              (0x00000001u)
#define PLLC_PLLCTL_PLLEN                       (0x00000001u)
#define PLLC_PLLCTL_PLLEN_SHIFT                 (0x00000000u)
#define PLLC_PREDIV_PREDEN                      (0x00008000u)
#define PLLC_POSTDIV_POSTDEN                    (0x00008000u)
#define PLLC_PLLSTAT_GOSTAT                     (0x00000001u)
#define PLLC_PLLDIV1_D1EN                       (0x00008000u)
#define PLLC_PLLDIV2_D2EN                       (0x00008000u)
#define PLLC_PLLDIV4_D4EN                       (0x00008000u)
#define PLLC_PLLDIV6_D6EN                       (0x00008000u)
#define PLLC_PLLDIV3_D3EN                       (0x00008000u)
#define PLLC_PLLDIV7_D7EN                       (0x00008000u)
#define PLLC_PLLCMD_GOSET                       (0x00000001u)
#define SYSCFG_CFGCHIP0_PLL_MASTER_LOCK         (0x00000010u)
#define SYSCFG_CFGCHIP0_PLL_MASTER_LOCK_SHIFT   (0x00000004u)
#define CLK_PLL0_SYSCLK3                        (0xFFFFFFF8u)
#define SYSCFG_CFGCHIP3_PLL1_MASTER_LOCK        (0x00000020u)
#define SYSCFG_CFGCHIP3_PLL1_MASTER_LOCK_SHIFT  (0x00000005u)
//
// System
//
#define SYSCFG0_KICK0R                          (*(volatile OS_U32*)(0x01C14038u))     // Kick 0 Register
#define SYSCFG0_KICK1R                          (*(volatile OS_U32*)(0x01C1403Cu))     // Kick 1 Register
#define SYSCFG0_CFGCHIP0                        (*(volatile OS_U32*)(0x01C1417Cu))     // Chip Configuration 0 Register
#define SYSCFG0_CFGCHIP3                        (*(volatile OS_U32*)(0x01C14188u))     // Chip Configuration 3 Register
#define SYSCFG_KICK0R_UNLOCK                    (0x83E70B13u)                          // Unlock/Lock code for KICK0
#define SYSCFG_KICK1R_UNLOCK                    (0x95A4F1E0u)                          // Unlock/Lock code for KICK1

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       MMU and cache configuration
*
*  Notes
*    The translation table has to be aligned on 16384 bytes boundary
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=16384
  static __no_init OS_U32 _TranslationTable[0x1000];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(16384) static OS_U32 _TranslationTable[0x1000];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_U32 _TranslationTable[0x1000] __attribute__ ((aligned (16384), section (".no_init")));
#endif

/*********************************************************************
*
*       Interrupt vector table
*
*  Notes
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=4
  static __no_init OS_ISR_HANDLER* _apIsrHandler[NUM_INT_VECTORS];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(4) static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_VECTORS];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_VECTORS] __attribute__ ((aligned (4), section (".no_init")));
#endif

static OS_U8 _aIntChannelPrio[NUM_INT_VECTORS]; // IRQ table for channel priorities

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif


/*********************************************************************
*
*       _InitAINTC()
*
*  Function description
*    Initialize interrupt controller
*/
static void _InitAINTC(void) {
  AINTC_GER   = 0;            // Disable interrupt controller
  AINTC_ECR1  = 0xFFFFFFFFu;  // Disable all interrupts
  AINTC_ECR2  = 0xFFFFFFFFu;
  AINTC_ECR3  = 0xFFFFFFFFu;
  AINTC_ECR4  = 0x1Fu;
  AINTC_SECR1 = 0xFFFFFFFFu;  // Clear all pending interrupts
  AINTC_SECR2 = 0xFFFFFFFFu;
  AINTC_SECR3 = 0xFFFFFFFFu;
  AINTC_SECR4 = 0x1Fu;
  AINTC_HIER  = 0x02u;        // Enable IRQ
  AINTC_VSR   = 0;            // Vector table ntry is 32bit
  AINTC_GER   = 0x01u;        // Enable interrupt controller

#if (ALLOW_NESTED_INTERRUPTS != 0)
  AINTC_CR = AINTC_CR_NESTMODE_GLOBAL << AINTC_CR_NESTMODE_SHIFT;
#else
  AINTC_CR = AINTC_CR_NESTMODE_NONEST << AINTC_CR_NESTMODE_SHIFT;
#endif
}

#if (OS_INIT_PLL != 0)
/*********************************************************************
*
*       _Delay()
*
*  Function description
*    Wait function for short delays
*/
static void _Delay(volatile unsigned int delay) {
  while (delay--) {
    ;
  }
}

/*********************************************************************
*
*       _PLL0_Init()
*
*  Function description
*    Initialize PLL0
*/
static void _PLL0_Init(unsigned char clk_src, unsigned char pllm, unsigned char prediv, unsigned char postdiv, unsigned char div1, unsigned char div3, unsigned char div7) {
  SYSCFG0_CFGCHIP0 &= ~SYSCFG_CFGCHIP0_PLL_MASTER_LOCK;
  PLLC0_PLLCTL     &= ~PLLC_PLLCTL_PLLENSRC;        // PLLENSRC must be cleared before PLLEN bit have any effect
  PLLC0_PLLCTL     &= ~PLLC_PLLCTL_EXTCLKSRC;       // PLLCTL.EXTCLKSRC bit 9 should be left at 0
  PLLC0_PLLCTL     &= ~PLLC_PLLCTL_PLLEN;           // PLLEN = 0 put pll in bypass mode
  _Delay(2);                                        // Wait for 2 counts to switch pll to the by pass mode
  PLLC0_PLLCTL     &= ~PLLC_PLLCTL_CLKMODE;         // Select the Clock Mode bit 8 as External Clock or On Chip Oscilator
  PLLC0_PLLCTL     |= (clk_src << 8);
  PLLC0_PLLCTL     &= ~PLLC_PLLCTL_PLLRST;          // Clear the PLLRST to reset the PLL
  PLLC0_PLLCTL     |= PLLC_PLLCTL_PLLDIS;           // Disable PLL out
  PLLC0_PLLCTL     &= ~PLLC_PLLCTL_PLLPWRDN;        // PLL initialization sequece, power up the PLL
  PLLC0_PLLCTL     &= ~PLLC_PLLCTL_PLLDIS;
  _Delay(300);                                      // Wait for 300 counts
  PLLC0_PLLM        = pllm;
  PLLC0_PREDIV      = PLLC_PREDIV_PREDEN | prediv;  // Program the required multiplier value
  PLLC0_POSTDIV     = PLLC_POSTDIV_POSTDEN | postdiv;
  while (PLLC0_PLLSTAT & PLLC_PLLSTAT_GOSTAT) {     // Check for the GOSTAT bit in PLLSTAT to clear to 0 to indicate that no GO operation is currently in progress
    ;
  }
  //
  // Divider values are assigned
  //
  PLLC0_PLLDIV1 = PLLC_PLLDIV1_D1EN | div1;
  PLLC0_PLLDIV2 = PLLC_PLLDIV2_D2EN | (((div1 + 1) * 2) - 1);
  PLLC0_PLLDIV4 = PLLC_PLLDIV4_D4EN | (((div1 + 1) * 4) - 1);
  PLLC0_PLLDIV6 = PLLC_PLLDIV6_D6EN | div1;
  PLLC0_PLLDIV3 = PLLC_PLLDIV3_D3EN | div3;
  PLLC0_PLLDIV7 = PLLC_PLLDIV7_D7EN | div7;
  PLLC0_PLLCMD |= PLLC_PLLCMD_GOSET;
  while (PLLC0_PLLSTAT & PLLC_PLLSTAT_GOSTAT) {  // Wait for the Gostat bit in PLLSTAT to clear to 0 ( completion of phase alignment)
    ;
  }
  _Delay(40);                                    // Wait for 40 counts
  PLLC0_PLLCTL     |= PLLC_PLLCTL_PLLRST;        // Set the PLLRST bit in PLLCTL to 1,bring the PLL out of reset
  _Delay(300);                                   // Wait for 300 counts
  PLLC0_PLLCTL     |= PLLC_PLLCTL_PLLEN;         // Removing pll from bypass mode
  SYSCFG0_CFGCHIP0 |= (1u << SYSCFG_CFGCHIP0_PLL_MASTER_LOCK_SHIFT) & SYSCFG_CFGCHIP0_PLL_MASTER_LOCK;  // Set PLL lock bit
  SYSCFG0_CFGCHIP3 &= CLK_PLL0_SYSCLK3;
}

/*********************************************************************
*
*       _PLL1_Init()
*
*  Function description
*    Initialize PLL1
*/
static void _PLL1_Init(unsigned char pllm, unsigned char postdiv, unsigned char div1, unsigned char div2, unsigned char div3) {
  SYSCFG0_CFGCHIP3 &= ~SYSCFG_CFGCHIP3_PLL1_MASTER_LOCK;  // Clear PLL lock bit
  PLLC1_PLLCTL     &= ~PLLC_PLLCTL_PLLENSRC;              // PLLENSRC must be cleared before PLLEN has any effect
  PLLC1_PLLCTL     &= ~PLLC_PLLCTL_EXTCLKSRC;             // PLLCTL.EXTCLKSRC bit 9 should be left at 0
  PLLC1_PLLCTL     &= ~PLLC_PLLCTL_PLLEN;                 // Set PLLEN=0 to put in bypass mode
  _Delay(4);                                              // wait for 4 cycles to allow PLLEN mux switches properly to bypass clock
  PLLC1_PLLCTL     &= ~PLLC_PLLCTL_PLLRST;                // Clear PLLRST bit to reset the PLL
  PLLC1_PLLCTL     &= PLLC_PLLCTL_PLLDIS;                 // Disable the PLL output
  //
  // PLL initialization sequence
  //
  PLLC1_PLLCTL     &= ~PLLC_PLLCTL_PLLPWRDN ;             // Power up the PLL by setting PWRDN bit set to 0
  PLLC1_PLLCTL     &= ~PLLC_PLLCTL_PLLDIS;                // Enable the PLL output
  _Delay(300);
  PLLC1_PLLM        = pllm;                               // Multiplier value is set
  PLLC1_POSTDIV     = PLLC_POSTDIV_POSTDEN | postdiv;
  while (PLLC1_PLLSTAT & PLLC_PLLCMD_GOSET) {
    ;
  }
  PLLC1_PLLDIV1 = PLLC_PLLDIV1_D1EN | div1;
  PLLC1_PLLDIV2 = PLLC_PLLDIV2_D2EN | div2;
  PLLC1_PLLDIV3 = PLLC_PLLDIV3_D3EN | div3;
  PLLC1_PLLCMD |= PLLC_PLLCMD_GOSET;                      // Set the GOSET bit in PLLCMD to 1 to initiate a new divider transition
  while (PLLC1_PLLSTAT & PLLC_PLLSTAT_GOSTAT) {
    ;
  }
  _Delay(40);                                             // Wait for the Gostat bit in PLLSTAT to clear to 0 ( complition of phase alignment )
  PLLC1_PLLCTL     |= PLLC_PLLCTL_PLLRST;                 // set the PLLRST bit in PLLCTL to 1,bring the PLL out of reset
  _Delay(300);
  PLLC1_PLLCTL     |= PLLC_PLLCTL_PLLEN;                  // Removing PLL from bypass mode
  SYSCFG0_CFGCHIP3 |= (1u << SYSCFG_CFGCHIP3_PLL1_MASTER_LOCK_SHIFT) & SYSCFG_CFGCHIP3_PLL1_MASTER_LOCK;  // set PLL lock bit
}
#endif  // OS_INIT_PLL

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return TIMER0_TIM34;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (TIMER0_INTCTLSTAT & PRDINTSTAT12);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  TIMER0_INTCTLSTAT |= PRDINTSTAT12;  // Reset pending flag
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize interrupt vector controller
  //
  _InitAINTC();
  //
  // Initialize PLLs
  //
#if (OS_INIT_PLL == 1)
  _PLL0_Init(PLL_CLK_SRC, PLL0_MUL, PLL0_PREDIV, PLL0_POSTDIV, PLL0_DIV1,PLL0_DIV3, PLL0_DIV7);
  _PLL1_Init(PLL1_MUL, PLL1_POSTDIV, PLL1_DIV1,PLL1_DIV2, PLL1_DIV3);
#endif
  //
  // Init timer
  //
  TIMER0_TGCR        = 0x00000000u;                        // Stop and reset timer.
  TIMER0_TCR         = 0x00000000u;
  TIMER0_EMUMGT      = EMUMGT_SOFT | EMUMGT_FREE;          // Free running timer
  TIMER0_TGCR        = TIMMODE_32BIT_UNCHAINED | TIM12RS;  // Config timer0 in 32-bit unchained mode.
  TIMER0_TIM12       = 0x00000000u;
  TIMER0_PRD12       = OS_TIMER_RELOAD;
  TIMER0_INTCTLSTAT |= PRDINTEN12;                         // Enable interrupts
  TIMER0_TCR        |= ENAMODE12_CONT;                     // Start timer
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler (TIMER0_IRQ_ID, (OS_ISR_HANDLER*)_OS_ISR_Tick);
  OS_ARM_ISRSetPrio(TIMER0_IRQ_ID, TIMER1_IRQ_PRIO);
  OS_ARM_EnableISR(TIMER0_IRQ_ID);
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  OS_U32          ActiveInterrupt;
  OS_U32          GlobalNestLevel;
  OS_U32          Index;
  OS_U32*         pSECR;

  //
  // Read active interrupt register for later branch to Irq service routine
  //
  ActiveInterrupt = AINTC_GPIR;               // Get active interrupt index
  GlobalNestLevel = AINTC_GNLR;               // Backup global nest level
  AINTC_GNLR      = 0x80000000u | _aIntChannelPrio[ActiveInterrupt];
  //
  // Clear interrupt pending bit. Interrupts are pending on pulses.
  // Therefore we have to clear the pending interrupt before executing
  // the Irq service routine as an immediate new pulse might result
  // in a lost interrupt.
  //
  Index   = ActiveInterrupt / 32u;
  pSECR   = (OS_U32*)AINTC_SECR1_ADDR;
  pSECR  += Index;
  *pSECR |= (1u << (ActiveInterrupt % 32u));
  OS_INT_Enter();
  //
  // Check if an interrupt is pending
  //
  if ((ActiveInterrupt & (1u << 31)) == 0u) {
    //
    // Branch to Irq service routine
    //
    pISR = (OS_ISR_HANDLER*)_apIsrHandler[ActiveInterrupt];
    if (pISR != NULL) {
      pISR();
    } else {
      #if (defined(DEBUG) || defined(OS_DEBUG))
        #if (OS_DEBUG != 0)
          OS_Error(OS_ERR_ISR_INDEX);
        #else
          while (1) {
            ;
          }
        #endif
      #endif
    }
  }
  OS_INT_Leave();
  AINTC_GNLR = 0x80000000 | GlobalNestLevel;  // Restore global nest level
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  papISR               = (OS_ISR_HANDLER**)_apIsrHandler;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  AINTC_EISR = ISRIndex;
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  AINTC_EICR = ISRIndex;
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U8* pPrio;
  int    OldPrio;
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pPrio                      = (OS_U8*)AINTC_CMR0_ADDR;
  OldPrio                    = pPrio[ISRIndex];
  pPrio[ISRIndex]            = Prio;
  _aIntChannelPrio[ISRIndex] = Prio;
  OS_INT_EnableConditional();
  return OldPrio;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
}
#endif
OS_INTERWORK int __low_level_init(void) {
  //
  // Disable watchdog
  //
  TIMER1_WDTCR = 0xA5C60000u;
  TIMER1_WDTCR = 0xDA7E0000u;
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Memory mapping should be complete, meaning 4096 entries.
  // Code below fills in ascending VAddr order
  //
  OS_ARM_MMU_InitTT(_TranslationTable);
  //                                         Mode                    VAddr  PAddr  Size[MB]
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0x000, 0x001, 0x001);  // PRUSS local RAM
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,    0x001, 0x001, 0x01A);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0x01B, 0x01B, 0x001);  // ETB RAM and control registers
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0x01C, 0x01C, 0x0E4);  // SFRs, non cacheable, non bufferable
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0x100, 0x100, 0x300);  // SFRs, non cacheable, non bufferable
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_C_B,   0x400, 0x400, 0x200);  // External memory, CS0
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0x600, 0x600, 0x200);  // External memory CS2, CS3, CS4, CS5
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_C_B,   0x800, 0x800, 0x001);  // Internal SRAM (128KByte)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0x801, 0x800, 0x001);  // Internal SRAM (128KByte)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,    0x802, 0x802, 0x1FE);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,    0x900, 0x900, 0x200);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0xB00, 0xB00, 0x100);  // DDR/mDDR controller
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_C_B,   0xC00, 0xC00, 0x080);  // DDR2 data, 128 MByte
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0xC80, 0xC00, 0x080);  // DDR2 data, 128 MByte, mirrored from 0xC0000000
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,    0xD00, 0xD00, 0x2FF);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_NC_NB, 0xFFF, 0xFFF, 0x001);  // ARM local RAM (8KByte) and ARM interrupt controller
  OS_ARM_MMU_Enable      (_TranslationTable);
  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();
  return 1;  // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
