/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit_AM3359.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#ifndef   IRAM
  #define IRAM  (0)
#endif

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ  (24000000u)
#define OS_TICK_FREQ   (1000u)
#define OS_INT_FREQ    (OS_TICK_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Device specific SFRs
*/
//
// Interrupt Vector Controller
//
#define MPUINTC_BASEADDR                     (0x48200000u)
#define INTCPS_SYSCONFIG                     (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000010u))
#define INTCPS_SYSSTATUS                     (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000014u))
#define INTCPS_SIR_IRQ                       (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000040u))
#define INTCPS_CONTROL                       (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000048u))
#define INTCPS_MIR0_CLR                      (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000088u))
#define INTCPS_MIR1_CLR                      (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000A8u))
#define INTCPS_MIR2_CLR                      (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000C8u))
#define INTCPS_MIR3_CLR                      (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000E8u))
#define INTCPS_MIR0_SET                      (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x0000008Cu))
#define INTCPS_MIR1_SET                      (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000ACu))
#define INTCPS_MIR2_SET                      (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000CCu))
#define INTCPS_MIR3_SET                      (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000ECu))
#define INTCPS_ILR_ADDR                                          (MPUINTC_BASEADDR + 0x00000100u)

#define NUM_INT_SOURCES                      (128u)  // index 126 and 127 are reserved

#define TIMER1_IRQ_ID                        (67u)
#define TIMER1_IRQ_PRIO                      (62u)

//
// DMTIMER_1MS
//
#define DMTIMER_1MS_BASEADDR                 (0x44E31000u)
#define DMTIMER_1MS_TISR                     (*(volatile OS_U32*)(DMTIMER_1MS_BASEADDR + 0x018u))
#define DMTIMER_1MS_TIER                     (*(volatile OS_U32*)(DMTIMER_1MS_BASEADDR + 0x01Cu))
#define DMTIMER_1MS_TCLR                     (*(volatile OS_U32*)(DMTIMER_1MS_BASEADDR + 0x024u))
#define DMTIMER_1MS_TCRR                     (*(volatile OS_U32*)(DMTIMER_1MS_BASEADDR + 0x028u))
#define DMTIMER_1MS_TLDR                     (*(volatile OS_U32*)(DMTIMER_1MS_BASEADDR + 0x02Cu))

#define OVF_IT_ENA                           (1u << 1)            // Overflow interrupt enable
#define OVF_IT_FLAG                          (1u << 1)            // Overflow interrupt flag

//
// Clock and power control, SDRAM
//
#define CM_CLKMODE_DPLL_CORE                 (*(volatile OS_U32*)(0x44E00490u))
#define CM_IDLEST_DPLL_CORE                  (*(volatile OS_U32*)(0x44E0045Cu))
#define CM_CLKSEL_DPLL_CORE                  (*(volatile OS_U32*)(0x44E00468u))
#define CM_DIV_M4_DPLL_CORE                  (*(volatile OS_U32*)(0x44E00480u))
#define CM_DIV_M5_DPLL_CORE                  (*(volatile OS_U32*)(0x44E00484u))
#define CM_DIV_M6_DPLL_CORE                  (*(volatile OS_U32*)(0x44E004D8u))
#define CM_CLKMODE_DPLL_MPU                  (*(volatile OS_U32*)(0x44E00488u))
#define CM_IDLEST_DPLL_MPU                   (*(volatile OS_U32*)(0x44E00420u))
#define CM_CLKSEL_DPLL_MPU                   (*(volatile OS_U32*)(0x44E0042Cu))
#define CM_DIV_M2_DPLL_MPU                   (*(volatile OS_U32*)(0x44E004A8u))
#define CM_CLKMODE_DPLL_PER                  (*(volatile OS_U32*)(0x44E0048Cu))
#define CM_IDLEST_DPLL_PER                   (*(volatile OS_U32*)(0x44E00470u))
#define CM_CLKSEL_DPLL_PERIPH                (*(volatile OS_U32*)(0x44E0049Cu))
#define CM_DIV_M2_DPLL_PER                   (*(volatile OS_U32*)(0x44E004ACu))
#define CM_CLKMODE_DPLL_DDR                  (*(volatile OS_U32*)(0x44E00494u))
#define CM_IDLEST_DPLL_DDR                   (*(volatile OS_U32*)(0x44E00434u))
#define CM_CLKSEL_DPLL_DDR                   (*(volatile OS_U32*)(0x44E00440u))
#define CM_DIV_M2_DPLL_DDR                   (*(volatile OS_U32*)(0x44E004A0u))
#define CM_PER_L3_CLKSTCTRL                  (*(volatile OS_U32*)(0x44E0000Cu))
#define CM_PER_L3_CLKCTRL                    (*(volatile OS_U32*)(0x44E000E0u))
#define CM_PER_EMIF_CLKCTRL                  (*(volatile OS_U32*)(0x44E00028u))
#define VTP_CTRL                             (*(volatile OS_U32*)(0x44E10E0Cu))
#define DDR_IO_CTRL                          (*(volatile OS_U32*)(0x44E10E04u))
#define DDR_CKE_CTRL                         (*(volatile OS_U32*)(0x44E1131Cu))
#define CMD0_REG_PHY_CTRL_SLAVE_RATIO_0      (*(volatile OS_U32*)(0x44E1201Cu))
#define CMD0_REG_PHY_DLL_LOCK_DIFF_0         (*(volatile OS_U32*)(0x44E12028u))
#define CMD0_REG_PHY_INVERT_CLKOUT_0         (*(volatile OS_U32*)(0x44E1202Cu))
#define CMD1_REG_PHY_CTRL_SLAVE_RATIO_0      (*(volatile OS_U32*)(0x44E12050u))
#define CMD1_REG_PHY_DLL_LOCK_DIFF_0         (*(volatile OS_U32*)(0x44E1205Cu))
#define CMD1_REG_PHY_INVERT_CLKOUT_0         (*(volatile OS_U32*)(0x44E12060u))
#define CMD2_REG_PHY_CTRL_SLAVE_RATIO_0      (*(volatile OS_U32*)(0x44E12084u))
#define CMD2_REG_PHY_DLL_LOCK_DIFF_0         (*(volatile OS_U32*)(0x44E12090u))
#define CMD2_REG_PHY_INVERT_CLKOUT_0         (*(volatile OS_U32*)(0x44E12094u))
#define DATA0_REG_PHY_RD_DQS_SLAVE_RATIO_0   (*(volatile OS_U32*)(0x44E120C8u))
#define DATA0_REG_PHY_WR_DQS_SLAVE_RATIO_0   (*(volatile OS_U32*)(0x44E120DCu))
#define DATA0_REG_PHY_WRLVL_INIT_RATIO_0     (*(volatile OS_U32*)(0x44E120F0u))
#define DATA0_REG_PHY_GATELVL_INIT_RATIO_0   (*(volatile OS_U32*)(0x44E120FCu))
#define DATA0_REG_PHY_FIFO_WE_SLAVE_RATIO_0  (*(volatile OS_U32*)(0x44E12108u))
#define DATA0_REG_PHY_WR_DATA_SLAVE_RATIO_0  (*(volatile OS_U32*)(0x44E12120u))
#define DATA0_REG_PHY_DLL_LOCK_DIFF_0        (*(volatile OS_U32*)(0x44E12138u))
#define DATA1_REG_PHY_RD_DQS_SLAVE_RATIO_0   (*(volatile OS_U32*)(0x44E1216Cu))
#define DATA1_REG_PHY_WR_DQS_SLAVE_RATIO_0   (*(volatile OS_U32*)(0x44E12180u))
#define DATA1_REG_PHY_WRLVL_INIT_RATIO_0     (*(volatile OS_U32*)(0x44E12194u))
#define DATA1_REG_PHY_GATELVL_INIT_RATIO_0   (*(volatile OS_U32*)(0x44E121A0u))
#define DATA1_REG_PHY_FIFO_WE_SLAVE_RATIO_0  (*(volatile OS_U32*)(0x44E121ACu))
#define DATA1_REG_PHY_WR_DATA_SLAVE_RATIO_0  (*(volatile OS_U32*)(0x44E121C4u))
#define DATA1_REG_PHY_DLL_LOCK_DIFF_0        (*(volatile OS_U32*)(0x44E121DCu))
#define DATA0_REG_PHY_USE_RANK0_DELAYS       (*(volatile OS_U32*)(0x44E12134u))
#define DDR_CMD0_IOCTRL                      (*(volatile OS_U32*)(0x44E11404u))
#define DDR_CMD1_IOCTRL                      (*(volatile OS_U32*)(0x44E11408u))
#define DDR_CMD2_IOCTRL                      (*(volatile OS_U32*)(0x44E1140Cu))
#define DDR_DATA0_IOCTRL                     (*(volatile OS_U32*)(0x44E11440u))
#define DDR_DATA1_IOCTRL                     (*(volatile OS_U32*)(0x44E11444u))
#define EMIF_DDR_PHY_CTRL_1                  (*(volatile OS_U32*)(0x4C0000E4u))
#define EMIF_DDR_PHY_CTRL_1_SHDW             (*(volatile OS_U32*)(0x4C0000E8u))
#define EMIF_DDR_PHY_CTRL_2                  (*(volatile OS_U32*)(0x4C0000ECu))
#define EMIF_SDRAM_TIM_1                     (*(volatile OS_U32*)(0x4C000018u))
#define EMIF_SDRAM_TIM_1_SHDW                (*(volatile OS_U32*)(0x4C00001Cu))
#define EMIF_SDRAM_TIM_2                     (*(volatile OS_U32*)(0x4C000020u))
#define EMIF_SDRAM_TIM_2_SHDW                (*(volatile OS_U32*)(0x4C000024u))
#define EMIF_SDRAM_TIM_3                     (*(volatile OS_U32*)(0x4C000028u))
#define EMIF_SDRAM_TIM_3_SHDW                (*(volatile OS_U32*)(0x4C00002Cu))
#define EMIF_SDRAM_CONFIG                    (*(volatile OS_U32*)(0x4C000008u))
#define EMIF_SDRAM_CONFIG_2                  (*(volatile OS_U32*)(0x4C00000Cu))
#define EMIF_SDRAM_REF_CTRL                  (*(volatile OS_U32*)(0x4C000010u))
#define EMIF_SDRAM_REF_CTRL_SHDW             (*(volatile OS_U32*)(0x4C000014u))
#define CM_WKUP_CLKSTCTRL                    (*(volatile OS_U32*)(0x44E00400u))
#define CM_WKUP_TIMER1_CLKCTRL               (*(volatile OS_U32*)(0x44E004C4u))

#define DPLL_SSC_EN                          (0x1000u)
#define DPLL_MULT                            ( 8u)
#define DPLL_DIV                             ( 0u)
#define DPLL_SD_DIV                          (24u)

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
extern unsigned int __vector;

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       MMU and cache configuration
*
*  Notes
*    The translation table has to be aligned on 16384 bytes boundary
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #if (IRAM != 0)
    #pragma location=0x4030C000           // Place at fixed address in iRAM, last 16KBytes, has to be reserved in the linker file
  #endif
  #pragma data_alignment=16384
  static __no_init OS_U32 _TranslationTable[0x1000];
#elif (defined(__CC_ARM))    // Keil
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(16384) static OS_U32 _TranslationTable[0x1000];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_U32 _TranslationTable[0x1000] __attribute__ ((aligned (16384), section (".non_init._TranslationTable")));
#endif

/*********************************************************************
*
*       Interrupt vector table
*
*  Notes
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=4
  static __no_init OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
#elif (defined(__CC_ARM))    // Keil
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(4) static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_SOURCES] __attribute__ ((aligned (4), section (".non_init._apIsrHandler")));
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USE_PARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USE_PARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USE_PARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USE_PARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when a non-installed interrupt was detected.
*    As interrupt pending condition of peripherals has to be reset,
*    program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(void) {
  volatile int Dummy;

  Dummy = 1;
  while (Dummy > 0) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
}

/*********************************************************************
*
*       _InitINTC()
*
*  Function description
*    Initialize interrupt controller by setting default vectors
*    and clearing all interrupts
*/
static void _InitINTC(void) {
  unsigned int i;

  INTCPS_SYSCONFIG = 2u;  // Reset interrupt controller;
  while ((INTCPS_SYSSTATUS & 1u) == 0) {
    ;                     // Wait until reset is finished
  }
  OS_ARM_SetVBAR((OS_U32)&__vector);
  for (i = 0; i < NUM_INT_SOURCES; i++) {
    _apIsrHandler[i] = (OS_ISR_HANDLER*)_OS_ISR_Undefined;
  }
}

#if (IRAM != 0)
/*********************************************************************
*
*       _InitPLL()
*
*  Function description
*    Initialize main clock and PLLs
*
*  Notes
*    (1) Inits Core DPLL. CLKDCOLDO      2000MHz
*                         CORE_CLKOUTM6   500MHz
*                         CORE_CLKOUTM4   200MHz
*                         CORE_CLKOUTM5   250MHz
*        Inits MPU DPLL.  MPU Clock       720MHz
*        Inits PER DPLL.  CLKDCOLDO       960MHz
*                         CLKOUT          192MHz
*        Inits DDR DPLL.  DDR Clock       266MHz
*/
static void _InitPLL(void) {
  //
  // Initialize core DPLL
  //
  CM_CLKMODE_DPLL_CORE  =  0x04u;                  // CORE_DPLL in baypass
  while (((CM_IDLEST_DPLL_CORE & 0x01u) == 0x01u) || ((CM_IDLEST_DPLL_CORE & 0x100u) == 0x00u)) {
    ;  // Wait DPLL enter bypass mode
  }
  CM_CLKMODE_DPLL_CORE &= ~DPLL_SSC_EN;            // disable Spread Spectrum Clocking
  CM_CLKSEL_DPLL_CORE   =  0x00u;                  // Set DPLL multiplier factor *1000
  CM_CLKSEL_DPLL_CORE  |= (1000u    << DPLL_MULT);
  CM_CLKSEL_DPLL_CORE  |= ((24u-1u) << DPLL_DIV);  // Set DPLL divider factor
  CM_DIV_M4_DPLL_CORE  &= ~0x1Fu;                  // Set DPLL post-divider factor M4
  CM_DIV_M4_DPLL_CORE  |=  0x0Au;
  CM_DIV_M5_DPLL_CORE  &= ~0x1Fu;                  // Set DPLL post-divider factor M5
  CM_DIV_M5_DPLL_CORE  |=  0x08u;
  CM_DIV_M6_DPLL_CORE  &= ~0x1Fu;                  // Set DPLL post-divider factor M5
  CM_DIV_M6_DPLL_CORE  |=  0x04u;
  CM_CLKMODE_DPLL_CORE  =  0x07u;                  // CORE_DPLL in baypass
  while (((CM_IDLEST_DPLL_CORE & 1u) == 0) || ((CM_IDLEST_DPLL_CORE & 0x100u) == 0x100u)) {
    ;  // Wait DPLL locks
  }
  //
  // Initialize MPU DPPL
  //
  CM_CLKMODE_DPLL_MPU  =  0x04u;                    // MPU_DPLL in baypass
  while (((CM_IDLEST_DPLL_MPU & 0x01u) == 0x01u) || ((CM_IDLEST_DPLL_MPU & 0x100u) == 0x00u)) {
    ;  // Wait DPLL enter bypass mode
  }
  CM_CLKMODE_DPLL_MPU &= ~DPLL_SSC_EN;              // disable Spread Spectrum Clocking
  CM_CLKSEL_DPLL_MPU   =  0x00u;                    // Set DPLL multiplier factor *720
  CM_CLKSEL_DPLL_MPU  |= (720u       << DPLL_MULT);
  CM_CLKSEL_DPLL_MPU  |= ((24u - 1u) << DPLL_DIV);  // Set DPLL divider factor
  CM_DIV_M2_DPLL_MPU  &= ~0x1Fu;                    // Set DPLL post-divider factor M2
  CM_DIV_M2_DPLL_MPU  |=  0x01u;
  CM_CLKMODE_DPLL_MPU  =  0x07u;                    // MPU_DPLL in baypass
  while (((CM_IDLEST_DPLL_MPU & 1u) == 0) || ((CM_IDLEST_DPLL_MPU & 0x100u) == 0x100u)) {
    ;  // Wait DPLL locks
  }
  //
  // Initialize peripheral DPPL
  //
  CM_CLKMODE_DPLL_PER    = 0x04u;                      // MPU_DPLL in baypass
  while (((CM_IDLEST_DPLL_PER & 0x01u) == 0x01u) || ((CM_IDLEST_DPLL_PER & 0x100u) == 0x00u)) {
    ;  // Wait DPLL enter bypass mode
  }
  CM_CLKMODE_DPLL_PER   &= ~DPLL_SSC_EN;               // disable Spread Spectrum Clocking
  CM_CLKSEL_DPLL_PERIPH  =  0x00u;                     // Set DPLL multiplier factor *960
  CM_CLKSEL_DPLL_PERIPH |= (960u     << DPLL_MULT);
  CM_CLKSEL_DPLL_PERIPH |= ((24 - 1) << DPLL_DIV);     // Set DPLL divider factor
  CM_CLKSEL_DPLL_PERIPH |= (4u       << DPLL_SD_DIV);  // Set DPLL Sigma-Delta divider
  CM_DIV_M2_DPLL_PER    &= ~0x7Fu;                     // Set DPLL post-divider factor M2
  CM_DIV_M2_DPLL_PER    |=  0x05u;
  CM_CLKMODE_DPLL_PER    =  0x07u;                     // MPU_DPLL in baypass
  while (((CM_IDLEST_DPLL_PER & 1u) == 0) || ((CM_IDLEST_DPLL_PER & 0x100u) == 0x100u)) {
    ;  // Wait DPLL locks
  }
  //
  // Initialize DDR DPLL
  //
  CM_CLKMODE_DPLL_DDR  =  0x04u;                  // DDR DPLL in baypass
  while (((CM_IDLEST_DPLL_DDR & 0x01u) == 0x01u) || ((CM_IDLEST_DPLL_DDR & 0x100u) == 0x00u)) {
    ;  // Wait DPLL enter bypass mode
  }
  CM_CLKMODE_DPLL_DDR &= ~DPLL_SSC_EN;            // disable Spread Spectrum Clocking
  CM_CLKSEL_DPLL_DDR   =  0x00u;                  // Set DPLL multiplier factor *266
  CM_CLKSEL_DPLL_DDR  |= (266u     << DPLL_MULT);
  CM_CLKSEL_DPLL_DDR  |= ((24 - 1) << DPLL_DIV);  // Set DPLL divider factor
  CM_DIV_M2_DPLL_MPU  &= ~0x1Fu;                  // Set DPLL post-divider factor M2
  CM_DIV_M2_DPLL_MPU  |=  0x01u;
  CM_CLKMODE_DPLL_DDR  =  0x07u;                  // MPU_DPLL in baypass
  while (((CM_IDLEST_DPLL_DDR & 1u) == 0) || ((CM_IDLEST_DPLL_DDR & 0x100u) == 0x100u)) {
    ;  // Wait DPLL locks
  }
}

/*********************************************************************
*
*       _InitDDR()
*
*  Function description
*    Initialize DDR controller
*
*/
static void _InitDDR(void) {
  volatile OS_U32 i;

  CM_PER_L3_CLKSTCTRL = 2u;  // Enable Clocks
  CM_PER_L3_CLKCTRL   = 2u;
  CM_PER_EMIF_CLKCTRL = 2u;   // Enable Func Clock
  while ((CM_PER_EMIF_CLKCTRL & 0x30000u) != 0) {
    ;  // Wait clock to get active
  }
  //
  // VTP Init
  //
  VTP_CTRL |=  0x40u;  // Dynamic VTP compensation mode
  VTP_CTRL &= ~0x01u;  // clears flops, start count again
  VTP_CTRL |=  0x01u;
  while ((VTP_CTRL & 0x20u) == 0) {
    ;  // wait ready
  }
  //
  // DDR Phy CMD0
  //
  CMD0_REG_PHY_CTRL_SLAVE_RATIO_0 = 0x80u;
  CMD0_REG_PHY_DLL_LOCK_DIFF_0    = 0x04u;
  CMD0_REG_PHY_INVERT_CLKOUT_0    = 0x00u;
  //
  // DDR Phy CMD1
  //
  CMD1_REG_PHY_CTRL_SLAVE_RATIO_0 = 0x80u;
  CMD1_REG_PHY_DLL_LOCK_DIFF_0    = 0x04u;
  CMD1_REG_PHY_INVERT_CLKOUT_0    = 0x00u;
  //
  // DDR Phy CMD2
  //
  CMD2_REG_PHY_CTRL_SLAVE_RATIO_0 = 0x80u;
  CMD2_REG_PHY_DLL_LOCK_DIFF_0    = 0x04u;
  CMD2_REG_PHY_INVERT_CLKOUT_0    = 0x00u;
  //
  //Init Data Macro 0
  //
  DATA0_REG_PHY_RD_DQS_SLAVE_RATIO_0  = (0x40u << 10) | (0x40u << 0);
  DATA0_REG_PHY_WR_DQS_SLAVE_RATIO_0  = (0x00u << 10) | (0x00u << 0);
  DATA0_REG_PHY_GATELVL_INIT_RATIO_0  = (0x00u << 10) | (0x00u << 0);
  DATA0_REG_PHY_WRLVL_INIT_RATIO_0    = (0x00u << 10) | (0x00u << 0);
  DATA0_REG_PHY_FIFO_WE_SLAVE_RATIO_0 = (0x56u << 10) | (0x56u << 0);
  DATA0_REG_PHY_WR_DATA_SLAVE_RATIO_0 = (0x40u << 10) | (0x40u << 0);
  DATA0_REG_PHY_DLL_LOCK_DIFF_0       = 0;
  //
  //Init Data Macro 1
  //
  DATA1_REG_PHY_RD_DQS_SLAVE_RATIO_0  = (0x40u << 10) | (0x40u << 0);
  DATA1_REG_PHY_WR_DQS_SLAVE_RATIO_0  = (0x00u << 10) | (0x00u << 0);
  DATA1_REG_PHY_WRLVL_INIT_RATIO_0    = (0x00u << 10) | (0x00u << 0);
  DATA1_REG_PHY_GATELVL_INIT_RATIO_0  = (0x00u << 10) | (0x00u << 0);
  DATA1_REG_PHY_FIFO_WE_SLAVE_RATIO_0 = (0x56u << 10) | (0x56u << 0);
  DATA1_REG_PHY_WR_DATA_SLAVE_RATIO_0 = (0x40u << 10) | (0x40u << 0);
  DATA1_REG_PHY_DLL_LOCK_DIFF_0       = 0;
  //
  // Rank 0 delays are used for all ranks. This must be set to 1 for DDR2 and mDDR
  //
  DATA0_REG_PHY_USE_RANK0_DELAYS = 1u;
  DATA0_REG_PHY_USE_RANK0_DELAYS = 1u;
  DDR_CMD0_IOCTRL                = 0x18Bu;
  DDR_CMD1_IOCTRL                = 0x18Bu;
  DDR_CMD2_IOCTRL                = 0x18Bu;
  DDR_DATA0_IOCTRL               = 0x18Bu;
  DDR_DATA1_IOCTRL               = 0x18Bu;
  DDR_IO_CTRL                   &= ~0x80000000u;
  DDR_IO_CTRL                   &= ~0x40000000u;
  DDR_IO_CTRL                   &= ~0x10000000u;
  DDR_CKE_CTRL                  |= 0x1u;
  EMIF_DDR_PHY_CTRL_1            = 0x5u;
  EMIF_DDR_PHY_CTRL_1_SHDW       = 0x5u;
  EMIF_DDR_PHY_CTRL_2            = 0x5u;
  EMIF_SDRAM_TIM_1               = 0x0666B3D6u;
  EMIF_SDRAM_TIM_1_SHDW          = 0x0666B3D6u;
  EMIF_SDRAM_TIM_2               = 0x143731DAu;
  EMIF_SDRAM_TIM_2_SHDW          = 0x143731DAu;
  EMIF_SDRAM_TIM_3               = 0x00000347u;
  EMIF_SDRAM_TIM_3_SHDW          = 0x00000347u;
  EMIF_SDRAM_CONFIG              = 0x43805332u;
  EMIF_SDRAM_CONFIG_2            = 0x43805332u;
  EMIF_SDRAM_REF_CTRL            = 0x00004650u;
  EMIF_SDRAM_REF_CTRL_SHDW       = 0x00004650u;
  for(i = 0; i <= 5000 ; i++) {
    ;
  }
  EMIF_SDRAM_REF_CTRL      = 0x0000081au;
  EMIF_SDRAM_REF_CTRL_SHDW = 0x0000081au;
  EMIF_SDRAM_CONFIG        = 0x43805332u;
  EMIF_SDRAM_CONFIG_2      = 0x43805332u;
}
#endif

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return DMTIMER_1MS_TCRR - (0xFFFFFFFFu - (OS_TIMER_FREQ / OS_INT_FREQ) + 1);
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return ((DMTIMER_1MS_TISR & OVF_IT_FLAG) != 0);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  DMTIMER_1MS_TISR |= OVF_IT_FLAG;  // Reset pending flag
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
void OS_InitHW(void) {
  volatile int i;

  OS_INT_IncDI();
  //
  // Ensure caches are synchronized
  //
  OS_ARM_CACHE_Sync();
  //
  // Initialize interrupt controller
  //
  _InitINTC();
#if (IRAM != 0)
  //
  // Initialize PLLs and DDR RAM when running from internal iRAM
  //
  _InitPLL();
  _InitDDR();
#endif
  //
  // Enable clock and power for DMTIMER_1MS
  //
  CM_WKUP_CLKSTCTRL      &= ~0x03u;
  CM_WKUP_CLKSTCTRL      |=  0x02u;
  CM_WKUP_TIMER1_CLKCTRL &= ~0x03u;
  CM_WKUP_TIMER1_CLKCTRL |=  0x02u;
  for (i = 0; i < 0x10; i++) {
    ;  // Short delay before accessing timer register
  }
  //
  // Initialize timer for embOS
  //
  DMTIMER_1MS_TLDR  = 0xFFFFFFFFu - (OS_TIMER_FREQ / OS_INT_FREQ) + 1;  // Timer reload register
  DMTIMER_1MS_TCRR  = 0xFFFFFFFFu - (OS_TIMER_FREQ / OS_INT_FREQ) + 1;  // Timer value
  DMTIMER_1MS_TIER |= OVF_IT_ENA;                                        // Enable Overflow interrupt
  DMTIMER_1MS_TCLR  = (1u << 1)                                          // Auto reload mode
                    | (1u << 0);                                         // Start Timer
  OS_ARM_InstallISRHandler(TIMER1_IRQ_ID, (OS_ISR_HANDLER*)_OS_ISR_Tick);
  OS_ARM_ISRSetPrio(TIMER1_IRQ_ID, TIMER1_IRQ_PRIO);
  OS_ARM_EnableISR(TIMER1_IRQ_ID);
  //
  // Inform embOS about the timer settings
  //
  {
    OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                      // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  unsigned int    ActiveInterrupt;
  unsigned int    SpuriousInterrupt;

  OS_INT_Enter();
  //
  // Read active interrupt register and branch to Irq service routine
  //
  ActiveInterrupt   = INTCPS_SIR_IRQ & 0x0000007Fu;  // Get active IRQ number
  SpuriousInterrupt = INTCPS_SIR_IRQ & 0xFFFFFF80u;  // Get spurious IRQ flags
  //
  //  Only call ISR when this is not a spurious interrupt
  //
  if (SpuriousInterrupt == 0) {
    pISR = (OS_ISR_HANDLER*)_apIsrHandler[ActiveInterrupt];
    pISR();
  }
  OS_INT_Disable();                                  // Disable interrupts and unlock
  INTCPS_CONTROL = 1u;                               // Reset NVIC
  //
  // Data Synchronization Barrier
  //
  __asm volatile ("dsb");
  OS_INT_Leave();
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Installs an interrupt handler
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, usually the interrupt vector number
*    pISRHandler: Address of the interrupt handler function
*
*  Return value
*    The address of the interrupt handler that was previously installed with the addressed interrupt source
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  papISR               = (OS_ISR_HANDLER**)_apIsrHandler;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Enables an interrupt
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  if (ISRIndex < 32) {
    INTCPS_MIR0_CLR = (1u << (ISRIndex));
  } else if (ISRIndex < 64) {
    INTCPS_MIR1_CLR = (1u << (ISRIndex - 32));
  } else if (ISRIndex < 96) {
    INTCPS_MIR2_CLR = (1u << (ISRIndex - 64));
  } else {
    INTCPS_MIR3_CLR = (1u << (ISRIndex - 96));
  }
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Disables an interrupt
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  if (ISRIndex < 32) {
    INTCPS_MIR0_SET = (1u << (ISRIndex));
  } else if (ISRIndex < 64) {
    INTCPS_MIR1_SET = (1u << (ISRIndex - 32));
  } else if (ISRIndex < 96) {
    INTCPS_MIR2_SET = (1u << (ISRIndex - 64));
  } else {
    INTCPS_MIR3_SET = (1u << (ISRIndex - 96));
  }
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Sets the priority of an interrupt
*
*  Parameters
*    ISRIndex: Index of the interrupt source for which to configure the priority
*    Prio    : The priority which should be set for the specific interrupt
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pPrio           = (OS_U32*)INTCPS_ILR_ADDR;
  OldPrio         = pPrio[ISRIndex];
  pPrio[ISRIndex] = ((Prio & 0x3Fu) << 2);
  OS_INT_EnableConditional();
  return OldPrio;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL.
*
*    Has to be modified if another CPU clock frequency should be
*    used. This function is called during startup and has to
*    return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
OS_INTERWORK int __low_level_init(void);
OS_INTERWORK int __low_level_init(void) {
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Code below fills all 4096 entries in ascending VAddr order.
  //
  OS_ARM_MMU_InitTT(_TranslationTable);
  //                                         Mode                                Access permissions        VAddr  PAddr  Size[MB]
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x000, 0x000, 0x402);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_WRITE_BACK_ALLOC,                            0x402, 0x402, 0x001);  // internal SRAM (48kByte), 1 MB (program area)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_WRITE_BACK_ALLOC,                            0x403, 0x403, 0x001);  // internal SRAM (64kByte), 1 MB (program area)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x404, 0x404, 0x03C);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_STRONGLY_ORDERED | OS_ARM_MMU_EXECUTE_NEVER, 0x440, 0x440, 0x0C0);  // SFRs, non cacheable, non bufferable
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x500, 0x500, 0x180);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_STRONGLY_ORDERED,                            0x680, 0x680, 0x080);  // L3 Interconnect (128 MByte)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x700, 0x700, 0x100);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_WRITE_BACK_ALLOC,                            0x800, 0x800, 0x100);  // external SDRAM (256 MByte), 1 MB (program area)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_STRONGLY_ORDERED,                            0x900, 0x800, 0x100);  // external SDRAM (256 MByte), non cacheable, non bufferable
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0xA00, 0x900, 0x600);  // Unmapped area
  //
  // Now enable MMU, Caches and branch prediction unit
  //
  OS_ARM_MMU_Enable(_TranslationTable);
  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();
  return 1;
}

/*************************** End of file ****************************/
