/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for TI TMDSSK3358 evalboard
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define CM_PER_BASEADDR       (0x44E00000u)
#define CM_PER_GPIO1_CLKCTRL  (*(volatile unsigned int*)(CM_PER_BASEADDR + 0xACu)) // Clock for GPIO1

#define GPIO1_BASEADDR        (0x4804C000u)
#define GPIO1_OE              (*(volatile unsigned int*)(GPIO1_BASEADDR + 0x134u)) // Output Enable Register
#define GPIO1_DATAOUT         (*(volatile unsigned int*)(GPIO1_BASEADDR + 0x13Cu)) // Data Output Register
#define GPIO_CLRDATAOUT       (*(volatile unsigned int*)(GPIO1_BASEADDR + 0x190u)) // Clear Data Output Register
#define GPIO_SETDATAOUT       (*(volatile unsigned int*)(GPIO1_BASEADDR + 0x194u)) // Set Data Output Register

#define LED1_BIT              (1u << 4u)
#define LED2_BIT              (1u << 5u)
#define LED_ALL_BITS          (LED1_BIT | LED2_BIT)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  CM_PER_GPIO1_CLKCTRL = 0x00040002;
  GPIO1_OE        &= ~LED_ALL_BITS; // Set LED pins to output
  GPIO_CLRDATAOUT  = LED_ALL_BITS;  // Turn off all LEDs
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if ( Index < 2 ) {
    GPIO_SETDATAOUT = (1u << (Index + 4u));
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if ( Index < 2 ) {
    GPIO_CLRDATAOUT = (1u << (Index + 4u));
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if ((GPIO1_DATAOUT & (1u << (Index + 4u))) == 0) {
    GPIO_SETDATAOUT = (1u << (Index + 4u));
  } else {
    GPIO_CLRDATAOUT = (1u << (Index + 4u));
  }
}

/****** End Of File *************************************************/
