/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/

#ifndef   IRAM
  #define IRAM  (0)  // Run per default in external DDR RAM
#endif

/*********************************************************************
*
*       Clock settings
*/
#define OS_SYSTEM_FREQ   (600000000u)

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ    (26000000u)
#define OS_TICK_FREQ     (1000u)
#define OS_INT_FREQ      (OS_TICK_FREQ)
#define OS_TIMER_RELOAD  (OS_TIMER_FREQ / OS_INT_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
//
// Interrupt Vector Controller
//
#define MPUINTC_BASEADDR                   (0x48200000u)
#define INTCPS_SYSCONFIG                   (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000010u))
#define INTCPS_SYSSTATUS                   (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000014u))
#define INTCPS_SIR_IRQ                     (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000040u))
#define INTCPS_CONTROL                     (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000048u))
#define INTCPS_MIR0_CLR                    (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000088u))
#define INTCPS_MIR1_CLR                    (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000A8u))
#define INTCPS_MIR2_CLR                    (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000C8u))
#define INTCPS_MIR0_SET                    (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x0000008Cu))
#define INTCPS_MIR1_SET                    (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000ACu))
#define INTCPS_MIR2_SET                    (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000CCu))
#define INTCPS_ILR_ADDR                    (MPUINTC_BASEADDR + 0x00000100u)

#define NUM_INT_VECTORS                    (96u)

#define TIMER1_IRQ_ID                      (37u)
#define TIMER1_IRQ_PRIO                    (62u)

#define UART3_IRQ_ID                       (74u)
#define UART3_IRQ_PRIO                     (63u)
//
// GPTIMER1
//
#define GPTIMER1_BASEADDR                  (0x48318000u)
#define TISR                               (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x018u))
#define TIER                               (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x01Cu))
#define TCLR                               (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x024u))
#define TCRR                               (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x028u))
#define TLDR                               (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x02Cu))

#define LCR_CONF_MODE_A                    (0x80u)       // Switch device to configuration mode A
#define LCR_CONF_MODE_B                    (0xBFu)       // Switch device to configuration mode B
#define MDR1_MODE_SELECT_UART16            (0x00u)       // UART mode selection UART 16x mode
#define OVF_IT_ENA                         (1u <<  1)    // Overflow interrupt enable
#define OVF_IT_FLAG                        (1u <<  1)    // Overflow interrupt flag
#define FCR_FIFO_EN                        (1u <<  0)    // Set the FIFO enable bit, enables TX/RX FIFO
#define FCR_RX_FIFO_CLEAR                  (1u << 11)    // Clears the receive FIFO buffer
#define FCR_TX_FIFO_CLEAR                  (1u <<  2)    // Clears the transmit FIFO buffer
#define FCR_DMA_MODE                       (1u <<  3)    // Enables DMA mode
#define FCR_TXFIFO_TRIG_LSB(x)             (x  <<  4)    // Two LSB bits of transmit FIFO trigger level
#define FCR_RXFIFO_TRIG_LSB(x)             (x  <<  6)    // Two LSB bits of receive FIFO trigger level
#define TLR_RX_FIFO_TRIG_DMA(x)            (x  <<  4)    // Two LSB bits of receive FIFO trigger level
#define TLR_TX_FIFO_TRIG_DMA(x)            (x & 0x0Fu)   // Two LSB bits of transmit FIFO trigger level
#define MCR_TCR_TLR                        (6u)
#define SCR_TX_TRIG_GRANU1                 (6u)          // Set Granularity of TX FIFO trigger level to 1
#define SCR_RX_TRIG_GRANU1                 (7u)          // Set Granularity of RX FIFO trigger level to 1
#define IER_RHR_IT                         (0u)          // Enable transmit holding register interrupt
#define IER_THR_IT                         (1u)          // Enable transmit holding register interrupt
#define IIR_IT_TYPE_MASK                   (0x1Fu << 1)  // Mask Bit for reading out interrupt type
#define IIR_IT_TYPE_THR                    (0x01u << 1)  // Transmit holding register interrupt
#define IIR_IT_TYPE_RHR                    (0x02u << 1)  // Receive holding register interrupt
#define IIR_IT_TYPE_RXTIMEOUT              (0x06u << 1)  // Receive Timeout interrupt


#define OUT_REGS(offSet, value)            (*(volatile OS_U16*)(offSet) = (OS_U16)(value))
#define OUT_REGL(offSet, value)            (*(volatile OS_U32*)(offSet) = (OS_U32)(value))
#define IN_REGL(offSet)                    (*(volatile OS_U32*)(offSet))
#define SETBIT_REGL(dest, mask)            (OUT_REGL((dest), IN_REGL((dest)) | (mask)))
#define LOAD_PINMUX(x)                     do { _pin_config_mux((x), ARRAY_ENTRIES(x)); } while (0)
#define ARRAY_ENTRIES(x)                   (sizeof(x) / sizeof(x[0]))

#define DDR_MEM_BASE                       (0x80000000u)
#define DDR_MEM_SIZE                       (0x10000000u)  // 256 MB
#define DDR_MEM_END                        (DDR_MEM_BASE + DDR_MEM_SIZE)

#define PULL_ENABLE_MASK                   (0x08u)
#define PULL_UP_MASK                       (0x10u)

#define MODE0                              (00u)
#define MODE1                              (01u)
#define MODE2                              (02u)
#define MODE3                              (03u)
#define MODE4                              (04u)
#define MODE5                              (05u)
#define MODE6                              (06u)
#define MODE7                              (07u)
#define MODE_SAFE                          MODE7

#define PUPD_EN                            (01u)
#define PUPD_DIS                           (00u)

#define PULL_UP                            (01u)
#define PULL_DOWN                          (00u)
#define INPUT_EN                           (02u)
#define INPUT_DIS                          (00u)

#define EMIF_STROBE_ENABLE                 (0u)

//
// Pinmuxing registers
//
#define PAD_CONFIG_BASE                    (0x48002000u)
#define CONTROL_PADCONF_sys_nirq_HI        (PAD_CONFIG_BASE + 2 + 0x01E0u)
#define CONTROL_PADCONF_sys_off_mode_HI    (PAD_CONFIG_BASE + 2 + 0x0A18u)
#define CONTROL_PADCONF_gpmc_a2            (PAD_CONFIG_BASE + 0 + 0x007Cu)
#define CONTROL_PADCONF_gpmc_a2_HI         (PAD_CONFIG_BASE + 2 + 0x007Cu)
#define CONTROL_PADCONF_gpmc_a4            (PAD_CONFIG_BASE + 0 + 0x0080u)
#define CONTROL_PADCONF_gpmc_a4_HI         (PAD_CONFIG_BASE + 2 + 0x0080u)
#define CONTROL_PADCONF_gpmc_a6            (PAD_CONFIG_BASE + 0 + 0x0084u)
#define CONTROL_PADCONF_gpmc_a6_HI         (PAD_CONFIG_BASE + 2 + 0x0084u)
#define CONTROL_PADCONF_gpmc_a8            (PAD_CONFIG_BASE + 0 + 0x0088u)
#define CONTROL_PADCONF_gpmc_a8_HI         (PAD_CONFIG_BASE + 2 + 0x0088u)
#define CONTROL_PADCONF_gpmc_a10           (PAD_CONFIG_BASE + 0 + 0x008Cu)
#define CONTROL_PADCONF_gpmc_a10_HI        (PAD_CONFIG_BASE + 2 + 0x008Cu)
#define CONTROL_PADCONF_gpmc_d1            (PAD_CONFIG_BASE + 0 + 0x0090u)
#define CONTROL_PADCONF_gpmc_d1_HI         (PAD_CONFIG_BASE + 2 + 0x0090u)
#define CONTROL_PADCONF_gpmc_d3            (PAD_CONFIG_BASE + 0 + 0x0094u)
#define CONTROL_PADCONF_gpmc_d3_HI         (PAD_CONFIG_BASE + 2 + 0x0094u)
#define CONTROL_PADCONF_gpmc_d5            (PAD_CONFIG_BASE + 0 + 0x0098u)
#define CONTROL_PADCONF_gpmc_d5_HI         (PAD_CONFIG_BASE + 2 + 0x0098u)
#define CONTROL_PADCONF_gpmc_d7            (PAD_CONFIG_BASE + 0 + 0x009Cu)
#define CONTROL_PADCONF_gpmc_d7_HI         (PAD_CONFIG_BASE + 2 + 0x009Cu)
#define CONTROL_PADCONF_gpmc_d9            (PAD_CONFIG_BASE + 0 + 0x00A0u)
#define CONTROL_PADCONF_gpmc_d9_HI         (PAD_CONFIG_BASE + 2 + 0x00A0u)
#define CONTROL_PADCONF_gpmc_d11           (PAD_CONFIG_BASE + 0 + 0x00A4u)
#define CONTROL_PADCONF_gpmc_d11_HI        (PAD_CONFIG_BASE + 2 + 0x00A4u)
#define CONTROL_PADCONF_gpmc_d13           (PAD_CONFIG_BASE + 0 + 0x00A8u)
#define CONTROL_PADCONF_gpmc_d13_HI        (PAD_CONFIG_BASE + 2 + 0x00A8u)
#define CONTROL_PADCONF_gpmc_d15           (PAD_CONFIG_BASE + 0 + 0x00ACu)
#define CONTROL_PADCONF_gpmc_d15_HI        (PAD_CONFIG_BASE + 2 + 0x00ACu)
#define CONTROL_PADCONF_gpmc_ncs1          (PAD_CONFIG_BASE + 0 + 0x00B0u)
#define CONTROL_PADCONF_gpmc_ncs1_HI       (PAD_CONFIG_BASE + 2 + 0x00B0u)
#define CONTROL_PADCONF_gpmc_ncs3          (PAD_CONFIG_BASE + 0 + 0x00B4u)
#define CONTROL_PADCONF_gpmc_ncs3_HI       (PAD_CONFIG_BASE + 2 + 0x00B4u)
#define CONTROL_PADCONF_gpmc_ncs5          (PAD_CONFIG_BASE + 0 + 0x00B8u)
#define CONTROL_PADCONF_gpmc_ncs5_HI       (PAD_CONFIG_BASE + 2 + 0x00B8u)
#define CONTROL_PADCONF_gpmc_ncs7          (PAD_CONFIG_BASE + 0 + 0x00BCu)
#define CONTROL_PADCONF_gpmc_ncs7_HI       (PAD_CONFIG_BASE + 2 + 0x00BCu)
#define CONTROL_PADCONF_gpmc_nadv_ale      (PAD_CONFIG_BASE + 0 + 0x00C0u)
#define CONTROL_PADCONF_gpmc_nadv_ale_HI   (PAD_CONFIG_BASE + 2 + 0x00C0u)
#define CONTROL_PADCONF_gpmc_nwe           (PAD_CONFIG_BASE + 0 + 0x00C4u)
#define CONTROL_PADCONF_gpmc_nwe_HI        (PAD_CONFIG_BASE + 2 + 0x00C4u)
#define CONTROL_PADCONF_gpmc_nbe1          (PAD_CONFIG_BASE + 0 + 0x00C8u)
#define CONTROL_PADCONF_gpmc_nbe1_HI       (PAD_CONFIG_BASE + 2 + 0x00C8u)
#define CONTROL_PADCONF_gpmc_wait0         (PAD_CONFIG_BASE + 0 + 0x00CCu)
#define CONTROL_PADCONF_gpmc_wait0_HI      (PAD_CONFIG_BASE + 2 + 0x00CCu)
#define CONTROL_PADCONF_sad2d_sbusflag_HI  (PAD_CONFIG_BASE + 2 + 0x0260u)
#define CONTROL_PADCONF_sdrc_cke1          (PAD_CONFIG_BASE + 0 + 0x0264u)
#define CONTROL_PADCONF_sdrc_dqs3_HI       (PAD_CONFIG_BASE + 2 + 0x0078u)

//
// PRM registers
//
#define IVA2_PRM                           (0x48306000u)
#define OCP_SYSTEM_REGISTERS_PRM           (0x48306800u)
#define MPU_PRM                            (0x48306900u)
#define CORE_PRM                           (0x48306A00u)
#define GFX_PRM                            (0x48306B00u)
#define WKUP_PRM                           (0x48306C00u)
#define CLOCK_CONTROL_REGISTERS_PRM        (0x48306D00u)
#define DSS_PRM                            (0x48306E00u)
#define CAM_PRM                            (0x48306F00u)
#define PER_PRM                            (0x48307000u)
#define EMU_PRM                            (0x48307100u)
#define GLOBAL_REGISTERS_PRM               (0x48307200u)
#define NEON_PRM                           (0x48307300u)
#define USBHOST_PRM                        (0x48307400u)

#define CM_CLKSEL1_EMU                     (EMU_CM + 0x40u)

#define PRM_CLKOUT_CTRL                    (CLOCK_CONTROL_REGISTERS_PRM + 0x70u)
#define CM_CLKOUT_CTRL                     (CLOCK_CONTROL_REGISTERS_CM  + 0x70u)

#define PRM_CLKSEL                         (CLOCK_CONTROL_REGISTERS_PRM + 0x40u)
#define PRM_CLCKSRC_CTRL                   (GLOBAL_REGISTERS_PRM        + 0x70u)

#define PRM_CLCKSRC_CTRL_SYSCLKDIV         (0x3u << 6)  // PRM_CLCKSRC_CTRL SYSCLKDIV Bitmask
#define PRM_CLCKSRC_CTRL_SYSCLKDIV_1       (0x1u << 6)  // PRM_CLCKSRC_CTRL SYSCLKDIV: Syst_clk is external clock / 1
#define PRM_CLCKSRC_CTRL_SYSCLKDIV_2       (0x2u << 6)  // PRM_CLCKSRC_CTRL SYSCLKDIV: Syst_clk is external clock / 2

#define CM_CLKEN_PLL                       (CLOCK_CONTROL_REGISTERS_CM + 0x00u)
#define CM_CLKEN2_PLL                      (CLOCK_CONTROL_REGISTERS_CM + 0x04u)
#define CM_CLKSEL1_PLL                     (CLOCK_CONTROL_REGISTERS_CM + 0x40u)
#define CM_CLKSEL2_PLL                     (CLOCK_CONTROL_REGISTERS_CM + 0x44u)
#define CM_CLKSEL3_PLL                     (CLOCK_CONTROL_REGISTERS_CM + 0x48u)

//
// CM registers
//
#define CLOCK_CONTROL_REGISTERS_CM         (0x48004D00u)
#define WKUP_CM                            (0x48004C00u)
#define PER_CM                             (0x48005000u)
#define MPU_CM                             (0x48004900u)
#define CORE_CM                            (0x48004A00u)
#define EMU_CM                             (0x48005100u)
#define DSS_CM                             (0x48004E00u)
#define CAM_CM                             (0x48004F00u)

//
// PER CM registers
//
#define CM_FCLKEN_PER                      (PER_CM)
#define CM_ICLKEN_PER                      (PER_CM + 0x10u)

//
// WakeUp CM registers
//
#define CM_FCLKEN_WKUP                     (WKUP_CM + 0x00u)
#define CM_ICLKEN_WKUP                     (WKUP_CM + 0x10u)
#define CM_CLKSEL_WKUP                     (WKUP_CM + 0x40u)

//
// MPU CM registers
//
#define CM_CLKEN_PLL_MPU                   (MPU_CM + 0x04u)
#define CM_IDLEST_PLL_MPU                  (MPU_CM + 0x24u)
#define CM_CLKSEL1_PLL_MPU                 (MPU_CM + 0x40u)
#define CM_CLKSEL2_PLL_MPU                 (MPU_CM + 0x44u)

//
// CORE CM registers
//
#define CM_CLKSEL_CORE                     (CORE_CM + 0x40u)
#define CM_FCLKEN1_CORE                    (CORE_CM + 0x00u)
#define CM_ICLKEN1_CORE                    (CORE_CM + 0x10u)
#define CM_ICLKEN2_CORE                    (CORE_CM + 0x14u)

//
// DSS CM registers
//
#define CM_CLKSEL_DSS                      (DSS_CM + 0x40u)
#define CM_FCLKEN_DSS                      (DSS_CM + 0x00u)
#define CM_ICLKEN_DSS                      (DSS_CM + 0x10u)

//
// CAM CM registers
//
#define CM_CLKSEL_CAM                      (CAM_CM + 0x40u)

//
// GPMC registers
//
#define GPMC_BASE_ADDR                     (0x6E000000u)
#define GPMC_SYSCONFIG                     (GPMC_BASE_ADDR + 0x010u)
#define GPMC_SYSSTATUS                     (GPMC_BASE_ADDR + 0x014u)
#define GPMC_IRQENABLE                     (GPMC_BASE_ADDR + 0x01Cu)
#define GPMC_TIMEOUT_CONTROL               (GPMC_BASE_ADDR + 0x040u)
#define GPMC_CONFIG                        (GPMC_BASE_ADDR + 0x050u)
#define GPMC_STATUS                        (GPMC_BASE_ADDR + 0x054u)
#define GPMC_CONFIG1_0                     (GPMC_BASE_ADDR + 0x060u)
#define GPMC_CONFIG2_0                     (GPMC_BASE_ADDR + 0x064u)
#define GPMC_CONFIG3_0                     (GPMC_BASE_ADDR + 0x068u)
#define GPMC_CONFIG4_0                     (GPMC_BASE_ADDR + 0x06Cu)
#define GPMC_CONFIG5_0                     (GPMC_BASE_ADDR + 0x070u)
#define GPMC_CONFIG6_0                     (GPMC_BASE_ADDR + 0x074u)
#define GPMC_CONFIG7_0                     (GPMC_BASE_ADDR + 0x078u)
#define GPMC_CONFIG1_2                     (GPMC_BASE_ADDR + 0x0C0u)
#define GPMC_CONFIG2_2                     (GPMC_BASE_ADDR + 0x0C4u)
#define GPMC_CONFIG3_2                     (GPMC_BASE_ADDR + 0x0C8u)
#define GPMC_CONFIG4_2                     (GPMC_BASE_ADDR + 0x0CCu)
#define GPMC_CONFIG5_2                     (GPMC_BASE_ADDR + 0x0D0u)
#define GPMC_CONFIG6_2                     (GPMC_BASE_ADDR + 0x0D4u)
#define GPMC_CONFIG7_2                     (GPMC_BASE_ADDR + 0x0D8u)

//
// EMIF registers
//
#define EMIF_BASE_ADDR                     (0x6D000000u)
#define EMIF_STATUS                        (EMIF_BASE_ADDR + 0x004u)
#define EMIF_SDRAM_CONFIG                  (EMIF_BASE_ADDR + 0x008u)
#define EMIF_DDR_PHY_CTRL_1                (EMIF_BASE_ADDR + 0x0E4u)
#define EMIF_DDR_PHY_CTRL_1_SHDW           (EMIF_BASE_ADDR + 0x0E8u)
#define EMIF_DDR_PHY_CTRL_2                (EMIF_BASE_ADDR + 0x0ECu)
#define EMIF_IODFT_TLGC                    (EMIF_BASE_ADDR + 0x060u)
#define EMIF_SDRAM_TIM_1                   (EMIF_BASE_ADDR + 0x018u)
#define EMIF_SDRAM_TIM_1_SHDW              (EMIF_BASE_ADDR + 0x01Cu)
#define EMIF_SDRAM_TIM_2                   (EMIF_BASE_ADDR + 0x020u)
#define EMIF_SDRAM_TIM_2_SHDW              (EMIF_BASE_ADDR + 0x024u)
#define EMIF_SDRAM_TIM_3                   (EMIF_BASE_ADDR + 0x028u)
#define EMIF_SDRAM_TIM_3_SHDW              (EMIF_BASE_ADDR + 0x02Cu)
#define EMIF_PWR_MGMT_CTRL                 (EMIF_BASE_ADDR + 0x038u)
#define EMIF_PWR_MGMT_CTRL_SHDW            (EMIF_BASE_ADDR + 0x03Cu)
#define EMIF_SDRAM_REF_CTRL                (EMIF_BASE_ADDR + 0x010u)
#define EMIF_SDRAM_REF_CTRL_SHDW           (EMIF_BASE_ADDR + 0x014u)

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
typedef struct PIN_CONFIG
{
   OS_U32  reg_addr;
   OS_U8   mode;
   OS_U8   pull_en;
   OS_U8   pull_type;
} PIN_CONFIG;

static void _OS_ISR_Undefined(void);

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
const PIN_CONFIG clockout_pin_mux[] = {
  {CONTROL_PADCONF_sys_nirq_HI,         MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // sys_clkout2 (gpio 186)
  {CONTROL_PADCONF_sys_off_mode_HI,     MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // sys_clkout1 (gpio 10 )
};

const PIN_CONFIG  emif_pin_mux_shiva[]=  {
  {CONTROL_PADCONF_sad2d_sbusflag_HI,   MODE0, PUPD_EN,  PULL_UP     | INPUT_EN },  // cke0
  {CONTROL_PADCONF_sdrc_cke1,           MODE0, PUPD_EN,  PULL_UP     | INPUT_EN }   // cke1
};

static const PIN_CONFIG gpmc_pin_mux[] = {
  {CONTROL_PADCONF_sdrc_dqs3_HI,        MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a1
  {CONTROL_PADCONF_gpmc_a2,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a2
  {CONTROL_PADCONF_gpmc_a2_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a3
  {CONTROL_PADCONF_gpmc_a4,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a4
  {CONTROL_PADCONF_gpmc_a4_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a5
  {CONTROL_PADCONF_gpmc_a6,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a6
  {CONTROL_PADCONF_gpmc_a6_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a7
  {CONTROL_PADCONF_gpmc_a8,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a8
  {CONTROL_PADCONF_gpmc_a8_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a9
  {CONTROL_PADCONF_gpmc_a10,            MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_a10

  {CONTROL_PADCONF_gpmc_a10_HI,         MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d0
  {CONTROL_PADCONF_gpmc_d1,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d1
  {CONTROL_PADCONF_gpmc_d1_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d2
  {CONTROL_PADCONF_gpmc_d3,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d3
  {CONTROL_PADCONF_gpmc_d3_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d4
  {CONTROL_PADCONF_gpmc_d5,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d5
  {CONTROL_PADCONF_gpmc_d5_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d6
  {CONTROL_PADCONF_gpmc_d7,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d7
  {CONTROL_PADCONF_gpmc_d7_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d8
  {CONTROL_PADCONF_gpmc_d9,             MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d9
  {CONTROL_PADCONF_gpmc_d9_HI,          MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d10
  {CONTROL_PADCONF_gpmc_d11,            MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d11
  {CONTROL_PADCONF_gpmc_d11_HI,         MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d12
  {CONTROL_PADCONF_gpmc_d13,            MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d13
  {CONTROL_PADCONF_gpmc_d13_HI,         MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d14
  {CONTROL_PADCONF_gpmc_d15,            MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_d15

  {CONTROL_PADCONF_gpmc_d15_HI,         MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_ncs0
  {CONTROL_PADCONF_gpmc_ncs1,           MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_ncs1
  {CONTROL_PADCONF_gpmc_ncs1_HI,        MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_ncs2
  {CONTROL_PADCONF_gpmc_ncs3,           MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_ncs3

  {CONTROL_PADCONF_gpmc_ncs3_HI,        MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_ncs4
  {CONTROL_PADCONF_gpmc_ncs5,           MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_ncs5
  {CONTROL_PADCONF_gpmc_ncs5_HI,        MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_ncs6
  {CONTROL_PADCONF_gpmc_ncs7,           MODE1, PUPD_EN,  PULL_DOWN   | INPUT_EN},   // gpmc_ncs7 -> gpmc_io_dir

  {CONTROL_PADCONF_gpmc_ncs7_HI,        MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_clk
  {CONTROL_PADCONF_gpmc_nadv_ale,       MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_adv_ale
  {CONTROL_PADCONF_gpmc_nadv_ale_HI,    MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_noe
  {CONTROL_PADCONF_gpmc_nwe,            MODE0, PUPD_DIS, PULL_DOWN   | INPUT_EN},   // gpmc_nwe

  {CONTROL_PADCONF_gpmc_nwe_HI,         MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_nbe0_cle LAB/LVSOM ?

  {CONTROL_PADCONF_gpmc_nbe1,           MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_nbe1
  {CONTROL_PADCONF_gpmc_nbe1_HI,        MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_nwp
  {CONTROL_PADCONF_gpmc_wait0_HI,       MODE0, PUPD_EN,  PULL_UP     | INPUT_EN},   // gpmc_wait1
};

/*********************************************************************
*
*       MMU and cache configuration
*
*  Notes
*    The translation table has to be aligned on 16384 bytes boundary
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #if (IRAM != 0)
    #pragma location=0x4020C000 // Place at fixed address in iRAM, last 16KBytes, has to be reserved in the linker file
  #endif
  #pragma data_alignment=16384
  static __no_init OS_U32 _TranslationTable[0x1000];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(16384) static OS_U32 _TranslationTable[0x1000];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_U32 _TranslationTable[0x1000] __attribute__ ((aligned (16384), section (".no_init")));
#endif

/*********************************************************************
*
*       Interrupt vector table
*
*  Notes
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=4
  static __no_init OS_ISR_HANDLER* _apIsrHandler[NUM_INT_VECTORS];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"         //   Has to be declared as separate section UNINIT in the scatter file
  __align(4) static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_VECTORS];
  #pragma arm section zidata // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_ISR_HANDLER* _apIsrHandler[NUM_INT_VECTORS] __attribute__ ((aligned (4), section (".no_init")));
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _InitVIC()
*
*  Function description
*    Initialize interrupt controller by setting default vectors
*    and clearing all interrupts
*/
static void _InitVIC(OS_U32 addr) {
  unsigned int i;

  INTCPS_SYSCONFIG = 0x02;                    // Reset interrupt controller;
  while ((INTCPS_SYSSTATUS & 0x01) == 0x00);  // Wait until reset is finished
  __MCR(15, 0, addr, 12, 0, 0);               // Set Vector Base addr
  for (i = 0; i < NUM_INT_VECTORS; i++) {
    _apIsrHandler[i] = (OS_ISR_HANDLER*) _OS_ISR_Undefined;
  }
}

/*********************************************************************
*
*       _DisableWatchdog()
*
*  Function description
*    Disables all watchdogs
*/
static void _DisableWatchdog(void) {
  //
  // Enable clocks
  //
  OUT_REGL(0x48004C10u, 0x20);          // enable Interface clock
  OUT_REGL(0x48004C00u, 0x20);          // enable functional clock
  while (IN_REGL(0x48004C20u) & 0x20);  // Check that module is Idle
  //
  // Disabler  watchdog 2
  //
  while (!(IN_REGL(0x48314014u) & 1u)) {
    ;  // Wait until reset complete
  }
  //
  // Disable 32Khz watchdog timer
  //
  OUT_REGL(0x48314048u, 0x0000AAAAu);
  while (IN_REGL(0x48314034u) & 0x10u) {
    ;
  }
  //
  // Disable 32Khz watchdog timer
  //
  OUT_REGL(0x48314048u, 0x00005555u);
  while (IN_REGL(0x48314034u) & 0x10u) {
    ;
  }
}

/*********************************************************************
*
*       _InitPower()
*
*  Function description
*    Enable power for the power domains
*/
static void _InitPower(void) {
  SETBIT_REGL(CORE_PRM                    + 0xE0u, 0x00000003u);  // Turn on CORE power domain (CPU *should* default to on)
  SETBIT_REGL(MPU_PRM                     + 0xE0u, 0x00000003u);  // Turn on MPU  power domain (CPU *should* default to on)
  SETBIT_REGL(IVA2_PRM                    + 0xE0u, 0x00000003u);
  SETBIT_REGL(OCP_SYSTEM_REGISTERS_PRM    + 0xE0u, 0x00000003u);
  SETBIT_REGL(MPU_PRM                     + 0xE0u, 0x00000003u);
  SETBIT_REGL(CORE_PRM                    + 0xE0u, 0x00000003u);
  SETBIT_REGL(GFX_PRM                     + 0xE0u, 0x00000003u);
  SETBIT_REGL(WKUP_PRM                    + 0xE0u, 0x00000003u);
  SETBIT_REGL(CLOCK_CONTROL_REGISTERS_PRM + 0xE0u, 0x00000003u);
  SETBIT_REGL(DSS_PRM                     + 0xE0u, 0x00000003u);
  SETBIT_REGL(CAM_PRM                     + 0xE0u, 0x00000003u);
  SETBIT_REGL(PER_PRM                     + 0xE0u, 0x00000003u);
  SETBIT_REGL(EMU_PRM                     + 0xE0u, 0x00000003u);
  SETBIT_REGL(GLOBAL_REGISTERS_PRM        + 0xE0u, 0x00000003u);
  SETBIT_REGL(NEON_PRM                    + 0xE0u, 0x00000003u);
  SETBIT_REGL(USBHOST_PRM                 + 0xE0u, 0x00000003u);
}

static void _config_pad(OS_U32 reg,OS_U8 mode,OS_U8 pull_en,OS_U8 pull_type) {
  OS_U16 tmp;

  tmp = 0
      | (mode & 0x7u)                   // pin mode
      | ((pull_en   & PUPD_EN)  << 3)  // pull enable
      | ((pull_type & PULL_UP)  << 4)  // pull type select
      | ((pull_type & INPUT_EN) << 7)  // input enable (bit 8)
      ;
  OUT_REGS(reg, tmp);
}

static void _pin_config_mux(const PIN_CONFIG pin_mux[], const OS_U32 no_of_pins) {
  OS_U32 i;

  for (i = 0; i < no_of_pins; i++) {
    _config_pad(pin_mux[i].reg_addr,
                pin_mux[i].mode,
                pin_mux[i].pull_en,
                pin_mux[i].pull_type);
  }
}

/*********************************************************************
*
*       _IoConfig()
*
*  Function description
*    Initialize IO
*/
static void _IoConfig(void) {
  OS_U32 temp;

  LOAD_PINMUX(clockout_pin_mux);
  OUT_REGL(PRM_CLKOUT_CTRL, 0x80u);  // Turn on clock out1, 26M CLK
  OUT_REGL(CM_CLKOUT_CTRL,  0x83u);  // Turn on clock out2, 54M CLK
  //
  // enable the function clock for GPIO 2-6
  //
  temp  = IN_REGL(CM_FCLKEN_PER);
  temp &= ~(0x1Fu << 13);
  temp |=  (0x1Fu << 13);
  OUT_REGL(CM_FCLKEN_PER, temp);
  //
  // enable the function clock for GPIO 1
  //
  temp  = IN_REGL(CM_FCLKEN_WKUP);
  temp &= ~(1u << 3);
  temp |=  (1u << 3);
  OUT_REGL(CM_FCLKEN_WKUP, temp);
  //
  // enable the interface clock for GPIO 2-6
  //
  temp = IN_REGL(CM_ICLKEN_PER);
  temp &= ~(0x1Fu << 13);
  temp |=  (0x1Fu << 13);
  OUT_REGL(CM_ICLKEN_PER, temp);
  //
  // enable the interface clock for GPIO 1
  //
  temp  = IN_REGL(CM_ICLKEN_WKUP);
  temp &= ~(1u << 3);
  temp |=  (1u << 3);
  OUT_REGL(CM_ICLKEN_WKUP, temp);
}

static OS_U32 sys_clock_divide_value(void) {
   return IN_REGL(PRM_CLCKSRC_CTRL) & PRM_CLCKSRC_CTRL_SYSCLKDIV;
}

/*********************************************************************
*
*       _InitClocks()
*
*  Function description
*    Initialize main clock and PLL
*
*  Notes
*    (2) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*/
static void _InitClocks(void) {
  OS_U32 temp;
  OS_U32 mult;
  OS_U32 div;
  OS_U32 freq;
  OS_U32 m2, m3, m4, m5, m6;

  //
  // Don't run if we're running in DDR
  //
  if (((OS_U32)_InitClocks <  DDR_MEM_BASE) ||
      ((OS_U32)_InitClocks >= DDR_MEM_END)) {
    //
    // Put CORE in low power bypass  (DPLL3)
    //
    temp  = IN_REGL(CLOCK_CONTROL_REGISTERS_CM);
    temp &= ~(0x7u << 0);
    temp |=  (0x5u << 0);
    OUT_REGL(CLOCK_CONTROL_REGISTERS_CM, temp);
    while ((IN_REGL(CLOCK_CONTROL_REGISTERS_CM + 0x20u) & 1u) == 1) {
      ;  // Wait for bypass mode
    }
    //
    // Put MPU in low power bypass  (DPLL1)
    //
    temp  = IN_REGL(CM_CLKEN_PLL_MPU);
    temp &= ~(0x7u << 0);
    temp |=  (0x5u << 0);
    OUT_REGL(CM_CLKEN_PLL_MPU, temp);
    while ((IN_REGL(CM_IDLEST_PLL_MPU) & 1u) == 1) {
      ;  // Wait for bypass mode
    }
    //
    // Put peripheral bus in low power stop mode  (DPLL4)
    //
    temp  = IN_REGL(CLOCK_CONTROL_REGISTERS_CM);
    temp &= ~(0x7u << 16);
    temp |=  (0x1u << 16);
    OUT_REGL(CLOCK_CONTROL_REGISTERS_CM, temp);
    while ((IN_REGL(CLOCK_CONTROL_REGISTERS_CM + 0x20u) & 2u) == 1) {
      ;  // Wait for stop mode
    }
    //
    // Put per2 in low power stop mode  (DPLL5)
    //
    temp  = IN_REGL(CLOCK_CONTROL_REGISTERS_CM + 0x04u);
    temp &= ~(0x7u << 0);
    temp |=  (0x1u << 0);
    OUT_REGL(CLOCK_CONTROL_REGISTERS_CM + 0x04u, temp);
    while ((IN_REGL(CLOCK_CONTROL_REGISTERS_CM + 0x24u) & 1u) == 1) {
      ;  // Wait for stop mode
    }
    OUT_REGL(PRM_CLKSEL, 0x3u);  // Set input clock frequency 26.0MHz
    //
    // Set the sys clock divider to 1
    //
    temp  = IN_REGL(PRM_CLCKSRC_CTRL);
    temp &= ~(PRM_CLCKSRC_CTRL_SYSCLKDIV);
    temp |= PRM_CLCKSRC_CTRL_SYSCLKDIV_1;
    OUT_REGL(PRM_CLCKSRC_CTRL, temp);
    //
    // Configure clock ratios for all L3, L4 targets
    //
    OUT_REGL(CM_CLKSEL_CORE, (2u << 0) |  // L3
                             (2u << 2) |  // L4
                             (2u << 4) |  // FSHOSTUSB
                             (0u << 6) |  // GPT10
                             (0u << 7) |  // GPT11
                             (3u << 8));  // SSI
    //
    // Configure MPU (DPLL 1) 600MHz
    //
    mult = 300u;  // (26 Mhz * 300)/(12 + 1) = 600 Mhz
    if (sys_clock_divide_value() == PRM_CLCKSRC_CTRL_SYSCLKDIV_2) {
      mult *= 2;  // (13 Mhz * 2 * 300)/(12 + 1) = 600 Mhz
    }
    div   = 12u;
    freq  =  7u;
    m2    =  1u;  // MPU_DPLL_CLKOUT (DPLL1 CLKOUTX2): 1000 Mhz
    temp  = IN_REGL(CM_CLKSEL1_PLL_MPU);
    temp &= ~(0x7FFu << 8);
    temp |=  (mult   << 8);
    temp &= ~(0x7Fu  << 0);
    temp |=  (div    << 0);
    OUT_REGL(CM_CLKSEL1_PLL_MPU, temp);
    temp  = IN_REGL(CM_CLKSEL2_PLL_MPU);
    temp &= ~(0x1Fu << 0);
    temp |=  (m2    << 0);
    OUT_REGL(CM_CLKSEL2_PLL_MPU, temp);
    temp  = IN_REGL(CM_CLKEN_PLL_MPU);
    temp &= ~(0xFu << 4);
    temp |=  (freq << 4);
    OUT_REGL(CM_CLKEN_PLL_MPU, temp);
    //
    // Configure CORE (DPLL 3) 166MHz
    // Reference clock is sys clock (26 Mhz) / 2
    //
    mult = 166u;    // (13 Mhz * 166)/(12 + 1) = 166 Mhz
    if (sys_clock_divide_value() == PRM_CLCKSRC_CTRL_SYSCLKDIV_2) {
       mult *= 2u;  // (6.5 Mhz * 2 * 166)/(12 + 1) = 166 Mhz
    }
    div   = 12u;
    freq  =  7u;
    m2    =  1u;    // CORE_DPLL_CLKOUT (166 Mhz)
    m3    =  2u;    // EMU_CORE_ALWON_CLK (83 Mhz)
    temp  = IN_REGL(CM_CLKSEL1_PLL);
    temp &= ~(0x1Fu  << 27);
    temp |=  (m2     << 27);
    temp &= ~(0x7FFu << 16);
    temp |=  (mult   << 16);
    temp &= ~(0x7Fu  <<  8);
    temp |=  (div    <<  8);
    OUT_REGL(CM_CLKSEL1_PLL, temp);
    temp  = IN_REGL(CM_CLKEN_PLL);
    temp &= ~(0xFu   <<  4);
    temp |=  (freq   <<  4);
    OUT_REGL(CM_CLKEN_PLL, temp);
    temp  = IN_REGL(CM_CLKSEL1_EMU);
    temp &= ~(0x1Fu  << 16);
    temp |=  (m3     << 16);
    OUT_REGL(CM_CLKSEL1_EMU, temp);
    //
    // Configure peripheral bus (DPLL 4) 864MHz
    // DPLL4 is derived from DPLL3
    //
    mult = 216u;   // (52 Mhz * 216)/(12 + 1) = 864 Mhz
    if (sys_clock_divide_value() == PRM_CLCKSRC_CTRL_SYSCLKDIV_2) {
      mult *= 2u;  // (26 Mhz * 2 * 216)/(12 + 1) = 864 Mhz
    }
    div   = 12u;
    freq  =  7u;
    m2    =  9u;   // 96 Mhz clock
    m3    = 16u;   // TV functional clock (54 Mhz)
    m4    =  9u;   // DSS1_ALWON_FCLK (96 Mhz)
    m5    =  4u;   // CAM_MCLK (216 Mhz)
    m6    =  3u;   // EMU_PER_ALWON_CLK (288 Mhz)
    temp  = IN_REGL(CM_CLKSEL2_PLL);
    temp &= ~(0x7Fu  <<  0);
    temp |=  (div    <<  0);
    temp &= ~(0x7FFu <<  8);
    temp |=  (mult   <<  8);
    OUT_REGL(CM_CLKSEL2_PLL, temp);
    temp  = IN_REGL(CM_CLKEN_PLL);
    temp &= ~(0xFu   << 20);
    temp |=  (freq   << 20);
    OUT_REGL(CM_CLKEN_PLL, temp);
    temp  = IN_REGL(CM_CLKSEL3_PLL);
    temp &= ~(0x1Fu  <<  0);
    temp |=  (m2     <<  0);
    OUT_REGL(CM_CLKSEL3_PLL, temp);
    temp  = IN_REGL(CM_CLKSEL_DSS);
    temp &= ~(0x1Fu  <<  8);
    temp |=  (m3     <<  8);
    OUT_REGL(CM_CLKSEL_DSS, temp);
    temp  = IN_REGL(CM_CLKSEL_DSS);
    temp &= ~(0x1Fu  <<  0);
    temp |=  (m4     <<  0);
    OUT_REGL(CM_CLKSEL_DSS, temp);
    temp  = IN_REGL(CM_CLKSEL_CAM);
    temp &= ~(0x1Fu  <<  0);
    temp |=  (m5     <<  0);
    OUT_REGL(CM_CLKSEL_CAM, temp);
    temp  = IN_REGL(CM_CLKSEL1_EMU);
    temp &= ~(0x1Fu  << 24);
    temp |=  (m6     << 24);
    OUT_REGL(CM_CLKSEL1_EMU, temp);
    //
    // Lock CORE
    //
    temp  = IN_REGL(CM_CLKEN_PLL);
    temp &= ~(0x7u << 0);
    temp |=  (0x7u << 0);
    OUT_REGL(CM_CLKEN_PLL, temp);
    while ((IN_REGL(CLOCK_CONTROL_REGISTERS_CM + 0x20u) & 1u) == 0) {
      ;  // Wait for locked
    }
    //
    // Lock MPU
    //
    temp  = IN_REGL(CM_CLKEN_PLL_MPU);
    temp &= ~(0x7u << 0);
    temp |=  (0x7u << 0);
    OUT_REGL(CM_CLKEN_PLL_MPU, temp);
    while ((IN_REGL(CM_IDLEST_PLL_MPU) & 0x00000001) == 0) {
      ;  // Wait for locked
    }
    //
    // Lock peripheral bus
    //
    temp  = IN_REGL(CM_CLKEN_PLL);
    temp &= ~(0x7u << 16);
    temp |=  (0x7u << 16);
    OUT_REGL(CM_CLKEN_PLL, temp);
    while ((IN_REGL(CLOCK_CONTROL_REGISTERS_CM + 0x20) & 0x00000002) == 0) {
      ;  // Wait for lock mode
    }
  }
  OUT_REGL(CM_FCLKEN1_CORE,  0xFFFFFFFFu);
  OUT_REGL(CM_ICLKEN1_CORE,  0xFFFFFFFFu);
  OUT_REGL(CM_ICLKEN2_CORE,  0xFFFFFFFFu);
  OUT_REGL(CM_FCLKEN_WKUP,   0xFFFFFFFFu);
  OUT_REGL(CM_ICLKEN_WKUP,   0xFFFFFFFFu);
  OUT_REGL(CM_FCLKEN_PER,    0xFFFFFFFFu);
  OUT_REGL(CM_ICLKEN_PER,    0xFFFFFFFFu);
  OUT_REGL(CM_FCLKEN_DSS,    0x00000007u);
  OUT_REGL(CM_ICLKEN_DSS,    0x00000001u);
}

/*********************************************************************
*
*       _InitGPMC()
*
*  Function description
*    Initialize GPMC
*/
static void _InitGPMC(void) {
  //
  // The GPMC has a single clock.  The GPMC_FCLK comes from the
  // PRCM module & runs at the L3 interconnect frequency.  For
  // each chip select, GPMC_FCLK can be divided by 1,2,3, or 4
  // using GPMC_CONFIG1 bits 0-1.
  // The L3 interconnect gets its clock from DPLL3.  DPLL3
  // is configured in init_clocks(), and is 166MHz.
  //
  LOAD_PINMUX(gpmc_pin_mux);
  //
  // Reset the GPMC
  //
  OUT_REGL(GPMC_SYSCONFIG, 0x2u);
  while (!IN_REGL(GPMC_SYSSTATUS)) {
    ;
  }
  //
  // GPMC Configuration
  //
  OUT_REGL(GPMC_SYSCONFIG,       0x00000010u);  // GPMC_SYSCONFIG(0x10): No idle, L3 clock free running
  OUT_REGL(GPMC_TIMEOUT_CONTROL, 0x00000000u);  // Time out disabled
  OUT_REGL(GPMC_IRQENABLE,       0x00000000u);  // GPMC_IRQENABLE(0x0C): All interrupts disabled
  OUT_REGL(GPMC_CONFIG,          0x00000010u);  // Write protect output pin high and Wait active Low
  OUT_REGL(GPMC_CONFIG1_0,       0x00001800u);
  OUT_REGL(GPMC_CONFIG2_0,       0x00080800u);
  OUT_REGL(GPMC_CONFIG3_0,       0x00080800u);
  OUT_REGL(GPMC_CONFIG4_0,       0x06000600u);
  OUT_REGL(GPMC_CONFIG5_0,       0x00080808u);
  OUT_REGL(GPMC_CONFIG6_0,       0x000003CFu);
  OUT_REGL(GPMC_CONFIG7_0,       0x00000F5Fu);
  OUT_REGL(GPMC_CONFIG1_2,       0x00001210u);
  OUT_REGL(GPMC_CONFIG2_2,       0x00101001u);
  OUT_REGL(GPMC_CONFIG3_2,       0x00020201u);
  OUT_REGL(GPMC_CONFIG4_2,       0x0f031003u);
  OUT_REGL(GPMC_CONFIG5_2,       0x000f1111u);
  OUT_REGL(GPMC_CONFIG6_2,       0x0f030080u);
  OUT_REGL(GPMC_CONFIG7_2,       0x00000C50u);  // 64MB
  OUT_REGL(GPMC_CONFIG,          0x00000010u);  // Don't use wait
}

/*********************************************************************
*
*       _InitDDR()
*
*  Function description
*    Initialize DDR RAM
*/
static void _InitDDR(void) {
  //
  // Don't initialize if we're running in DDR
  //
  if (((OS_U32)_InitDDR <  DDR_MEM_BASE) ||
      ((OS_U32)_InitDDR >= DDR_MEM_END )) {
    //
    // EMIF initilization for Micron DDR2 [166MHz] access
    //
    LOAD_PINMUX(emif_pin_mux_shiva);  // Changing muxmode 1 for cke0 & cke1
    //
    // Configure EMIF to Access Micron DDR2
    //
    OUT_REGL(EMIF_DDR_PHY_CTRL_1,      (0x8046u | (EMIF_STROBE_ENABLE << 7)));
    OUT_REGL(EMIF_DDR_PHY_CTRL_1_SHDW, (0x8046u | (EMIF_STROBE_ENABLE << 7)));
    OUT_REGL(EMIF_DDR_PHY_CTRL_2,      0);
    OUT_REGL(EMIF_IODFT_TLGC,          (IN_REGL(EMIF_IODFT_TLGC) | 0x400u));
    while((IN_REGL(EMIF_STATUS) & 4u) == 0) {
      ;
    }
    OUT_REGL(EMIF_IODFT_TLGC, IN_REGL(EMIF_IODFT_TLGC) | 0x1); // VRD - Debug Normal Mode
    //
    // EMIF DDR phy PLL is ready
    //
    OUT_REGL(EMIF_SDRAM_TIM_1,         0x04448279u);
    OUT_REGL(EMIF_SDRAM_TIM_1_SHDW,    0x04448279u);
    OUT_REGL(EMIF_SDRAM_TIM_2,         0x342231CBu);
    OUT_REGL(EMIF_SDRAM_TIM_2_SHDW,    0x342231CBu);
    OUT_REGL(EMIF_SDRAM_TIM_3,         0x00000217u);
    OUT_REGL(EMIF_SDRAM_TIM_3_SHDW,    0x00000217u);
    OUT_REGL(EMIF_PWR_MGMT_CTRL,       0x80000000u);
    OUT_REGL(EMIF_PWR_MGMT_CTRL_SHDW,  0x80000000u);
    OUT_REGL(EMIF_SDRAM_REF_CTRL,      0x0000050Fu);
    OUT_REGL(EMIF_SDRAM_REF_CTRL_SHDW, 0x0000050Fu);
    OUT_REGL(EMIF_SDRAM_CONFIG,        0x43801432u);
  }
}

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when an not installed interrupt was detected.
*    As interrupt pending condition of peripherals has to be reset,
*    program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(void) {
  volatile int Dummy;

  Dummy = 1;
  while (Dummy > 0) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return TCRR - (0xFFFFFFFF - OS_TIMER_RELOAD + 1);
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (TISR & OVF_IT_FLAG);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
* Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  TISR |= OVF_IT_FLAG;  // Reset pending flag
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Set interrupt vector base address
  //
#if (IRAM != 0)
  _InitVIC(0x40200000);
#else
  _InitVIC(0x80000000);
#endif
  //
  // Initialize hardware
  //
  _DisableWatchdog();  // Disable the watchdog timer
  _InitPower();        // Turn on MPU & core
  _IoConfig();         // Init pins and clock outs
  _InitClocks();       // Initialze clocks
  _InitGPMC();         // Init chip selects
  _InitDDR();          // Init DDR RAM
  //
  // Enable clock and power for GPTIMER1
  //
  (*(volatile OS_U32*)CM_FCLKEN_WKUP) |= (1u << 0);  // Enable function clock
  (*(volatile OS_U32*)CM_ICLKEN_WKUP) |= (1u << 0);  // Enable interface clock
  (*(volatile OS_U32*)CM_CLKSEL_WKUP) |= (1u << 0);  // Set SysClk as clock source
  //
  // Init GPTIMER1 as System tick timer
  //
  TLDR  = 0xFFFFFFFFu - OS_TIMER_RELOAD + 1;         // Timer reload register
  TCRR  = 0xFFFFFFFFu - OS_TIMER_RELOAD + 1;         // Timer value
  TIER |= OVF_IT_ENA;                                // Enable Overflow interrupt
  TCLR  = 0
        | (1u << 1)                                  // Auto reload mode
        | (1u << 0)                                  // Start Timer
        ;
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler(TIMER1_IRQ_ID, (OS_ISR_HANDLER*)_OS_ISR_Tick);
  OS_ARM_ISRSetPrio(TIMER1_IRQ_ID, TIMER1_IRQ_PRIO);
  OS_ARM_EnableISR(TIMER1_IRQ_ID);
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  int             ActiveInterrupt;

  OS_INT_Enter();
  //
  // Read active interrupt register and branch to Irq service routine
  //
  ActiveInterrupt = INTCPS_SIR_IRQ;
  pISR            = (OS_ISR_HANDLER*)_apIsrHandler[ActiveInterrupt];
  pISR();
  INTCPS_CONTROL = 0x01u;  // Reset NVIC
  OS_INT_Leave();
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  papISR               = (OS_ISR_HANDLER**)_apIsrHandler;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  if (ISRIndex < 32) {
    INTCPS_MIR0_CLR = (1u << (ISRIndex));
  } else if (ISRIndex < 64) {
    INTCPS_MIR1_CLR = (1u << (ISRIndex - 32));
  } else if (ISRIndex < 96) {
    INTCPS_MIR2_CLR = (1u << (ISRIndex - 64));
  }
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  if (ISRIndex < 32) {
    INTCPS_MIR0_SET = (1u << (ISRIndex));
  } else if (ISRIndex < 64) {
    INTCPS_MIR1_SET = (1u << (ISRIndex - 32));
  } else if (ISRIndex < 96) {
    INTCPS_MIR2_SET = (1u << (ISRIndex - 64));
  }
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pPrio           = (OS_U32*)INTCPS_ILR_ADDR;
  OldPrio         = pPrio[ISRIndex];
  pPrio[ISRIndex] = ((Prio & 0x3Fu) << 2);
  OS_INT_EnableConditional();
  return OldPrio;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
}
#endif
OS_INTERWORK int __low_level_init(void) {
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Memory mapping should be complete, meaning 4096 entries.
  // Code below fills in ascending VAddr order
  //
  OS_ARM_MMU_InitTT(_TranslationTable);
  //                                         Mode                                Access permissions        VAddr  PAddr  Size[MB]
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x000, 0x000, 0x402);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_WRITE_BACK_ALLOC,                            0x402, 0x402, 0x001);  // internal SRAM (48kByte), 1 MB (program area)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x403, 0x403, 0x07D);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_STRONGLY_ORDERED | OS_ARM_MMU_EXECUTE_NEVER, 0x480, 0x480, 0x080);  // SFRs, non cacheable, non bufferable
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x500, 0x500, 0x300);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_STRONGLY_ORDERED,                            0x680, 0x680, 0x080);  // L3 Interconnect (128 MByte)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x700, 0x700, 0x100);  // Unmapped area
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_CACHEMODE_WRITE_BACK_ALLOC,                            0x800, 0x800, 0x100);  // external SDRAM (256 MByte), 1 MB (program area)
  OS_ARM_MMU_AddTTEntries(_TranslationTable, OS_ARM_MMU_UNMAPPED,                                          0x900, 0x900, 0x700);  // Unmapped area
  OS_ARM_MMU_Enable      (_TranslationTable);
  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();
  return 1;  // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
