;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Part one of the system initialization code,
;; contains low-level
;; initialization.
;;
;; Copyright 2011 IAR Systems. All rights reserved.
;;
;; $Revision: #2 $
;;

        MODULE  ?cstartup

;; Forward declaration of sections.
        SECTION IRQ_STACK:DATA:NOROOT(3)
        SECTION FIQ_STACK:DATA:NOROOT(3)
        SECTION SVC_STACK:DATA:NOROOT(3)
        SECTION ABT_STACK:DATA:NOROOT(3)
        SECTION UND_STACK:DATA:NOROOT(3)
        SECTION CSTACK:DATA:NOROOT(3)


; --------------------
; Mode, correspords to bits 0-5 in CPSR

MODE_MSK DEFINE 0x1F            ; Bit mask for mode bits in CPSR

USR_MODE DEFINE 0x10            ; User mode
FIQ_MODE DEFINE 0x11            ; Fast Interrupt Request mode
IRQ_MODE DEFINE 0x12            ; Interrupt Request mode
SVC_MODE DEFINE 0x13            ; Supervisor mode
ABT_MODE DEFINE 0x17            ; Abort mode
UND_MODE DEFINE 0x1B            ; Undefined Instruction mode
SYS_MODE DEFINE 0x1F            ; System mode


CP_DIS_MASK         DEFINE  0xFFFFEFFA


;
; The module in this file are included in the libraries, and may be
; replaced by any user-defined modules that define the PUBLIC symbol
; __iar_program_start or a user defined start symbol.
;
; To override the cstartup defined in the library, simply add your
; modified version to the workbench project.

; This section should be at the begin of the code

        SECTION .intvec:CODE:NOROOT(2)

        PUBLIC  __vector
        PUBLIC  __iar_program_start
        EXTERN  Undefined_Handler
        EXTERN  SWI_Handler
        EXTERN  Prefetch_Handler
        EXTERN  Abort_Handler
        EXTERN  IRQ_Handler
        EXTERN  FIQ_Handler
        EXPORT  Undefined_Addr

        ARM

__iar_init$$done:               ; The vector table is not needed
                                ; until after copy initialization is done

__vector:
        ; All default exception handlers (except reset) are
        ; defined as weak symbol definitions.
        ; If a handler is defined by the application it will take precedence.
        LDR     PC,Reset_Addr           ; Reset
        LDR     PC,Undefined_Addr       ; Undefined instructions
        LDR     PC,SWI_Addr             ; Software interrupt (SWI/SVC)
        LDR     PC,Prefetch_Addr        ; Prefetch abort
        LDR     PC,Abort_Addr           ; Data abort
        LDR     PC,Unused_Addr          ; RESERVED
        LDR     PC,IRQ_Addr             ; IRQ
        LDR     PC,FIQ_Addr             ; FIQ

        DATA

Reset_Addr:     DCD   __iar_program_start

Undefined_Addr: DCD   Undefined_Handler
SWI_Addr:       DCD   SWI_Handler
Prefetch_Addr:  DCD   Prefetch_Handler
Abort_Addr:     DCD   Abort_Handler
Unused_Addr:    DCD   .
IRQ_Addr:       DCD   IRQ_Handler
FIQ_Addr:       DCD   FIQ_Handler


        SECTION .text:CODE:NOROOT(2)

        EXTERN  __cmain
        REQUIRE __vector
        EXTWEAK __iar_init_core
        EXTWEAK __iar_init_vfp


        ARM
__iar_program_start:
?cstartup:

; Execution starts here.
; After a reset, the mode is ARM, Supervisor, interrupts disabled.

; Disable Addr translation, D cache and enable I cache
        MRC         p15,0,R1,C1,C0,0
        LDR         R0,=CP_DIS_MASK      ;; 0xFFFFEFFA
        AND         R1,R1,R0
        ORR         R1,R1,#(1<<12)
        MCR         p15,0,R1,C1,C0,0

//==================================================================
// Invalidate Data and Instruction TLBs and branch predictor
//==================================================================

        mov     R0, #0
        mcr     P15, 0, R0, C8, C7, 0      // I-TLB and D-TLB invalidation
        mcr     P15, 0, R0, C7, C5, 6      // BPIALL - Invalidate entire branch predictor array
        
//==================================================================
// Cache Invalidation code for Cortex-A
//==================================================================

        // Invalidate L1 Instruction Cache

        mrc     P15, 1, R0, C0, C0, 1      // Read Cache Level ID Register (CLIDR)
        tst     R0, #0x3                   // Harvard Cache?
        mov     R0, #0                     // SBZ
        mcrne   P15, 0, R0, C7, C5, 0      // ICIALLU - Invalidate instruction cache and flush branch target cache

        // Invalidate Data/Unified Caches

        mrc     P15, 1, R0, C0, C0, 1      // Read CLIDR
        ands    R3, R0, #0x07000000        // Extract coherency level
        mov     R3, R3, LSR #23            // Total cache levels << 1
        beq     Finished                   // If 0, no need to clean

        mov     R10, #0                    // R10 holds current cache level << 1
Loop1:  add     R2, R10, R10, LSR #1       // R2 holds cache "Set" position
        mov     R1, R0, LSR R2             // Bottom 3 bits are the Cache-type for this level
        and     R1, R1, #7                 // Isolate those lower 3 bits
        cmp     R1, #2
        blt     Skip                       // No cache or only instruction cache at this level

        mcr     P15, 2, R10, C0, C0, 0     // Write the Cache Size selection register
        isb                                // isb to sync the change to the CacheSizeID reg
        mrc     P15, 1, R1, C0, C0, 0      // Reads current Cache Size ID register
        and     R2, R1, #7                 // Extract the line length field
        add     R2, R2, #4                 // Add 4 for the line length offset (log2 16 bytes)
        ldr     R4, =0x3FF
        ands    R4, R4, R1, LSR #3         // R4 is the max number on the way size (right aligned)
        clz     R5, R4                     // R5 is the bit position of the way size increment
        ldr     R7, =0x7FFF
        ands    R7, R7, R1, LSR #13        // R7 is the max number of the index size (right aligned)

Loop2:  mov     R9, R4                     // R9 working copy of the max way size (right aligned)

Loop3:  orr     R11, R10, R9, LSL R5       // Factor in the Way number and cache number into R11
        orr     R11, R11, R7, LSL R2       // Factor in the Set number
        mcr     P15, 0, R11, C7, C6, 2     // Invalidate by Set/Way
        subs    R9, R9, #1                 // Decrement the Way number
        bge     Loop3
        subs    R7, R7, #1                 // Decrement the Set number
        bge     Loop2
Skip:   add     R10, R10, #2               // increment the cache number
        cmp     R3, R10
        bgt     Loop1

Finished:

; Initialize the stack pointers.
; The pattern below can be used for any of the exception stacks:
; FIQ, IRQ, SVC, ABT, UND, SYS.
; The USR mode uses the same stack as SYS.
; The stack segments must be defined in the linker command file,
; and be declared above.
        mrs     r0,cpsr                             ; Original PSR value
        bic     r0,r0,#MODE_MSK                     ; Clear the mode bits
        orr     r0,r0,#SVC_MODE                     ; Set Supervisor mode bits
        msr     cpsr_c,r0                           ; Change the mode
        ldr     sp,=SFE(SVC_STACK)                  ; End of SVC_STACK

        bic     r0,r0,#MODE_MSK                     ; Clear the mode bits
        orr     r0,r0,#ABT_MODE                     ; Set Abort mode bits
        msr     cpsr_c,r0                           ; Change the mode
        ldr     sp,=SFE(ABT_STACK)                  ; End of ABT_STACK

        bic     r0,r0,#MODE_MSK                     ; Clear the mode bits
        orr     r0,r0,#UND_MODE                     ; Set Undefined mode bits
        msr     cpsr_c,r0                           ; Change the mode
        ldr     sp,=SFE(UND_STACK)                  ; End of UND_STACK

        bic     r0,r0,#MODE_MSK                     ; Clear the mode bits
        orr     r0,r0,#FIQ_MODE                     ; Set FIR mode bits
        msr     cpsr_c,r0                           ; Change the mode
        ldr     sp,=SFE(FIQ_STACK)                  ; End of FIQ_STACK

        bic     r0,r0,#MODE_MSK                     ; Clear the mode bits
        orr     r0,r0,#IRQ_MODE                     ; Set IRQ mode bits
        msr     cpsr_c,r0                           ; Change the mode
        ldr     sp,=SFE(IRQ_STACK)                  ; End of IRQ_STACK

        bic     r0,r0,#MODE_MSK                     ; Clear the mode bits
        orr     r0,r0,#SYS_MODE                     ; Set System mode bits
        msr     cpsr_c,r0                           ; Change the mode
        ldr     sp,=SFE(CSTACK)                     ; End of CSTACK

        ;; Turn on core features assumed to be enabled.
        BL      __iar_init_core
	
        ;; Initialize VFP (if needed).
        BL      __iar_init_vfp


; Add more initialization here

; Continue to ?main for more IAR specific system startup
        B       __cmain

                END
