/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for TI Hercules RM46 HDK eval board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define HET1_GCR    (*(volatile unsigned int*)(0xFFF7B800))   // HET1 Global Configuration Register
#define HET1_PULDIS (*(volatile unsigned int*)(0xFFF7B864))   // HET1 NHET Pull Disable Register
#define HET1_DIR    (*(volatile unsigned int*)(0xFFF7B84C))   // HET1 NHET Direction Register
#define HET1_DIN    (*(volatile unsigned int*)(0xFFF7B850))   // NHET Data Input Register
#define HET1_DOUT   (*(volatile unsigned int*)(0xFFF7B854))   // NHET Data Output Register

#define LED_HET1_17 (1u << 17) // Left Top
#define LED_HET1_31 (1u << 31) // Top
#define LED_HET1_00 (1u << 00) // Right Top
#define LED_HET1_25 (1u << 25) // Right Bottom
#define LED_HET1_18 (1u << 18) // Bottom
#define LED_HET1_29 (1u << 29) // Left Bottom
#define LED_HET1_27 (1u << 27) // Left
#define LED_HET1_05 (1u << 05) // Right

#define LED_ALL (LED_HET1_17 | LED_HET1_31 | LED_HET1_00 | LED_HET1_25 | LED_HET1_18 | LED_HET1_29 | LED_HET1_27 | LED_HET1_05)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  // 
  // NHET1 is configured as Master, NHET1 is turned On
  //
	HET1_GCR     = 0x01000001;  // Turn HET on
  HET1_PULDIS  = 0x00000000;  // PULL functinality is enabled
  HET1_DIR    |= LED_ALL;     // Configure NHET pins as output
  HET1_DOUT   &= ~LED_ALL;    // Turn all LEDs off
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
	if (Index == 0) {
    HET1_DOUT = HET1_DIN | LED_HET1_31;
	} else {
		if (Index == 1) {
      HET1_DOUT = HET1_DIN | LED_HET1_18;
	  } 
	}
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
	if (Index == 0) {
    HET1_DOUT = HET1_DIN & ~LED_HET1_31;
	} else {
		if (Index == 1) {
      HET1_DOUT = HET1_DIN & ~LED_HET1_18;
	  } 
	}
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
	if (Index == 0) {
		if (HET1_DIN & LED_HET1_31) {
      HET1_DOUT = HET1_DIN & ~LED_HET1_31;
		} else {
			HET1_DOUT = HET1_DIN | LED_HET1_31;
		}
	} else {
		if (Index == 1) {
		  if (HET1_DIN & LED_HET1_18) {
        HET1_DOUT = HET1_DIN & ~LED_HET1_18;
		  } else {
			  HET1_DOUT = HET1_DIN | LED_HET1_18;
		  }
		}
	}
}

/****** End Of File *************************************************/
