/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : UART implementation

Additional information:

  Device : STR912
  Board  : STR912 SK

  Unit | UART   | Board connector
  ===============================
  0    | UART0  | RS232_1
  1    | UART1  | RS232_2
  2    | UART2  | -

*/

#include "BSP_UART.h"
#include "RTOS.h"      // For OS_ARM_* and OS_INT_* functions.

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/

#define SYSTEM_FREQ                (7500000uL * 8)
#define UART_FREQ                  (SYSTEM_FREQ / 4uL)

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/

#define UART0                        (0)
#define UART1                        (1)
#define UART2                        (2)

#define UART0_BASE_ADDR              (0xFFF7F400uL)
#define UART0_CCR                    (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x03uL))
#define UART0_CTL1                   (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x07uL))
#define UART0_CTL2                   (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x0BuL))
#define UART0_CTL3                   (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x0FuL))
#define UART0_RXST                   (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x13uL))
#define UART0_HBAUD                  (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x17uL))
#define UART0_MBAUD                  (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x1BuL))
#define UART0_LBAUD                  (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x1FuL))
#define UART0_RXBUF                  (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x27uL))
#define UART0_TXBUF                  (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x2BuL))
#define UART0_PC1                    (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x2FuL))
#define UART0_PC2                    (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x33uL))
#define UART0_PC3                    (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x37uL))

#define UART1_BASE_ADDR              (0xFFF7F500uL)
#define UART1_CCR                    (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x03uL))
#define UART1_CTL1                   (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x07uL))
#define UART1_CTL2                   (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x0BuL))
#define UART1_CTL3                   (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x0FuL))
#define UART1_RXST                   (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x13uL))
#define UART1_HBAUD                  (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x17uL))
#define UART1_MBAUD                  (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x1BuL))
#define UART1_LBAUD                  (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x1FuL))
#define UART1_RXBUF                  (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x27uL))
#define UART1_TXBUF                  (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x2BuL))
#define UART1_PC1                    (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x2FuL))
#define UART1_PC2                    (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x33uL))
#define UART1_PC3                    (*(volatile unsigned char*)(UART1_BASE_ADDR + 0x37uL))

#define UART2_BASE_ADDR              (0xFFF7F600uL)
#define UART2_CCR                    (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x03uL))
#define UART2_CTL1                   (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x07uL))
#define UART2_CTL2                   (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x0BuL))
#define UART2_CTL3                   (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x0FuL))
#define UART2_RXST                   (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x13uL))
#define UART2_HBAUD                  (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x17uL))
#define UART2_MBAUD                  (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x1BuL))
#define UART2_LBAUD                  (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x1FuL))
#define UART2_RXBUF                  (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x27uL))
#define UART2_TXBUF                  (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x2BuL))
#define UART2_PC1                    (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x2FuL))
#define UART2_PC2                    (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x33uL))
#define UART2_PC3                    (*(volatile unsigned char*)(UART2_BASE_ADDR + 0x37uL))

#define UART0_RX_IRQ_ID  (10)
#define UART0_TX_IRQ_ID  (20)
#define UART1_RX_IRQ_ID  (17)
#define UART1_TX_IRQ_ID  (26)
#define UART2_RX_IRQ_ID  ( 0)
#define UART2_TX_IRQ_ID  ( 0)

#define UART_BAUDDIVIDE(Baudrate)    ((UART_FREQ + Baudrate * 4uL) / (Baudrate * 8uL) - 1)

#define UART_TX_ACTION_ENA_MASK      (1uL << 3)  // 3, Tx DMA or interrupt enable
#define UART_RX_ACTION_ENA_MASK      (1uL << 4)  // 4, Rx DMA or interrupt enable
#define UART_RXENA_MASK              (1uL << 0)  // 0, Rx enable
#define UART_TXENA_MASK              (1uL << 0)  // 0, Tx enable
#define UART_SW_NRESET_MASK          (1uL << 7)  // 7, Software reset (active low)
#define UART_RX_FUNC_MASK            (1uL << 1)  // 1, SCIRX pin function
#define UART_TX_FUNC_MASK            (1uL << 1)  // 1, SCITX pin function
#define UART_CLOCK_MASK              (1uL << 5)  // 5, Internal clock enable
#define UART_TIMING_MODE_ASYNC_MASK  (1uL << 4)  // Timing Mode
#define UART_CHAR_8_MASK             (7uL << 0)  // Character length control 8

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static BSP_UART_TX_CB* _pfWriteCB;
static BSP_UART_RX_CB* _pfReadCB;

/*********************************************************************
*
*       Prototypes
*
*  Declare ISR handler here to avoid "no prototype" warning.
*  They are not declared in any CMSIS header.
*
**********************************************************************
*/

#if defined(__cplusplus)
  extern "C" {                // Make sure we have C-declarations in C++ programs.
#endif

static void UART0_RX_IRQHandler(void);
static void UART0_TX_IRQHandler(void);
static void UART1_RX_IRQHandler(void);
static void UART1_TX_IRQHandler(void);
static void UART2_RX_IRQHandler(void);
static void UART2_TX_IRQHandler(void);

#if defined(__cplusplus)
}                             // Make sure we have C-declarations in C++ programs.
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _SetBaudrate()
*
*  Function description
*    Configures the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
static void _SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  BSP_UART_USE_PARA(Unit);
  BSP_UART_USE_PARA(Baudrate);
}

/*********************************************************************
*
*       Global functions, IRQ handler
*
**********************************************************************
*/

/*********************************************************************
*
*       UART0_RX_IRQHandler()
*
*  Function description
*    UART Rx interrupt handler.
*/
static void UART0_RX_IRQHandler(void) {
  unsigned char Data;
  unsigned char Status;
  unsigned int  Unit;

  Unit   = UART0;
  Data   = UART0_RXBUF;
  Status = UART0_RXST;
  if (Status & 0x3Fu) {  // Check if errors occurred
    UART0_RXST = 0x3Fu;  // Clear all errors
  } else {
    if (_pfReadCB != 0) {
      _pfReadCB(Unit, Data);
    }
  }
}

/*********************************************************************
*
*       UART0_TX_IRQHandler()
*
*  Function description
*    UART Tx interrupt handler.
*/
static void UART0_TX_IRQHandler(void) {
  unsigned int Unit;

  Unit = UART0;
  if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {  // No more characters to send ?
    UART0_CTL3 &= ~UART_TX_ACTION_ENA_MASK;            // disable Tx interrrupt
  }
}

/*********************************************************************
*
*       UART1_RX_IRQHandler()
*
*  Function description
*    UART Rx interrupt handler.
*/
static void UART1_RX_IRQHandler(void) {
  unsigned char Data;
  unsigned char Status;
  unsigned int  Unit;

  Unit   = UART1;
  Data   = UART1_RXBUF;
  Status = UART1_RXST;
  if (Status & 0x3Fu) {  // Check if errors occurred
    UART1_RXST = 0x3Fu;  // Clear all errors
  } else {
    if (_pfReadCB != 0) {
      _pfReadCB(Unit, Data);
    }
  }
}

/*********************************************************************
*
*       UART1_TX_IRQHandler()
*
*  Function description
*    UART Tx interrupt handler.
*/
static void UART1_TX_IRQHandler(void) {
  unsigned int Unit;

  Unit = UART1;
  if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {  // No more characters to send ?
    UART1_CTL3 &= ~UART_TX_ACTION_ENA_MASK;            // disable Tx interrrupt
  }
}

/*********************************************************************
*
*       UART2_RX_IRQHandler()
*
*  Function description
*    UART Rx interrupt handler.
*/
static void UART2_RX_IRQHandler(void) {
  unsigned char Data;
  unsigned char Status;
  unsigned int  Unit;

  Unit   = UART2;
  Data   = UART2_RXBUF;
  Status = UART2_RXST;
  if (Status & 0x3Fu) {  // Check if errors occurred
    UART2_RXST = 0x3Fu;  // Clear all errors
  } else {
    if (_pfReadCB != 0) {
      _pfReadCB(Unit, Data);
    }
  }
}

/*********************************************************************
*
*       UART2_TX_IRQHandler()
*
*  Function description
*    UART Tx interrupt handler.
*/
static void UART2_TX_IRQHandler(void) {
  unsigned int Unit;

  Unit = UART2;
  if ((_pfWriteCB != 0) && (_pfWriteCB(Unit) != 0)) {  // No more characters to send ?
    UART2_CTL3 &= ~UART_TX_ACTION_ENA_MASK;            // disable Tx interrrupt
  }
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_UART_Init()
*
*  Function description
*    Initializes the UART peripheral.
*
*  Parameters
*    Unit       : Unit number (typically zero-based).
*    Baudrate   : Baud rate to configure [Hz].
*    NumDataBits: Number of data bits to use.
*    Parity     : One of the following values:
*                   * BSP_UART_PARITY_NONE
*                   * BSP_UART_PARITY_ODD
*                   * BSP_UART_PARITY_EVEN
*    NumStopBits: Number of stop bits to use.
*/
void BSP_UART_Init(unsigned int Unit, unsigned long Baudrate, unsigned char NumDataBits, unsigned char Parity, unsigned char NumStopBits) {
  BSP_UART_USE_PARA(NumDataBits);
  BSP_UART_USE_PARA(Parity);
  BSP_UART_USE_PARA(NumStopBits);
  OS_INT_IncDI();
  switch (Unit) {
  case UART0:
    UART0_CTL3 &= ~UART_SW_NRESET_MASK;                               // Reset SCI state machine
    UART0_CCR   = UART_TIMING_MODE_ASYNC_MASK + UART_CHAR_8_MASK;     // Async, 8-bit Char
    UART0_CTL1 |= UART_RXENA_MASK;                                    // RX enabled
    UART0_CTL2 |= UART_TXENA_MASK;                                    // TX enabled
    UART0_CTL3 |= UART_CLOCK_MASK + UART_RX_ACTION_ENA_MASK;          // Internal clock, enable Rx interrrupt
    UART0_HBAUD = (UART_BAUDDIVIDE(Baudrate) >> 16) & 0xFFuL;
    UART0_MBAUD = (UART_BAUDDIVIDE(Baudrate) >>  8) & 0xFFuL;
    UART0_LBAUD =  UART_BAUDDIVIDE(Baudrate) & 0xFFuL;
    UART0_PC2  |= UART_RX_FUNC_MASK;                                  // SCIRX is the SCI receive pin
    UART0_PC3  |= UART_TX_FUNC_MASK;                                  // SCITX is the SCI transmit pin
    UART0_CTL3 |= UART_SW_NRESET_MASK;                                // Configure SCI2 state machine
    //
    // Setup interrupt controller for OS UART
    //
    OS_ARM_InstallISRHandler(UART0_RX_IRQ_ID, &UART0_RX_IRQHandler);  // OS UART interrupt handler Rx
    OS_ARM_InstallISRHandler(UART0_TX_IRQ_ID, &UART0_TX_IRQHandler);  // OS UART interrupt handler Tx
    OS_ARM_EnableISR        (UART0_RX_IRQ_ID);                        // Enable OS UART interrupts
    OS_ARM_EnableISR        (UART0_TX_IRQ_ID);                        // Enable OS UART interrupts
    break;
  case UART1:
    UART1_CTL3 &= ~UART_SW_NRESET_MASK;                               // Reset SCI state machine
    UART1_CCR   = UART_TIMING_MODE_ASYNC_MASK | UART_CHAR_8_MASK;     // Async, 8-bit Char
    UART1_CTL1 |= UART_RXENA_MASK;                                    // RX enabled
    UART1_CTL2 |= UART_TXENA_MASK;                                    // TX enabled
    UART1_CTL3 |= UART_CLOCK_MASK | UART_RX_ACTION_ENA_MASK;          // Internal clock, enable Rx interrrupt
    UART1_HBAUD = (UART_BAUDDIVIDE(Baudrate) >> 16) & 0xFFuL;
    UART1_MBAUD = (UART_BAUDDIVIDE(Baudrate) >>  8) & 0xFFuL;
    UART1_LBAUD =  UART_BAUDDIVIDE(Baudrate) & 0xFFuL;
    UART1_PC2  |= UART_RX_FUNC_MASK;                                  // SCIRX is the SCI receive pin
    UART1_PC3  |= UART_TX_FUNC_MASK;                                  // SCITX is the SCI transmit pin
    UART1_CTL3 |= UART_SW_NRESET_MASK;                                // Configure SCI2 state machine
    //
    // Setup interrupt controller for OS UART
    //
    OS_ARM_InstallISRHandler(UART1_RX_IRQ_ID, &UART1_RX_IRQHandler);  // OS UART interrupt handler Rx
    OS_ARM_InstallISRHandler(UART1_TX_IRQ_ID, &UART1_TX_IRQHandler);  // OS UART interrupt handler Tx
    OS_ARM_EnableISR        (UART1_RX_IRQ_ID);                        // Enable OS UART interrupts
    OS_ARM_EnableISR        (UART1_TX_IRQ_ID);                        // Enable OS UART interrupts
    break;
  case UART2:
    UART2_CTL3 &= ~UART_SW_NRESET_MASK;                               // Reset SCI state machine
    UART2_CCR   = UART_TIMING_MODE_ASYNC_MASK | UART_CHAR_8_MASK;     // Async, 8-bit Char
    UART2_CTL1 |= UART_RXENA_MASK;                                    // RX enabled
    UART2_CTL2 |= UART_TXENA_MASK;                                    // TX enabled
    UART2_CTL3 |= UART_CLOCK_MASK | UART_RX_ACTION_ENA_MASK;          // Internal clock, enable Rx interrrupt
    UART2_HBAUD = (UART_BAUDDIVIDE(Baudrate) >> 16) & 0xFFuL;
    UART2_MBAUD = (UART_BAUDDIVIDE(Baudrate) >>  8) & 0xFFuL;
    UART2_LBAUD =  UART_BAUDDIVIDE(Baudrate) & 0xFFuL;
    UART2_PC2  |= UART_RX_FUNC_MASK;                                  // SCIRX is the SCI receive pin
    UART2_PC3  |= UART_TX_FUNC_MASK;                                  // SCITX is the SCI transmit pin
    UART2_CTL3 |= UART_SW_NRESET_MASK;                                // Configure SCI2 state machine
    //
    // Setup interrupt controller for OS UART
    //
    OS_ARM_InstallISRHandler(UART2_RX_IRQ_ID, &UART2_RX_IRQHandler);  // OS UART interrupt handler Rx
    OS_ARM_InstallISRHandler(UART2_TX_IRQ_ID, &UART2_TX_IRQHandler);  // OS UART interrupt handler Tx
    OS_ARM_EnableISR        (UART2_RX_IRQ_ID);                        // Enable OS UART interrupts
    OS_ARM_EnableISR        (UART2_TX_IRQ_ID);                        // Enable OS UART interrupts
    break;
  }
  OS_INT_DecRI();
}

/*********************************************************************
*
*       BSP_UART_DeInit()
*
*  Function description
*    De-initializes the UART peripheral.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*/
void BSP_UART_DeInit(unsigned int Unit) {
  BSP_UART_USE_PARA(Unit);
}

/*********************************************************************
*
*       BSP_UART_SetBaudrate()
*
*  Function description
*    Configures/changes the UART baud rate.
*
*  Parameters
*    Unit    : Unit number (typically zero-based).
*    Baudrate: Baud rate to configure [Hz].
*/
void BSP_UART_SetBaudrate(unsigned int Unit, unsigned long Baudrate) {
  _SetBaudrate(Unit, Baudrate);
}

/*********************************************************************
*
*       BSP_UART_SetReadCallback()
*
*  Function description
*    Sets the callback to execute upon an Rx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetReadCallback(unsigned Unit, BSP_UART_RX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfReadCB = pf;
}

/*********************************************************************
*
*       BSP_UART_SetWriteCallback()
*
*  Function description
*    Sets the callback to execute upon a Tx interrupt.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    pf  : Callback to execute.
*/
void BSP_UART_SetWriteCallback(unsigned int Unit, BSP_UART_TX_CB* pf) {
  BSP_UART_USE_PARA(Unit);
  _pfWriteCB = pf;
}

/*********************************************************************
*
*       BSP_UART_Write1()
*
*  Function description
*    Sends one byte via UART.
*
*  Parameters
*    Unit: Unit number (typically zero-based).
*    Data: (First) data byte to send.
*
*  Additional information
*    The first byte of a transfer is typically sent from application
*    context. Further bytes of the transfer are then sent from the
*    Tx interrupt handler by also calling this function from interrupt
*    context.
*/
void BSP_UART_Write1(unsigned int Unit, unsigned char Data) {
  switch (Unit) {
  case UART0:
    UART0_TXBUF  = Data;                     // Send character
    UART0_CTL3  |= UART_TX_ACTION_ENA_MASK;  // enable Tx Empty interrupt
    break;
  case UART1:
    UART1_TXBUF  = Data;                     // Send character
    UART1_CTL3  |= UART_TX_ACTION_ENA_MASK;  // enable Tx Empty interrupt
    break;
  case UART2:
    UART2_TXBUF  = Data;                     // Send character
    UART2_CTL3  |= UART_TX_ACTION_ENA_MASK;  // enable Tx Empty interrupt
    break;
  }
}

/*************************** End of file ****************************/
