;*********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                *
;*        Solutions for real time microcontroller applications       *
;*********************************************************************
;*                                                                   *
;*      (C) 2005- 2007 SEGGER Microcontroller Systeme GmbH           *
;*                  www.segger.com                                   *
;*                                                                   *
;*********************************************************************
;
;---------------------------------------------------------------------
;File   : TMS470R1B1M_Startup.s
;Purpose: Startup file for OS and IAR embedded workbench 
;         running on TMS470
;-------- END-OF-HEADER ----------------------------------------------
;

;---------------------------------------------------------------
; ?RESET
; Reset Vector.
; Normally, segment INTVEC is linked at address 0.
; For debugging purposes, INTVEC may be placed at other
; addresses.
; A debugger that honors the entry point will start the
; program in a normal way even if INTVEC is not at address 0.
;---------------------------------------------------------------

        MODULE  ?cstartup

        ;; Forward declaration of sections.
        SECTION IRQ_STACK:DATA:NOROOT(3)
        SECTION FIQ_STACK:DATA:NOROOT(3)
        SECTION CSTACK:DATA:NOROOT(3)

        SECTION .intvec:CODE:NOROOT(2)

        PUBLIC  __vector
        PUBLIC  __vector_0x14
        PUBLIC  __iar_program_start
        EXTERN  Undefined_Handler
        EXTERN  SWI_Handler
        EXTERN  Prefetch_Handler
        EXTERN  Abort_Handler
        EXTERN  IRQ_Handler
        EXTERN  FIQ_Handler

        ARM

__iar_init$$done:               ; The vector table is not needed
                                ; until after copy initialization is done

__vector:
        ; All default exception handlers (except reset) are
        ; defined as weak symbol definitions.
        ; If a handler is defined by the application it will take precedence.

        ldr     PC,Reset_Addr           ; To allow remap, a relative branch is required !
        ldr     PC,Undefined_Addr       ; Undefined instructions
        ldr     PC,SWI_Addr             ; Software interrupt (SWI/SVC)
        ldr     PC,Prefetch_Addr        ; Prefetch abort
        ldr     PC,Abort_Addr           ; Data abort
__vector_0x14:
        dcd     0                       ; RESERVED
        ldr     PC,IRQ_Addr             ; IRQ
        ldr     PC,FIQ_Addr             ; FIQ

        DATA
    
Reset_Addr:     DCD   __iar_program_start
Undefined_Addr: DCD   Undefined_Handler
SWI_Addr:       DCD   SWI_Handler
Prefetch_Addr:  DCD   Prefetch_Handler
Abort_Addr:     DCD   Abort_Handler
IRQ_Addr:       DCD   IRQ_Handler
FIQ_Addr:       DCD   FIQ_Handler

__vector_end:


/**********************************************************************
* CSTARTUP
*
* Execution starts here.
* After a reset, the mode is ARM, Supervisor, interrupts disabled.
*/

        SECTION .text:CODE:NOROOT(2)

        EXTERN  ?main
        REQUIRE __vector

        ARM

__iar_program_start:
        ;; Set up the interrupt stack pointer.

        msr     cpsr_c,#0xd2            ; Change mode to IRQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(IRQ_STACK)     ; End of IRQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary
	
        ;; Set up the fast interrupt stack pointer.

        msr     cpsr_c,#0xd1            ; Change mode to FIQ mode, FIQ and IRQ disabled
        ldr     sp, =SFE(FIQ_STACK)     ; End of FIQ_STACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary

        ;; Set up the normal stack pointer.

        msr     cpsr_c,#0xdf            ; Change mode to SYS mode, FIQ and IRQ disabled
        ldr     sp, =SFE(CSTACK)        ; End of CSTACK
        bic     sp,sp,#0x7              ; Make sure SP is aligned on 8 byte boundary

Remap:
// Initialize Chip selects, enable internal RAM, if not already done by debugger
// if ((MFBALR0 & (1 << 8)) == 0) {  // Memory mapping enabled?
        mvn      R0,#+255
        bic      R0,R0,#0x100        // R0 now points to MFBAH0 (0xfffffe00)
        ldr      R1,[R0, #+0x04]     // load MFBALR0  (0xfffffe04)
        tst      R1,#0x100
        bne      RemapEnd

// Set memory select 0, whole FLASH, block size to 1M */
// MFBALR0  = 0x000000B0;
        mov      R1,#+176
        str      R1,[R0, #+0x04]    // Store MFBALR0  (0xfffffe04)
// MFBAHR0  = 0x00000000;
        mov      R1,#+0
        str      R1,[R0, #+0]       // Store MFBAHR0  (0xfffffe00)

// Disable FLASH which is connected to memory select 1
// MFBALR1  = 0x00000000;
// MFBAHR1  = 0x00000000;
        str      R1,[R0, #+0x0c]    // Store MFBAHR1  (0xfffffe08)
        str      R1,[R0, #+0x08]    // Store MFBAHR1  (0xfffffe08)

// activate 64 K RAM at 0x00400000 that is connected to memory select 2 */
// MFBAHR2  = 0x00000040;
//      mov      R2,#SFB(DATA_I) >> 16
        mov      R2,#0x40
        str      R2,[R0, #+0x10]    // Store MFBAHR2  (0xfffffe10)
// MFBALR2  = 0x00000070;
        mov      R2,#+112
        str      R2,[R0, #+0x14]    // Store MFBALR2  (0xfffffe14)

// Disable RAM which is connected to memory select 3 */
// MFBAHR3  = 0x00000000;
        str      R1,[R0, #+0x18]    // Store MFBAHR3  (0xfffffe18)
// MFBALR3  = 0x00000000;
        str      R1,[R0, #+0x1c]    // Store MFBALR3  (0xfffffe1c)

// Activate HET RAM at 0x00800000 1k that is connected to memory select 4 */
// MFBAHR4  = 0x00000080;
        mov      R2,#+128
        str      R2,[R0, #+0x20]    // Store MFBAHR4  (0xfffffe20)

// MFBALR4  = 0x00000020;
        mov      R2,#+32
        str      R2,[R0, #+0x24]    // Store MFBALR4  (0xfffffe24)

// Activate new mapping by writing 1 to MFBALR0.8 */
// MFBALR0  = 0x000001B0;
        mov      R1,#+432
        str      R1,[R0, #+0x04]    // Store MFBALR0  (0xfffffe04)
RemapEnd:

; Jump to startup code in library
        ldr     r0,=?main
        bx      r0

; ********************************************************************

        END

; ***** End of File **************************************************
