/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit_TMS570LS12x.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ  (4000000u)  // RTICLK / (RTICPUC0 + 1) = (VCLK = OSCIN = 16 MHz / 2) / (RTICPUC0 + 1)
#define OS_TICK_FREQ   (1000u)
#define OS_INT_FREQ    (OS_TICK_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_DISABLED
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Device specific SFRs
*/
#define RCLKSRC               (*(volatile unsigned int*)(0xFFFFFF50))   // RTI Clock Source Register
#define RTIGCTRL              (*(volatile unsigned int*)(0xFFFFFC00))   // RTI Global Control Register Section
#define RTITBCTRL             (*(volatile unsigned int*)(0xFFFFFC04))   // RTI Timebase Control Register Section
#define RTICAPCTRL            (*(volatile unsigned int*)(0xFFFFFC08))   // RTI Capture Control Register Section
#define RTICOMPCTRL           (*(volatile unsigned int*)(0xFFFFFC0C))   // RTI Compare Control Register Section
#define RTIFRC0               (*(volatile unsigned int*)(0xFFFFFC10))   // RTI Free Running Counter 0 Register Section
#define RTIUC0                (*(volatile unsigned int*)(0xFFFFFC14))   // RTI Up Counter 0 Register Section
#define RTICPUC0              (*(volatile unsigned int*)(0xFFFFFC18))   // RTI Compare Up Counter 0 Register Section
#define RTICAFRC0             (*(volatile unsigned int*)(0xFFFFFC20))   // RTI Capture Free Running Counter 0 Register Section
#define RTICAUC0              (*(volatile unsigned int*)(0xFFFFFC24))   // RTI Capture Up Counter 0 Register Section
#define RTICOMP0              (*(volatile unsigned int*)(0xFFFFFC50))   // RTI Compare 0 Register Section
#define RTIUDCP0              (*(volatile unsigned int*)(0xFFFFFC54))   // RTI Update Compare 0 Register Section
#define RTISETINTENA          (*(volatile unsigned int*)(0xFFFFFC80))   // RTI Set Interrupt Enable Register Section
#define RTIINTFLAG            (*(volatile unsigned int*)(0xFFFFFC88))   // RTI Interrupt Flag Register Section

#define RTI_INT_ID            (2u)
#define RTIINTFLAG_INT0_MASK  (1u << 0)
#define VIM_RAM_ADDR          (0xFFF82000u) // Fixed vector table start addr in RAM

#define FIRQPR0               (*(volatile unsigned int*)(0xFFFFFE10))   // FIQ/IRQ Program Control Register 0
#define FIRQPR1               (*(volatile unsigned int*)(0xFFFFFE14))   // FIQ/IRQ Program Control Register 1
#define FIRQPR2               (*(volatile unsigned int*)(0xFFFFFE18))   // FIQ/IRQ Program Control Register 2
#define FIRQPR3               (*(volatile unsigned int*)(0xFFFFFE1C))   // FIQ/IRQ Program Control Register 3
#define REQENASET0            (*(volatile unsigned int*)(0xFFFFFE30))   // Interrupt Enable Set Register 0
#define REQENASET1            (*(volatile unsigned int*)(0xFFFFFE34))   // Interrupt Enable Set Register 1
#define REQENASET2            (*(volatile unsigned int*)(0xFFFFFE38))   // Interrupt Enable Set Register 2
#define REQENASET3            (*(volatile unsigned int*)(0xFFFFFE3C))   // Interrupt Enable Set Register 3
#define REQENACLR0            (*(volatile unsigned int*)(0xFFFFFE40))   // Interrupt Enable CLR Register 0
#define REQENACLR1            (*(volatile unsigned int*)(0xFFFFFE44))   // Interrupt Enable CLR Register 1
#define REQENACLR2            (*(volatile unsigned int*)(0xFFFFFE48))   // Interrupt Enable CLR Register 2
#define REQENACLR3            (*(volatile unsigned int*)(0xFFFFFE4C))   // Interrupt Enable CLR Register 3
#define IRQVECREG             (*(volatile unsigned int*)(0xFFFFFE70))   // IRQ Interrupt Vector Register

#define NUM_INT_SOURCES       (120u)

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USE_PARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USE_PARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USE_PARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USE_PARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when a non-installed interrupt was detected.
*/
static void _OS_ISR_Undefined(void) {
  volatile int Dummy;

  Dummy = 1;
  while (Dummy > 0) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
}

/*********************************************************************
*
*       _InitVIM()
*
*  Function description
*    Initializes the Vector interrupt controller manager
*/
static void _InitVIM(void) {
  OS_ISR_HANDLER** papISR;
  OS_U32 i;
  //
  // Map all interrupts to IRQ
  //
  FIRQPR0 = 0u;
  FIRQPR1 = 0u;
  FIRQPR2 = 0u;
  FIRQPR3 = 0u;
  //
  // Fill vector table with default interrupt handler
  //
  papISR = (OS_ISR_HANDLER**)VIM_RAM_ADDR;
  for (i = 0 ; i < NUM_INT_SOURCES ; i++) {
    *(papISR + i) = &_OS_ISR_Undefined;
  }
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  unsigned int Compare;

  Compare = RTICOMP0;
  return RTIFRC0 - (Compare - (OS_TIMER_FREQ / OS_INT_FREQ));
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return RTIINTFLAG & RTIINTFLAG_INT0_MASK;
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
static void _OS_ISR_Tick(void) {
  RTIINTFLAG = RTIINTFLAG_INT0_MASK; // Clear the pending interrupts flag
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/
/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize interrupt controller
  //
  _InitVIM();
  //
  // Initialize timer for embOS
  //
  RCLKSRC      = 0x08;                         // Clock source 8 i.e VClk, div the clk source 0 by 1
  RTIGCTRL     = 0x00000000;                   // Disable RTIUC0 and RTIUC1
  RTIUC0       = 0x00000000;                   // Initialize up Counter
  RTIFRC0      = 0x00000000;                   // Initialize Free Running Counter
  RTICPUC0     = 0x00000001;                   // Set Compare Up Counter value to Prescale RTICLK.
  RTICOMP0     = OS_TIMER_FREQ / OS_INT_FREQ;  // Compare value
  RTIUDCP0     = OS_TIMER_FREQ / OS_INT_FREQ;  // This value is added to generate new compare value since the timer register counts continuously
  RTIINTFLAG   = 0x0000000F;                   // Clearing the pending Interrupts Flags
  RTICOMPCTRL  = 0x00000000;                   // Initializing the Compare Counter register
  RTISETINTENA = 0x00000001;                   // Enabling compare interrupt 0
  RTIGCTRL     = 0x00000001;                   // Enable the RTIUC0
  OS_ARM_InstallISRHandler(RTI_INT_ID, _OS_ISR_Tick);
  OS_ARM_EnableISR(RTI_INT_ID);
  //
  // Inform embOS about the timer settings
  //
  {
    OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                      // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;

  pISR = (OS_ISR_HANDLER*)IRQVECREG;  // Read interrupt vector
  OS_INT_Enter();                     // Inform OS that interrupt handler is running
  pISR();                             // Call interrupt service routine
  OS_INT_Leave();                     // Leave interrupt, perform task switch if required
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Installs an interrupt handler
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, usually the interrupt vector number
*    pISRHandler: Address of the interrupt handler function
*
*  Return value
*    The address of the interrupt handler that was previously installed with the addressed interrupt source
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  papISR                     = (OS_ISR_HANDLER**)VIM_RAM_ADDR;
  pOldHandler                = *(papISR + (ISRIndex + 1u));
  *(papISR + (ISRIndex + 1)) = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Enables an interrupt
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  if (ISRIndex < 32) {
    REQENASET0 = (1u << (ISRIndex -  0));
  } else if (ISRIndex < 64) {
    REQENASET1 = (1u << (ISRIndex - 32));
  } else if (ISRIndex < 96) {
    REQENASET2 = (1u << (ISRIndex - 64));
  } else {
    REQENASET3 = (1u << (ISRIndex - 96));
  }
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Disables an interrupt
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  if (ISRIndex < 32) {
    REQENACLR0 = (1u << (ISRIndex -  0));
  } else if (ISRIndex < 64) {
    REQENACLR1 = (1u << (ISRIndex - 32));
  } else if (ISRIndex < 96) {
    REQENACLR2 = (1u << (ISRIndex - 64));
  } else {
    REQENACLR3 = (1u << (ISRIndex - 96));
  }
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL.
*
*    Has to be modified if another CPU clock frequency should be
*    used. This function is called during startup and has to
*    return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
OS_INTERWORK int __low_level_init(void);
OS_INTERWORK int __low_level_init(void) {
  //
  // Enable MPU, Caches and branch prediction unit
  //
  OS_ARM_DCACHE_Enable();
  OS_ARM_ICACHE_Enable();
  OS_ARM_MPU_Init();
  //
  // Add MPU regions to set cache settings for different memory sections
  // The following entries are just an example and need to be modified according to the application
  //
  OS_ARM_MPU_AddEntry(0u, (void*)0x00000000, 0x00140000, OS_ARM_MPU_READONLY,  OS_ARM_CACHEMODE_WRITE_BACK_ALLOC);  // FLASH
  OS_ARM_MPU_AddEntry(1u, (void*)0x08000000, 0x00030000, OS_ARM_MPU_READWRITE, OS_ARM_CACHEMODE_WRITE_BACK_ALLOC);  // RAM
  OS_ARM_MPU_Enable();
  return 1;
}

/*************************** End of file ****************************/
