/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose: Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Configuration
*/
#ifndef   IMAGE_TARGET          // Special version for eval-kit
  #define IMAGE_TARGET     (0)  // Defaults to 0, may be overwritten by project settings
#endif

#ifndef   SDRAM_TARGET
  #define SDRAM_TARGET     (0)
#endif

#ifndef   RAM_TARGET
  #define RAM_TARGET       (0)
#endif

#ifndef   SDRAM_BASE_ADDR
  #define SDRAM_BASE_ADDR  (0x40000000u)
#endif

/*********************************************************************
*
*       Clock settings
*/
#define OS_SYSTEM_FREQ      (192000000u)

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ       (OS_SYSTEM_FREQ / 2 / 2)
#define OS_TICK_FREQ        (1000u)
#define OS_INT_FREQ         (OS_TICK_FREQ)
#define OS_TIMER_RELOAD     ((OS_TIMER_FREQ / OS_INT_FREQ) - 1)
#define OS_TIMER_BASE_ADDR  (TIMER5_BASE_ADDR)
#define OS_TIMER_INT_INDEX  (4)
#define OS_TIMER_INT_PRIO   (INT_PRIORITY_LOWEST)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKDCC.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
//
// System control
//
#define SYSCR_BASE_ADDR         (0xF0050000u)
#define SYSCR0                  (*(volatile OS_U32*)(SYSCR_BASE_ADDR + 0x0000u))
#define SYSCR1                  (*(volatile OS_U32*)(SYSCR_BASE_ADDR + 0x0004u))
#define SYSCR2                  (*(volatile OS_U32*)(SYSCR_BASE_ADDR + 0x0008u))
#define SYSCR3                  (*(volatile OS_U32*)(SYSCR_BASE_ADDR + 0x000cu))
#define SYSCR4                  (*(volatile OS_U32*)(SYSCR_BASE_ADDR + 0x0010u))
#define SYSCR5                  (*(volatile OS_U32*)(SYSCR_BASE_ADDR + 0x0014u))
#define SYSCR6                  (*(volatile OS_U32*)(SYSCR_BASE_ADDR + 0x0018u))
#define SYSCR7                  (*(volatile OS_U32*)(SYSCR_BASE_ADDR + 0x001cu))
//
// Timer
//
#define TIMER0_BASE_ADDR        (0xF0040000u)
#define TIMER1_BASE_ADDR        (0xF0040100u)
#define TIMER2_BASE_ADDR        (0xF0041000u)
#define TIMER3_BASE_ADDR        (0xF0041100u)
#define TIMER4_BASE_ADDR        (0xF0042000u)
#define TIMER5_BASE_ADDR        (0xF0042100u)

#define OS_TIMER_LOAD           (*(volatile OS_U32*)(OS_TIMER_BASE_ADDR + 0x0000u))
#define OS_TIMER_VALUE          (*(volatile OS_U32*)(OS_TIMER_BASE_ADDR + 0x0004u))
#define OS_TIMER_CTRL           (*(volatile OS_U32*)(OS_TIMER_BASE_ADDR + 0x0008u))
#define OS_TIMER_INTCLR         (*(volatile OS_U32*)(OS_TIMER_BASE_ADDR + 0x000Cu))
#define OS_TIMER_RIS            (*(volatile OS_U32*)(OS_TIMER_BASE_ADDR + 0x0010u))
#define OS_TIMER_MODE           (*(volatile OS_U32*)(OS_TIMER_BASE_ADDR + 0x001Cu))

#define OS_CLKCR5               (*(volatile OS_U32*)(0xF0050054u))
//
// MATRIX + EBI interface
//
#define MATRIX_BASE_ADDR        (0xFFFFEE00u)                                    // MATRIX Base Address
#define MATRIX_MCFG             (*(volatile OS_U32*)(MATRIX_BASE_ADDR + 0x00u))  // MATRIX Master configuration register
#define MATRIX_EBICSA           (*(volatile OS_U32*)(MATRIX_BASE_ADDR + 0x30u))  // MATRIX EBI Chip Select Assignment register
//
// Vectored interrupt controller (VIC)
//
#define VIC_BASE_ADDR           (0xF4000000u)
#define VIC_IRQSTATUS           (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0000u))
#define VIC_FIQSTATUS           (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0004u))
#define VIC_RAWINTERRUPT        (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0008u))
#define VIC_INTSELECT           (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x000Cu))
#define VIC_INTENABLE           (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0010u))
#define VIC_INTENABLECLEAR      (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0014u))
#define VIC_SOFTINT             (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0018u))
#define VIC_SOFTINTCLEAR        (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x001Cu))
#define VIC_VECTORADDR          (*(volatile OS_U32*)(VIC_BASE_ADDR + 0x0F00u))

#define VIC_VECTADDR_BASE_ADDR  (VIC_BASE_ADDR + 0x0100u)
#define VIC_VECTPRIO_BASE_ADDR  (VIC_BASE_ADDR + 0x0200u)

#ifndef   NUM_INT_VECTORS
  #define NUM_INT_VECTORS       (32u)
#endif

#define INT_PRIORITY_MASK       (0x0Fu)
#define NUM_INT_PRIORITIES      (16u)

#define INT_PRIORITY_LOWEST     (0)
//
// PMC
//
#define PMC_BASE_ADDR           (0xF0020000u)
#define PMCDRV                  (*(volatile OS_U32*)(PMC_BASE_ADDR + 0x260u))
//
// SDRAM controller
//
#define DDR_MPMC_BASE_ADDR      (0xf4300000u)
#define DMC_MEMC_CMD            (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0004u))
#define DMC_DIRECT_CMD          (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0008u))
#define DMC_MEMORY_CFG          (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x000Cu))
#define DMC_REFRESH_PRD         (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0010u))
#define DMC_CAS_LATENCY         (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0014u))
#define DMC_T_DQSS              (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0018u))
#define DMC_T_MRD               (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x001Cu))
#define DMC_T_RAS               (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0020u))
#define DMC_T_RC                (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0024u))
#define DMC_T_RCD               (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0028u))
#define DMC_T_RFC               (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x002Cu))
#define DMC_T_RP                (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0030u))
#define DMC_T_RRD               (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0034u))
#define DMC_T_WR                (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0038u))
#define DMC_T_WTR               (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x003Cu))
#define DMC_T_XP                (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0040u))
#define DMC_T_XSR               (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0044u))
#define DMC_T_ESR               (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0048u))
#define DMC_CHIP_0_CFG          (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0200u))
#define DMC_CHIP_1_CFG          (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0204u))
#define DMC_CHIP_2_CFG          (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0208u))
#define DMC_CHIP_3_CFG          (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x020Cu))
#define DMC_USER_CONFIG         (*(volatile OS_U32*)(DDR_MPMC_BASE_ADDR + 0x0304u))

#define MEMC_CMD_GO             (0)

/*********************************************************************
*
*       local data
*
**********************************************************************
*/

/*********************************************************************
*
*       MMU and cache configuration
*
*  Notes
*    The translation table has to be aligned on 16384 bytes boundary
*    The translation table has to be located in un-initialized memory,
*    because it may be filled from startup, before segments and variables
*    are initialized.
*/
#if   (defined(__ICCARM__))  // IAR
  #pragma data_alignment=16384
  static __no_init OS_U32 _TranslationTable[0x1000];
#elif (defined(__CC_ARM))
  #pragma arm section zidata = "NO_INIT"  // Has to be declared as separate section UNINIT in the scatter file
  __align(16384) static OS_U32 _TranslationTable[0x1000];
  #pragma arm section zidata              // back to default (.bss section)
#elif (defined(__GNUC__))    // GCC
  static OS_U32 _TranslationTable[0x1000] __attribute__ ((aligned (16384), section (".no_init")));
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  OS_USEPARA(Channel);  // Not supported by this version, avoid compiler warning
  OS_COM_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  OS_USEPARA(Channel);  // Not supported by this version, suppress warning
  OS_COM_OnTx();
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USEPARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USEPARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _InitVIC()
*
*  Function description
*    Clears and disables all interrupt vectors in VIC.
*
*  Note
*    This function may be called from __low_level_init() and therefore
*    must not use or call any function which relies on any variable,
*    because variables are not initialized before __low_level_init() is called !
*/
static void _InitVIC(void) {
  OS_ISR_HANDLER** papISR;
  unsigned int     Index;

  VIC_INTENABLECLEAR  = 0xFFFFFFFFu;           // Disable all source interrupts
  VIC_VECTORADDR      = 0;                     // Write to vector, reset VIC
  papISR              = (OS_ISR_HANDLER**) VIC_VECTADDR_BASE_ADDR;
  for (Index = 0; Index < NUM_INT_VECTORS; Index++) {
    *(papISR + Index) = NULL;                  // Set vector to 0;
  }
  VIC_SOFTINTCLEAR    = 0xFFFFFFFFu;           // Disable all soft interrupts
  VIC_INTSELECT       = 0;                     // Set all channels to generate IRQ
}

/*********************************************************************
*
*       _InitClocks()
*
*  Function description
*    Initialize main clock and PLL
*
*  Notes
*    (2) This function may be called from __low_level_init() and therefore
*        must not use or call any function which relies on any variable,
*        because variables are not initialized before __low_level_init()
*        is called !
*    (1) Settings used here are for TMPA-910 eval board
*        and may have to be modified for other target boards or frequency
*/
static void _InitClocks(void) {
  unsigned int RegData;

  SYSCR3 =  0x07u;        // Set PLL multiplier (7 => multiply by 8)
  SYSCR4 = (0x96u << 4)   // RS value, PLL output frequency range selection: >= 140MHz
         | (0x01u << 2)   // IS setting, Always write 1!
         | (0x01u << 0)   // FS value, PLL output frequency range selection: >= 140MHz
         ;
  SYSCR3 = 0x87u;
  do {
   RegData = SYSCR2;
  } while ((RegData & 0x1u) == 0);  // Wait until PLL is locked
  SYSCR2 = 0x02u;                   // Select PLL output as system clock source
#if ((defined __ICCARM__) && (__VER__ < 6300000))
  //
  // Wait 5 clock cycles
  //
  __asm("nop");
  __asm("nop");
  __asm("nop");
  __asm("nop");
  __asm("nop");
  SYSCR1 = 0;  // System clock gear = 0 => Full speed
  //
  // Wait 5 clock cycles
  //
  __asm("nop");
  __asm("nop");
  __asm("nop");
  __asm("nop");
  __asm("nop");
#else
  //
  // Wait 5 clock cycles
  //
  __asm volatile("nop");
  __asm volatile("nop");
  __asm volatile("nop");
  __asm volatile("nop");
  __asm volatile("nop");
  SYSCR1 = 0;  // System clock gear = 0 => Full speed
  //
  // Wait 5 clock cycles
  //
  __asm volatile("nop");
  __asm volatile("nop");
  __asm volatile("nop");
  __asm volatile("nop");
  __asm volatile("nop");
#endif
}

/*********************************************************************
*
*       _InitSDRAM()
*
* Function description
*   Initialize SDRAM controller
*
* NOTES: (1) Settings used here are for TMPA-910 eval board
*            and may have to be modified for other target boards or frequency
*
*        (2) This function may be called from __low_level_init() and therefore
*            must not use or call any function which relies on any variable,
*            because variables are not initialized before __low_level_init()
*            is called !
*/
static void _InitSDRAM(void) {
  #if (SDRAM_TARGET == 0)  // Must not be initialized, if program was downloaded to SDRAM
  //
  // Port drive setting
  //
  PMCDRV          = 0x00000071u;
  //
  // Initialize SDRAM timing paramater
  //
  DMC_CAS_LATENCY = 0x00000006u;
  DMC_T_DQSS      = 0x00000000u;
  DMC_T_MRD       = 0x00000002u;
  DMC_T_RAS       = 0x00000007u;
  DMC_T_RC        = 0x0000000Au;
  DMC_T_RCD       = 0x00000013u;
  DMC_T_RFC       = 0x0000010Au;
  DMC_T_RP        = 0x00000013u;
  DMC_T_RRD       = 0x00000002u;
  DMC_T_WR        = 0x00000002u;
  DMC_T_WTR       = 0x00000001u;
  DMC_T_XP        = 0x0000000Au;
  DMC_T_XSR       = 0x0000000Bu;
  DMC_T_ESR       = 0x00000014u;
  //
  // Configure SDRAM type parameter
  //
  DMC_MEMORY_CFG  = 0x00008011u;
  DMC_USER_CONFIG = 0x00000011u;   // 32 bit memory interface
  DMC_REFRESH_PRD = 0x00000A60u;
  DMC_CHIP_0_CFG  = 0x000140FFu;
  DMC_DIRECT_CMD  = 0x000C0000u;   // NOP
  DMC_DIRECT_CMD  = 0x00000000u;   // Precharge All
  DMC_DIRECT_CMD  = 0x00040000u;   // AutoRefresh
  DMC_DIRECT_CMD  = 0x00040000u;   // AutoRefresh
  DMC_DIRECT_CMD  = 0x00080031u;   // Mode setup, 3 CAS delay, 2 byte burst
  //
  // Finally start SDRAM
  //
  DMC_MEMC_CMD    = MEMC_CMD_GO;
  #endif
}

/*********************************************************************
*
*       _CopyToSDRAM()
*
* Function description
*   Copy whole application from Flash to SDRAM, which has to be initialized before
*
* NOTES: (1) Settings used here are for TMPA-910 eval board
*            and may have to be modified for other target boards
*
*        (2) This function may be called from __low_level_init() and therefore
*            must not use or call any function which relies on any variable,
*            because variables are not initialized before __low_level_init()
*            is called !
*
*        (3) The function has to be called, before MMU is initialized and
*            SDRAM is remapped to 0x0.
*/
#if (RAM_TARGET == 0) // Not used for target application which runs in internal RAM, avoid warning
#if (defined(__ICCARM__)) && (__VER__ >= 500)
static void _CopyToSDRAM(void) {
  unsigned long Size;
  #pragma segment="PROGRAM_DATA"
  #pragma section=".intvec"
  Size = (unsigned long) __segment_end("PROGRAM_DATA") - (unsigned long) __segment_begin(".intvec");
  memcpy((void*)SDRAM_BASE_ADDR, (void*)__segment_begin(".intvec"), Size);
}
#else
static void _CopyToSDRAM(void) {
  unsigned long Size;
  #pragma segment="PROGRAM_END"
  #pragma segment="INTVEC"
  Size = (unsigned long) __segment_begin("PROGRAM_END") - (unsigned long) __segment_begin("INTVEC");
  memcpy((void*)SDRAM_BASE_ADDR, (void*)__segment_begin("INTVEC"), Size);
}
#endif  // __ICCARM__ && __VER__ >= 500
#endif  // RAM_TARGET == 0

/*********************************************************************
*
*       _OS_ISR_Undefined()
*
*  Function description
*    Is called when a not installed interrupt was detected.
*    As interrupt pending condition of peripherals has to be reset,
*    program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(void) {
#if (OS_DEBUG != 0)
  volatile OS_U32 Dummy;

  Dummy = 1;
  while (Dummy > 0) {
    ;  // You may set a breakpoint here to detect non-installed ISR-handler
  }
#endif
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return OS_TIMER_VALUE;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return OS_TIMER_RIS & (1u << 0);
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*
*  Note:
*    Two hardware timer share the same interrupt vector.
*    If user application uses the second timer and interrupts are
*    enabled for the second timer, this interrupt has to be handled
*    and cleared here.
*/
static void _OS_ISR_Tick(void) {
  OS_TIMER_INTCLR = 1u;  // Writing any value Clears OS timer interrupt
  OS_TICK_Handle();      // Call OS tick handler
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_Process();
#endif
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  OS_ARM_CACHE_Sync();              // Ensure, caches are synchronized
  OS_TIMER_CTRL = 0;                // Stop timer
  OS_CLKCR5    |= 0x0Fu;            // Default value after Reset = fHCLK / 2
  OS_TIMER_LOAD = OS_TIMER_RELOAD;  // Set timer interval
  OS_TIMER_CTRL = (1u << 7)
                | (1u << 6)
                | (1u << 5)
                | (0u << 4)
                | (0u << 3)
                | (0u << 2)
                | (1u << 1)
                | (0u << 0)
                ;
  //
  // Enable timer interrupt in interrupt controller
  //
  OS_ARM_InstallISRHandler(OS_TIMER_INT_INDEX, _OS_ISR_Tick);
  OS_ARM_ISRSetPrio(OS_TIMER_INT_INDEX, INT_PRIORITY_LOWEST);
  OS_ARM_EnableISR(OS_TIMER_INT_INDEX);
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKDCC_SendCharOnChannelNB(1, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
*  Function description
*    Detect reason for IRQ and call corresponding service routine.
*    OS_irq_handler is called from the IRQ_handler function
*    found in RTOS.s.
*    IRQ_Handler has to be inserted in the vector table in the
*    startup code.
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;

  pISR = (OS_ISR_HANDLER*)VIC_VECTORADDR;  // Get current interrupt handler
  OS_INT_EnterNestable();
  if (pISR != NULL) {
    pISR();                                // Call installed interrupt service routine
  } else {
    _OS_ISR_Undefined();
  }
  VIC_VECTORADDR = 0;                      // Clear current interrupt pending condition, reset VIC
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler()
*
*  Function description
*    Used to install a specific interrupt vector when ARM
*    CPUs with vectored interrupt controller are used.
*
*  Parameters
*    ISRIndex:    Index of the interrupt source, normally the interrupt vector number.
*    pISRHandler: Address of the interrupt handler function.
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler(int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  papISR               = (OS_ISR_HANDLER**)VIC_VECTADDR_BASE_ADDR;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_INT_EnableConditional();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR()
*
*  Function description
*    Used to enable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be enabled.
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  VIC_INTENABLE |= (1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_DisableISR()
*
*  Function description
*    Used to disable interrupt acceptance of a specific interrupt
*    source in a vectored interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be disabled.
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  VIC_INTENABLECLEAR = (1u << ISRIndex);
  OS_INT_EnableConditional();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio()
*
*  Function description
*    Used to set or modify the priority of a specific interrupt
*    source by programming the interrupt controller.
*
*  Parameters
*    ISRIndex: Index of the interrupt source which should be modified.
*    Prio:     The priority which should be set for the specific interrupt.
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if (OS_DEBUG != 0)
  if ((unsigned int)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
  }
#endif
  OS_INT_Disable();
  pPrio           = (OS_U32*)VIC_VECTPRIO_BASE_ADDR;
  OldPrio         = pPrio[ISRIndex];
  pPrio[ISRIndex] = (OldPrio & ~INT_PRIORITY_MASK) | (Prio & INT_PRIORITY_MASK);
  OS_INT_EnableConditional();
  return OldPrio & INT_PRIORITY_MASK;
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Initialize memory controller, clock generation and PLL
*
*    Has to be modified, if another CPU clock frequency should be
*    used. This function is called during startup and
*    has to return 1 to perform segment initialization.
*
*    Because variables are not initialized before __low_level_init() is called,
*    no access or call which relies on any variable can be used here.
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  //
  //  Initialize main clock oscillator and PLLs
  //
  _InitClocks();
  //
  //  Initialize SDRAM
  //
  _InitSDRAM();
  //
  // Copy program from Flash to SDRAM
  //
#if (RAM_TARGET == 0)  // If application is not built to run in internal RAM
  _CopyToSDRAM();
#endif
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Memory mapping should be complete, meaning 4096 entries.
  // Code below fills in ascending VAddr order
  //
  //                                             Mode                    VAddr  PAddr  Size[MB]
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x000, 0x400, 0x020);    // SDRAM, the first 32 MB (program area) are remapped to 0 to map vectors to correct address, cacheable, bufferable
#if (IMAGE_TARGET == 1)                                                                           // Special version for eval kit software
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x020, 0x0,   0x1E2);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x202, 0x400, 0x00E);    // SDRAM remapped over Flash area, 14MB
#else
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x020, 0x0,   0x1E0);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x200, 0x400, 0x010);    // SDRAM remapped over Flash area, 16MB
#endif
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x220, 0x0,   0x1E0);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,   0x400, 0x400, 0x040);    // SDRAM, 64MB , original address, cacheable, bufferable
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x440, 0x400, 0x040);    // SDRAM, 64MB data area, remapped, cacheable, NON bufferable
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x480, 0x0,   0x180);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0x600, 0x600, 0x001);    // CS1 area, 1MB, NON cacheable, NON bufferable
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x601, 0x0,   0x1FF);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_B,  0x800, 0x200, 0x020);    // NOR Flash, 32MB, remapped, second 16MB are used for data storage
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_MMU_UNMAPPED,    0x820, 0x0,   0x6E0);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0xf00, 0xf00, 0x080);    // Internal SFRs, original address range, NON cacheable, NON bufferable
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB, 0xf80, 0xf80, 0x080);    // Internal SRAM, original address range, NON cacheable, NON bufferable
  OS_ARM_MMU_Enable      (&_TranslationTable[0]);
  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();
  //
  // Initialize interrupt controller
  //
  _InitVIC();
  return 1;  // Always return 1 to enable segment initialization!
}

/*************************** End of file ****************************/
